import React from 'react';
import PropTypes from 'prop-types';
import { injectIntl } from 'react-intl';
import { useSelector, useDispatch } from 'react-redux';
import { BootstrapTable, TableHeaderColumn, SearchField } from 'react-bootstrap-table';
import { trOddEvenClassFormat } from 'utils/htmlUtils';
import { DEFAULT_VALUES, BOOTSTRAP_TABLE_SORT } from 'common/constants';
import { TABLES, VEHICLE_CERTIFICATION } from 'common/ui-labels';
import { selectCheckedVMs, selectVehicles, selectCheckedVehicles } from '../../store/selectors';
import {
    openDeleteVehicleCertModal,
    selectTableVehicle,
    selectTableVehicles,
    unselectTableVehicle,
    openEditVehicleModal,
} from '../../store/actions';


function btnGroup(singleCertSelected, vehiclesIsSelected, callbackDel, callbackEdit) {
    if (!singleCertSelected) return null;
    return (
        <section className="Table">
            <i
                className="fas fa-trash-alt fa-lg table-action-icon"
                data-id="vehicle-cert-class-button-trash"
                onClick={() => {
                    if (vehiclesIsSelected) {
                        callbackDel();
                    }
                }}
            />
            <i
                className="fa fa-plus fa-lg table-action-icon"
                data-id="vehicle-cert-class-button-plus"
                onClick={callbackEdit}
            />
        </section>);
}

function VehiclesTable(props) {
    const { intl } = props;
    const dispatch = useDispatch();
    const vehicles = useSelector(selectVehicles);
    const vehicleModels = useSelector(selectCheckedVMs);
    const selectedVehicles = useSelector(selectCheckedVehicles);
    const tableData = vehicleModels.length > 0 ? vehicles.filter(vehicle => vehicleModels.some(vm => vehicle.vehicleModelId === vm.id)) : vehicles;
    const singleCertSelected = vehicleModels.length === 1;
    
    function openDeleteModal() {
        dispatch(openDeleteVehicleCertModal());
    }
    
    function openEditModal() {
        dispatch(openEditVehicleModal());
    }
    
    const options = {
        btnGroup: () => btnGroup(singleCertSelected, selectedVehicles.length > 0, openDeleteModal, openEditModal),
        searchField: () => <SearchField data-id="vehicles-search" placeholder={TABLES.SEARCH(intl)} />,
        noDataText: TABLES.NO_DATA_TO_DISPLAY,
        beforeShowError: () => true,
        defaultSortName: 'name',
        defaultSortOrder: BOOTSTRAP_TABLE_SORT.ASC,
    };
    const headerStyle = {
        marginBottom: '0',
    };
    
    const selectRow = {
        mode: singleCertSelected ? 'checkbox' : null,
        bgColor: DEFAULT_VALUES.SELECTED_ROW_COLOR,
        clickToSelect: !!singleCertSelected, // click to select, default is false
        selected: selectedVehicles.map(cert => cert.obj_uid),
        onSelect: (row) => {
            dispatch(selectTableVehicle(row));
        },
        onSelectAll: (isSelected, data) => {
            if (!isSelected) {
                dispatch(unselectTableVehicle(data));
            } else {
                dispatch(selectTableVehicles(data));
            }
        },
    };
    
    return (
        <BootstrapTable
            trClassName={trOddEvenClassFormat}
            tableBodyClass="automation-vehicles-table"
            data={tableData}
            options={options}
            selectRow={selectRow}
            headerStyle={headerStyle}
            bodyContainerClass="Admin-BootstrapTable"
            className="middle-align"
            condensed
            search
            hover
        >
            <TableHeaderColumn
                dataField="obj_uid"
                searchable={false}
                hidden
                isKey
            >
                Uid
            </TableHeaderColumn>
            <TableHeaderColumn
                dataField="name"
                tdAttr={{ 'data-id': 'vehicle-name' }}
                width="170"
                headerAlign="center"
                dataSort
            >
                {VEHICLE_CERTIFICATION.NAME}
            </TableHeaderColumn>
            <TableHeaderColumn
                dataField="vehicleModelName"
                tdAttr={{ 'data-id': 'cert-class-name' }}
                dataAlign="center"
                width="120"
                headerAlign="center"
                dataSort
            >
                {VEHICLE_CERTIFICATION.MODEL}
            </TableHeaderColumn>
        </BootstrapTable>
    );
}

VehiclesTable.propTypes = {
    intl: PropTypes.shape({
        formatMessage: PropTypes.func,
    }),
};

VehiclesTable.defaultProps = {
    intl: {
        formatMessage: () => {
        },
    },
};

export default injectIntl(VehiclesTable);
