import React from 'react';
import { connect } from 'react-redux';
import PropTypes from 'prop-types';
import Modal from 'react-modal';
import { history } from 'stores/configureStore';
import { settingUp, userLogout, openUserSessionExpirationModal } from 'stores/actions';
import store from 'stores/LocationStore';
import handleKeyDown from 'utils/downloadUtils';
import { handlePackageVersion, stringToBoolean } from 'utils/utils';
import { isSessionExpired } from 'utils/timeUtils';
import { getLocalStorageValue, getQueryParam } from 'utils';
import { CUSTOM_LOCAL_STORE_KEYS } from 'common';
import Modals from 'modals';

import Login from 'components/Login';
import App from './App';

class AppContainer extends React.Component {
    constructor(props) {
        super(props);
        this.state = {
            timeout: null,
            interval: null,
            hasError: false,
            lastActivityTs: new Date().getTime(),
        };
        const hostName = store.getHostName();
        const targetServer = getQueryParam('target_server');
        const search = targetServer ? `?target_server=${targetServer}` : '';

        // Dynamically add the favicon
        const link = document.createElement('link');
        link.rel = 'shortcut icon';
        link.href = `${hostName}/logo/company-logo-small.png${search}`;
        link.type = 'image/png';
        document.head.appendChild(link);
    }
    
    componentDidMount() {
        Modal.setAppElement('body');
        handlePackageVersion();
        document.addEventListener('keydown', handleKeyDown);
        document.addEventListener('mousemove', this.handleUserActivity);
        document.addEventListener('click', this.handleUserActivity);
        this.interval = setInterval(() => {
            const sessionExpired = isSessionExpired(this.props.userRole, this.props.usersAccessControl, this.state.lastActivityTs, new Date().getTime());
            if (sessionExpired) {
                this.props.dispatch(openUserSessionExpirationModal());
            }
        }, 10_000);
        if (navigator.cookieEnabled) {
            this.props.dispatch(settingUp());
        }
    }

    componentDidCatch(error, errorInfo) {
        console.error('error ', error);
        console.error('errorInfo ', errorInfo);
        this.setState({ hasError: true });
        if (!stringToBoolean(getLocalStorageValue(CUSTOM_LOCAL_STORE_KEYS.APP_DEBUG))) {
            this.props.dispatch(userLogout());
        }
    }
    
    componentWillUnmount() {
        document.removeEventListener('keydown', handleKeyDown);
        document.removeEventListener('mousemove', this.handleUserActivity);
        document.removeEventListener('click', this.handleUserActivity);
    }

    handleUserActivity = () => {
        if (this.state.timeout) {
            clearTimeout(this.state.timeout);
        }
        this.state.timeout = setTimeout(() => {
            this.setState({ lastActivityTs: new Date().getTime() - 1000 })
        }, 1000);
    }
    
    resetError = () => {
        this.setState({ hasError: false });
    }
    
    render() {
        // todo: implement routing for login
        if (this.props.logged && !this.state.hasError) {
            return (
                <div id="reactRootElement">
                    <App history={history} />
                    <Modals />
                </div>
            );
        }
    
        return (
            <div id="reactRootElement">
                <Login resetError={this.resetError} />
                <Modals />
            </div>
        );
    }
}

AppContainer.propTypes = {
    dispatch: PropTypes.func,
    userRole: PropTypes.string,
    logged: PropTypes.bool,
    usersAccessControl: PropTypes.arrayOf(PropTypes.shape({})),
};

AppContainer.defaultProps = {
    dispatch: () => { },
    userRole: '',
    logged: false,
    usersAccessControl: [],
};

const mapStateToProps = (state) => ({
    logged: state.user ? state.user.logged : false,
    userRole: state.user?.onBehalf?.access?.role || state.user?.access?.role,
    usersAccessControl: state.entities.usersAccessControl,
});

export default connect(mapStateToProps)(AppContainer);
