import _Get from 'lodash/get';

import Trackable from 'stores/classes/Trackable';
import { formatPwsConfig } from 'utils/pwsConfigUtils';
import { calculateAllowDefault, separateTrackableAndGroupExceptions } from 'utils/zoneUtils';
import { convertMetersToMillimeters } from 'utils/formatters';
import { makeCopy } from 'utils/simpleUtils';
import { ZONE_MODE, OBJECT_TYPES, T2T_TYPE, DEFAULT_VALUES } from 'common/constants';

export const formatAttachedZone = (params, oldTrackable) => {
    const zone = {};

    zone.dynamic = true;
    zone.zone_mode = ZONE_MODE.ALWAYS_ACTIVE;
    
    zone.zone_exceptions = params.zone_exceptions;
    zone.zone_exceptions_groups = params.zone_exceptions_groups;
    zone.shape = 'circle';
    zone.trackable_object_uid = oldTrackable ? oldTrackable.uid : params.uid;
    zone.zone_geometry = {};
    zone.zone_geometry.radius = params.buffer_radius;
    zone.name = `Buffer (${params.name})`;
    zone.default_allowed = calculateAllowDefault({ apply_to: params.apply_to });
    zone.zone_type_name = 'keep_out';
    zone.priority = 0;
    
    return zone;
};

/**
 * separate trackables_and_groups to tko exceptions and group exceptions
 * @param trackables_and_groups Array[Trackable|Group]
 * @returns {{zone_exceptions_groups: Array[string], zone_exceptions: Array[string]}}
 */
export function getZoneExceptions(trackables_and_groups = []) {
    const zone_exceptions = [];
    const zone_exceptions_groups = [];
    trackables_and_groups.forEach((item) => {
        // if (item.objectType && item.objectType() === OBJECT_TYPES.TRACKABLE) {
        if (item._type === OBJECT_TYPES.TRACKABLE) {
            zone_exceptions.push(item.uid);
        } else if (item._type === OBJECT_TYPES.GROUP) {
            zone_exceptions_groups.push(item.uid);
        }
    });
    return { zone_exceptions_groups, zone_exceptions };
}

/**
 * compare t2t of group and trackable, if not equal return
 * @param params
 * @returns {Object<t2t_cfg>}
 */
export function calculateT2T(params) {
    return { t2t_cfg: formatPwsConfig(params, params.category) };
}

export const formatTrackable = (params, oldTo = {}) => {
    const trackable = {
        uid: params.uid,
        name: params.name,
        tag_config: {
            stationary: params.stationary || false,
        },
        sensor_config: {
            sensors: params.sensor_config,
        },
        trackables_and_groups: params.trackables_and_groups || [],
        custom_info: params.custom_info,
        category: params.category, // requires field API trackable_object POST PUT
    };

    if (params.user_params) {
        trackable.user_params = params.user_params;
    }

    trackable.t2t_no_alarm_uid = [];
    if (params.t2t_no_alarm_uid && params.role === T2T_TYPE.MONITOR) {
        trackable.t2t_no_alarm_uid = params.t2t_no_alarm_uid.split(',');
    }

    const { t2t_cfg } = calculateT2T(params);
    trackable.tag_config.t2t_cfg = t2t_cfg || null;

    if (trackable.tag_config.stationary) {
        trackable.position = {
            x: params.x,
            y: params.y,
            z: convertMetersToMillimeters(params.defaultHeight),
            sl_id: params.area,
        };
    }
    const groups = params.groupList && params.groupList.map((x) => x.uid);
    trackable.default_height = params.defaultHeight ? params.defaultHeight * 1000 : DEFAULT_VALUES.trackable_default_height * 1000;
    trackable.position_update_interval = !Number.isNaN(params.position_update_interval) ? params.position_update_interval : DEFAULT_VALUES.trackable_update_interval;
    
    let create_attached_zone;
    let update_attached_zone;
    let delete_attached_zone_uid;
    
    if (params.buffer) {
        // self zone:  create attached zone
        // buffer_radius zone radius
        // active_buffer make zone active
        // buffer_exceptions => bufferExceptionLists  trackable objects witch can move in attached zone without alarms
        if (!params.self_zone_uid) {
            create_attached_zone = formatAttachedZone(params);
            if (create_attached_zone) {
                create_attached_zone.zone_mode = params.active_buffer ? ZONE_MODE.ALWAYS_ACTIVE : ZONE_MODE.DEACTIVATED;
                // trackable.self_zone_uid = create_attached_zone.uid;
                const { zone_exceptions, zone_exceptions_groups } = getZoneExceptions(trackable.trackables_and_groups);
                
                create_attached_zone.zone_exceptions = zone_exceptions;
                create_attached_zone.zone_exceptions_groups = zone_exceptions_groups;
            }
        } else {
            update_attached_zone = params.attached_zone;
            if (update_attached_zone) {
                update_attached_zone.default_allowed = calculateAllowDefault({ apply_to: params.apply_to });
                update_attached_zone.zone_mode = params.active_buffer ? ZONE_MODE.ALWAYS_ACTIVE : ZONE_MODE.DEACTIVATED;
                
                const { zone_exceptions, zone_exceptions_groups } = getZoneExceptions(trackable.trackables_and_groups);
                
                update_attached_zone.zone_exceptions = zone_exceptions;
                update_attached_zone.zone_exceptions_groups = zone_exceptions_groups;
                update_attached_zone.zone_geometry.radius = params.buffer_radius; // * 1000*/;
            }
        }
    } else if (!params.buffer && oldTo.self_zone_uid) {
        delete_attached_zone_uid = oldTo.self_zone_uid;
        trackable.self_zone_uid = null;
    }
    
    return {
        trackable,
        groups,
        create_attached_zone,
        update_attached_zone,
        delete_attached_zone_uid,
    };
};

export const formatCreatedTrackable = (values, { trackables, groups }) => {
    const valueCopy = makeCopy(values);
    valueCopy.groupList = values.groupList || [];
    delete valueCopy.selector;
    valueCopy.sensor_config = DEFAULT_VALUES.empty_array;
    const { trackablesExceptions, groupsExceptions } = separateTrackableAndGroupExceptions(valueCopy.trackables_and_groups, trackables, groups);
    
    valueCopy.zone_exceptions = trackablesExceptions;
    valueCopy.zone_exceptions_groups = groupsExceptions;
    const t2t_cfg = formatPwsConfig(valueCopy, values.category);
    valueCopy.tag_config = { t2t_cfg };
    valueCopy.low_latency_link_config = t2t_cfg;

    return valueCopy;
};

/**
 *
 * @param trackable
 * @param groupList
 * @param editedTrackable
 * @param isEqualLLL {bool} is lll config for group changes
 * @param nodesMap
 * @returns Object { hasProperVersion<boolean>, hasNode<boolean>, node<object> }
 */
export function getValidationInfo(trackable, groupList, editedTrackable, isEqualLLL, nodesMap) {
    const mac = _Get(trackable, 'node.mac_address', null);
    const node = nodesMap.byMac[(mac || '').toUpperCase()];
    
    if (!node || isEqualLLL) {
        return { hasProperVersion: true, hasNode: !!node, node };
    }

    const tko = new Trackable(trackable);
    const hasRole = tko.hasLowLatencyConfigurations();
    const hasProperVersion = hasRole ? tko.hasProperVersionForLowLatencyLink(node) : !hasRole;
    const hasNode = !!node;
    
    return { hasProperVersion, hasNode, node };
}
