import React, { Component } from 'react';
import PropTypes from 'prop-types';
import _sortBy from 'lodash/sortBy';
import _Get from 'lodash/get';
import { SortDirection } from 'react-virtualized';

import { DEFAULT_VALUES, BOOTSTRAP_TABLE_SORT } from 'common/constants';
import { trOddEvenClassFormat } from 'utils/htmlUtils';
import { changeSelectedItems, selectItems, clearAdminPageSelectedItems } from 'stores/actions/adminPageActions';


const withSortSelect = WrappedComponent => (
    class extends Component {
        static propTypes = {
            totalCount: PropTypes.number,
            defaultSortName: PropTypes.string,
            defaultSortOrder: PropTypes.string,
            allItems: PropTypes.oneOfType([PropTypes.shape(), PropTypes.array]),
            dispatch: PropTypes.func,
        };
        static defaultProps = {
            totalCount: 0,
            defaultSortName: 'name',
            defaultSortOrder: BOOTSTRAP_TABLE_SORT.ASC,
            allItems: new Map(),
            dispatch: () => {},
        };
        
        constructor(props) {
            super(props);
            
            this.totalCount = props.totalCount;
            this.dispatch = props.dispatch;
            this.myRef = React.createRef();

            this.state = {
                sortBy: props.defaultSortName,
                sortDirection: props.defaultSortOrder,
                allSelected: false,
                selectedElements: [],
                filteredItems: props.allItems,
                isItemsFiltered: false,
            };
        }
        
        static getDerivedStateFromProps(props, state) {
            const filteredItems = !state.isItemsFiltered
                ? [...props.allItems.values()]
                : state.filteredItems.reduce((result, filteredItem) => {
                    const item = props.allItems.get(filteredItem.uid);
                    if (item) { result.push(item); }
                    return result;
                }, []);
            if (!state.selectedElements || !props.selectedItems || state.selectedElements.length === props.selectedItems.length) {
                return {
                    filteredItems,
                };
            }
            const selectedElements = props.selectedItems.map(item => item.uid);
            return {
                filteredItems,
                selectedElements,
            };
        }
        
        componentDidUpdate() {
            const selectedCount = this.state.selectedElements.length;
            const itemsCount = this.totalCount;
            
            if (this._hasCurrentInRef() && this.myRef.current.indeterminate && (selectedCount === itemsCount || selectedCount === 0)) {
                this.myRef.current.indeterminate = false;
            } else if (this._hasCurrentInRef() && !this.myRef.current.indeterminate && selectedCount > 0 && (selectedCount !== itemsCount)) {
                this.myRef.current.indeterminate = true;
            }
        }
        
        // FILTER SECTION
        _setFilterItems = (filteredItems, isItemsFiltered = false) => {
            this.setState({
                filteredItems,
                isItemsFiltered,
            });
        };
    
        // SORTING SECTION
        _sort = ({ sortBy, sortDirection }) => {
            // sortBy use in child class
            this.setState({ sortBy, sortDirection });
        };
    
        _sortFunction = (item) => {
            const { sortBy } = this.state;
            const itemToSort = item[sortBy];
            if (sortBy === 'since') {
                return itemToSort.range;
            } else if (sortBy === 'battery') {
                return _Get(itemToSort, 'pr', -1);
            }
            return String(itemToSort).toLowerCase();
        };
    
        _sortDisplayItems = (initialItems) => {
            if (this.state.sortBy) {
                const displayItems = _sortBy(initialItems, [this._sortFunction]);
                if (this.state.sortDirection === SortDirection.DESC) {
                    return displayItems.reverse();
                } else if (this.state.sortDirection === SortDirection.ASC) {
                    return displayItems;
                }
            }
            return initialItems;
        };
    
        // CHECKBOX SECTION
        _hasCurrentInRef = () => this.myRef && this.myRef.current;
        
        _setHeaderCheckboxView = (selectedCount, itemsCount) => {
            if (this._hasCurrentInRef() && selectedCount !== itemsCount) {
                this.myRef.current.indeterminate = selectedCount > 0;
            } else if (this._hasCurrentInRef() && selectedCount === itemsCount) {
                this.myRef.current.indeterminate = false;
            }
        };
        
        _handleHeaderCheckboxClick = (items) => {
            if (!this.state.allSelected && !this.state.selectedElements.length) {
                this.dispatch(selectItems(items));
                this.setState({ allSelected: true, selectedElements: items.map(item => item.uid) });
            } else {
                this.dispatch(clearAdminPageSelectedItems());
                this.setState({ allSelected: false, selectedElements: [] });
            }
        };
        
        _headerCheckbox = items => (
            <input
                type="checkbox"
                ref={this.myRef}
                checked={this.state.allSelected}
                onChange={() => {}}
                onClick={() => { this._handleHeaderCheckboxClick(items); }}
            />
        );
        
        _handleSelectRow = (itemsCount, { event, rowData }) => {
            if (event.target.getAttribute('data-info') === 'actions-section') {
                return false;
            }
            
            const { selectedElements } = this.state;
            const selectedItemIndex = selectedElements.indexOf(rowData.uid);
            
            if (selectedItemIndex === -1) {
                selectedElements.push(rowData.uid);
                const selectedCount = selectedElements.length;
                
                this.setState({
                    selectedElements,
                    allSelected: selectedCount === itemsCount,
                }, () => {
                    this._setHeaderCheckboxView(selectedCount, itemsCount);
                });
            } else {
                selectedElements.splice(selectedItemIndex, 1);
                this.setState({
                    selectedElements,
                    allSelected: false,
                }, () => {
                    this._setHeaderCheckboxView(selectedElements.length, itemsCount);
                });
            }
            
            this.dispatch(changeSelectedItems(rowData));
            return true;
        };
        
        _checkboxButton = (rowData) => {
            const isChecked = this.state.selectedElements.indexOf(rowData.uid) !== -1;
            return (
                <section>
                    <input type="checkbox" checked={isChecked} onChange={() => {}} />
                </section>
            );
        };
        
        _trClassFormat = (items, { index }) => {
            const currentItem = items[index];
            if (index >= 0 && currentItem) {
                return (currentItem.project_uid ? trOddEvenClassFormat(currentItem, index) : 'Node-Unassigned');
            }
            return 'Virtual-Table-Header-Row';
        };
        
        _trStyleFormat = (items, { index }) => {
            const style = { alignItems: 'stretch' };
            const currentItem = items[index];
            if (currentItem && this.state.selectedElements.indexOf(currentItem.uid) !== -1) {
                return { ...style, backgroundColor: DEFAULT_VALUES.SELECTED_ROW_COLOR };
            }
            return style;
        };
    
        render() {
            return (
                <WrappedComponent
                    sort={this._sort}
                    trClassFormat={this._trClassFormat}
                    trStyleFormat={this._trStyleFormat}
                    checkboxButton={this._checkboxButton}
                    headerCheckbox={this._headerCheckbox}
                    handleSelectRow={this._handleSelectRow}
                    sortDisplayItems={this._sortDisplayItems}
                    setFilterItems={this._setFilterItems}
                    sortBy={this.state.sortBy}
                    filteredItems={this.state.filteredItems}
                    sortDirection={this.state.sortDirection}
                    {...this.props}
                />
            );
        }
    }
);

export default withSortSelect;
