import React, { Component, Fragment } from 'react';
import PropTypes from 'prop-types';
import { injectIntl } from 'react-intl';
import { connect } from 'react-redux';
import { getTimeZoneOfProject, getFullDateByTz } from 'utils/timeUtils';
import { ALARM_TYPES } from 'common/constants';
import { DB_NAMES, ALARM_NOTIFICATIONS } from 'common/ui-labels';
import { Alarm } from 'stores/classes';
import { Zone as ZoneSelector, Trackable as TrackableSelector } from 'stores/classes/Selectors';


class AlarmNotifications extends Component {
    static propTypes = {
        isAlarmsListShowed: PropTypes.bool,
        alarms: PropTypes.arrayOf(PropTypes.shape({})),
        timeZone: PropTypes.string,
        showAlarmsList: PropTypes.func,
        intl: PropTypes.shape({
            formatMessage: PropTypes.func,
        }),
        hideAlarmsList: PropTypes.func,
    };
    
    static defaultProps = {
        isAlarmsListShowed: false,
        alarms: [],
        timeZone: 'Etc/GMT',
        showAlarmsList: () => {
        },
        intl: {
            formatMessage: () => {
            },
        },
        hideAlarmsList: () => {

        },
    };
    
    constructor(props) {
        super(props);
        
        this.state = {
            dismissedAlarms: [],
        };

        this.alarmsList = React.createRef();
    }

    componentDidMount() {
        document.addEventListener('mousedown', this.handleClickOutsideAlarmsButton);
    }

    componentDidUpdate(prevProps) {
        const { alarms: prevAlarms } = prevProps;
        const { alarms: currentAlarms, showAlarmsList } = this.props;
    
        if (!prevAlarms.length && currentAlarms.length) {
            showAlarmsList();
        }
    }
    
    componentWillUnmount() {
        document.removeEventListener('mousedown', this.handleClickOutsideAlarmsButton);
    }

    getTrackableName = (uid) => {
        const trackable = TrackableSelector.getTrackableByUid(uid);
        return trackable ? trackable.name : 'Undefined';
    };

    getZoneName = (uid) => {
        const zone = ZoneSelector.getZoneByUid(uid);
        return {
            isKeepIn: (zone && zone.zone_type_name === 'keep_in') || false,
            isKeepOut: (zone && zone.zone_type_name === 'keep_out') || false,
            zoneName: (zone && zone.name) || 'Undefined',
        };
    };

    getAlarmDate = (createdAtTime, timeZone) => getFullDateByTz(createdAtTime, timeZone);

    getAlarmText = (alarm) => {
        let result = '';
        const { intl } = this.props;
        const trackableName = alarm.obj_uid && this.getTrackableName(alarm.obj_uid);
        switch (alarm.alarm_type_id) {
            case ALARM_TYPES.ANCHOR_MOVED:
                result = DB_NAMES.ANCHOR_MOVED(intl);
                break;
            case ALARM_TYPES.BUFFER_VIOLATION: {
                const { zoneName } = this.getZoneName(alarm.zone_uid);
                result = ALARM_NOTIFICATIONS.ZONE_VIOLATION_ENTERED(intl, trackableName, zoneName);
                break;
            }
            case ALARM_TYPES.EMERGENCY_BUTTON:
                result = ALARM_NOTIFICATIONS.EMERGENCY_BUTTON(intl, trackableName);
                break;
            case ALARM_TYPES.ZONE_VIOLATION: {
                const { zoneName, isKeepIn, isKeepOut } = this.getZoneName(alarm.zone_uid);
                if (isKeepIn) {
                    result = ALARM_NOTIFICATIONS.ZONE_VIOLATION_EXITED(intl, trackableName, zoneName);
                } else if (isKeepOut) {
                    result = ALARM_NOTIFICATIONS.ZONE_VIOLATION_ENTERED(intl, trackableName, zoneName);
                }
                break;
            }
            case ALARM_TYPES.DEVICE_ABSENT: {
                const { zoneName } = this.getZoneName(alarm.zone_uid);
                result = ALARM_NOTIFICATIONS.DEVICE_ABSENT(intl, trackableName, zoneName);
                break;
            }
            case ALARM_TYPES.NODE_LOW_BATTERY:
                result = ALARM_NOTIFICATIONS.NODE_LOW_BATTERY(intl, trackableName);
                break;
            case ALARM_TYPES.MAC_ADDRESS_CONFLICT:
                // todo: implement
                result = DB_NAMES.MAC_ADDRESS_CONFLICT(intl);
                break;
            case ALARM_TYPES.PWS_SAFETY: {
                const { name, contact_name } = alarm.additional_info || {};
                result = ALARM_NOTIFICATIONS.PWS_SAFETY(intl, name, contact_name);
                break;
            }
            case ALARM_TYPES.UNCERTIFIED_OPERATION:
                result = `${DB_NAMES.UNCERTIFIED_OPERATION(intl)}: ${trackableName}`;
                break;
            default:
                result = 'Unknown alarm type';
        }
        return result;
    };

    handleClickOutsideAlarmsButton = (event) => {
        const alarmsList = this.alarmsList.current; // from ref
        const { hideAlarmsList } = this.props;

        if (alarmsList && !alarmsList.contains(event.target)) {
            hideAlarmsList();
        }
    };

    render() {
        const { timeZone, alarms, isAlarmsListShowed, hideAlarmsList } = this.props;
        const { dismissedAlarms } = this.state;
        const alarmsLength = alarms.length;

        const sortedAlarms = alarms
            .filter(alarm => !dismissedAlarms.find(disAlarm => disAlarm.uid === alarm.uid))
            .sort((prevAlarm, nextAlarm) => {
                if (nextAlarm.created_at && prevAlarm.created_at) {
                    const createdAtTimeNext = new Date(nextAlarm.created_at).getTime();
                    const createdAtTimePrev = new Date(prevAlarm.created_at).getTime();
                    return createdAtTimeNext - createdAtTimePrev;
                }
                return 0;
            });

        return (
            <Fragment>
                {
                    isAlarmsListShowed &&
                    <div
                        className="alarms-list"
                        ref={this.alarmsList}
                    >
                        <button
                            className="cross"
                            onClick={hideAlarmsList}
                        >
                            <i className="far fa-times-circle" />
                        </button>
                        <div className="alarms-list-wrapper">
                            {
                                alarmsLength > 0
                                    ? sortedAlarms.map(alarm => (
                                        <div className="alarm" key={alarm.uid}>
                                            <div className="alarm-text">
                                                {this.getAlarmText(alarm, timeZone)}
                                            </div>
                                            <div className="alarm-date-time">
                                                {this.getAlarmDate(alarm.created_at || 0, timeZone)}
                                            </div>
                                        </div>))
                                    : ALARM_NOTIFICATIONS.NO_ALARMS
                            }
                        </div>
                    </div>
                }
            </Fragment>
        );
    }
}

const mapStateToProps = (state) => {
    const project = state.appContext.project;
    const timeZone = getTimeZoneOfProject(project);

    return {
        alarms: Alarm.getAllAlarms(state),
        timeZone,
    };
};

export default connect(mapStateToProps)(injectIntl(AlarmNotifications));
