import React from 'react';
import { connect } from 'react-redux';
import { LayerGroup, GeoJSON } from 'react-leaflet';
import DivIcon from 'react-leaflet-div-icon';
import PropTypes from 'prop-types';
import Position from 'stores/classes/Position';
import { calculateGridData, calculateGridGeometry } from 'utils/formatters';
import { getGridStepByZoom } from 'utils/simpleUtils';
import { DEFAULT_VALUES, LEAFLET_LABEL_TYPE } from 'common/constants';

class LeafletScaling extends LayerGroup {
    static propTypes = {
        zoom: PropTypes.number,
        isShowingScale: PropTypes.bool,
        bounds: PropTypes.shape({}),
    };
    
    static defaultProps = {
        zoom: DEFAULT_VALUES.MAP_ZOOM.DEFAULT,
        isShowingScale: true,
        bounds: {},
    };

    renderGridLines = (grid) => {
        const { gridGeometry, axesGeometry, axesKey, gridKey  } = grid;
        const result = [];

        if (Array.isArray(gridGeometry) && gridGeometry.length > 0) {
            result.push(<GeoJSON
                data={{ type: 'Feature', geometry: { type: 'GeometryCollection', geometries: gridGeometry } }}
                key={gridKey}
                style={() => ({
                    color: 'darkblue',
                    weight: 0.3,
                })}
            />);
        }

        if (Array.isArray(axesGeometry) && axesGeometry.length > 0) {
            result.push(<GeoJSON
                data={{ type: 'Feature', geometry: { type: 'GeometryCollection', geometries: axesGeometry } }}
                key={axesKey}
                style={() => ({
                    color: 'green',
                    weight: 1.5,
                })}
            />);
        }
        return result;
    };

    renderGridLabels = (gridData) => {
        const { labelPositions, } = gridData;
        return labelPositions.map((label, index) => {
            const position = new Position(label);
            const style = label.type === LEAFLET_LABEL_TYPE.HORIZONTAL ? { fontSize: '2em', marginLeft: '20px', color: 'grey' } : { fontSize: '2em', marginTop: '-30px', color: 'grey' };
            return (
                <div key={index}>
                    <DivIcon position={position.latlng}>
                        <div style={style}>{label.val}</div>
                    </DivIcon>
                </div>
            );
        });
    };

    render() {
        const { zoom, bounds, isShowingScale } = this.props;
        if (!bounds || Object.keys(bounds).length === 0) return null;
        const step = getGridStepByZoom(zoom);
        const gridData = calculateGridData(step, bounds);
        const grid = calculateGridGeometry(gridData, step);

        return (
            <div>
                {isShowingScale && this.renderGridLabels(gridData) }
                {isShowingScale && this.renderGridLines(grid) }
            </div>
        );
    }
}

const mapStateToProps = (state, props) => ({
    zoom: props.zoom ? Math.round(props.zoom) : Math.round(state.siteViewFilter.leaflet.zoom),
    isShowingScale: props.isShowingScale || state.siteViewFilter.layerFilter.isShowingScale,
    bounds: props.bounds || state.siteViewFilter.leaflet.bounds,
});

export default connect(mapStateToProps)(LeafletScaling);
