import React, { Fragment } from 'react';
import { LayerGroup, Polyline } from 'react-leaflet';
import _isEqual from 'lodash/isEqual';
import { LeafletLine, SpaghettiMarker } from 'stores/classes';


class LeafletSpaghettiDiagram extends LayerGroup {
    shouldComponentUpdate(nextProps) {
        return !_isEqual(this.props.timeframe, nextProps.timeframe)
            || !_isEqual(this.props.filteredTrailUids, nextProps.filteredTrailUids);
    }

    getMarkerInTrail = (positions, checkPoint, trackableName, color, timeZone) => {
        let marker;
        let minDistance;
        const positionsCount = positions.length;
        for (let i = 0; i < positionsCount - 1; i += 1) {
            const startPos = positions[i];
            const endPos = positions[i + 1];

            const line = new LeafletLine(startPos, endPos);
            const currentDistance = line.deltaBetweenLineLengthAndPointSegments(checkPoint);

            if (!minDistance || currentDistance < minDistance) {
                minDistance = currentDistance;
                marker = new SpaghettiMarker(startPos, endPos, checkPoint, trackableName, color, timeZone).getMarker();
            }
        }

        return marker;
    };

    render() {
        const { trackables, showPolylineMarker, filteredTrailUids, timeZone } = this.props;

        // filter trackable positions according to the trails filter
        const trackablesToDisplay = Object.values(trackables).reduce((memo, trackable) => {
            if (filteredTrailUids.includes(trackable.uid)) {
                memo.push({
                    uid: trackable.uid,
                    color: trackable.color,
                    trackableName: trackable.name,
                    positions: trackable.positions,
                });
            }
            return memo;
        }, []);

        return (
            <Fragment>
                {
                    trackablesToDisplay.map((tko) => {
                        const { uid, positions, trackableName, color } = tko;

                        return positions.map((positionsSubList, index) => (
                            <Polyline
                                ref={`poly${uid}_${index}`}
                                key={index}
                                smoothFactor={0}
                                weight={2}
                                positions={positionsSubList}
                                color={color}
                                clickTolerance={2}
                                onClick={(e) => {
                                    const marker = this.getMarkerInTrail(positionsSubList, e.latlng, trackableName, color, timeZone);
                                    if (marker) {
                                        showPolylineMarker(marker);
                                    }
                                }}
                            />
                        ));
                    })
                }
            </Fragment>
        );
    }
}

export default LeafletSpaghettiDiagram;

