import React from 'react';
import PropTypes from 'prop-types';
import { connect } from 'react-redux';
import { injectIntl } from 'react-intl';
import { LayerGroup, Popup } from 'react-leaflet';
import DivIcon from 'react-leaflet-div-icon';
import _filter from 'lodash/filter';
import _isEmpty from 'lodash/isEmpty';
import _negate from 'lodash/negate';
import _flatten from 'lodash/flatten';

import { selectIdleTimeMarker } from '../../stores/actions';
import { translateDegreesToMillimeters, degreeToLength, millisecondsToHumanReadable } from '../../utils';
import { SITE_ANALYTICS_LABELS } from '../../common';


class LeafletSpaghettiIdleTimeMarkers extends LayerGroup {
    static propTypes = {
        dispatch: PropTypes.func,
        step: PropTypes.number,
        markers: PropTypes.arrayOf(PropTypes.array),
        isShowingGrid1m: PropTypes.bool,
        isShowingGrid5m: PropTypes.bool,
    };

    static defaultProps = {
        dispatch: () => {
        },
        step: 1,
        markers: [],
        isShowingGrid1m: false,
        isShowingGrid5m: false,
    };

    onIdleTimeMarkerClick = (idleTimeMarker) => {
        const { dispatch } = this.props;

        dispatch(selectIdleTimeMarker({
            ...idleTimeMarker,
        }));
    };

    onClosePopup = () => this.props.dispatch(selectIdleTimeMarker(null));

    getIdlePositions = (trackables) => {
        const { sublocation } = this.props;

        return Object.values(trackables).map(trackable => (_isEmpty(trackable.idle_positions) ? [] : trackable.idle_positions.map((to) => {
            const pos = translateDegreesToMillimeters(to, sublocation);
            return {
                x: pos.x,
                y: pos.y,
                pos: to,
                ts: to.ts,
                idle: to.idle,
                name: trackable.name,
                color: trackable.color,
            };
        })));
    };

    positionInMetersToDisplay = (pos) => {
        const posInMeters = {
            x: parseInt(degreeToLength(pos.lng)) / 10,
            y: parseInt(degreeToLength(pos.lat)) / 10,
        };
        return `( ${posInMeters.x}; ${posInMeters.y} )`;
    };

    render() {
        const { selectedIdleTimeMarker, trackables, filteredTrailUids, intl } = this.props;

        const trackablesToDisplay = Object.values(trackables).reduce((memo, trackable) => {
            if (filteredTrailUids.includes(trackable.uid)) {
                memo.push({
                    name: trackable.name,
                    color: trackable.color,
                    idle_positions: trackable.idle_positions,
                });
            }
            return memo;
        }, []);

        const trackablesWithIdleTime = _flatten(_filter(this.getIdlePositions(trackablesToDisplay), _negate(_isEmpty)));
        const isSelectedMarkerVisible = selectedIdleTimeMarker && Object.values(trackablesWithIdleTime)
            .some(to => to.ts === selectedIdleTimeMarker.ts);

        return (
            <div>
                {
                    trackablesWithIdleTime.length > 0 && trackablesWithIdleTime.map((idleTimePos, i) => (
                        <DivIcon
                            key={i}
                            ref={i}
                            className="leaflet-idletime-marker far fa-clock"
                            position={idleTimePos.pos}
                            styles={{ color: 'red' }}
                            onClick={this.onIdleTimeMarkerClick.bind(this, idleTimePos)}
                        >
                            <span />
                        </DivIcon>
                    ))
                }
                {
                    selectedIdleTimeMarker && isSelectedMarkerVisible &&
                    <Popup
                        className="spaghetti-idleTime-marker"
                        position={selectedIdleTimeMarker.pos}
                        onClose={this.onClosePopup}
                    >
                        <div>
                            <div className="RPP-BlockHeader" title={selectedIdleTimeMarker.trackableName}>
                                <h4
                                    className="RPP-BlockBtnTitle title-popup text-center"
                                    style={{ color: selectedIdleTimeMarker.color }}
                                    title={selectedIdleTimeMarker.name}
                                >
                                    {selectedIdleTimeMarker.name}
                                </h4>
                            </div>
                            <div className="RPP-PopUpRowsWrapper">
                                <div className="popup-row row">
                                    <div className="col-md-4 popup-field-label">{SITE_ANALYTICS_LABELS.IDLE_TIME(intl)}</div>
                                    <div className="col-md-8 popup-field-value">
                                        {millisecondsToHumanReadable(selectedIdleTimeMarker.idle)}
                                    </div>
                                </div>
                                <div className="popup-row row">
                                    <div className="col-md-4 popup-field-label">{SITE_ANALYTICS_LABELS.LOCATION(intl)}</div>
                                    <div className="col-md-8 popup-field-value">
                                        {
                                            this.positionInMetersToDisplay(selectedIdleTimeMarker.pos)
                                        }
                                    </div>
                                </div>
                            </div>
                        </div>
                    </Popup>
                }
            </div>
        );
    }
}

const mapStateToProps = state => ({
    selectedIdleTimeMarker: state.spaghetti.selectedIdleTimeMarker,
    sublocation: state.siteView.selectedSublocation,
});

export default connect(mapStateToProps)(injectIntl(LeafletSpaghettiIdleTimeMarkers));

