import React, { PureComponent } from 'react';
import PropTypes from 'prop-types';
import { injectIntl } from 'react-intl';
import { connect } from 'react-redux';
import Control from 'react-leaflet-control';
import _isEqual from 'lodash/isEqual';
import _Unique from 'lodash/uniq';
import _Difference from 'lodash/difference';
import deepcopy from 'deepcopy';

import { FILTER, WIN_ANCHOR_TYPE, SITE_VIEW_FILTER, FILTER_UNRESPONSIVE, STORAGE_KEY, USER_ROLES } from 'common';
import { formatDbNameToHumanReadable } from 'utils/htmlUtils';
import { userHasPrivileges } from 'utils/utils';
import { getTrackablesBySublocation } from 'utils/trackableUtils';
import {
    openFilter,
    openCloseGroupFilter,
    openCloseUnresponsiveFilter,
    closeFilters,
    changeFilter,
    changeGroupFilter,
    changeUnresponsiveFilter,
} from 'stores/actions';
import { Group } from 'stores/classes';
import locationStore from 'stores/LocationStore';


class SiteViewFilter extends PureComponent {
    static propTypes = {
        groups: PropTypes.arrayOf(PropTypes.shape()),
        filterGroupUids: PropTypes.arrayOf(PropTypes.string),
        unselectedUids: PropTypes.arrayOf(PropTypes.string),
        objectFilter: PropTypes.shape(),
        dispatch: PropTypes.func,
        intl: PropTypes.shape({
            formatMessage: PropTypes.func,
        }),
        position: PropTypes.string,
        currentRole: PropTypes.string,
        showGroups: PropTypes.bool,
        isSpaghettiShowing: PropTypes.bool,
    };
    
    static defaultProps = {
        groups: [],
        filterGroupUids: [],
        unselectedUids: [],
        objectFilter: {},
        dispatch: () => {
        },
        intl: {
            formatMessage: () => {
            },
        },
        position: '',
        currentRole: '',
        showGroups: true,
        isSpaghettiShowing: false,
    };
    
    constructor(props) {
        super(props);

        this.dataInfo = 'NoClose';
        this.groupCheckbox = null;
        this.unresponsiveCheckbox = null;
    }
    
    componentDidMount() {
        this.checkIndeterminateStatuses();
    }
    
    componentDidUpdate() {
        this.checkIndeterminateStatuses();
    }
    
    checkIndeterminateStatuses() {
        if (this.groupCheckbox) {
            this.addIndeterminateToGroupCheckbox();
        }
        if (this.unresponsiveCheckbox) {
            this.addIndeterminateToUnresponsiveCheckbox();
        }
    }

    addRemoveFilter = (_value) => {
        const value = {};
        const _objectFilter = this.props.objectFilter;
        switch (_value) {
            case FILTER.ANCHOR: {
                value.showAnchor = !_objectFilter.showAnchor;
                break;
            }
            case FILTER.BRIDGE: {
                value.showBridges = !_objectFilter.showBridges;
                break;
            }
            case FILTER.GROUP: {
                value.showGroups = !_objectFilter.showGroups;
                value.selectedGroups = value.showGroups ? locationStore.getGroups().map(g => g.uid).concat(FILTER.NA) : [];
                break;
            }
            case FILTER.STATIONARY: {
                value.showStationary = !_objectFilter.showStationary;
                break;
            }
            default: {
                break;
            }
        }
    
        const objectFilter = {
            ..._objectFilter,
            ...value,
            isOpenObjectFilter: true,
        };
        locationStore.setToLocalStorage(STORAGE_KEY.LAST_STATE_OF_OBJECT_FILTER, objectFilter);
        
        this.props.dispatch(changeFilter(objectFilter));
    };

    addRemoveGroupFilter = (filter) => {
        const _objectFilter = deepcopy(this.props.objectFilter);
        let selectedGroups;
        const oldSelectedGroups = _objectFilter.selectedGroups;
        const idx = oldSelectedGroups.indexOf(filter);
        if (~idx) {
            // remove
            oldSelectedGroups.splice(idx, 1);
            selectedGroups = [...oldSelectedGroups];
        } else {
            // add
            selectedGroups = [...oldSelectedGroups, filter];
        }
    
        const objectFilter = {
            ..._objectFilter,
            selectedGroups,
            isOpenObjectFilter: true,
            isOpenGroupFilter: true,
        };
    
        locationStore.setToLocalStorage(STORAGE_KEY.LAST_STATE_OF_OBJECT_FILTER, objectFilter);
        
        this.props.dispatch(changeGroupFilter(objectFilter));
    };
    
    addIndeterminateToGroupCheckbox() {
        const { filterGroupUids, unselectedUids } = this.props;
        this.groupCheckbox.indeterminate = unselectedUids.length > 0 && !_isEqual(unselectedUids, filterGroupUids);
    }
    
    addIndeterminateToUnresponsiveCheckbox() {
        const { showInactive, inactivityValue } = this.props.objectFilter;
        this.unresponsiveCheckbox.indeterminate = showInactive && !!inactivityValue;
    }

    updateUnresponsiveFilter = ({ filter, value }) => {
        const _objectFilter = this.props.objectFilter;
        const _value = {};
        if (filter === FILTER.UNRESPONSIVE) {
            _value.showInactive = !_objectFilter.showInactive;
            _value.inactivityValue = null;
        } else if (value) {
            _value.showInactive = true;
            _value.inactivityValue = value;
        }
    
        const objectFilter = {
            ..._objectFilter,
            ..._value,
            isOpenObjectFilter: true,
        };
    
        locationStore.setToLocalStorage(STORAGE_KEY.LAST_STATE_OF_OBJECT_FILTER, objectFilter);
    
        this.props.dispatch(changeUnresponsiveFilter(objectFilter));
    };
    
    openCloseGroupList = (event) => {
        if (event.relatedTarget && event.relatedTarget.getAttribute('data-name') !== 'groups') {
            this.props.dispatch(openCloseGroupFilter());
        }
    };
    
    openCloseUnresponsiveList = (event) => {
        if (event.relatedTarget && event.relatedTarget.getAttribute('data-name') !== 'inactive') {
            this.props.dispatch(openCloseUnresponsiveFilter());
        }
    };
    
    openList = () => {
        this.props.dispatch(openFilter()).then(() => {
            this.checkIndeterminateStatuses();
        });
    };
    
    closeFilter = (event) => {
        if (event.relatedTarget && event.relatedTarget.getAttribute('data-info') !== this.dataInfo) {
            this.props.dispatch(closeFilters());
        }
    };
    
    renderItem = group => (
        <li
            key={group.uid}
            className="Group-Elem-2"
            data-name="groups"
            data-info={this.dataInfo}
        >
            <input
                type="checkbox"
                data-name="groups"
                id={group.uid}
                data-info={this.dataInfo}
                checked={!!this.props.objectFilter.selectedGroups.find(filter => filter === group.uid)}
                onChange={this.addRemoveGroupFilter.bind(this, group.uid)}
            />

            <label
                style={{
                    marginBottom: '-1px',
                }}
                htmlFor={group.uid}
                data-name="groups"
                data-info={this.dataInfo}
            >
                {group.name}
            </label>
        </li>
    );
    
    renderColumnList = (items, columnCount) => (
        <ul
            style={{
                columns: columnCount,
                listStyleType: 'none',
                padding: 0,
                columnGap: 0,
                background: columnCount > 1 && '#fff',
            }}
            className="DropDown-List-Group"
            onMouseOut={(event) => {
                this.openCloseGroupList(event);
                this.closeFilter(event);
            }}
            data-name="groups"
            data-info={this.dataInfo}
        >
            {items.map(group => this.renderItem(group))}
        </ul>
    );
    
    renderFilterGroupList = () => {
        const sorted = this.props.groups.sort((a, b) => (a.name > b.name ? 1 : -1));
        const items = [{ name: FILTER.NA, uid: FILTER.NA }].concat(sorted);
        const len = items.length;
        const columnCount = Math.min(4, Math.ceil(len / 14));
        
        const itemCount = Math.min(14, Math.ceil(len / columnCount));
        const width = len < 56 ? (columnCount * 195) : (columnCount * 195) + 20;
        
        return (
            <div
                className="DropDown-List-Wrap"
                data-info={this.dataInfo}
                style={{
                    height: `calc(${itemCount * 40} + ${2} px)`, // 14 items with height 40
                    width: `${width}px`, // 200 column width
                }}
                data-name="groups"
            >
                {this.renderColumnList(items, columnCount)}
            </div>
        );
    };
    
    renderFilterUnresponsiveList = () => (
        <div
            className="DropDown-List-Wrap"
            data-info={this.dataInfo}
            style={{
                width: '195px',
            }}
            data-name="groups"
        >
            <div
                className="DropDown-List-Group"
                onMouseOut={(event) => {
                    this.openCloseUnresponsiveList(event);
                    this.closeFilter(event);
                }}
                data-info={this.dataInfo}
            >
                {FILTER_UNRESPONSIVE.ranges.map(range => (
                    <div
                        className="Group-Elem"
                        data-name="inactive"
                        key={range.value}
                        data-info={this.dataInfo}
                    >
                        <input
                            type="radio"
                            data-name="inactive"
                            id={range.value}
                            data-info={this.dataInfo}
                            checked={this.props.objectFilter.inactivityValue === range.value}
                            onChange={((value) => {
                                this.updateUnresponsiveFilter({ value });
                            }).bind(this, range.value)}
                        />
                        <label
                            htmlFor={range.value}
                            data-name="inactive"
                            data-info={this.dataInfo}
                        >
                            {formatDbNameToHumanReadable(this.props.intl, range.label)}
                        </label>
                    </div>
                ))}
            </div>
        </div>
    );
    
    groupsFilter = intl => (
        <div
            className="DropDown-Buttons"
            onMouseOut={(event) => {
                this.openCloseGroupList(event);
                this.closeFilter(event);
            }}
            onMouseOver={this.openCloseGroupList}
            data-name="groups"
            data-info={this.dataInfo}
        >
            <input
                type="checkbox"
                ref={(groupCheckbox) => { this.groupCheckbox = groupCheckbox; }}
                id={FILTER.GROUP}
                data-info={this.dataInfo}
                data-name="groups"
                checked={this.props.showGroups}
                onChange={this.addRemoveFilter.bind(this, FILTER.GROUP)}
            />
            <label
                htmlFor={FILTER.GROUP}
                data-name="groups"
                data-info={this.dataInfo}
            >
                {SITE_VIEW_FILTER.TITLES.GROUPS_FUNC(intl)}
            </label>
        </div>
    );
    
    stationaryFilter = intl => (
        <div
            className="DropDown-Buttons"
            onMouseOut={this.closeFilter}
            data-info={this.dataInfo}
        >
            <input
                type="checkbox"
                id={FILTER.STATIONARY}
                data-info={this.dataInfo}
                checked={this.props.objectFilter.showStationary}
                onChange={this.addRemoveFilter.bind(this, FILTER.STATIONARY)}
            />
            <label
                htmlFor={FILTER.STATIONARY}
                data-info={this.dataInfo}
            >
                {SITE_VIEW_FILTER.TITLES.STATIONARY(intl)}
            </label>
        </div>
    );
    anchorFilter = (currentRole, intl) => (
        userHasPrivileges(currentRole, USER_ROLES.PROJECT_MAINTAINER)
        && (
            <div
                className="DropDown-Buttons"
                onMouseOut={this.closeFilter}
                data-info={this.dataInfo}
            >
                <input
                    type="checkbox"
                    id={WIN_ANCHOR_TYPE.ANCHOR}
                    data-info={this.dataInfo}
                    checked={this.props.objectFilter.showAnchor}
                    onChange={this.addRemoveFilter.bind(this, FILTER.ANCHOR)}
                />
                <label
                    htmlFor={WIN_ANCHOR_TYPE.ANCHOR}
                    data-info={this.dataInfo}
                >
                    {SITE_VIEW_FILTER.TITLES.ANCHORS_FUNC(intl)}
                </label>
            </div>
        )
    );
    
    bridgeFilter = (currentRole, intl) => (
        userHasPrivileges(currentRole, USER_ROLES.PROJECT_MAINTAINER)
        && (
            <div
                className="DropDown-Buttons"
                onMouseOut={this.closeFilter}
                data-info={this.dataInfo}
            >
                <input
                    type="checkbox"
                    id={WIN_ANCHOR_TYPE.BRIDGE}
                    data-info={this.dataInfo}
                    checked={this.props.objectFilter.showBridges}
                    onChange={this.addRemoveFilter.bind(this, FILTER.BRIDGE)}
                />
                <label
                    htmlFor={WIN_ANCHOR_TYPE.BRIDGE}
                    data-info={this.dataInfo}
                >
                    {SITE_VIEW_FILTER.TITLES.BRIDGES_FUNC(intl)}
                </label>
            </div>
        )
    );
    
    unresponsiveFilter = intl => (
        <div
            className="DropDown-Buttons"
            onMouseOut={(event) => {
                this.openCloseUnresponsiveList(event);
                this.closeFilter(event);
            }}
            onMouseOver={this.openCloseUnresponsiveList}
            data-name="inactive"
            data-info={this.dataInfo}
        >
            <input
                type="checkbox"
                ref={(unresponsiveCheckbox) => {
                    this.unresponsiveCheckbox = unresponsiveCheckbox;
                }}
                id={FILTER.UNRESPONSIVE}
                data-info={this.dataInfo}
                data-name="inactive"
                checked={this.props.objectFilter.showInactive}
                onChange={() => {
                    this.updateUnresponsiveFilter({ filter: FILTER.UNRESPONSIVE });
                }}
            />
            <label
                htmlFor={FILTER.UNRESPONSIVE}
                className="text-capitalize"
                data-name="inactive"
                data-info={this.dataInfo}
            >
                {SITE_VIEW_FILTER.TITLES.UNRESPONSIVE_FUNC(intl)}
            </label>
        </div>
    );
    
    renderFilterList = () => {
        const { intl, currentRole } = this.props;
        return (
            <div className="DropDown-List" data-info={this.dataInfo}>
                {this.groupsFilter(intl)}
                {this.props.objectFilter.isOpenGroupFilter ? this.renderFilterGroupList() : ''}

                {this.stationaryFilter(intl)}
    
                {this.anchorFilter(currentRole, intl)}
                
                {this.bridgeFilter(currentRole, intl)}
                
                {this.unresponsiveFilter(intl)}
                {this.props.objectFilter.isOpenInactiveFilter ? this.renderFilterUnresponsiveList() : ''}
            </div>
        );
    };
    
    render() {
        const { intl, position, isSpaghettiShowing } = this.props;
        return (
            <Control position={position}>
                <div className="RPP-Site-Filter" style={{ display: isSpaghettiShowing ? 'none' : 'block' }}>
                    <div>
                        <button
                            type="button"
                            className="Filter-Button"
                            data-info={this.dataInfo}
                            onMouseOut={this.closeFilter}
                            onMouseOver={this.openList}
                        >
                            {SITE_VIEW_FILTER.TITLES.FILTER_FUNC(intl)}
                        </button>
                        {this.props.objectFilter.isOpenObjectFilter ? this.renderFilterList() : ''}
                    </div>
                </div>
            </Control>
        );
    }
}

const getFilterGroups = (state) => {
    const trackables = getTrackablesBySublocation(state);
    const groups = Group.getAllGroupsArray(state);
    
    const all_group_uids = [];
    trackables.forEach((tko) => {
        if (tko.groups) {
            all_group_uids.push(...tko.groups);
        }
    });
    const unique_groups = _Unique(all_group_uids);
    
    return groups.reduce((memo, group) => {
        if (unique_groups.indexOf(group.uid) !== -1) {
            memo.push(group);
        }
        return memo;
    }, []);
};

const mapStateToProps = (state) => {
    const { objectFilter } = state.siteViewFilter;
    const groups = getFilterGroups(state);
    const filterGroupUids = groups.map(g => g.uid).concat(FILTER.NA);
    const unselectedUids = _Difference(filterGroupUids, objectFilter.selectedGroups);

    return {
        groups,
        filterGroupUids,
        unselectedUids,
        showGroups: unselectedUids.length === 0,
        objectFilter: state.siteViewFilter.objectFilter,
        currentRole: state.user.access.currentRole,
    };
};

export default connect(mapStateToProps)(injectIntl(SiteViewFilter));
