import React from 'react';
import PropTypes from 'prop-types';
import CloneDeep from 'lodash/cloneDeep';
import {
    getAnchorType, getDeviceTypeClass, getKeyValueHtml, getLastReported, getLeftCornerRibbonHtml,
    isAnchorLive, macToUpperCase, positionOffset, userHasPrivileges
} from 'utils';
import store from 'stores/LocationStore';
import { TRACKABLE_POP_UP, USER_ROLES, COMMON_LABELS } from 'common';
import _Get from 'lodash/get';
import { PopUpWrapper } from 'components/leaflet/PopUpWrapper';


export function AnchorPopUp(props) {
    const { sublocation, onClose, intl } = props;
    const anchor = CloneDeep(props.anchor);
    const user = store.getCurrentUser();
    const hasPrivileges = userHasPrivileges(user?.access?.role || user?.access?.currentRole, USER_ROLES.PROJECT_MAINTAINER);
    const node = store.getNodeByMacAddress(_Get(anchor, 'node.mac_address', false));

    if (node) {
        anchor.node.last_heard = node.last_heard;
    }

    if (!hasPrivileges || !anchor || !anchor.uid || !sublocation) return null;

    const anchorPosition = positionOffset(anchor.position, sublocation);
    if (!anchorPosition) {
        return null;
    }

    const info = {
        [COMMON_LABELS.MAC_ADDRESS_FUNC(intl)]: macToUpperCase(_Get(anchor, 'node.mac_address') || ''),
        [TRACKABLE_POP_UP.REPORTED(intl)]: getLastReported(_Get(anchor, 'node.last_heard', null), intl),
    };

    const isAlive = isAnchorLive(anchor);
    const cornerRibbonTextLabel = getAnchorType(intl, anchor);

    return (
        <PopUpWrapper
            position={anchorPosition}
            closeButton={false}
            onClose={onClose}
            className="RPP-BlockBody"
        >
            <div className="RPP-BlockBody-Wrapper" onClick={onClose}>
                <div className="RPP-BlockHeader">
                    {getLeftCornerRibbonHtml(cornerRibbonTextLabel, getDeviceTypeClass(anchor))}
                    <h4 className="RPP-BlockBtnTitle multiple-lines-ellipsis">
                        {anchor.name}
                    </h4>
                    {!isAlive && <div className="unresponsive-message text-danger">
                        {TRACKABLE_POP_UP.NOT_RESPONDING(intl)}!
                    </div>}
                    <button
                        type="button"
                        className="RPP-BtnPopUp"
                        onClick={onClose}
                    >
                        <i className="fa fa-times" />
                    </button>
                </div>
                <div className="RPP-PopUpRowsWrapper">
                    {Object.keys(info).map((key, i) => getKeyValueHtml(key, info[key], i))}
                </div>
            </div>
        </PopUpWrapper>
    );
}

AnchorPopUp.defualtProps = {
    anchor: PropTypes.shape({}),
    sublocation: PropTypes.shape({}),
    onClose: PropTypes.func,
    intl: PropTypes.shape({
        formatMessage: PropTypes.func,
    })
};

AnchorPopUp.propTypes = {
    anchor: {},
    sublocation: {},
    onClose: () => {},
    intl: {
        formatMessage: () => {},
    }
};
