import React, { Fragment } from 'react';
import PropTypes from 'prop-types';
import { Field } from 'redux-form';
import _Get from 'lodash/get';
import RenderTooltipHelper from 'components/RenderTooltipHelper';
import RenderInputField from 'components/modals/RenderInputField';
import {
    COMPONENTS,
    TRACKABLE_MODAL,
    DISABLED_FIELD_TEXT,
    INDICATION,
    DB_NAMES,
    DURATION_FIELD_DESCRIPTION,
} from 'common/ui-labels';
import { DEFAULT_VALUES, T2T_TYPE, PWS_PIN } from 'common/constants';
import RenderSimpleTimeDurationField from 'components/modals/RenderSimpleTimeDurationField';
import { preventInputNotNumber, isUnspecified, isPerson, isVehicle, isAssetTag, showPwsFields } from 'utils/utils';
import { formatDbNameToHumanReadable } from 'utils/htmlUtils';
import RenderSelectField from '../RenderSelectField';
import CoverageAnnouncerWidget from './CoverageAnnouncerWidget';
import MultiSelectComponent from '../MultiSelectComponent';


const renderInputField = RenderInputField();

class LowLatencyLinkComponent extends React.Component {
    static propTypes = {
        isAdvanced: PropTypes.bool,
        dispatch: PropTypes.func,
        intl: PropTypes.shape({
            formatMessage: PropTypes.func,
        }),
        role: PropTypes.string,
        pin: PropTypes.string,
        safe_distance: PropTypes.number,
        distance: PropTypes.number,
        alert_distance: PropTypes.number,
        angle: PropTypes.number,
        coverageAnnouncerWidgetUiSettings: PropTypes.shape({
            id: PropTypes.string,
            fillColor: PropTypes.string,
            size: PropTypes.number,
        }),
        distanceDefaults: PropTypes.shape({
            min: PropTypes.number,
            max: PropTypes.number,
        }),
        trackables: PropTypes.arrayOf(PropTypes.shape()),
        groups: PropTypes.arrayOf(PropTypes.shape()),
        t2tNoAlarmUid: PropTypes.string,
        showAlarmDisableForField: PropTypes.bool,
        anglePredefinedValues: PropTypes.arrayOf(PropTypes.number),
        uid: PropTypes.string,
        formDispatchChange: PropTypes.func,
        trackableCategory: PropTypes.string,
        isStationary: PropTypes.bool,
    };

    static defaultProps = {
        isAdvanced: undefined,
        intl: {
            formatMessage: () => {},
        },
        role: '',
        pin: DEFAULT_VALUES.LLL_TRACKABLE_PIN,
        safe_distance: DEFAULT_VALUES.LLL_COVERAGE_SAFE_DISTANCE,
        distance: DEFAULT_VALUES.LLL_COVERAGE_DISTANCE, // distance default value
        alert_distance: DEFAULT_VALUES.LLL_COVERAGE_ALERT_DISTANCE,
        angle: DEFAULT_VALUES.LLL_COVERAGE_ANGLE, // angle default value
        coverageAnnouncerWidgetUiSettings: {
            id: 'coverageAnnouncer',
            fillColor: 'lightBlue',
            size: 100, // announcer width & height
        },
        distanceDefaults: {
            min: DEFAULT_VALUES.LLL_DISTANCE_MIN,
            max: DEFAULT_VALUES.LLL_DISTANCE_MAX,
            step: DEFAULT_VALUES.LLL_DISTANCE_STEP,
        },
        trackables: [],
        groups: [],
        t2tNoAlarmUid: '',
        showAlarmDisableForField: false,
        anglePredefinedValues: [80, 180, 360],
        uid: '',
        formDispatchChange: () => {},
        trackableCategory: '',
        isStationary: false,
    };

    filterByLllType = (values, labelFilter) => {
        const { t2tNoAlarmUid, uid } = this.props;
        const except = (t2tNoAlarmUid || '').split(',');
        return values.filter(trackable => (trackable
            && _Get(trackable, 'low_latency_link_config.role')
            && trackable.low_latency_link_config.role === T2T_TYPE.ANNOUNCER
            && trackable.uid !== uid
            && trackable.name.includes(labelFilter)
            && !except.includes(trackable.uid)
        ));
    };

    changeLLL = (e) => {
        const { formDispatchChange } = this.props;
        if (e.target.value === T2T_TYPE.MONITOR) {
            formDispatchChange('alarm_back', true);
        } else if (e.target.value === T2T_TYPE.ANNOUNCER) {
            formDispatchChange('angle', DEFAULT_VALUES.LLL_COVERAGE_ANGLE);
            formDispatchChange('alert_distance', DEFAULT_VALUES.LLL_COVERAGE_ALERT_DISTANCE);
            formDispatchChange('distance', DEFAULT_VALUES.LLL_COVERAGE_DISTANCE);
            formDispatchChange('safe_distance', DEFAULT_VALUES.LLL_COVERAGE_SAFE_DISTANCE);
            formDispatchChange('delay', DEFAULT_VALUES.LLL_DELAY);
        }
        formDispatchChange('pin', DEFAULT_VALUES.LLL_TRACKABLE_PIN);
        formDispatchChange('haptic', DEFAULT_VALUES.LLL_HAPTIC);
        formDispatchChange('buzzer', DEFAULT_VALUES.LLL_BUZZER);
    };

    getAngleCoverageFieldHtml = (anglePredefinedValues, angle, intl) => (
        <div className="Form-Line row ">
            <div className="Form-Label col-md-5">
                {TRACKABLE_MODAL.EDIT.FORM.PWS_PARAMS.ANGLE_OF_COVERAGE(intl)}
                <RenderTooltipHelper
                    tooltipText={TRACKABLE_MODAL.EDIT.DESCRIPTION.ANGLE_OF_COVERAGE(intl)}
                />
                <div style={{ display: 'block', fontSize: '10px' }}>
                    {TRACKABLE_MODAL.EDIT.FORM.PWS_PARAMS.FULL_RTLS_ONLY(intl)}
                </div>
            </div>
            <div className="col-md-7 row" data-id="trackable-edit-angle">
                {
                    anglePredefinedValues.map((value, idx) => (
                        <div className="col-md-4 text-left" key={idx}>
                            <Field
                                key={value}
                                name="angle"
                                className="col-md-2"
                                component="input"
                                checked={(Number(angle) === value)}
                                value={value}
                                type="radio"
                            />
                            <div style={{ textIndent: '8px' }}>{value}°</div>
                        </div>
                    ))
                }
            </div>
        </div>
    );

    renderDelayFragment = () => (
        <Fragment key="delay">
            <div className="Form-Line row ">
                <div className="Form-Label col-md-5">
                    {TRACKABLE_MODAL.DELAY}
                    <RenderTooltipHelper
                        tooltipText={TRACKABLE_MODAL.EDIT.DESCRIPTION.DELAY(this.props.intl)}
                    />
                    <div style={{ display: 'block', fontSize: '10px' }}>
                        {TRACKABLE_MODAL.EDIT.DESCRIPTION.LOWER_TEXT_DELAY(this.props.intl)}
                    </div>
                </div>
                <div
                    className="col-md-7"
                    data-cy="delay-div"
                >
                    <Field
                        type="text"
                        className="input"
                        name="delay"
                        component={RenderSimpleTimeDurationField}
                        fieldDescription={DURATION_FIELD_DESCRIPTION.FORMAT_1M1S}
                        defaultValue={0}
                    />
                </div>
            </div>
        </Fragment>
    )

    renderPinFragment = () => (
        <Fragment key="pin">
            <div className="Form-Line row">
                <div className="Form-Label col-md-5">
                    {TRACKABLE_MODAL.PIN}
                </div>
                <div className="col-md-7">
                    <Field
                        className="form-control input-sm Hw-Params-dropdown-item-size "
                        name="pin"
                        dataId="pin"
                        onChange={this.changeLLL}
                        component={RenderSelectField}
                    >
                        <option value={PWS_PIN.PULSE}>{DB_NAMES.ON(this.props.intl)}</option>
                        <option value={PWS_PIN.OFF}>{DB_NAMES.OFF(this.props.intl)}</option>
                    </Field>
                </div>
            </div>
        </Fragment>
    )

    renderHapticBuzzerFragment = () => (
        <Fragment key="haptic_buzzer">
            <div className="Form-Label col-md-5" />
            <div className="col-md-7">
                <div className="Form-Line row">
                    <div className="col-md-12">
                        <div className="row">
                            <div className="col-md-1">
                                <Field
                                    name="haptic"
                                    data-cy="haptic"
                                    id="haptic"
                                    className="Form-Input"
                                    component="input"
                                    type="checkbox"
                                />
                            </div>
                            <div className="col-md-10" style={{ marginTop: '5px' }}>
                                <label htmlFor="haptic">
                                    {INDICATION.HAPTIC(this.props.intl)}
                                </label>
                            </div>
                        </div>
                        <div className="row">
                            <div className="col-md-1">
                                <Field
                                    name="buzzer"
                                    data-cy="buzzer"
                                    id="buzzer"
                                    className="Form-Input"
                                    component="input"
                                    type="checkbox"
                                />
                            </div>
                            <div className="col-md-10" style={{ marginTop: '5px' }}>
                                <label htmlFor="buzzer">
                                    {INDICATION.BUZZER(this.props.intl)}
                                </label>
                            </div>
                        </div>
                    </div>
                </div>
            </div>
        </Fragment>
    )

    render() {
        const {
            trackables,
            isAdvanced,
            intl,
            role,
            pin,
            coverageAnnouncerWidgetUiSettings,
            angle,
            distanceDefaults,
            anglePredefinedValues,
            trackableCategory,
            isStationary,
        } = this.props;

        const isCategoryAsset = isAssetTag(trackableCategory);
        const isCategoryUnspecified = isUnspecified(trackableCategory);
        const isCategoryVehicle = isVehicle(trackableCategory);
        const isCategoryPerson = isPerson(trackableCategory);

        const isRoleDisabled = isCategoryAsset || (isCategoryUnspecified && !isStationary);
        const isCategoryAssetOrUnspecified = isRoleDisabled && (isCategoryAsset || isCategoryUnspecified);
        const isUnspecifiedStationary = isUnspecified(trackableCategory) && isStationary;
        const isPinOn = pin === PWS_PIN.PULSE;

        const {
            showDistance, showAlertDistance, showSafeDistance,
            showDelay, showPin, showAngle, showAlarmBack, showsT2tNoAlarmUid,
        } = showPwsFields(role, trackableCategory);

        return (
            <div className="Form-Line RPP-Group-advanced container-fluid" style={{ display: isAdvanced ? 'block' : 'none' }}>
                <h4 className="RPP-Group-advanced-border-title">
                    {COMPONENTS.FIELDS.NODE.PWS_TITLE}
                </h4>
                <div style={{ paddingLeft: '70px', paddingRight: '50px' }}>
                    <Fragment key="role">
                        <div className="Form-Line row">
                            <div className="Form-Label col-md-5">
                                {TRACKABLE_MODAL.ROLE}
                                <RenderTooltipHelper
                                    tooltipText={TRACKABLE_MODAL.EDIT.DESCRIPTION.ROLE(intl)}
                                />
                            </div>
                            <div className="col-md-7">
                                <Field
                                    className="form-control input-sm Hw-Params-dropdown-item-size "
                                    name="role"
                                    dataId="role"
                                    disabled={isRoleDisabled}
                                    onChange={this.changeLLL}
                                    component={RenderSelectField}
                                    disabledTooltipText={isCategoryAssetOrUnspecified
                                        ? DISABLED_FIELD_TEXT.PWS_UNAVAILABLE_4_CATEGORY(intl, formatDbNameToHumanReadable(intl, trackableCategory))
                                        : ''}
                                >
                                    <option value="">{DB_NAMES.NONE(intl)}</option>
                                    <option
                                        value={T2T_TYPE.MONITOR}
                                        disabled={isCategoryVehicle}
                                        title={isCategoryVehicle
                                            ? DISABLED_FIELD_TEXT.MONITOR_UNSUPPORTED_4_VEHICLE(intl)
                                            : ''}
                                    >
                                        {TRACKABLE_MODAL.EDIT.FORM.PWS_PARAMS.MONITOR(intl)}
                                    </option>
                                    <option
                                        value={T2T_TYPE.ANNOUNCER}
                                        disabled={isUnspecifiedStationary}
                                        title={isUnspecifiedStationary
                                            ? DISABLED_FIELD_TEXT.ANNOUNCER_UNSUPPORTED_4_STATIONARY(intl)
                                            : ''}
                                    >
                                        {TRACKABLE_MODAL.EDIT.FORM.PWS_PARAMS.ANNOUNCER(intl)}
                                    </option>
                                </Field>
                            </div>
                        </div>
                    </Fragment>

                    {showSafeDistance &&
                        <Fragment key="safe_distance">
                            <div className="Form-Line row ">
                                <div className="Form-Label col-md-5">
                                    {TRACKABLE_MODAL.SAFE_DISTANCE}
                                    <RenderTooltipHelper
                                        tooltipText={TRACKABLE_MODAL.EDIT.DESCRIPTION.SAFE_DISTANCE(intl)}
                                    />
                                    <div style={{ display: 'block', fontSize: '10px' }}>
                                        {TRACKABLE_MODAL.EDIT.DESCRIPTION.LOWER_TEXT_SAFE_DISTANCE(intl)}
                                    </div>
                                </div>
                                <div className="col-md-7">
                                    <Field
                                        simpleValue
                                        onChange={(e) => {
                                            preventInputNotNumber(e);
                                        }}
                                        clearable={false}
                                        className="Form-Input"
                                        name="safe_distance"
                                        label={TRACKABLE_MODAL.SAFE_DISTANCE}
                                        component={renderInputField}
                                    />
                                </div>
                            </div>
                        </Fragment>
                    }
                    {showDistance &&
                        <Fragment key="warning_distance">
                            <div className="Form-Line row">
                                <div className="Form-Label col-md-5">
                                    {TRACKABLE_MODAL.DISTANCE}
                                    <RenderTooltipHelper
                                        tooltipText={TRACKABLE_MODAL.EDIT.DESCRIPTION.DISTANCE(intl)}
                                    />
                                    <div style={{ display: 'block', fontSize: '10px' }}>
                                        {TRACKABLE_MODAL.EDIT.DESCRIPTION.LOWER_TEXT_DISTANCE(intl)}
                                    </div>
                                </div>
                                <div className="col-md-7">
                                    <Field
                                        simpleValue
                                        clearable={false}
                                        className="Form-Input"
                                        name="distance"
                                        data-cy="near-miss-distance"
                                        dataCy="near-miss-distance"
                                        label={TRACKABLE_MODAL.DISTANCE}
                                        lowerIndexText={TRACKABLE_MODAL.EDIT.DESCRIPTION.LOWER_TEXT_DISTANCE(intl)}
                                        tooltip={TRACKABLE_MODAL.EDIT.DESCRIPTION.DISTANCE(intl)}
                                        component={renderInputField}
                                    />
                                </div>
                            </div>
                        </Fragment>
                    }
                    {showAlertDistance &&
                        <Fragment key="alert_distance">
                            <div className="Form-Line row ">
                                <div className="Form-Label col-md-5">
                                    {TRACKABLE_MODAL.ALERT_DISTANCE}
                                    <RenderTooltipHelper
                                        tooltipText={TRACKABLE_MODAL.EDIT.DESCRIPTION.ALERT_DISTANCE(intl)}
                                    />
                                    <div style={{ display: 'block', fontSize: '10px' }}>
                                        {TRACKABLE_MODAL.EDIT.DESCRIPTION.LOWER_TEXT_ALERT_DISTANCE(intl)}
                                    </div>
                                </div>
                                <div className="col-md-7" data-id="trackable-edit-alert-distance">
                                    <Field
                                        clearable={false}
                                        className="input"
                                        data-cy="alert_distance"
                                        dataCy="alert_distance"
                                        name="alert_distance"
                                        placeholder={COMPONENTS.SELECT_COMPONENT_PLACEHOLDER(intl)}
                                        type="text"
                                        component={renderInputField}
                                    />
                                </div>
                            </div>
                        </Fragment>
                    }

                    {showPin && this.renderPinFragment()}
                    {showPin && isPinOn && isCategoryPerson && this.renderHapticBuzzerFragment()}
                    {showDelay && isPinOn && this.renderDelayFragment()}

                    {showAngle && (
                        <Fragment key="angle">
                            <Fragment key="angleCoverageFieldHtml">
                                {this.getAngleCoverageFieldHtml(anglePredefinedValues, angle, intl)}
                            </Fragment>
                            <Fragment key="coverageAnnouncerWidget">
                                <div className="Form-Line row text-center">
                                    <div className="col-md-12">
                                        <CoverageAnnouncerWidget
                                            uiSetting={coverageAnnouncerWidgetUiSettings}
                                            angle={angle}
                                            safe_distance={distanceDefaults.max}
                                            distance={distanceDefaults.max}
                                            alert_distance={distanceDefaults.max}
                                            distance_max={distanceDefaults.max}
                                        />
                                    </div>
                                </div>
                            </Fragment>
                        </Fragment>
                    )}
                    {showAlarmBack &&
                        <Fragment key="alarm_back">
                            <div className="Form-Line row">
                                <div className="Form-Label col-md-5">
                                    {TRACKABLE_MODAL.EDIT.FORM.PWS_PARAMS.REPORTS_BACK_TO_ANNOUNCER}
                                    <RenderTooltipHelper
                                        tooltipText={TRACKABLE_MODAL.EDIT.DESCRIPTION.REPORTS_BACK_TO_ANNOUNCER(intl)}
                                    />
                                </div>
                                <div className="col-md-1" data-id="trackable-edit-alarm-back">
                                    <Field
                                        name="alarm_back"
                                        data-cy="alarm_back"
                                        id="alarm_back"
                                        component="input"
                                        type="checkbox"
                                        style={{ width: '19px', height: '19px' }}
                                    />
                                </div>
                            </div>
                        </Fragment>
                    }
                    {showsT2tNoAlarmUid &&
                        <Fragment key="monitors_settings_Alarm_disable_for_t2t_no_alarm_uid">
                            <div className="Form-Line row">
                                <div className="Form-Label col-md-5">
                                    {TRACKABLE_MODAL.EDIT.FORM.PWS_PARAMS.ALARM_DISABLED}
                                    <RenderTooltipHelper
                                        tooltipText={TRACKABLE_MODAL.EDIT.DESCRIPTION.ALARM_DISABLED(intl)}
                                    />
                                </div>
                                <div className="col-md-7" data-id="trackable-edit-no-alarm">
                                    <Field
                                        dataId="t2t_no_alarm_uid"
                                        name="t2t_no_alarm_uid"
                                        label="Alarm disable for"
                                        options={trackables}
                                        simpleValue
                                        placeholder={COMPONENTS.SELECT_COMPONENT_PLACEHOLDER(intl)}
                                        noResultsText={COMPONENTS.SELECT_COMPONENT_NO_RESULTS_TEXT}
                                        filterOptions={this.filterByLllType}
                                        component={MultiSelectComponent}
                                        scrollMenuIntoView
                                    />
                                </div>
                            </div>
                        </Fragment>
                    }
                </div>
            </div>
        );
    }
}

export default LowLatencyLinkComponent;
