import React, { PureComponent } from 'react';
import { Map, LayersControl, Marker } from 'react-leaflet';
import PropTypes from 'prop-types';
import _Has from 'lodash/has';

import L from 'leaflet';
import LeafletFloorplan from 'components/leaflet/LeafletFloorplan';
import LeafletScaling from 'components/leaflet/LeafletScaling';
import { DEFAULT_VALUES, TRACKABLE_MODAL } from 'common';
import { MAP_BOUNDS, degreeToLength } from 'utils';
import { saveStationaryPosition } from 'stores/actions';

const { Overlay } = LayersControl;


class SelectedFloorplan extends PureComponent {
    static propTypes = {
        intl: PropTypes.shape({
            formatMessage: PropTypes.func,
        }),
        selectedFloorplan: PropTypes.oneOfType([PropTypes.object, PropTypes.bool]),
        unSelectFloorplan: PropTypes.func,
        saveStationaryUIPosition: PropTypes.func,
        leafletCursorPosition: PropTypes.shape({}),
        dispatch: PropTypes.func,
        stationaryPosition: PropTypes.shape({
            lat: PropTypes.number,
            lng: PropTypes.number,
        }),
    };

    static defaultProps = {
        intl: {
            formatMessage: () => {},
        },
        selectedFloorplan: false,
        unSelectFloorplan: () => {},
        saveStationaryUIPosition: () => {},
        leafletCursorPosition: null,
        dispatch: () => {},
        stationaryPosition: null,
    };

    constructor(props) {
        super(props);
        this.selectedMap = React.createRef();
        this.state = {
            zoom: 19,
            boundsForScale: {},
            isStationaryTagAdded: props.stationaryPosition && props.stationaryPosition.lat && props.stationaryPosition.lng,
            leafletCursorPosition: null,
        };
    }

    componentDidMount() {
        const selectedMap = this.selectedMap.current;
        if (selectedMap) {
            const { leafletElement } = selectedMap;
            const boundsForScale = leafletElement ? leafletElement.getBounds() : {};
            this.setState({ leafletElement, boundsForScale });

            const stationaryTagIcon = L.divIcon({ className: 'stationary-tag' });
            this.setState({ stationaryTagIcon });
        }
    }

    static getDerivedStateFromProps(props, state) {
        return {
            ...state,
            isStationaryTagAdded: props.stationaryPosition && props.stationaryPosition.lat && props.stationaryPosition.lng,
        };
    }
    
    handleMove = (event) => {
        const { leafletElement } = this.state;
        this.setState({
            // eslint-disable-next-line
            zoom: event.target._zoom,
            boundsForScale: leafletElement ? leafletElement.getBounds() : {},
        });
    };

    stationaryUIPosBoxTemplate = (stationaryLeafletPosition) => {
        const stationaryUIPos = {
            x: parseInt(degreeToLength(stationaryLeafletPosition.lng)) / 10,
            y: parseInt(degreeToLength(stationaryLeafletPosition.lat)) / 10,
        };
        return `x,y(m): ${stationaryUIPos.x}, ${stationaryUIPos.y}`;
    };

    render() {
        const {
            intl,
            selectedFloorplan,
            unSelectFloorplan,
            dispatch,
            stationaryPosition,
            saveStationaryUIPosition,
        } = this.props;

        const {
            stationaryTagIcon, leafletCursorPosition, zoom, boundsForScale, isStationaryTagAdded, 
        } = this.state;

        return (
            <div className="floorPlanContainer text-center">
                <div
                    className="modal-floorplan-leaflet-selected"
                    title={!isStationaryTagAdded ? TRACKABLE_MODAL.EDIT.FORM.IS_STATIONARY_TAG_ADDED_FUNC(intl) : ''}
                    onDoubleClick={() => {
                        unSelectFloorplan();
                    }}
                >
                    <Map
                        center={[0, 0]}
                        ref={this.selectedMap}
                        style={{ height: '99.3%' }}
                        bounds={MAP_BOUNDS({}, selectedFloorplan)}
                        zoomAnimation={false}
                        zoomSnap={0}
                        scrollWheelZoom="center"
                        minZoom={DEFAULT_VALUES.MAP_ZOOM.MIN}
                        doubleClickZoom={false}
                        maxZoom={DEFAULT_VALUES.MAP_ZOOM.MAX}
                        zoom={DEFAULT_VALUES.MAP_ZOOM.DEFAULT}
                        onMoveend={this.handleMove}
                        onclick={(ev) => {
                            if (!isStationaryTagAdded) {
                                dispatch(saveStationaryPosition({
                                    stationaryPosition: {
                                        lat: ev.latlng.lat,
                                        lng: ev.latlng.lng,
                                    },
                                }));
                                saveStationaryUIPosition(dispatch, ev.latlng);
                                this.setState({ isStationaryTagAdded: true });
                            }
                        }}
                        onmousemove={(ev) => {
                            this.setState({ leafletCursorPosition: {
                                lat: ev.latlng.lat,
                                lng: ev.latlng.lng,
                            } });
                        }}
                    >
                        <LayersControl position="topright">
                            <Overlay checked name="Floorplan">
                                <LeafletFloorplan floorplan={selectedFloorplan} />
                            </Overlay>

                            <LeafletScaling
                                zoom={zoom}
                                floorplan={selectedFloorplan}
                                bounds={boundsForScale}
                                isShowingScale
                            />
                            {
                                _Has(stationaryPosition, 'lat') && _Has(stationaryPosition, 'lng')
                                && (
                                    <Marker
                                        icon={stationaryTagIcon}
                                        draggable
                                        position={stationaryPosition}
                                        className="stationary-tag"
                                        onDrag={(ev) => {
                                            this.setState({
                                                latlng: {
                                                    lat: ev.latlng.lat,
                                                    lng: ev.latlng.lng,
                                                },
                                            }, () => {
                                                dispatch(saveStationaryPosition({ stationaryPosition: this.state.latlng }));
                                            });
                                        }}
                                        onDragEnd={() => {
                                            dispatch(saveStationaryPosition({ stationaryPosition: this.state.latlng }));
                                            saveStationaryUIPosition(dispatch, {
                                                lat: this.state.latlng.lat,
                                                lng: this.state.latlng.lng,
                                            });
                                        }}
                                    />
                                )
                            }
                        </LayersControl>
                    </Map>
                    {
                        leafletCursorPosition
                        && (
                            <div id="coordinates-box">
                                {this.stationaryUIPosBoxTemplate(leafletCursorPosition)}
                            </div>
                        )
                    }
                </div>
            </div>
        );
    }
}

export default SelectedFloorplan;
