import React from 'react';
import Modal from 'react-modal';
import { injectIntl } from 'react-intl';
import { connect } from 'react-redux';
import { reduxForm } from 'redux-form';
import PropTypes from 'prop-types';

import { NAME_OF_FORMS, ALARMS_ACKNOWLEDGE_MODAL, COMPONENTS } from '../../common';
import { closeAlarmAcknowledgeModal, acknowledgeAlarm } from '../../stores/actions';
import { Alarm } from '../../stores/classes/Selectors';
import { timeFormatter, dateFormatter, formatDbNameToHumanReadable, minToMs, getUiConstForAdditionalInfoText, getTimeZoneOffsetByProject } from '../../utils';

let AlarmAcknowledgeForm = (props) => {
    const {
        close, submit, className, alarm, intl, timeZoneOffset,
    } = props;
    
    return (
        <form className={className}>
            {alarm &&
            <div className="row">
                <div className="col-md-2" />
                <div className="col-md-8" data-id="alarm-acknowledge-alarm">
                    {[alarm].map((a, i) => {
                        const created = new Date(new Date(a.created_at).getTime() + minToMs(timeZoneOffset));
                        const name = a.trackableName;
                        const date = dateFormatter(created);
                        const time = timeFormatter(created);
                        const text = a.additional_info && a.additional_info.text ? getUiConstForAdditionalInfoText(intl, a.additional_info.text) : '';
                        return (
                            <div
                                key={i}
                                className="Form-Batch-List-Element"
                            >
                                {name}<br />{time} {date}<br />{text}
                            </div>
                        );
                    })}
                </div>
            </div>}
            <div className="Form-DeleteTitle">
                {ALARMS_ACKNOWLEDGE_MODAL.WARNING}
            </div>
            <div className="Form-Buttons">
                <button
                    type="button"
                    onClick={submit}
                    data-id="alarm-acknowledge-acknowledge"
                >
                    {ALARMS_ACKNOWLEDGE_MODAL.BUTTON_ACK}
                </button>
                <button type="button" data-id="alarm-acknowledge-cancel" onClick={close}>{COMPONENTS.BUTTONS.CANCEL}</button>
            </div>
        </form>
    );
};

AlarmAcknowledgeForm.propTypes = {
    close: PropTypes.func,
    submit: PropTypes.func,
    className: PropTypes.string,
    timeZoneOffset: PropTypes.number,
    alarm: PropTypes.shape({
        additional_info: PropTypes.shape({
            text: PropTypes.string,
        }),
        trackableName: PropTypes.string,
        created_at: PropTypes.oneOfType([
            PropTypes.string,
            PropTypes.instanceOf(Date),
        ]),
    }),
    intl: PropTypes.shape({
        formatMessage: PropTypes.func,
    }),
};

AlarmAcknowledgeForm.defaultProps = {
    close: () => {
    },
    submit: () => {
    },
    className: '',
    timeZoneOffset: 0,
    alarm: {
        additional_info: {
            text: '',
        },
        trackableName: '',
        created_at: '',
    },
    intl: PropTypes.shape({
        formatMessage: PropTypes.func,
    }),
};

AlarmAcknowledgeForm = reduxForm({
    form: NAME_OF_FORMS.ALARM_ACKNOWLEDGE,
})(injectIntl(AlarmAcknowledgeForm));

const mapStateToForm = (state) => {
    const project = state.appContext.project;
    const timeZoneOffset = getTimeZoneOffsetByProject(project);
    if (state.modals.alarmModals.alarmAcknowledge
        && state.modals.alarmModals.alarmAcknowledge.params
        && state.modals.alarmModals.alarmAcknowledge.params.uid) {
        return {
            alarm: state.modals.alarmModals.alarmAcknowledge.params,
            timeZoneOffset,
        };
    }
    return {
        alarm: null,
        timeZoneOffset,
    };
};

AlarmAcknowledgeForm = connect(mapStateToForm)(AlarmAcknowledgeForm);

class AlarmAcknowledgeModal extends React.Component {
    static propTypes = {
        dispatch: PropTypes.func,
        style: PropTypes.shape(),
        intl: PropTypes.shape({
            formatMessage: PropTypes.func,
        }),
        alarmAcknowledge: PropTypes.shape({
            open: PropTypes.bool,
            params: PropTypes.oneOfType([
                PropTypes.shape({
                    uid: PropTypes.string,
                }),
                PropTypes.arrayOf(PropTypes.object),
            ]),
        }),
    };
    
    static defaultProps = {
        dispatch: () => {
        },
        style: {},
        intl: {
            formatMessage: () => {
            },
        },
        alarmAcknowledge: {
            open: false,
            params: {
                uid: '',
            },
        },
    };
    
    close = () => {
        this.props.dispatch(closeAlarmAcknowledgeModal());
    };
    
    submit = () => {
        if (this.props.alarmAcknowledge.params.uid) {
            if (Alarm.getAlarm(this.props.alarmAcknowledge.params.uid)) {
                this.props.dispatch(acknowledgeAlarm(this.props.alarmAcknowledge.params));
            } else {
                console.warn(`alarms ${this.props.alarmAcknowledge.params.uid} was cleared`);
            }
        } else {
            this.props.alarmAcknowledge.params.forEach((alarm) => {
                if (Alarm.getAlarm(alarm.uid)) {
                    this.props.dispatch(acknowledgeAlarm(alarm));
                } else {
                    console.warn(`alarms ${alarm.uid} was cleared`);
                }
            });
        }
        this.props.dispatch(closeAlarmAcknowledgeModal());
    };
    
    render() {
        const { style, intl } = this.props;
        const { params } = this.props.alarmAcknowledge;
        
        const localizeAlarmName = params && params.alarm_name ? formatDbNameToHumanReadable(intl, params.alarm_name).toLowerCase() : '';
    
        return (
            <Modal
                contentLabel="AlarmAcknowledge"
                isOpen={this.props.alarmAcknowledge.open}
                onRequestClose={this.close}
                style={style}
            >
                
                <div className="RPP-Modal">
                    <span className="Modal-Header">
                        {params && params.uid
                            ? ALARMS_ACKNOWLEDGE_MODAL.HEADER_FUNC(localizeAlarmName)
                            : ALARMS_ACKNOWLEDGE_MODAL.HEADER_MULTI_FUNC()
                        }
                    </span>
                    <AlarmAcknowledgeForm
                        className="Modal-Form"
                        close={this.close}
                        onSubmit={this.submit}
                    />
                </div>
            </Modal>
        );
    }
}

const mapStateToProps = state => ({
    alarmAcknowledge: state.modals.alarmModals.alarmAcknowledge,
});

export default connect(mapStateToProps)(injectIntl(AlarmAcknowledgeModal));
