import React from 'react';
import Modal from 'react-modal';
import { connect } from 'react-redux';
import { Field, reduxForm } from 'redux-form';
import PropTypes from 'prop-types';
import { injectIntl } from 'react-intl';
import _get from 'lodash/get';

import SelectComponent from 'components/modals/SelectComponent';
import { NAME_OF_FORMS, TRACKABLE_TAG_ASSOCIATION, COMPONENTS } from 'common';
import { macToUpperCase, sortByField, isOpenLllValidationModal, filterSelectBoxOptions } from 'utils';
import { closeTagAssignModal, openAssignLowLatencyLinkValidateModal, createTrackableAssociations } from 'stores/actions';
import { Trackable, Group } from 'stores/classes';

let TagAssignForm = (props) => {
    const {
        close, submit, className, notAssignTrackables, intl, formState,
        submitSucceeded,
    } = props;

    const formValues = formState && formState[NAME_OF_FORMS.TAG_ASSIGN];
    const selectedTrackable = _get(formValues, 'values.trackable');

    return (
        <form className={className}>
            <div className="Form-Line row">
                <div htmlFor="node" className="Form-Label col-md-3">{COMPONENTS.FIELDS.NODE.TRACKABLE}</div>
                <div className="col-md-9 assign-trackable-field">
                    <Field
                        data-id="tag-assign-trackable"
                        name="trackable"
                        style={{ fontSize: '14px' }}
                        props={{ disabled: false }}
                        placeholder={COMPONENTS.SELECT_COMPONENT_PLACEHOLDER(intl)}
                        noResultsText={COMPONENTS.SELECT_COMPONENT_NO_RESULTS_TEXT}
                        filterOptions={filterSelectBoxOptions}
                        options={notAssignTrackables}
                        component={SelectComponent}
                    />
                </div>
            </div>
            <div className="Form-Buttons">
                <button
                    type="button"
                    data-id="tag-assign-assign"
                    onClick={submit}
                    disabled={!selectedTrackable || submitSucceeded}
                >
                    {COMPONENTS.BUTTONS.ASSIGN}
                </button>
                <button type="button" data-id="tag-assign-cancel" onClick={close}>{COMPONENTS.BUTTONS.CANCEL}</button>
            </div>
        </form>
    );
};

TagAssignForm.propTypes = {
    close: PropTypes.func,
    submit: PropTypes.func,
    className: PropTypes.string,
    notAssignTrackables: PropTypes.arrayOf(PropTypes.object),
    submitSucceeded: PropTypes.bool,
    intl: PropTypes.shape({
        formatMessage: PropTypes.func,
    }),
    formState: PropTypes.shape({}),
};

TagAssignForm.defaultProps = {
    close: () => {
    },
    submit: () => {
    },
    className: '',
    notAssignTrackables: [{}],
    submitSucceeded: false,
    intl: {
        formatMessage: () => {
        },
    },
    formState: {},
};

TagAssignForm = reduxForm({
    form: NAME_OF_FORMS.TAG_ASSIGN,
})(TagAssignForm);

const mapStateToForm = (state) => {
    const trackables = Trackable.getNotAssociatedTrackables(state).map(trackable => trackable.formattedTrackablesAddUidName());
    return {
        notAssignTrackables: sortByField(trackables, 'name'),
        formState: state.form,
    };
};
TagAssignForm = connect(mapStateToForm)(injectIntl(TagAssignForm));

class TagAssignModal extends React.Component {
    static propTypes = {
        dispatch: PropTypes.func,
        tagAssign: PropTypes.shape({
            params: PropTypes.shape({
                mac_address: PropTypes.string,
            }),
            open: PropTypes.bool,
        }),
        groups: PropTypes.arrayOf(PropTypes.shape),
        style: PropTypes.shape({
            overlay: PropTypes.object,
            content: PropTypes.object,
        }),
    };
    
    static defaultProps = {
        dispatch: () => {
        },
        tagAssign: {
            params: {
                mac_address: '',
            },
            open: false,
        },
        groups: [],
        style: {
            overlay: {},
            content: {},
        },
    };
    
    close = () => {
        this.props.dispatch(closeTagAssignModal());
    };
    
    submit = (value) => {
        if (value.trackable) {
            const node = this.props.tagAssign.params;
            const { trackable } = value;
            
            const isOpen = isOpenLllValidationModal({ trackable: new Trackable(trackable), node });
            
            if (isOpen) {
                this.props.dispatch(openAssignLowLatencyLinkValidateModal({
                    mac_address: node.mac_address,
                    trackable_uid: value.trackable.uid,
                }));
            } else {
                this.props.dispatch(createTrackableAssociations({
                    mac_address: node.mac_address,
                    trackable_uid: value.trackable.uid,
                }));
            }
        }
        this.props.dispatch(closeTagAssignModal());
    };
    
    render() {
        const { style, tagAssign } = this.props;
        const { params } = tagAssign;
        
        return (
            <Modal
                contentLabel="tagAssign"
                isOpen={tagAssign.open}
                onRequestClose={this.close}
                style={style}
            >
                
                <div className="RPP-Modal">
                    <span className="Modal-Header">
                        {TRACKABLE_TAG_ASSOCIATION.FORM.ASSIGN_TAG_FUNC(params && params.mac_address ? macToUpperCase(params.mac_address) : '')}
                    </span>
                    <TagAssignForm
                        className="Modal-Form"
                        close={this.close}
                        onSubmit={this.submit}
                    />
                </div>
            </Modal>
        );
    }
}

const mapStateToProps = state => ({
    tagAssign: state.modals.assignModals.tagAssign,
    groups: Group.getAllGroupsArray(state),
});

export default connect(mapStateToProps)(TagAssignModal);
