import React from 'react';
import Modal from 'react-modal';
import PropTypes from 'prop-types';
import { connect } from 'react-redux';
import { Field, reduxForm } from 'redux-form';
import { injectIntl } from 'react-intl';
import RenderField from 'components/modals/RenderField';
import { NAME_OF_FORMS, STORAGE_KEY, COMPONENTS, NAME_OF_GEAR_BUTTONS } from 'common';
import { closeEditProfile, editUser, openChangePassword, startLiveMode, stopLiveMode } from 'stores/actions';
import { getSessionStorageValue } from 'utils';
import { makeCopy } from 'utils/simpleUtils';


const validate = (values) => {
    const errors = {};
    if (!values.first_name) {
        errors.first_name = COMPONENTS.VALIDATION.REQUIRED;
    }
    if (!values.last_name) {
        errors.last_name = COMPONENTS.VALIDATION.REQUIRED;
    }
    
    return errors;
};

const renderField = RenderField(4, 8);

let EditProfileForm = (props) => {
    const {
        close, submit, className, invalid, openChangePasswordModal, intl,
    } = props;
    
    return (
        <form className={className}>
            <div className="Form-Line row">
                <Field
                    className="Form-Input"
                    name="first_name"
                    dataId="edit-profile-first-name"
                    component={renderField}
                    label={COMPONENTS.FIELDS.USER.FIRST_NAME_FUNC(intl)}
                    type="text"
                />
            </div>
            
            <div className="Form-Line row">
                <Field
                    className="Form-Input"
                    name="last_name"
                    component={renderField}
                    dataId="edit-profile-last-name"
                    label={COMPONENTS.FIELDS.USER.LAST_NAME_FUNC(intl)}
                    type="text"
                />
            </div>
            
            <div className="Form-Line row">
                <div htmlFor="address" className="Form-Label col-md-4">{COMPONENTS.FIELDS.USER.PHONE}</div>
                <div className="col-md-8">
                    <Field
                        className="Form-Input input"
                        name="phone_number"
                        data-id="edit-profile-phone"
                        component="input"
                        type="text"
                    />
                </div>
            </div>
            
            <button
                type="button"
                onClick={openChangePasswordModal}
                className="Form-ChangePassword"
                data-id="edit-profile-change-password"
            >
                {COMPONENTS.BUTTONS.CHANGE_PASSWORD}
            </button>
            <div className="Form-Buttons">
                <button type="button" data-id="edit-profile-save" onClick={submit} disabled={invalid}>{COMPONENTS.BUTTONS.SAVE}</button>
                <button type="button" data-id="edit-profile-cancel" onClick={close}>{COMPONENTS.BUTTONS.CANCEL}</button>
            </div>
        </form>
    );
};
EditProfileForm.propTypes = {
    close: PropTypes.func,
    submit: PropTypes.func,
    openChangePasswordModal: PropTypes.func,
    className: PropTypes.string,
    invalid: PropTypes.bool,
    intl: PropTypes.shape({
        formatMessage: PropTypes.func,
    }),
};

EditProfileForm.defaultProps = {
    close: () => {
    },
    submit: () => {
    },
    openChangePasswordModal: () => {
    },
    className: '',
    invalid: false,
    intl: {
        formatMessage: () => {
        },
    },
};


EditProfileForm = reduxForm({
    form: NAME_OF_FORMS.EDIT_PROFILE,
    validate,
})(EditProfileForm);

const mapStateToForm = state => ({
    initialValues: state.modals.authModals.editProfile,
});
EditProfileForm = connect(mapStateToForm)(injectIntl(EditProfileForm));


class EditProfileModal extends React.PureComponent {
    static propTypes = {
        dispatch: PropTypes.func,
        projectScope: PropTypes.shape({}),
        editProfile: PropTypes.shape({
            open: PropTypes.bool,
            phone_number: PropTypes.string,
            user_details: PropTypes.shape({
                phone_number: PropTypes.string,
            }),
            uid: PropTypes.string,
        }),
        form: PropTypes.shape({
            values: PropTypes.shape({}),
        }),
        style: PropTypes.shape({}),
    };
    
    static defaultProps = {
        editProfile: {
            open: false,
            phone_number: '',
            user_details: {
                phone_number: '',
            },
            uid: '',
        },
        form: {
            values: {},
        },
        dispatch: () => {},
        projectScope: null,
        style: {},
    };
    
    componentDidUpdate(prevProps) {
        if (this.props && this.props.editProfile && this.props.editProfile.user_details && !this.props.editProfile.phone_number) {
            this.props.editProfile.phone_number = this.props.editProfile.user_details.phone_number;
        }
        const openInProjectScope = this.props.projectScope && this.props.editProfile.open;
        const closeInProjectScope = this.props.projectScope && !this.props.editProfile.open;

        if (openInProjectScope && !prevProps.editProfile.open) {
            this.props.dispatch(stopLiveMode());
        } else if (closeInProjectScope && prevProps.editProfile.open) {
            this.props.dispatch(startLiveMode());
        }
        return null;
    }

    close = () => {
        this.props.dispatch(closeEditProfile());
    };

    submit = (values) => {
        const user = makeCopy(values);

        this.props.dispatch(editUser(user));
        this.close();
    };

    openChangePasswordModal = () => {
        this.props.dispatch(openChangePassword({
            uid: this.props.editProfile.uid,
            email: getSessionStorageValue(STORAGE_KEY.USER_EMAIL),
            token: getSessionStorageValue(STORAGE_KEY.USER_TOKEN),
            openEditProfile: this.props.form.values,
        }));

        this.close();
    };

    render() {
        const { editProfile, style } = this.props;
        
        return (
            <Modal
                contentLabel="EditProfile"
                isOpen={editProfile.open}
                onRequestClose={this.close}
                style={style}
            >
                <div className="RPP-Modal">
                    <span className="Modal-Header">
                        {NAME_OF_GEAR_BUTTONS.EDIT_PROFILE}
                    </span>
                    
                    <EditProfileForm
                        className="Modal-Form"
                        openChangePasswordModal={this.openChangePasswordModal}
                        close={this.close}
                        onSubmit={this.submit}
                    />
                </div>
            </Modal>
        );
    }
}

const mapStateToProps = state => ({
    editProfile: state.modals.authModals.editProfile,
    projectScope: state.appContext.project,
    form: state.form[NAME_OF_FORMS.EDIT_PROFILE],
});

export default connect(mapStateToProps)(EditProfileModal);
