import React, { Fragment } from 'react';
import { connect } from 'react-redux';
import { injectIntl } from 'react-intl';
import PropTypes from 'prop-types';
import Modal from 'react-modal';
import { Field, reduxForm, touch, formValueSelector } from 'redux-form';
import moment from 'moment';
import momentTz from 'moment-timezone';
import _isEmpty from 'lodash/isEmpty';
import {
    closeSiteViewAnalyticsModal,
    getSiteViewAnalyticsTrackables,
    siteViewAnalyticsModalTrackablesListLoading,
    siteViewAnalyticsModalChangeSelectedTrackables,
    stopLiveMode,
    startLiveMode,
    setLastMode,
    clearPlayback,
    clearHeatmap,
    changeSpaghettiViewMode,
    generateSpaghetti,
    changeMode,
    generateNearMissEventReport,
    createHeatmap,
    changePlaybackSelectTrackables,
    loadAssociationForPlayback,
    changePlaybackParams,
    generatePlayback,
    clearLastMode,
} from 'stores/actions';
import RenderField from 'components/modals/RenderField';
import { clearNearMissReport, clearSpaghettiDiagram } from 'stores/actions/commonActions';
import {
    SITE_ANALYTICS_MODAL,
    COMPONENTS,
    NAME_OF_FORMS,
    SITE_MODE,
    SITE_ANALYTICS_LABELS,
    NAMES_OF_TABS,
} from 'common';
import MultiSelectComponent from 'components/modals/MultiSelectComponent';
import RenderDatePicker from 'components/modals/RenderDatePicker';
import RenderSlider from 'components/modals/RenderSlider';
import { getTimeZoneOfProject, filterSelectBoxOptions } from 'utils';
import { makeCopy } from 'utils/simpleUtils';
import { AppContext } from '../../context';

const analyticsTypes = {
    spaghetti: 'spaghetti',
    heatmap: 'heatmap',
    playback: 'playback',
    nearMiss: 'nearMiss',
};

const renderField = RenderField(4, 2);
const required = (value) => (value || typeof value === 'number' ? undefined : 'Required');
const positive = (value) => (Number(value) >= 0.5 ? undefined : 'Distance should be greater that 0.5 m');
const multiple = (value) => (Number.isInteger(Number(value) * 10) ? undefined : 'Minimal distance resolution equal to 0.1 m');

let SiteViewAnalyticsForm = (props) => {
    const { intl, closeModal, handleSubmit, modalParams, timeZone, onChange, type, invalid, showNearMiss } = props;
    const { isCustomInterval, trackables, sentTrackablesRequest } = modalParams;

    const isTypeHeatmap = type === analyticsTypes.heatmap;
    const isTypeNearMiss = type === analyticsTypes.nearMiss;

    return (
        <form className="Modal-Form Form-Line" onSubmit={handleSubmit} onChange={onChange}>
            <div className="Form-Line row">
                <div className="Form-Label col-md-4">{COMPONENTS.FIELDS.COMMON.TYPE}</div>
                <div className="col-md-8" data-id="site-analytics-type">
                    {showNearMiss && <div className="row">
                        <div className="col-md-5">
                            <label className="field-label">
                                <Field
                                    name="type"
                                    component="input"
                                    type="radio"
                                    data-cy="view_analytics_type"
                                    value={analyticsTypes.nearMiss}
                                />
                                <span className="label-text text-capitalize">{SITE_ANALYTICS_MODAL.NEAR_MISS}</span>
                            </label>
                        </div>
                    </div>}
                    <div className="row">
                        <div className="col-md-5">
                            <label className="field-label">
                                <Field
                                    name="type"
                                    component="input"
                                    type="radio"
                                    data-cy="view_analytics_type"
                                    value={analyticsTypes.spaghetti}
                                />
                                <span className="label-text text-capitalize">{SITE_ANALYTICS_MODAL.SPAGHETTI}</span>
                            </label>
                        </div>
                    </div>
                    <div className="row">
                        <div className="col-md-5">
                            <label className="field-label">
                                <Field
                                    name="type"
                                    component="input"
                                    type="radio"
                                    data-cy="view_analytics_type"
                                    value={analyticsTypes.heatmap}
                                />
                                <span className="label-text">{SITE_ANALYTICS_MODAL.HEATMAP}</span>
                            </label>
                        </div>
                        {
                            isTypeHeatmap && <div className="col-md-7">
                                <div className="slider-wrapper heatmap-weight-slider">
                                    <Field
                                        name="weight"
                                        component={RenderSlider}
                                    />
                                    <span className="label-text text-capitalize">{SITE_ANALYTICS_MODAL.WEIGHT}</span>
                                </div>
                            </div>
                        }
                    </div>
                    <div>
                        <label className="field-label">
                            <Field
                                name="type"
                                component="input"
                                type="radio"
                                value={analyticsTypes.playback}
                            />
                            <span className="label-text">{SITE_ANALYTICS_MODAL.PLAYBACK}</span>
                        </label>
                    </div>
                </div>
            </div>

            {
                isCustomInterval && <Fragment>
                    <div className="Form-Line row">
                        <div className="Form-Label col-md-4">{COMPONENTS.FIELDS.SITE.FROM}</div>
                        <div className="col-md-5" data-id="site-analytics-interval-from">
                            <Field
                                name="from"
                                disabled={sentTrackablesRequest}
                                component={RenderDatePicker}
                                placeholder={SITE_ANALYTICS_LABELS.SELECT_DATE(intl)}
                                timeZone={timeZone}
                                showToday={false}
                            />
                        </div>
                    </div>
                    <div className="Form-Line row">
                        <div className="Form-Label col-md-4">{COMPONENTS.FIELDS.SITE.TO}</div>
                        <div className="col-md-5" data-id="site-analytics-interval-to">
                            <Field
                                name="to"
                                disabled={sentTrackablesRequest}
                                component={RenderDatePicker}
                                placeholder={SITE_ANALYTICS_LABELS.SELECT_DATE(intl)}
                                showToday
                                timeZone={timeZone}
                            />
                        </div>
                    </div>
                </Fragment>
            }

            {!isTypeNearMiss && <div className="Form-Line row">
                <div className="Form-Label col-md-4">{NAMES_OF_TABS.TRACKABLE_OBJECTS}</div>
                <div className={(sentTrackablesRequest) ? 'col-md-8 spinner' : 'col-md-8'} data-id="site-analytics-trackables-groups">
                    <Field
                        name="trackables_and_groups"
                        component={MultiSelectComponent}
                        disabled={sentTrackablesRequest}
                        placeholder={COMPONENTS.SELECT_COMPONENT_PLACEHOLDER(intl)}
                        noResultsText={COMPONENTS.SELECT_COMPONENT_NO_RESULTS_TEXT}
                        options={trackables}
                        filterOptions={filterSelectBoxOptions}
                    />
                </div>
            </div>}
            {
                isTypeNearMiss && <div className="Form-Line row">
                    <Field
                        className="Form-Input"
                        name="distance"
                        component={renderField}
                        dataCy="near-miss-distance"
                        label={SITE_ANALYTICS_LABELS.DISTANCE(intl)}
                        type="number"
                        validate={[required, positive, multiple]}
                        min={0.5}
                        step={0.1}
                    />
                </div>
            }
            <div className="Form-Buttons">
                <button
                    data-id="create-site-analytics"
                    type="submit"
                    disabled={invalid || sentTrackablesRequest}
                >
                    {COMPONENTS.BUTTONS.CREATE}
                </button>
                <button data-id="cancel-site-analytics" onClick={closeModal}>
                    {COMPONENTS.BUTTONS.CANCEL}
                </button>
            </div>
        </form>
    );
};

SiteViewAnalyticsForm.propTypes = {
    closeModal: PropTypes.func,
    handleSubmit: PropTypes.func,
    onChange: PropTypes.func,
    intl: PropTypes.shape({
        formatMessage: PropTypes.func,
    }),
    modalParams: PropTypes.shape(),
    type: PropTypes.string,
    timeZone: PropTypes.string,
    invalid: PropTypes.bool,
    showNearMiss: PropTypes.bool,
};

SiteViewAnalyticsForm.defaultProps = {
    closeModal: () => {
    },
    handleSubmit: () => {
    },
    onChange: () => {
    },
    intl: {
        formatMessage: () => {
        },
    },
    modalParams: {},
    type: '',
    timeZone: '',
    invalid: false,
    showNearMiss: true,
};

const validate = (values, props) => {
    const errors = {};
    const now = moment().utc().valueOf();
    
    if (_isEmpty(values.trackables_and_groups)) {
        errors.trackables_and_groups = COMPONENTS.VALIDATION.REQUIRED_FUNC(props.intl);
    } else if (values.type === analyticsTypes.spaghetti && values.trackables_and_groups.length > 5) {
        errors.trackables_and_groups = SITE_ANALYTICS_MODAL.ERROR_MESSAGE(props.intl, 5);
        props.dispatch(touch(NAME_OF_FORMS.SITE_VIEW_ANALYTICS, 'trackables_and_groups'));
    } else if (values.trackables_and_groups.length > 20) {
        errors.trackables_and_groups = SITE_ANALYTICS_MODAL.ERROR_MESSAGE(props.intl, 20);
        props.dispatch(touch(NAME_OF_FORMS.SITE_VIEW_ANALYTICS, 'trackables_and_groups'));
    }
    
    if (_isEmpty(values.from)) {
        errors.from = COMPONENTS.VALIDATION.REQUIRED_FUNC(props.intl);
    }
    
    if (_isEmpty(values.to)) {
        errors.to = COMPONENTS.VALIDATION.REQUIRED_FUNC(props.intl);
    }
    
    if (_isEmpty(values.type)) {
        errors.type = COMPONENTS.VALIDATION.REQUIRED_FUNC(props.intl);
    }
    
    if (values.from && now < props.timeZoneFormat(values.from)) {
        props.dispatch(touch(NAME_OF_FORMS.SITE_VIEW_ANALYTICS, 'from'));
        errors.from = SITE_ANALYTICS_MODAL.FROM_SHOULD_BE_LESS_THEN_NOW(props.intl);
    }
    if (values.to && now < props.timeZoneFormat(values.to)) {
        props.dispatch(touch(NAME_OF_FORMS.SITE_VIEW_ANALYTICS, 'to'));
        errors.to = SITE_ANALYTICS_MODAL.TO_SHOULD_BE_LESS_THEN_NOW(props.intl);
    }
    if (values.from && values.to && props.timeZoneFormat(values.from) > props.timeZoneFormat(values.to)) {
        props.dispatch(touch(NAME_OF_FORMS.SITE_VIEW_ANALYTICS, 'from'));
        errors.from = SITE_ANALYTICS_MODAL.FROM_SHOULD_BE_LESS_THEN_TO(props.intl);
    }

    return errors;
};

SiteViewAnalyticsForm = reduxForm({
    form: NAME_OF_FORMS.SITE_VIEW_ANALYTICS,
    enableReinitialize: true,
    keepDirtyOnReinitialize: true,
    validate,
})(SiteViewAnalyticsForm);

const selector = formValueSelector(NAME_OF_FORMS.SITE_VIEW_ANALYTICS);

const mapStateToForm = (state, props) => {
    const { selectedTrackables, isPreselectedTimePeriod, selectedTime, isCustomInterval } = props.modalParams;

    const initialValues = {
        weight: 1,
        distance: 1,
        isCustomInterval,
        selectedPeriodInHours: selectedTime,
        trackables_and_groups: selectedTrackables || [],
    };

    if (isPreselectedTimePeriod) {
        const { from: _from, to: _to } = props.modalParams;

        initialValues.from = _from;
        initialValues.to = _to;
    }

    return {
        initialValues,
        type: selector(state, 'type'),
    };
};
SiteViewAnalyticsForm = connect(mapStateToForm)(injectIntl(SiteViewAnalyticsForm));

class ViewSiteAnalyticsModal extends React.Component {
    static propTypes = {
        dispatch: PropTypes.func,
        intl: PropTypes.shape({
            formatMessage: PropTypes.func,
        }),
        modalParams: PropTypes.shape(),
        style: PropTypes.shape(),
        sublocation: PropTypes.shape(),
        timeFormat: PropTypes.string,
        timeZone: PropTypes.string,
        isOpenPlayBackControl: PropTypes.bool,
        currentHeatmaps: PropTypes.arrayOf(PropTypes.shape({})),
    };
    
    static defaultProps = {
        intl: {
            formatMessage: () => {
            },
        },
        dispatch: () => {
        },
        modalParams: {},
        style: {},
        sublocation: {},
        timeFormat: 'YYYY-MM-DD HH:mm',
        isOpenPlayBackControl: false,
        timeZone: 'Etc/Gmt',
        currentHeatmaps: [],
    };

    static contextType = AppContext

    componentDidUpdate(prevProps) {
        const { isPreselectedTimePeriod, sentTrackablesRequest, from, to } = this.props.modalParams;
        
        if (isPreselectedTimePeriod && !sentTrackablesRequest && this.isValidTimeRange(from, to)) {
            this.loadTrackablesForPeriod(from, to);
        }

        const openModal = this.props.modalParams.isModalOpened && this.props.modalParams.isModalOpened !== prevProps.modalParams.isModalOpened;
        if (openModal) {
            this.props.dispatch(stopLiveMode());
        }
    }

    handleSubmitNearMissEventsReport = (isOpenPlayBackControl, currentHeatmaps, sublocation, start, end, distance) => (dispatch) => {
        if (isOpenPlayBackControl) {
            dispatch(clearPlayback());
        }
    
        if (currentHeatmaps.length > 0) {
            dispatch(clearHeatmap());
        }
        dispatch(clearNearMissReport());
        dispatch(clearSpaghettiDiagram());
        dispatch(stopLiveMode());
    
        const params = {
            sublocation,
            dateFrom: start,
            dateTo: end,
            distance: distance * 1000, // convert meters to millimeters
        };
        dispatch(generateNearMissEventReport(params));

        const NearMissEventsReport = () => {
            dispatch(clearNearMissReport());
            dispatch(startLiveMode());
        };

        dispatch(clearLastMode());
        dispatch(changeMode({
            name: SITE_MODE.NEAR_MISS,
            onReturn: NearMissEventsReport,
        }));
    };

    handleSubmitHeatmap = (start, end, weight, uids, names, intl, isCustomInterval, selectedPeriodInHours, sublocation_uid) => (dispatch) => {
        const heatMapSettings = { start, end, brashAdaptiveCoef: weight, uids, names, isCustomInterval, selectedPeriodInHours, sublocation_uid };

        dispatch(clearNearMissReport());
        dispatch(createHeatmap(heatMapSettings, intl));

        const heatmapToLive = () => {
            dispatch(clearHeatmap());
            dispatch(startLiveMode());
        };

        dispatch(clearLastMode());
        dispatch(changeMode({
            name: SITE_MODE.HEATMAP,
            onReturn: heatmapToLive,
        }));
    };

    handleSubmitPlayback = (sublocation, start, end, trackables_and_groups, selectedTrackables) => (dispatch) => {
        const params = {
            sublocation_uid: sublocation.uid,
            from: start,
            to: end,
        };

        dispatch(clearSpaghettiDiagram());
        dispatch(clearNearMissReport());
        dispatch(clearPlayback());
        dispatch(changePlaybackSelectTrackables({ selectedTrackables: trackables_and_groups }));
        dispatch(loadAssociationForPlayback(params));

        const setting = {
            playFrom: start,
            playTo: end,
            sublocation_uid: sublocation.uid,
            sequence: sublocation.id,
            idProgressBar: start,
        };

        dispatch(changePlaybackParams({
            startTs: start,
            currentTs: start,
            finishTs: end,
            setting,
        }));

        const playbackToLive = () => {
            dispatch(clearPlayback());
        };
        dispatch(generatePlayback({ setting, trackables: selectedTrackables }));
        dispatch(clearLastMode());
        dispatch(changeMode({
            name: SITE_MODE.PLAYBACK,
            onReturn: playbackToLive,
        }));
    };

    submit = (submitData) => {
        const {
            from, to, trackables_and_groups, weight, type, isCustomInterval, selectedPeriodInHours, distance,
        } = submitData;
        const {
            intl, sublocation, dispatch, isOpenPlayBackControl, currentHeatmaps,
        } = this.props;

        const uids = [];
        const names = [];
        const selectedTrackables = (trackables_and_groups && trackables_and_groups.map((trackable) => {
            uids.push(trackable.value);
            names.push(trackable.label);
            return trackable.uid;
        })) || [];

        const start = typeof from === 'number' ? from : this.getUtcTime(from);
        const end = typeof to === 'number' ? to : this.getUtcTime(to);
        
        switch (type) {
            case analyticsTypes.spaghetti: {
                dispatch(this.handleSubmitSpaghetti(isOpenPlayBackControl, currentHeatmaps, sublocation, start, end, trackables_and_groups));
                dispatch(setLastMode(this.handleSubmitSpaghetti, [isOpenPlayBackControl, currentHeatmaps, sublocation, start, end, trackables_and_groups]));
                break;
            }
            case analyticsTypes.heatmap: {
                dispatch(this.handleSubmitHeatmap(start, end, weight, uids, names, intl, isCustomInterval, selectedPeriodInHours, sublocation.uid));
                dispatch(setLastMode(this.handleSubmitHeatmap, [start, end, weight, uids, names, intl, isCustomInterval, selectedPeriodInHours, sublocation.uid]));
                break;
            }
            case analyticsTypes.playback: {
                dispatch(this.handleSubmitPlayback(sublocation, start, end, trackables_and_groups, selectedTrackables));
                dispatch(setLastMode(this.handleSubmitPlayback, [sublocation, start, end, trackables_and_groups, selectedTrackables]));
                break;
            }
            case analyticsTypes.nearMiss: {
                dispatch(this.handleSubmitNearMissEventsReport(isOpenPlayBackControl, currentHeatmaps, sublocation, start, end, distance));
                dispatch(setLastMode(this.handleSubmitNearMissEventsReport, [isOpenPlayBackControl, currentHeatmaps, sublocation, start, end, distance]));
                break;
            }
            default: {
                break;
            }
        }

        this.props.dispatch(closeSiteViewAnalyticsModal());
    };
    
    loadTrackablesForPeriod = (fromUtc, toUtc) => {
        const { sublocation, dispatch } = this.props;
        const params = {
            sublocation_uid: sublocation.uid,
            from: fromUtc,
            to: toUtc,
            isPreselectedTimePeriod: false,
        };
        dispatch(siteViewAnalyticsModalTrackablesListLoading());
        dispatch(getSiteViewAnalyticsTrackables(params));
    };
    
    onFormChange = (values, dispatch, props, previousValues) => {
        const { isCustomInterval } = this.props.modalParams;
        const timeWasChanged = values && previousValues && values.to && values.from && (values.from !== previousValues.from || values.to !== previousValues.to);
        if (isCustomInterval && timeWasChanged) {
            const fromUtc = this.getUtcTime(values.from);
            const toUtc = this.getUtcTime(values.to);
            if (this.isValidTimeRange(fromUtc, toUtc)) {
                this.loadTrackablesForPeriod(fromUtc, toUtc);
            }
        }
    };

    handleSubmitSpaghetti = (isOpenPlayBackControl, currentHeatmaps, sublocation, start, end, trackables_and_groups) => (dispatch) => {
        if (isOpenPlayBackControl) {
            dispatch(clearPlayback());
        }

        if (currentHeatmaps.length > 0) {
            dispatch(clearHeatmap());
        }

        dispatch(clearSpaghettiDiagram());
        dispatch(clearNearMissReport());
        dispatch(stopLiveMode());
        dispatch(changeSpaghettiViewMode({
            isSpaghettiDiagramShowing: true,
        }));
        const params = {
            sublocation,
            spaghettiFrom: start,
            spaghettiTo: end,
            trackables: trackables_and_groups,
        };

        dispatch(generateSpaghetti(params));

        const spaghettiToLive = () => {
            dispatch(clearSpaghettiDiagram());
            dispatch(startLiveMode());
        };

        dispatch(clearLastMode());
        dispatch(changeMode({
            name: SITE_MODE.SPAGHETTI,
            onReturn: spaghettiToLive,
        }));
    };

    getUtcTime = (date) => momentTz.tz(moment(date).format(this.props.timeFormat), this.props.timeZone).utc().valueOf();

    isValidTimeRange = (fromUtc, toUtc) => {
        if (!fromUtc || !toUtc) {
            return false;
        }
        const now = moment().utc().valueOf();
        return toUtc > fromUtc && fromUtc < now && toUtc < now;
    };
    
    closeModal = () => {
        this.props.dispatch(startLiveMode());
        this.props.dispatch(closeSiteViewAnalyticsModal());
    };

    render() {
        const { modalParams: { isModalOpened }, timeZone, style } = this.props;
        const { config } = this.context;
        const _style = makeCopy(style);
        _style.content.width = '700px';

        return (
            <Modal
                contentLabel="Site-Analytics-Modal"
                isOpen={isModalOpened}
                style={_style}
            >
                <div className="RPP-Modal">
                    <div
                        className="Modal-Header"
                        data-id="view_analytics_header"
                    >
                        {SITE_ANALYTICS_MODAL.TITLE}
                    </div>
                    <SiteViewAnalyticsForm
                        timeZoneFormat={this.getUtcTime}
                        modalParams={this.props.modalParams}
                        showNearMiss={config.ANALYTICS.NEAR_MISS_EVENTS}
                        closeModal={this.closeModal}
                        onSubmit={this.submit}
                        timeZone={timeZone}
                        onChange={this.onFormChange}
                        loadTrackablesForPeriod={this.loadTrackablesForPeriod}
                    />
                </div>
            </Modal>
        );
    }
}

const mapStateToProps = (state) => {
    const { project } = state.appContext;
    const timeZone = getTimeZoneOfProject(project);

    return {
        timeZone,
        modalParams: state.modals.commonModals.siteAnalyticsModal,
        sublocation: state.siteView.selectedSublocation,
        isOpenPlayBackControl: state.playback.isOpenPlayBackControl,
        currentHeatmaps: state.heatmap.currentHeatmaps,
    };
};

export default connect(mapStateToProps)(injectIntl(ViewSiteAnalyticsModal));
