import React from 'react';
import Modal from 'react-modal';
import PropTypes from 'prop-types';
import { connect } from 'react-redux';
import { Field, reduxForm } from 'redux-form';
import { injectIntl } from 'react-intl';

import RenderField from 'components/modals/RenderField';
import store from 'stores/LocationStore';
import { NAME_OF_FORMS } from 'common/constants';
import { COMPANY_EDIT_MODAL, COMPONENTS } from 'common/ui-labels';
import { closeCompanyEditModalSystemScope, editCompany, createCompany } from 'stores/actions';

const validate = (values) => {
    const errors = {};
    if (!values.name) {
        errors.name = COMPONENTS.VALIDATION.REQUIRED;
    } else if (!!store.getCompanyByName(values.name) && store.getCompanyByName(values.name).uid !== values.uid) {
        errors.name = COMPONENTS.VALIDATION.NAME_EXIST;
    } else if (values.name.length < 3) {
        errors.name = COMPONENTS.VALIDATION.MIN_LENGTH_FUNC(3);
    }
    
    return errors;
};

const renderField = RenderField(4, 8);

let CompanyEditForm = (props) => {
    const {
        close, submit, className, invalid, formState, intl, submitSucceeded,
    } = props;
    
    let tag_allocation;
    const companyEditForm = formState[NAME_OF_FORMS.COMPANY_EDIT];
    
    if (companyEditForm && companyEditForm.values) {
        tag_allocation = companyEditForm.values.tag_allocation;
    } else {
        tag_allocation = false;
    }
    
    return (
        <form className={className}>
            <div className="Form-Line row">
                <Field
                    className="Form-Input"
                    name="name"
                    dataId="company-edit-system-name"
                    component={renderField}
                    label={COMPANY_EDIT_MODAL.FORM.LABEL_COMPANY_NAME_FUNC(intl)}
                    type="text"
                />
            </div>
            
            <div className="Form-Line row">
                <Field
                    className="Form-Input"
                    name="website"
                    dataId="company-edit-system-website"
                    component={renderField}
                    label={COMPANY_EDIT_MODAL.FORM.WEBSITE_FUNC(intl)}
                    type="text"
                />
            </div>
            
            <div className="Form-Line row">
                <Field
                    className="Form-Input"
                    name="phone"
                    dataId="company-edit-system-phone"
                    component={renderField}
                    label={COMPONENTS.FIELDS.USER.PHONE_FUNC(intl)}
                    type="text"
                />
            </div>
            
            <div style={{ textAlign: 'center', marginTop: '15px', display: 'none' }} data-id="company-edit-system-tag-allocation">
                <Field
                    name="tag_allocation"
                    component="input"
                    type="checkbox"
                    checked={tag_allocation}
                />
                <span style={{ marginLeft: '10px' }}>
                    {COMPANY_EDIT_MODAL.FORM.ALLOW_AUTO_TAG_ALLOCATION}
                </span>
            </div>
            
            <div className="Form-Buttons">
                <button
                    type="button"
                    onClick={submit}
                    disabled={invalid || submitSucceeded}
                    data-id="company-edit-system-save"
                >
                    {COMPONENTS.BUTTONS.SAVE}
                </button>
                <button data-id="company-edit-system-cancel" type="button" onClick={close}>{COMPONENTS.BUTTONS.CANCEL}</button>
            </div>
        </form>
    );
};

CompanyEditForm.propTypes = {
    close: PropTypes.func,
    submit: PropTypes.func,
    className: PropTypes.string,
    invalid: PropTypes.bool,
    formState: PropTypes.shape(),
    intl: PropTypes.shape({
        formatMessage: PropTypes.func,
    }),
    submitSucceeded: PropTypes.bool,
};

CompanyEditForm.defaultProps = {
    close: () => {
    },
    submit: () => {
    },
    invalid: false,
    className: '',
    formState: {},
    intl: {
        formatMessage: () => {},
    },
    submitSucceeded: false,
};

CompanyEditForm = reduxForm({
    form: NAME_OF_FORMS.COMPANY_EDIT,
    validate,
})(CompanyEditForm);

const mapStateToForm = state => ({
    initialValues: state.modals.companyModals.companyEdit.params,
    formState: state.form,
});
CompanyEditForm = connect(mapStateToForm)(injectIntl(CompanyEditForm));


class CompanyEditModal extends React.Component {
    static propTypes = {
        dispatch: PropTypes.func,
        companyEdit: PropTypes.shape(),
        style: PropTypes.shape(),
    };
    
    static defaultProps = {
        dispatch: () => {
        },
        companyEdit: {},
        style: {},
    };
    
    close = () => {
        this.props.dispatch(closeCompanyEditModalSystemScope());
    };
    
    submit = (values) => {
        if (this.props.companyEdit.params && this.props.companyEdit.params.uid) {
            this.props.dispatch(editCompany(values));
        } else {
            this.props.dispatch(createCompany(values));
        }
        this.props.dispatch(closeCompanyEditModalSystemScope());
    };
    
    render() {
        const { style, companyEdit } = this.props;
        const title = companyEdit.params && companyEdit.params.uid
            ? companyEdit.params.name
            : COMPANY_EDIT_MODAL.TITLE_ADD_NEW;
        
        return (
            <Modal
                contentLabel="companyEdit"
                isOpen={companyEdit.openSystemScope}
                onRequestClose={this.close}
                style={style}
            >
                <div className="RPP-Modal">
                    <span className="Modal-Header multiple-lines-ellipsis">
                        {title}
                    </span>
                    <CompanyEditForm
                        className="Modal-Form"
                        close={this.close}
                        onSubmit={this.submit}
                    />
                </div>
            </Modal>
        );
    }
}


const mapStateToProps = state => ({
    companyEdit: state.modals.companyModals.companyEdit,
});

export default connect(mapStateToProps)(CompanyEditModal);
