import React from 'react';
import { injectIntl } from 'react-intl';

import PropTypes from 'prop-types';
import Modal from 'react-modal';
import { connect } from 'react-redux';
import { Field, reduxForm } from 'redux-form';
import _IsEmpty from 'lodash/isEmpty';

import { NAME_OF_FORMS } from 'common/constants';
import { GROUP_BULK_EDIT_MODAL, COMPONENTS } from 'common/ui-labels';
import { getAdminPageSelectedGroup, filterSelectBoxOptions } from 'utils/utils';
import { closeBulkEditGroupsModal, bulkEditGroup } from 'stores/actions';
import MultiSelectComponent from 'components/modals/MultiSelectComponent';
import SaveButtons from 'components/modals/SaveButtons';

import { Trackable } from 'stores/classes';

const required = value => (_IsEmpty(value) ? COMPONENTS.VALIDATION.REQUIRED : undefined);

let GroupBulkEditForm = (props) => {
    const { close, className, availableTrackables, handleSubmit, intl, invalid, submitted } = props;

    return (
        <form className={className} onSubmit={handleSubmit}>
            <div className="Form-Line row">
                <div className="Form-Label col-md-4">{GROUP_BULK_EDIT_MODAL.TRACKABLE_OR_TRACKABLES}*</div>
                <div className="col-md-8">
                    <Field
                        name="trackables_list"
                        dataId="group_bulk_edit_trackables_list"
                        component={MultiSelectComponent}
                        placeholder={COMPONENTS.SELECT_COMPONENT_PLACEHOLDER(intl)}
                        noResultsText={COMPONENTS.SELECT_COMPONENT_NO_RESULTS_TEXT}
                        options={availableTrackables}
                        filterOptions={filterSelectBoxOptions}
                        validate={[required]}
                    />
                </div>
            </div>
            <SaveButtons
                close={close}
                submit={handleSubmit}
                disabled={invalid}
                submitted={submitted}
                dataId="group_bulk_edit-add"
            />
        </form>
    );
};

GroupBulkEditForm.propTypes = {
    intl: PropTypes.shape({
        formatMessage: PropTypes.func,
    }),
    close: PropTypes.func,
    submit: PropTypes.func,
    className: PropTypes.string,
    availableTrackables: PropTypes.oneOfType([PropTypes.shape(), PropTypes.array]),
    handleSubmit: PropTypes.func,
    invalid: PropTypes.bool,
    submitted: PropTypes.bool,
};

GroupBulkEditForm.defaultProps = {
    intl: {
        formatMessage: () => {
        },
    },
    close: () => {
    },
    submit: () => {
    },
    className: '',
    availableTrackables: [{}],
    handleSubmit: () => {
    },
    invalid: false,
    submitted: false,
};

GroupBulkEditForm = reduxForm({
    form: NAME_OF_FORMS.GROUP_BULK_EDIT,
})(injectIntl(GroupBulkEditForm));

const mapStateToForm = (state) => {
    const selectedGroup = getAdminPageSelectedGroup(state);
    const allTrackables = Trackable.getAllTrackablesArray(state);

    const availableTrackables = allTrackables.filter(to => !to.groups || !to.groups.some(uid => uid === selectedGroup.uid)).map(x => ({
        value: x.uid,
        label: x.name,
        groups: x.groups ? [...x.groups] : [], /* for avoiding the store mutation */
        uid: x.uid,
    }));

    return ({
        availableTrackables,
        selectedGroup,
    });
};

GroupBulkEditForm = connect(mapStateToForm)(GroupBulkEditForm);


class GroupBulkEditModal extends React.Component {
    static propTypes = {
        dispatch: PropTypes.func,
        selectedGroup: PropTypes.shape({}),
        groupBulkEdit: PropTypes.shape({
            open: PropTypes.bool,
        }),
        style: PropTypes.shape({
            overlay: PropTypes.object,
            content: PropTypes.object,
        }),
    };
    
    static defaultProps = {
        dispatch: () => {
        },
        selectedGroup: {},
        groupBulkEdit: {
            open: false,
        },
        style: {
            overlay: {},
            content: {},
        },
    };

    constructor(props) {
        super(props);
        this.state = {
            submitted: false,
        };
    }
    
    close = () => {
        this.props.dispatch(closeBulkEditGroupsModal());
    };
    
    submit = (values) => {
        const { trackables_list } = values;
        const { selectedGroup, dispatch } = this.props;

        this.setState({ submitted: true });

        this.close();

        dispatch(bulkEditGroup({ trackables: trackables_list, groupUid: selectedGroup.uid }));
    };
    
    render() {
        const { style, groupBulkEdit } = this.props;
        return (
            <Modal
                contentLabel="groupBulkEdit"
                isOpen={groupBulkEdit.open}
                onRequestClose={this.close}
                shouldCloseOnOverlayClick={false}
                onAfterClose={() => {
                    this.setState({ submitted: false });
                }}
                style={style}
            >
                <div className="RPP-Modal">
                    <div className="Modal-Header capitalizedText">
                        {GROUP_BULK_EDIT_MODAL.TITLE}
                    </div>
                    <div className="Modal-Header-Subtitle">
                        {GROUP_BULK_EDIT_MODAL.SUB_TITLE}
                    </div>
                    <GroupBulkEditForm
                        className="Modal-Form"
                        submitted={this.state.submitted}
                        close={this.close}
                        onSubmit={this.submit}
                    />
                </div>
            </Modal>
        );
    }
}

const mapStateToProps = state => ({
    groupBulkEdit: state.modals.groupModals.groupBulkEdit,
    selectedGroup: getAdminPageSelectedGroup(state),
});

export default connect(mapStateToProps)(GroupBulkEditModal);
