import { Field, formValueSelector, reduxForm } from 'redux-form';
import PropTypes from 'prop-types';
import connect from 'react-redux/es/connect/connect';
import { injectIntl } from 'react-intl';
import React from 'react';
import RenderField from 'components/modals/RenderField';
import RenderTextArea from 'components/modals/RenderTextArea';
import SubmitButtons from 'components/modals/forms/SubmitButtons';
import RenderBox from 'components/modals/RenderBox';
import CustomInfoFieldComponent from 'components/modals/CustomInfoField';
import {
    DEFAULT_VALUES,
    ERROR_TEXT,
    GROUP_EDIT_MODAL,
    LABELS,
    NAME_OF_FORMS,
    T2T_TYPE, COMPONENTS,
} from 'common';
import { checkboxGroupClick } from 'stores/actions/modalActions';
import { getLength } from 'utils/utils';

import SensorInterfaceSection from './SensorInterfaceSection';
import { getParseGroupEdit } from './utils';

const renderField = RenderField(4, 8);

let GroupEditForm = (props) => {
    const {
        close, submit, className, invalid, intl, dispatch,
        isAdvanced, uid, customInfoLength, submitSucceeded, maxHeight,
    } = props;

    return (
        <form className={className} onSubmit={(e) => { e.preventDefault(); }}>
            <div
                className="col-md-12"
                style={{ overflowY: 'auto', overflowX: 'hidden', maxHeight }}
            >
                <div className="Form-Line row">
                    <Field
                        autoFocus={!uid}
                        className="Form-Input"
                        name="name"
                        dataId="group-edit-name"
                        component={renderField}
                        label={`${GROUP_EDIT_MODAL.FORM.NAME_FUNC(intl)} *`}
                        type="text"
                    />
                </div>
                <div className="Form-Line row">
                    <div htmlFor="companyColor" className="Form-Label col-md-4">{COMPONENTS.FIELDS.COMMON.COLOR}</div>
                    <div className="col-md-8" data-id="group-edit-color">
                        <Field
                            name="color"
                            className="Form-Input"
                            component="input"
                            type="color"
                        />
                    </div>
                </div>
                <br />

                {/* ADVANCED */}
                <div className="Form-Line row" style={{ marginTop: '2px' }}>
                    <div className="Form-Label col-md-4">{COMPONENTS.FIELDS.NODE.ADVANCED}</div>
                    <div className="col-md-1 Custom-Rotated-Checkbox" data-id="group-edit-checkbox">
                        <Field
                            onChange={e => dispatch(checkboxGroupClick({ advanced: e.target.checked }))}
                            onBlur={() => dispatch(checkboxGroupClick({}))}
                            name="advanced"
                            id="advanced"
                            component={RenderBox}
                            type="checkbox"
                        />
                    </div>
                    <div
                        className="col-md-7"
                    />
                </div>

                <CustomInfoFieldComponent
                    placeholder={LABELS.CUSTOM_INFO_PLACEHOLDER(intl)}
                    dataId="group-edit-custom-info"
                    customInfoLength={customInfoLength}
                />

                <div className="Form-Line container-fluid RPP-Group-advanced" style={{ display: isAdvanced ? 'block' : 'none' }}>
                    <h4 className="RPP-Group-advanced-border-title">{GROUP_EDIT_MODAL.FORM.ADVANCED_CUSTOM_PARAMETER}</h4>
                    <div className="col-md-12" style={{ marginTop: '15px' }}>
                        <Field
                            name="customParams"
                            dataId="group-edit-advanced-parameter"
                            component={RenderTextArea}
                            rows="5"
                            maxLength={100}
                            onKeyPress={(e) => {
                                if (e.key === 'Enter') {
                                    e.preventDefault();
                                }
                            }}
                            placeholder={GROUP_EDIT_MODAL.FORM.ADVANCED_CUSTOM_PARAMETER_PLACEHOLDER_FUNC(intl)}
                        />
                    </div>
                </div>

                {SensorInterfaceSection({ isAdvanced })}
            </div>
            <SubmitButtons
                className="Form-Buttons"
                submit={submit}
                invalid={invalid || submitSucceeded}
                close={close}
                saveDataId="zone-add-configuration-site-page-save"
                cancelDataId="zone-add-configuration-site-page-cancel"
                saveDataCy="group-edit-save"
                cancelDataCy="group-edit-cancel"
            />
        </form>
    );
};

GroupEditForm.propTypes = {
    close: PropTypes.func,
    submit: PropTypes.func,
    invalid: PropTypes.bool,
    className: PropTypes.string,
    dispatch: PropTypes.func,
    intl: PropTypes.shape({
        formatMessage: PropTypes.func,
    }),
    uid: PropTypes.string,
    isAdvanced: PropTypes.bool,
    customInfoLength: PropTypes.number,
    maxHeight: PropTypes.string,
    submitSucceeded: PropTypes.bool,
};

GroupEditForm.defaultProps = {
    close: () => {
    },
    submit: () => {
    },
    invalid: false,
    className: '',
    dispatch: () => {
    },
    intl: {
        formatMessage: () => {
        },
    },
    uid: PropTypes.string,
    isAdvanced: false,
    customInfoLength: 0,
    maxHeight: '60vh',
    submitSucceeded: false,
};

const validate = (values, props) => {
    const errors = {};
    const nameExist = Object.values(props.groups).find(group => (group.name === values.name && group.uid !== values.uid));
    
    if (!values.name) {
        errors.name = COMPONENTS.VALIDATION.REQUIRED;
    } else if (nameExist) {
        errors.name = COMPONENTS.VALIDATION.NAME_EXIST;
    }
    if (values.custom_info && getLength(values.custom_info) > DEFAULT_VALUES.CUSTOM_INFO_LIMIT) {
        errors.custom_info = ERROR_TEXT.CUSTOM_INFO_ERROR();
    }

    if (values.trackable_statinary_count > 0 && values.role === T2T_TYPE.ANNOUNCER) {
        errors.role = GROUP_EDIT_MODAL.ERROR.STATIONARY_PWS_CONFLICT;
    }

    return errors;
};

GroupEditForm = reduxForm({
    form: NAME_OF_FORMS.GROUP_EDIT,
    validate,
})(GroupEditForm);

const mapStateToForm = (state) => {
    const selector = formValueSelector(NAME_OF_FORMS.GROUP_EDIT);
    const {
        uid, advanced, custom_info,
    } = selector(state, 'uid', 'advanced', 'custom_info');

    return {
        groups: state.entities.groups.byUid,
        initialValues: getParseGroupEdit(state).params,
        isAdvanced: advanced,
        customInfoLength: custom_info ? getLength(custom_info) : 0,
        uid: uid || '',
    };
};

GroupEditForm = connect(mapStateToForm)(injectIntl(GroupEditForm));

export default GroupEditForm;
