import React from 'react';
import Modal from 'react-modal';
import { connect } from 'react-redux';
import { injectIntl } from 'react-intl';
import { reduxForm } from 'redux-form';
import PropTypes from 'prop-types';

import { NAME_OF_FORMS, NODE_DETAILS_MODAL, NODE_TYPE, COMPONENTS } from 'common';
import { durationFormatting, timeFormat, macToUpperCase, getNodeBatteryLevel, getTimeZoneOfProject, getFullDateByTz } from 'utils';
import { closeDetailsNodeModal } from 'stores/actions';
import { Trackable } from 'stores/classes';


const findProjectTimeZoneByNode = (projects, node) => {
    let timeZone = 'Etc/GMT';
    if (node.project_uid) {
        const nodeProject = projects.find(project => (project.uid === node.project_uid));
        timeZone = getTimeZoneOfProject(nodeProject);
    }
    return timeZone;
};

let NodeDetailsForm = (props) => {
    const {
        close, params, className, projects, projectContext, intl,
    } = props;
    let timeZone = 'Etc/GMT';
    
    if (projectContext) {
        timeZone = getTimeZoneOfProject(projectContext);
    } else {
        timeZone = findProjectTimeZoneByNode(projects, params);
    }

    const now = new Date().getTime();
    let heardDuration;
    if (params.last_heard !== '') {
        const last_heard = new Date(params.last_heard).getTime();
        heardDuration = now - last_heard;
    }
    
    const created_at = getFullDateByTz(params.created_at, timeZone);
    const updated_at = getFullDateByTz(params.updated_at, timeZone);

    return (
        <form className={className}>
            <div className="Form-Line row">
                <div className="Form-Label-Grey col-md-6">{COMPONENTS.FIELDS.COMMON.PROJECT}:</div>
                <div className="Form-Value-Bold col-md-6 multiple-lines-ellipsis" data-id="node-details-project-name">{params.project_name}</div>
            </div>
            <div className="Form-Line row">
                <div className="Form-Label-Grey col-md-6">{COMPONENTS.FIELDS.NODE.TRACKABLE}:</div>
                <div className="Form-Value-Bold col-md-6 multiple-lines-ellipsis" data-id="node-details-trackable-name">{params.trackable_name}</div>
            </div>
            <div className="Form-Line row">
                <div className="Form-Label-Grey col-md-6">{NODE_DETAILS_MODAL.FORM.PARENT}</div>
                <div className="Form-Value-Bold col-md-6" data-id="node-details-parent_mac">
                    {params.parent_mac_address ? macToUpperCase(params.parent_mac_address) : ''}
                </div>
            </div>
            <div className="Form-Line row">
                <div className="Form-Label-Grey col-md-6">{NODE_DETAILS_MODAL.FORM.BRIDGE}</div>
                <div className="Form-Value-Bold col-md-6" data-id="node-details-bridge_mac">
                    {params.bridge_mac_address ? macToUpperCase(params.bridge_mac_address) : ''}
                </div>
            </div>
            <div className="Form-Line row">
                <div className="Form-Label-Grey col-md-6">{COMPONENTS.FIELDS.NODE.LAST_REPORTED}:</div>
                <div className="Form-Value-Bold col-md-6" data-id="node-details-last_heard">
                    {durationFormatting(heardDuration, intl).text}
                </div>
            </div>
            <div className="Form-Line row">
                <div className="Form-Label-Grey col-md-6">{NODE_DETAILS_MODAL.FORM.DEVICE_TYPE}</div>
                <div className="Form-Value-Bold col-md-6" data-id="node-details-type">{params.node_type}</div>
            </div>
            <div className="Form-Line row">
                <div className="Form-Label-Grey col-md-6">{NODE_DETAILS_MODAL.FORM.CREATED_AT}</div>
                <div className="Form-Value-Bold col-md-6" data-id="node-details-created_at">{created_at}</div>
            </div>
            <div className="Form-Line row">
                <div className="Form-Label-Grey col-md-6">{NODE_DETAILS_MODAL.FORM.UPDATED_AT}</div>
                <div className="Form-Value-Bold col-md-6" data-id="node-details-updated-at">{updated_at}</div>
            </div>
            <div className="Form-Line row">
                <div className="Form-Label-Grey col-md-6">{NODE_DETAILS_MODAL.FORM.BATTERY}</div>
                <div className="Form-Value-Bold col-md-6" data-id="node-details-battery">
                    {getNodeBatteryLevel(params)}
                </div>
            </div>
            <div className="Form-Buttons OneBtn">
                <button type="button" data-id="node-details-cancel" onClick={close}>{COMPONENTS.BUTTONS.CLOSE}</button>
            </div>
        </form>
    );
};

NodeDetailsForm.propTypes = {
    close: PropTypes.func,
    className: PropTypes.string,
    projectContext: PropTypes.shape(),
    projects: PropTypes.arrayOf(PropTypes.shape({})),
    params: PropTypes.shape({}),
    intl: PropTypes.shape({
        formatMessage: PropTypes.func,
    }),
};

NodeDetailsForm.defaultProps = {
    close: () => {
    },
    className: '',
    projectContext: null,
    projects: [],
    params: {},
    intl: {
        formatMessage: () => {
        },
    },
};


NodeDetailsForm = reduxForm({
    form: NAME_OF_FORMS.NODE_DETAILS,
})(NodeDetailsForm);

NodeDetailsForm = connect()(injectIntl(NodeDetailsForm));

class NodeDetailsModal extends React.Component {
    static propTypes = {
        dispatch: PropTypes.func,
        nodeDetails: PropTypes.shape({
            params: PropTypes.shape({
                mac_address: PropTypes.string,
            }),
            open: PropTypes.bool,
        }),
        projects: PropTypes.arrayOf(PropTypes.shape({})),
        projectContext: PropTypes.shape(),
        trackablesMap: PropTypes.shape({}),
        style: PropTypes.shape({}),
        intl: PropTypes.shape({
            formatMessage: PropTypes.func,
        }),
    };
    
    static defaultProps = {
        dispatch: () => {
        },
        nodeDetails: {
            params: {
                mac_address: '',
            },
            open: false,
        },
        projects: [],
        projectContext: null,
        trackablesMap: {},
        style: {},
        intl: {
            formatMessage: () => {
            },
        },
    };
    
    timeFormatter = (_time) => {
        const time = new Date(_time);
        const zeroTime = new Date(0);
        if (time.getFullYear() === zeroTime.getFullYear()) {
            return COMPONENTS.FIELDS.COMMON.NEVER_FUNC(this.props.intl);
        }
        return timeFormat(time);
    };
    
    formatNodes = (node, projects, trackablesMap) => {
        if (node) {
            projects.forEach((project) => {
                if (project.uid === node.project_uid) {
                    node.project_name = project.name;
                }
            });
            node.last_heard = this.timeFormatter(node.last_heard);
            node.created_at = this.timeFormatter(node.created_at);
            node.updated_at = this.timeFormatter(node.updated_at);
            if (node.node_type === NODE_TYPE.CONFIGURATION_PENDING) {
                node.node_type = NODE_DETAILS_MODAL.TEXT.UNDEFINED_FUNC(this.props.intl);
            }
            if (node.node_type === NODE_TYPE.TRACKABLE) {
                node.node_type = COMPONENTS.FIELDS.NODE.TRACKABLE_FUNC(this.props.intl);
            }
            if (node.node_type === NODE_TYPE.WIN_ANCHOR) {
                node.node_type = NODE_DETAILS_MODAL.TEXT.WIN_FUNC(this.props.intl);
            }

            const trackable = Trackable.getTrackableByMacAddress(node.mac_address, trackablesMap);
            if (trackable) {
                node.trackable_name = trackable.name;
            }
        }
        return node;
    };
    
    close = () => {
        this.props.dispatch(closeDetailsNodeModal());
    };
    
    render() {
        let node = this.props.nodeDetails.params;
        const { style, projects, trackablesMap, projectContext } = this.props;
        node = this.formatNodes(node, projects, trackablesMap);
        return (
            <Modal
                contentLabel="projectDetails"
                isOpen={this.props.nodeDetails.open}
                onRequestClose={this.close}
                style={style}
            >
                <div className="RPP-Modal">
                    <span
                        className="Modal-Header multiple-lines-ellipsis"
                        data-cy="node-details-header"
                    >
                        {COMPONENTS.FIELDS.COMMON.VIEW_INFORMATION_FUNC(this.props.nodeDetails.params ? macToUpperCase(this.props.nodeDetails.params.mac_address) : '')}
                    </span>
                    <NodeDetailsForm
                        className="Modal-Form"
                        close={this.close}
                        params={node}
                        projects={projects}
                        projectContext={projectContext}
                    />
                </div>
            </Modal>
        );
    }
}

const mapStateToProps = state => ({
    nodeDetails: state.modals.nodeModals.nodeDetails,
    projectContext: state.appContext.project,
    projects: state.entities.projects,
    trackablesMap: state.entities.trackables.byUid,
});

export default connect(mapStateToProps)(injectIntl(NodeDetailsModal));
