import React from 'react';
import PropTypes from 'prop-types';
import Modal from 'react-modal';
import { connect } from 'react-redux';
import { Field, reduxForm } from 'redux-form';
import { injectIntl } from 'react-intl';
import Select from 'react-select';

import RenderField from 'components/modals/RenderField';
import RenderSelectField from 'components/modals/RenderSelectField';
import RenderTooltipHelper from 'components/RenderTooltipHelper';

import { closeNotificationEditModal, updateNotification, createNotification } from 'stores/actions';
import {
    NOTIFICATION_MODAL,
    NAME_OF_FORMS,
    NOTIFICATION_TYPE,
    COMPONENTS,
    DEFAULT_VALUES,
} from 'common';
import { alarmLevelStyle, getNotificationTypes } from 'utils';

import Notification from 'stores/classes/Notification';

const MultiSelectComponentWithError = ({
    options, placeholder, meta: { touched, error }, input: { value, onChange, onBlur },
}) => (
    <div>
        <Select
            options={options}
            className="menu-outer-top"
            multi
            placeholder={placeholder}
            onChange={onChange}
            value={value}
            onBlur={() => onBlur(value)}
        />
        {touched && error && <span className="error">{error}</span>}
    </div>
);

MultiSelectComponentWithError.propTypes = {
    options: PropTypes.arrayOf(PropTypes.shape({
        value: PropTypes.string,
        label: PropTypes.string,
    })),
    meta: PropTypes.shape({
        touched: PropTypes.bool,
        error: PropTypes.string,
    }),
    input: PropTypes.shape({
        onChange: PropTypes.func,
        value: PropTypes.arrayOf(PropTypes.shape({})),
        onBlur: PropTypes.func,
    }),
    placeholder: PropTypes.string,
};
MultiSelectComponentWithError.defaultProps = {
    options: [],
    meta: {
        touched: PropTypes.bool,
        error: PropTypes.string,
    },
    input: {
        onChange: () => {},
        value: [],
        onBlur: () => {},
    },
    placeholder: '',
};

const validate = (values, props) => {
    const errors = {};
    if (!values.name) {
        errors.name = COMPONENTS.VALIDATION.REQUIRED_FUNC(props.intl);
        // for case of duplicate name
    } else {
        const subscriber = props.notifications.find(x => x.name === values.name);
        if (subscriber && subscriber.uid !== values.uid) {
            errors.name = COMPONENTS.VALIDATION.NAME_EXISTS_FUNC(props.intl);
        }
    }
    
    if (!values.alarm_type_list || (values.alarm_type_list && values.alarm_type_list.length === 0)) {
        errors.alarm_type_list = COMPONENTS.VALIDATION.REQUIRED_FUNC(props.intl);
    }
    return errors;
};

const renderField_6_6 = RenderField(6, 6);

let NotificationEditForm = (props) => {
    const {
        close, submit, className, invalid, intl, alarmTypes, initialValues, submitSucceeded,
    } = props;

    return (
        <form className={className}>
            <div className="Form-Line row">
                <Field
                    className="Form-Input"
                    name="name"
                    dataId="notification-edit-name"
                    component={renderField_6_6}
                    label={`${NOTIFICATION_MODAL.FORM.LABEL.NOTIFICATION_NAME_FUNC(intl)} *`}
                    type="text"
                    autoFocus={!(initialValues && initialValues.uid)}
                />
            </div>

            <div className="Form-Line row">
                <div htmlFor="alarm_type_list" className="Form-Label col-md-6">
                    {NOTIFICATION_MODAL.FORM.LABEL.NOTIFICATION_TYPE} *
                </div>
                <div htmlFor="alarm_type_list" className="col-md-6" data-id="notification-edit-alarm-type">
                    <Field
                        name="alarm_type_list"
                        type="text"
                        label="Objective"
                        placeholder={COMPONENTS.SELECT_COMPONENT_PLACEHOLDER(intl)}
                        options={alarmTypes}
                        component={MultiSelectComponentWithError}
                    />
                </div>
            </div>
            
            <div className="Form-Line row">
                <div htmlFor="zone_severity_level" className="Form-Label col-md-6">
                    {COMPONENTS.FIELDS.ZONE.SEVERITY_LEVEL}
                    <RenderTooltipHelper
                        tooltipText={NOTIFICATION_MODAL.FORM.LABEL.CREATE_ZONE_SEVERITY_LEVEL_COMMENT_FUNC(intl)}
                    />
                </div>
                <div className="Form-Label col-md-6">
                    <Field
                        className="form-control input-sm "
                        name="zone_severity_level"
                        dataId="notification-edit-severity"
                        style={{ fontSize: '14px' }}
                        component={RenderSelectField}
                    >
                        <option />
                        <option value={1} style={alarmLevelStyle(1)}>1</option>
                        <option value={2} style={alarmLevelStyle(2)}>2</option>
                        <option value={3} style={alarmLevelStyle(3)}>3</option>
                        <option value={4} style={alarmLevelStyle(4)}>4</option>
                    </Field>
                </div>
            </div>
            <div className="Form-Line row">
                <div htmlFor="email" className="Form-Label col-md-6">
                    {NOTIFICATION_MODAL.FORM.LABEL.NOTIFICATION_METHOD}
                </div>
                <div className="col-md-6" data-id="notification-edit-email">
                    <div className="row">
                        <div className="col-md-1">
                            <Field
                                name="email"
                                data-cy="notification-edit-email"
                                className="Form-Input"
                                component="input"
                                type="checkbox"
                            />
                        </div>
                        <div className="col-md-10" style={{ marginTop: '5px' }}>{COMPONENTS.FIELDS.USER.EMAIL}</div>
                    </div>
                    <div className="row">
                        <div className="col-md-1">
                            <Field
                                name="phone"
                                data-cy="notification-edit-sms"
                                className="Form-Input"
                                component="input"
                                type="checkbox"
                            />
                        </div>
                        <div className="col-md-10" style={{ marginTop: '5px' }}>{NOTIFICATION_MODAL.FORM.LABEL.SMS_TEXT}</div>
                    </div>
                </div>
            </div>
            <div className="Form-Line row">
                <div htmlFor="active" className="Form-Label col-md-6">
                    {COMPONENTS.FIELDS.COMMON.ACTIVE}
                </div>
                <div className="col-md-6" data-id="notification-edit-active">
                    <Field
                        name="active"
                        data-cy="notification-edit-active"
                        className="Form-Input"
                        component="input"
                        type="checkbox"
                    />
                </div>
            </div>
            
            <div className="Form-Buttons">
                <button
                    type="button"
                    data-id="notification-edit-save"
                    onClick={submit}
                    disabled={invalid || submitSucceeded}
                >
                    {COMPONENTS.BUTTONS.SAVE}
                </button>
                <button
                    type="button"
                    data-id="notification-edit-cancel"
                    onClick={close}
                >
                    {COMPONENTS.BUTTONS.CANCEL}
                </button>
            </div>
        </form>
    );
};

NotificationEditForm.propTypes = {
    close: PropTypes.func,
    submit: PropTypes.func,
    invalid: PropTypes.bool,
    className: PropTypes.string,
    intl: PropTypes.shape({
        formatMessage: PropTypes.func,
    }),
    initialValues: PropTypes.shape({}),
    alarmTypes: PropTypes.arrayOf(PropTypes.shape({
        uid: PropTypes.string,
        name: PropTypes.string,
    })),
    submitSucceeded: PropTypes.bool,
};

NotificationEditForm.defaultProps = {
    close: () => {
    },
    submit: () => {
    },
    invalid: false,
    className: '',
    intl: {
        formatMessage: () => {
        },
    },
    initialValues: {},
    alarmTypes: [],
    submitSucceeded: false,
};

NotificationEditForm = reduxForm({
    form: NAME_OF_FORMS.NOTIFICATION_EDIT,
    validate,
})(NotificationEditForm);

const mapStateToForm = (state, { intl }) => ({
    notifications: Notification.getAllNotifications(state),
    formState: state.form,
    initialValues: state.modals.notificationModals.notificationEdit.params,
    alarmTypes: getNotificationTypes(Notification.getNotificationsTypes(state), intl),
});

NotificationEditForm = injectIntl(connect(mapStateToForm)(injectIntl(NotificationEditForm)));

const getTitle = (notificationEdit) => {
    if (notificationEdit.params && notificationEdit.params.notification_type === NOTIFICATION_TYPE.ALARM) {
        return notificationEdit.params && notificationEdit.params.uid && notificationEdit.params.name
            ? notificationEdit.params.name
            : NOTIFICATION_MODAL.TITLE;
    }
    
    return null;
};

class NotificationEditModal extends React.Component {
    static propTypes = {
        dispatch: PropTypes.func,
        notificationEdit: PropTypes.shape({
            params: PropTypes.shape({
                uid: PropTypes.string,
            }),
            open: PropTypes.bool,
        }),
        style: PropTypes.shape({}),
    };
    
    static defaultProps = {
        dispatch: () => {
        },
        notificationEdit: {
            params: {
                uid: '',
            },
            open: false,
        },
        style: {},
    };
    
    close = () => {
        this.props.dispatch(closeNotificationEditModal());
    };
    
    submit = (_values) => {
        const values = {
            uid: _values.uid,
            name: _values.name,
            notification_type: NOTIFICATION_TYPE.ALARM,
            active: _values.active ? 1 : 0,
            conditions: {
                zone_severity_level: _values.zone_severity_level || 1,
                alarm_type_list: _values.alarm_type_list ? _values.alarm_type_list.map(x => x.uid) : DEFAULT_VALUES.empty_array,
            },
            types: {
                email: _values.email,
                phone: _values.phone,
            },
        };
        
        if (this.props.notificationEdit.params && this.props.notificationEdit.params.uid) {
            this.props.dispatch(updateNotification(values));
        } else {
            this.props.dispatch(createNotification(values));
        }
        this.props.dispatch(closeNotificationEditModal());
    };
    
    render() {
        const { style, notificationEdit } = this.props;
        
        const title = getTitle(notificationEdit);
        
        return (
            <Modal
                contentLabel="notificationEdit"
                isOpen={this.props.notificationEdit.open}
                onRequestClose={this.close}
                style={style}
            >
                
                <div className="RPP-Modal">
                    <span className="Modal-Header multiple-lines-ellipsis" data-id="notification-edit-title">
                        {title || NOTIFICATION_MODAL.TITLE}
                    </span>
                    <NotificationEditForm
                        className="Modal-Form"
                        close={this.close}
                        onSubmit={this.submit}
                    />
                </div>
            </Modal>
        );
    }
}

const mapStateToProps = state => ({
    notificationEdit: state.modals.notificationModals.notificationEdit,
});

export default connect(mapStateToProps)(NotificationEditModal);
