import React from 'react';
import Modal from 'react-modal';
import PropTypes from 'prop-types';
import { connect } from 'react-redux';
import { Field, reduxForm, touch } from 'redux-form';
import { injectIntl } from 'react-intl';
import { NAME_OF_FORMS } from 'common/constants';
import { SHIFTS_MODAL, COMPONENTS } from 'common/ui-labels';
import { closeShiftEditModal, locationShiftUpdate, shiftEditDisable } from 'stores/actions';
import { sortByField, getShifts } from 'utils/utils';
import { shiftOverlapped } from 'utils/timeUtils';
import RenderSelectField from 'components/modals/RenderSelectField';
import RenderWeekdayPicker from 'components/modals/RenderWeekdayPicker';
import RenderTimePicker from 'components/modals/RenderTimePicker';
import RenderField from 'components/modals/RenderField';

const validate = (values, props) => {
    const { name, location_uid, start_time, end_time, weekdays } = values;
    const location = props.locations.find((loc) => loc.uid === location_uid);
    const errors = {};
    if (location && values && shiftOverlapped(values, location.shift)) {
        errors.end_time = SHIFTS_MODAL.EDIT.WARNINGS.SHOULD_NOT_OVERLAP;
    }
    if (!name || !name.trim()) {
        errors.name = COMPONENTS.VALIDATION.REQUIRED;
    }
    if (!location_uid) {
        errors.location_uid = COMPONENTS.VALIDATION.REQUIRED;
    }
    if (!start_time || !end_time) {
        props.dispatch(touch(NAME_OF_FORMS.SHIFT_EDIT, 'end_time'));

        errors.end_time = COMPONENTS.VALIDATION.REQUIRED;
    }

    const areWeekdaysNotChecked = !weekdays || weekdays.length === 0;
    if (areWeekdaysNotChecked) {
        errors.weekdays = COMPONENTS.VALIDATION.REQUIRED;
    }

    return errors;
};

const warn = (values, props) => {
    const { name, uid } = values;
    const warnings = {};

    if (props.shifts && props.shifts.find((x) => x.name === name && x.uid !== uid)) {
        warnings.name = SHIFTS_MODAL.EDIT.WARNINGS.NAME_DUPLICATES;
    }

    return warnings;
};

const renderField = RenderField(4, 8);
const positive = (value) => ((Number(value) >= 0 || !value) ? undefined : SHIFTS_MODAL.EDIT.WARNINGS.NUMBER_POSITIVE_OR_EMPTY);

let ShiftEditForm = (props) => {
    const { intl, close, className, invalid, isEditMode, submitSucceeded, handleSubmit, locations } = props;

    return (
        <form className={className} onSubmit={handleSubmit}>
            <div className="Form-Line row">
                <div className="col-md-4 Form-Label">{COMPONENTS.FIELDS.SITE.SITE}</div>
                <div className="col-md-8">
                    <Field
                        className="form-control input-sm "
                        name="location_uid"
                        dataCy="shift-edit-location"
                        disabled={isEditMode}
                        style={{ fontSize: '14px' }}
                        component={RenderSelectField}
                    >
                        <option value="" />
                        {
                            locations.map((location) => (
                                <option key={location.uid} value={location.uid}>{location.name}</option>
                            ))
                        }
                    </Field>
                </div>
            </div>

            <div className="Form-Line row">
                <Field
                    className="Form-Input"
                    name="name"
                    dataId="shift-name"
                    component={renderField}
                    label={COMPONENTS.FIELDS.COMMON.NAME_FUNC(intl)}
                    type="text"
                />
            </div>

            <div className="Form-Line row">
                <div className="Form-Label col-md-4">{SHIFTS_MODAL.EDIT.START_END}</div>
                <div className="col-md-8 multicomponent-field">
                    <Field
                        className="Form-Input"
                        name="start_time"
                        dataId="shift-start_time"
                        placeholder={SHIFTS_MODAL.EDIT.SELECT_TIME(intl)}
                        component={RenderTimePicker}
                    />
                    <div className="field-separator">-</div>
                    <Field
                        className="Form-Input"
                        name="end_time"
                        dataId="shift-end_time"
                        placeholder={SHIFTS_MODAL.EDIT.SELECT_TIME(intl)}
                        component={RenderTimePicker}
                    />
                </div>
            </div>

            <div className="Form-Line row">
                <div className="Form-Label col-md-4">{COMPONENTS.FIELDS.SHIFT.WEEKDAYS}</div>
                <div className="col-md-8">
                    <Field
                        className="Form-Input"
                        name="weekdays"
                        dataId="shift-weekdays"
                        component={RenderWeekdayPicker}
                    />
                </div>
            </div>
            <div className="Form-Line row" style={{ marginTop: '2px', display: 'block' }}>
                <div className="Form-Label col-md-4">{COMPONENTS.FIELDS.COMMON.ACTIVE}</div>
                <div className="col-md-1" data-id="trackable-edit-active-buffer">
                    <Field
                        name="active"
                        id="active"
                        data-cy="shift-active"
                        component="input"
                        type="checkbox"
                        style={{ width: '19px', height: '19px' }}
                    />
                </div>
                <div className="col-md-7" />
            </div>

            <div className="Form-Line row">
                <Field
                    className="Form-Input"
                    name="expected_person"
                    dataId="shift-expected-person"
                    component={renderField}
                    type="number"
                    validate={[positive]}
                    label={SHIFTS_MODAL.EXPECTED_PERSON}
                    placeholder={SHIFTS_MODAL.NOT_SPECIFIED_TXT(intl)}
                />
            </div>

            <div className="Form-Line row">
                <Field
                    className="Form-Input"
                    name="expected_vehicles"
                    dataId="shift-expected-vehicles"
                    component={renderField}
                    type="number"
                    validate={[positive]}
                    label={SHIFTS_MODAL.EXPECTED_VEHICLE}
                    placeholder={SHIFTS_MODAL.NOT_SPECIFIED_TXT(intl)}
                />
            </div>

            <div className="Form-Line row">
                <Field
                    className="Form-Input"
                    name="expected_others"
                    dataId="shift-expected-others"
                    component={renderField}
                    type="number"
                    validate={[positive]}
                    label={SHIFTS_MODAL.EXPECTED_OTHERS}
                    placeholder={SHIFTS_MODAL.NOT_SPECIFIED_TXT(intl)}
                />
            </div>

            <div className="Form-Buttons">
                <button
                    type="submit"
                    disabled={invalid || submitSucceeded}
                    data-id="shift-save"
                >
                    {COMPONENTS.BUTTONS.SAVE}
                </button>
                <button
                    type="button"
                    data-id="shift-cancel"
                    onClick={close}
                >
                    {COMPONENTS.BUTTONS.CANCEL}
                </button>
            </div>
        </form>
    );
};

ShiftEditForm.propTypes = {
    close: PropTypes.func,
    handleSubmit: PropTypes.func,
    className: PropTypes.string,
    invalid: PropTypes.bool,
    isEditMode: PropTypes.bool,
    intl: PropTypes.shape({
        formatMessage: PropTypes.func,
    }),
    submitSucceeded: PropTypes.bool,
    locations: PropTypes.arrayOf(PropTypes.shape()),
};

ShiftEditForm.defaultProps = {
    close: () => {},
    handleSubmit: () => {},
    invalid: false,
    isEditMode: false,
    className: '',
    intl: {
        formatMessage: () => {},
    },
    submitSucceeded: false,
    locations: [],
};

ShiftEditForm = reduxForm({
    form: NAME_OF_FORMS.SHIFT_EDIT,
    validate,
    warn,
    touchOnChange: true,
})(ShiftEditForm);

const mapStateToForm = (state) => ({
    initialValues: state.modals.shiftModals.shiftEdit.params,
    shifts: getShifts(state.entities.locations),
});
ShiftEditForm = connect(mapStateToForm)(injectIntl(ShiftEditForm));

class ShiftEditModal extends React.Component {
    static propTypes = {
        dispatch: PropTypes.func,
        locations: PropTypes.arrayOf(PropTypes.shape()),
        shiftEdit: PropTypes.shape(),
        style: PropTypes.shape(),
    };
    
    static defaultProps = {
        dispatch: () => { },
        locations: [],
        shiftEdit: {},
        style: {},
    };
    
    close = () => {
        this.props.dispatch(closeShiftEditModal());
    };

    shiftEditDisable = () => {
        this.props.dispatch(shiftEditDisable());
    };

    updateLocationShift = (shift) => {
        this.props.dispatch(locationShiftUpdate(shift));
    };
    
    submit = (values) => {
        const { locations } = this.props;
        const location = locations.find((loc) => loc.uid === values.location_uid);

        if (location) {
            this.updateLocationShift(values);
        }
        this.close();
        this.shiftEditDisable();
    };
    
    render() {
        const { style, shiftEdit, locations } = this.props;
        const { open: isModalOpen } = shiftEdit;
        const isEditMode = !!this.props.shiftEdit.params;
        const title = this.props.shiftEdit.params && this.props.shiftEdit.params.uid ? this.props.shiftEdit.params.name : SHIFTS_MODAL.EDIT.TITLE_ADD_NEW;
        
        return (
            <Modal
                contentLabel="shiftEdit"
                isOpen={isModalOpen}
                onRequestClose={this.close}
                style={style}
            >
                <div className="RPP-Modal">
                    <span
                        className="Modal-Header multiple-lines-ellipsis"
                        data-cy="shift-create-header"
                    >
                        {title}
                    </span>
                    <ShiftEditForm
                        className="Modal-Form"
                        close={this.close}
                        onSubmit={this.submit}
                        isEditMode={isEditMode}
                        locations={sortByField(locations, 'name')}
                    />
                </div>
            </Modal>
        );
    }
}

const mapStateToProps = (state) => ({
    shiftEdit: state.modals.shiftModals.shiftEdit,
    locations: state.entities.locations,
});

export default connect(mapStateToProps)(ShiftEditModal);
