import React from 'react';
import PropTypes from 'prop-types';
import Modal from 'react-modal';
import { connect } from 'react-redux';
import { injectIntl } from 'react-intl';
import { reduxForm, Field } from 'redux-form';
import {
    NAME_OF_FORMS,
    EMAIL_REGEXP,
    NOTIFICATION_TYPE,
    SUBSCRIBER_EDIT_MODAL,
    COMPONENTS,
    NAMES_OF_TABS,
} from 'common';
import { closeSubscriberEditModal, createSubscriber, updateSubscriber } from 'stores/actions';
import { addValueLabelToValue, sortByField, filterSelectBoxOptions } from 'utils';
import { makeCopy } from 'utils/simpleUtils';
import RenderField from 'components/modals/RenderField';
import MultiSelectComponent from 'components/modals/MultiSelectComponent';
import { Subscriber, Notification } from 'stores/classes';


const validateCode = (code) => {
    if (!code) {
        return true;
    }
    if (code && code.trim) {
        return /^\+?[\d -]+$/g.test(code.trim()); // possibility start with '+' other numbers only
    }
    return /^\+?[\d -]+$/g.test(code);
};

const validatePhone = (phone) => {
    if (!phone) {
        return true;
    }
    if (phone && phone.trim) {
        return /^[\d -]+$/g.test(phone.trim()); // possibility start with '+' other numbers only
    }
    return /^[\d -]+$/g.test(phone);
};

const validate = (values, props) => {
    const errors = {};
    if (!values.name) {
        errors.name = COMPONENTS.VALIDATION.REQUIRED_FUNC(props.intl);
    } else {
        const subscriber = props.subscribers.find(x => x.name === values.name);
        if (subscriber && subscriber.uid !== values.uid) {
            errors.name = COMPONENTS.VALIDATION.NAME_EXISTS_FUNC(props.intl);
        }
    }
    
    if (values.phoneNumber || values.phoneCode) {
        const codeValid = validateCode(values.phoneCode);
        const phoneValid = validatePhone(values.phoneNumber);
        
        if (!codeValid) {
            errors.phoneCode = SUBSCRIBER_EDIT_MODAL.VALIDATION.PHONE_CODE_INVALID_FUNC(props.intl);
        }
        if (!phoneValid) {
            errors.phoneNumber = SUBSCRIBER_EDIT_MODAL.VALIDATION.PHONE_FORMAT_INVALID_FUNC(props.intl);
        }
        
        if (codeValid && !values.phoneNumber) {
            errors.phoneNumber = SUBSCRIBER_EDIT_MODAL.VALIDATION.PHONE_FORMAT_INVALID_FUNC(props.intl);
        }
        if (phoneValid && !values.phoneCode) {
            errors.phoneCode = SUBSCRIBER_EDIT_MODAL.VALIDATION.PHONE_CODE_INVALID_FUNC(props.intl);
        }
    }
    
    if (values.email && !EMAIL_REGEXP.test(values.email)) {
        errors.email = SUBSCRIBER_EDIT_MODAL.VALIDATION.USER_EMAIL_FORMAT_INVALID_FUNC(props.intl);
    }
    return errors;
};

const renderField_5_7 = RenderField(5, 7);

const renderPhone = ({
    input, dataId, label, type, meta: { error },
}) => (
    <div>
        <input
            {...input}
            type={type}
            data-id={dataId}
            placeholder={label}
            className="input"
        />
        <br />
        {error && <span className="error">{error}</span>}
    </div>
);

renderPhone.propTypes = {
    options: PropTypes.arrayOf(PropTypes.shape({
        value: PropTypes.string,
        label: PropTypes.string,
    })),
    meta: PropTypes.shape({
        touched: PropTypes.bool,
        error: PropTypes.string,
    }),
    input: PropTypes.shape({
        onChange: PropTypes.func,
        value: PropTypes.string,
        onBlur: PropTypes.func,
    }),
    dataId: PropTypes.string,
    label: PropTypes.string,
    type: PropTypes.string,
};

renderPhone.defaultProps = {
    options: [],
    meta: {
        touched: PropTypes.bool,
        error: PropTypes.string,
    },
    input: {
        onChange: () => {},
        value: '',
        onBlur: () => {},
    },
    dataId: '',
    label: '',
    type: '',
};

let SubscriberEditForm = (props) => {
    const {
        close, submit, className, notifications, invalid, intl, initialValues, submitSucceeded,
    } = props;

    return (
        <form className={className}>
            <div className="Form-Line row">
                <Field
                    className="Form-Input"
                    name="name"
                    dataId="subscribers-edit-subscribers"
                    component={renderField_5_7}
                    label={`${SUBSCRIBER_EDIT_MODAL.FORM.LABEL.SUBSCRIBER_NAME_FUNC(intl)} *`}
                    type="text"
                    autoFocus={!(initialValues && initialValues.uid)}
                />
            </div>
            
            <div className="Form-Line row">
                <div htmlFor="notifications" className="Form-Label col-md-5">
                    {NAMES_OF_TABS.NOTIFICATIONS}
                </div>
                <div className="col-md-7" data-id="subscribers-edit-notifications">
                    <Field
                        className="form-control input-sm "
                        name="notificationsLists"
                        type="text"
                        label="Objective"
                        placeholder={COMPONENTS.SELECT_COMPONENT_PLACEHOLDER(intl)}
                        noResultsText={COMPONENTS.SELECT_COMPONENT_NO_RESULTS_TEXT}
                        options={notifications}
                        component={MultiSelectComponent}
                        filterOptions={filterSelectBoxOptions}
                    />
                </div>
            </div>
            
            <div className="Form-Line row">
                <Field
                    className="Form-Input"
                    name="email"
                    dataId="subscribers-edit-email"
                    component={renderField_5_7}
                    label={COMPONENTS.FIELDS.USER.EMAIL_ADDRESS_FUNC(intl)}
                    type="text"
                />
            </div>
            
            <div className="Form-Line row">
                <div className="Form-Label col-md-5">{COMPONENTS.FIELDS.USER.PHONE_NUMBER_FUNC(intl)}</div>
                <div className="col-md-3">
                    <Field
                        className="Form-Input"
                        name="phoneCode"
                        component={renderPhone}
                        dataId="subscribers-edit-code"
                        label={SUBSCRIBER_EDIT_MODAL.FORM.LABEL.PHONE_CODE_FUNC(intl)}
                        type="text"
                    />
                </div>
                <div className="col-md-4">
                    <Field
                        className="Form-Input"
                        name="phoneNumber"
                        dataId="subscribers-edit-number"
                        component={renderPhone}
                        label={COMPONENTS.FIELDS.USER.PHONE_NUMBER_FUNC(intl)}
                        type="text"
                    />
                </div>
            </div>
            
            <div className="Form-Buttons">
                <button
                    type="button"
                    data-id="subscribers-edit-save"
                    onClick={submit}
                    disabled={invalid || submitSucceeded}
                >
                    {COMPONENTS.BUTTONS.SAVE}
                </button>
                <button
                    type="button"
                    data-id="subscribers-edit-cancel"
                    onClick={close}
                >
                    {COMPONENTS.BUTTONS.CANCEL}
                </button>
            </div>
        </form>
    );
};

SubscriberEditForm.propTypes = {
    close: PropTypes.func,
    submit: PropTypes.func,
    invalid: PropTypes.bool,
    className: PropTypes.string,
    notifications: PropTypes.arrayOf(PropTypes.shape({
        value: PropTypes.string,
        label: PropTypes.string,
    })),
    initialValues: PropTypes.shape({
        uid: PropTypes.string,
    }),
    intl: PropTypes.shape({
        formatMessage: PropTypes.func,
    }),
    submitSucceeded: PropTypes.bool,
};

SubscriberEditForm.defaultProps = {
    close: () => {
    },
    submit: () => {
    },
    invalid: false,
    className: '',
    intl: {
        formatMessage: () => {
        },
    },
    initialValues: {
        uid: '',
    },
    notifications: [],
    submitSucceeded: false,
};

SubscriberEditForm = reduxForm({
    validate,
    form: NAME_OF_FORMS.SUBSCRIPTION_EDIT,
})(SubscriberEditForm);

const mapStateToForm = (state) => {
    const { subscriberEdit } = state.modals.subscriberModals;
    
    if (subscriberEdit.params && subscriberEdit.params.notifications) {
        subscriberEdit.params.notificationsLists = subscriberEdit.params.notifications.map((uid) => {
            const notification = Notification.getAllNotifications(state).find(y => y.uid === uid);
            if (notification) {
                return Object.assign({}, notification, { value: notification.uid, label: notification.name });
            }
            return undefined;
        }).filter(x => x);
    }
    const notifications = Notification.getAllNotifications(state)
        .filter(x => x.notification_type === NOTIFICATION_TYPE.ALARM)
        .map(addValueLabelToValue);
    
    return {
        subscribers: Subscriber.getAllSubscribers(state),
        notifications: sortByField(notifications, 'name'),
        initialValues: subscriberEdit.params,
    };
};

SubscriberEditForm = connect(mapStateToForm)(injectIntl(SubscriberEditForm));

class SubscriberEditModal extends React.Component {
    static propTypes = {
        dispatch: PropTypes.func,
        subscriberEdit: PropTypes.shape({
            params: PropTypes.shape({
                uid: PropTypes.string,
            }),
            open: PropTypes.bool,
        }),
        style: PropTypes.shape(),
    };
    
    static defaultProps = {
        dispatch: PropTypes.func,
        subscriberEdit: PropTypes.func,
        style: {},
    };
    
    close = () => {
        this.props.dispatch(closeSubscriberEditModal());
    };
    
    submit = (values) => {
        values.notifications = values.notificationsLists
            ? values.notificationsLists.reduce((r, x) => {
                if (!r.some(w => w === x.uid)) {
                    r.push(x.uid);
                }
                return r;
            }, [])
            : [];
        values.addresses = {};
        // if (values.phone) {
        //     values.addresses.phone = values.phone;
        // }
        if (values.phoneCode && values.phoneNumber) {
            values.addresses.phone = values.phoneCode.toString().trim() + values.phoneNumber.trim();
        }
        
        if (values.email) {
            values.addresses.email = values.email;
        }

        if (this.props.subscriberEdit.params && this.props.subscriberEdit.params.uid) {
            this.props.dispatch(updateSubscriber(values));
        } else {
            this.props.dispatch(createSubscriber(values));
        }
        this.props.dispatch(closeSubscriberEditModal());
        return undefined;
    };
    
    render() {
        const { subscriberEdit } = this.props;
        const style = makeCopy(this.props.style);
        style.content.width = '500px';
        
        const title = subscriberEdit.params && subscriberEdit.params.uid
            ? subscriberEdit.params.name
            : SUBSCRIBER_EDIT_MODAL.TITLE;
        
        return (
            <Modal
                contentLabel="subscriberEdit"
                isOpen={this.props.subscriberEdit.open}
                onRequestClose={this.close}
                style={style}
            >
                <div className="RPP-Modal">
                    <span className="Modal-Header multiple-lines-ellipsis" data-id="subscribers-edit-title">
                        {title}
                    </span>
                    <SubscriberEditForm
                        className="Modal-Form"
                        close={this.close}
                        onSubmit={this.submit}
                    />
                </div>
            </Modal>
        );
    }
}

const mapStateToProps = state => ({
    subscriberEdit: state.modals.subscriberModals.subscriberEdit,
});

export default connect(mapStateToProps)(SubscriberEditModal);
