import React from 'react';
import PropTypes from 'prop-types';
import { injectIntl } from 'react-intl';
import { connect } from 'react-redux';
import { Field, reduxForm } from 'redux-form';
import 'react-select/dist/react-select.css';
import { COMPONENTS, TRACKABLE_MODAL, NAMES_OF_TABS } from 'common/ui-labels';
import { NAME_OF_FORMS, DEFAULT_VALUES, ERROR_TEXT, POS_UPDATE_INTERVAL } from 'common/constants';
import { getLength, isAssetTag, sortByField } from 'utils/utils';
import { validatePWS } from 'utils/pwsConfigUtils';
import { onMouseDownScrollPrevent } from 'utils/htmlUtils';
import MultiSelectComponent from 'components/modals/MultiSelectComponent';
import Trackable from 'stores/classes/Trackable';
import Group from 'stores/classes/Group';
import { Buttons } from './Buttons';
import { CustomInfoForm } from './CustomInfoForm';
import { CategoryForm } from './CategoryForm';
import { GroupsForm } from './GroupsForm';
import { BufferForm } from './BufferForm';
import { AdvancedSectionForm } from './AdvancedSectionForm';
import { PwsForm } from './PwsForm';


const validate = (values, props) => {
    const { intl } = props;
    const errors = {};

    if (!Array.isArray(values.editedTrackables) || !values.editedTrackables.length) {
        errors.editedTrackables = COMPONENTS.VALIDATION.REQUIRED;
    }
    
    if (values.category_checkbox && !values.category) {
        errors.category = COMPONENTS.VALIDATION.REQUIRED;
    }
    if (values.buffer && values.buffer_radius <= 0) {
        errors.buffer_radius = TRACKABLE_MODAL.EDIT.FORM.ERROR_MUST_BE_POSITIVE;
    }
    if (values.custom_info_checkbox && values.custom_info && getLength(values.custom_info) > DEFAULT_VALUES.CUSTOM_INFO_LIMIT) {
        errors.custom_info = ERROR_TEXT.CUSTOM_INFO_ERROR();
    }
    
    if (values.advanced_checkbox
        && (values.position_update_interval < POS_UPDATE_INTERVAL.MIN || values.position_update_interval > POS_UPDATE_INTERVAL.MAX)
        && isAssetTag(values.category)
    ) {
        errors.position_update_interval = TRACKABLE_MODAL.EDIT.FORM.ERROR_WRONG_UPDATE_INTERVAL(POS_UPDATE_INTERVAL.MIN, POS_UPDATE_INTERVAL.MAX / (60 * 60));
    }

    if (values.pws_checkbox) {
        validatePWS(values, errors, intl);
    }

    const { advanced_checkbox, buffer, category_checkbox, custom_info_checkbox, groups_checkbox, pws_checkbox } = values || {};

    if (!category_checkbox && !groups_checkbox && !(category_checkbox && buffer) && !advanced_checkbox && !pws_checkbox && !custom_info_checkbox) {
        errors._error = 'Nothing to change';
    }
    return errors;
};

let TrackableBulkEditForm = (props) => {
    const { close, submit, groups, intl, dispatch, invalid, trackables, submitted, className, formState } = props;
    const { custom_info, custom_info_checkbox, category_checkbox, groups_checkbox, buffer_checkbox, advanced_checkbox, pws_checkbox } = formState?.values || {};
    const { category, role, safe_distance, distance, alert_distance, t2t_no_alarm_uid, delay, angle, editedTrackables, pin } = formState?.values || {};
    const editedUids = (editedTrackables || []).map((tko) => tko.value);
    const trackablesOptions = trackables.filter((tko) => !editedUids.includes(tko.uid));

    return (
        <form className={className}>
            <div className="col-md-12" style={{ overflowY: 'auto', overflowX: 'hidden', maxHeight: '60vh' }} onMouseDown={onMouseDownScrollPrevent}>
                <div className="Form-Line row">
                    <div className="Form-Label col-md-4">{NAMES_OF_TABS.TRACKABLE_OBJECTS}*</div>
                    <div className="col-md-8" data-id="trackable-bulk-edit-trackbale-list">
                        <Field
                            name="editedTrackables"
                            type="text"
                            label="Objective"
                            options={trackables}
                            placeholder={COMPONENTS.SELECT_COMPONENT_PLACEHOLDER(intl)}
                            noResultsText={COMPONENTS.SELECT_COMPONENT_NO_RESULTS_TEXT}
                            component={MultiSelectComponent}
                        />
                    </div>
                </div>
                <CategoryForm
                    intl={intl}
                    dispatch={dispatch}
                    category_checkbox={category_checkbox}
                    category={category}
                />
                <GroupsForm
                    intl={intl}
                    groups_checkbox={groups_checkbox}
                    groups={groups}
                />
                <BufferForm
                    intl={intl}
                    groups={groups}
                    trackables={trackables}
                    category_checkbox={category_checkbox}
                    buffer_checkbox={buffer_checkbox}
                    category={category}
                />
                <AdvancedSectionForm
                    category={category}
                    intl={intl}
                    advanced_checkbox={advanced_checkbox}
                />
                <PwsForm
                    pws_checkbox={pws_checkbox}
                    category_checkbox={category_checkbox}
                    category={category}
                    safe_distance={safe_distance}
                    distance={distance}
                    alert_distance={alert_distance}
                    delay={delay}
                    angle={angle}
                    dispatch={dispatch}
                    intl={intl}
                    role={role}
                    t2tNoAlarmUid={t2t_no_alarm_uid}
                    trackables={trackablesOptions}
                    groups={groups}
                    trackableCategory={category}
                    isStationary={false}
                    pin={pin}
                />
                <CustomInfoForm
                    intl={intl}
                    custom_info={custom_info}
                    custom_info_checkbox={custom_info_checkbox}
                    dispatch={dispatch}
                />
            </div>
            <Buttons
                close={close}
                submit={submit}
                submitted={submitted}
                invalid={invalid}
            />
        </form>
    );
};

TrackableBulkEditForm.propTypes = {
    dispatch: PropTypes.func,
    close: PropTypes.func,
    submit: PropTypes.func,
    className: PropTypes.string,
    invalid: PropTypes.bool,
    submitted: PropTypes.bool,
    groups: PropTypes.arrayOf(PropTypes.shape({})),
    intl: PropTypes.shape({
        formatMessage: PropTypes.func,
    }),
    trackables: PropTypes.arrayOf(PropTypes.shape({})),
    formState: PropTypes.shape({}),
};

TrackableBulkEditForm.defaultProps = {
    dispatch: () => {},
    close: () => {},
    submit: () => {},
    className: '',
    invalid: true,
    submitted: false,
    groups: [],
    intl: { formatMessage: () => {} },
    trackables: [],
    formState: {

    }
};

TrackableBulkEditForm = reduxForm({
    form: NAME_OF_FORMS.TRACKABLE_BULK_EDIT,
    validate,
})(TrackableBulkEditForm);

const mapStateToForm = (state) => {
    const bulkEdit = state.modals.trackableModals.trackableBulkEdit;
    const initialValues = {
        editedTrackables: bulkEdit.editedTrackables.map(tko => ({ value: tko.uid, label: tko.name })),
        defaultHeight: DEFAULT_VALUES.trackable_default_height,
        position_update_interval: DEFAULT_VALUES.trackable_update_interval,
        buffer_radius: DEFAULT_VALUES.trackable_buffer_radius,
    };
    const groups = sortByField(Group.getAllGroupsArray(state), 'name').map((group) => (group.addValueLabelToValue()));
    const trackables = sortByField(Trackable.getAllTrackablesArray(state), 'name').map((tko) => (tko.addValueLabelToValue()));

    return {
        submitted: bulkEdit.submitted,
        initialValues,
        groups,
        trackables,
        formState: state.form[NAME_OF_FORMS.TRACKABLE_BULK_EDIT],
    };
};

TrackableBulkEditForm = connect(mapStateToForm)(injectIntl(TrackableBulkEditForm));

export { TrackableBulkEditForm };
