import React, { Fragment } from 'react';
import PropTypes from 'prop-types';
import { injectIntl } from 'react-intl';
import { connect } from 'react-redux';
import { Field, reduxForm, change, formValueSelector } from 'redux-form';
import _Get from 'lodash/get';
import 'react-select/dist/react-select.css';
import { TRACKABLE_MODAL, COMPONENTS, LABELS } from 'common/ui-labels';
import { NAME_OF_FORMS, DEFAULT_VALUES, ERROR_TEXT, POS_UPDATE_INTERVAL } from 'common/constants';
import { getLength, sortByField, isAssetTag, isPerson, isVehicle, isUnspecified, filterSelectBoxOptions } from 'utils/utils';
import { onMouseDownScrollPrevent } from 'utils/htmlUtils';
import { degreeToLength } from 'utils/formatters';
import { validatePWS } from 'utils/pwsConfigUtils';
import { checkboxClick, closeStatusNotificationPopup, openStatusNotificationPopup, showMapClickTrackableEditModal } from 'stores/actions';
import { Group, Location, Sublocation, Floorplan, Trackable, TrackableTemplate } from 'stores/classes';
import MultiSelectComponent from 'components/modals/MultiSelectComponent';
import FloorplanComponent from 'components/modals/forms/Floorplan';
import SelectedFloorplan from 'components/modals/forms/SelectedFloorplan';
import LowLatencyLinkComponent from 'components/modals/LowLatencyLink';
import CustomInfoFieldComponent from 'components/modals/CustomInfoField';
import RenderTooltipHelper from 'components/RenderTooltipHelper';
import SelectComponent from 'components/modals/SelectComponent';
import {
    ALL_TYPE_VALUES, CATEGORIES, getParsedTrackable, renderField, isSelectedFields,
    getFloorplansBySublocationUid, getUpdateIntervalValues,
} from './trackableModalUtils';
import renderBufferSection from './bufferSection';
import advancedOptionsSection from './advancedOptionsSection';
import { UserTemplatesSection } from './userParamsSection';

const validate = (values, props) => {
    const { intl } = props;
    const errors = { user_params: {} };
    if (!values.name) {
        errors.name = COMPONENTS.VALIDATION.REQUIRED;
    }

    const templateProperties = TrackableTemplate.getTemplatePropertyByCategory(values.category, props.trackableTemplates);
    templateProperties.forEach((templateProperty) => {
        const userParamsOrVariableNotExists = !values.user_params || (values.user_params && !values.user_params[templateProperty.name]);
        const property = values.user_params && values.user_params[templateProperty.name];
        if (templateProperty.required && userParamsOrVariableNotExists) {
            errors.user_params[templateProperty.name] = COMPONENTS.VALIDATION.REQUIRED;
        }
        if (templateProperty.minLength && property && property.length < templateProperty.minLength) {
            errors.user_params[templateProperty.name] = COMPONENTS.VALIDATION.MIN_LENGTH_FUNC(templateProperty.minLength);
        }
        if (templateProperty.maxLength && property && property.length > templateProperty.maxLength) {
            errors.user_params[templateProperty.name] = COMPONENTS.VALIDATION.MAXIMUM_LENGTH_FUNC(templateProperty.maxLength);
        }
    });
    if (values.buffer_radius <= 0) {
        errors.buffer_radius = TRACKABLE_MODAL.EDIT.FORM.ERROR_MUST_BE_POSITIVE;
    }
    if (values.stationary) {
        if (!values.pos_x) {
            errors.pos_x = COMPONENTS.VALIDATION.REQUIRED;
        } else if (isNaN(values.pos_x)) {
            errors.pos_x = COMPONENTS.VALIDATION.NOT_A_NUMBER;
        }
        if (!values.pos_y) {
            errors.pos_y = COMPONENTS.VALIDATION.REQUIRED;
        } else if (isNaN(values.pos_y)) {
            errors.pos_y = COMPONENTS.VALIDATION.NOT_A_NUMBER;
        }
        if (!values.defaultHeight) {
            errors.defaultHeight = COMPONENTS.VALIDATION.REQUIRED;
        } else if (isNaN(values.defaultHeight)) {
            errors.defaultHeight = COMPONENTS.VALIDATION.NOT_A_NUMBER;
        }
    }
    if (values.custom_info && getLength(values.custom_info) > DEFAULT_VALUES.CUSTOM_INFO_LIMIT) {
        errors.custom_info = ERROR_TEXT.CUSTOM_INFO_ERROR();
    }
    
    if ((values.position_update_interval < POS_UPDATE_INTERVAL.MIN || values.position_update_interval > POS_UPDATE_INTERVAL.MAX) && isAssetTag(values.category)) {
        errors.position_update_interval = TRACKABLE_MODAL.EDIT.FORM.ERROR_WRONG_UPDATE_INTERVAL(POS_UPDATE_INTERVAL.MIN, POS_UPDATE_INTERVAL.MAX / (60 * 60));
    }

    validatePWS(values, errors, intl);

    return errors;
};

const warn = (values, props) => {
    const { name, uid } = values;
    const warnings = {};
    if (props.trackables && props.trackables.find((tko) => tko.name === name && tko.uid !== uid)) {
        warnings.name = TRACKABLE_MODAL.EDIT.WARNING.NAME_DUPLICATES;
    }

    const mac_address = _Get(props, 'initialValues.node.mac_address', null);
    const position_update_interval = props.initialValues.position_update_interval.value || props.initialValues.position_update_interval;

    if (props.trackableCategory === 'ASSET' && mac_address
        && (values.category !== props.initialValues.category || values.position_update_interval !== position_update_interval)) {
        warnings.position_update_interval = TRACKABLE_MODAL.EDIT.WARNING.NEED_REBOOT(mac_address);
    }

    return warnings;
};

const saveStationaryUIPosition = (dispatch, stationaryLeafletPosition) => {
    const stationaryUIPosition = {
        x: parseInt(degreeToLength(stationaryLeafletPosition.lng)) / 10,
        y: parseInt(degreeToLength(stationaryLeafletPosition.lat)) / 10,
    };

    dispatch(change(NAME_OF_FORMS.TRACKABLE_EDIT, 'pos_x', stationaryUIPosition.x));
    dispatch(change(NAME_OF_FORMS.TRACKABLE_EDIT, 'pos_y', stationaryUIPosition.y));
};

let TrackableEditForm = (props) => {
    const {
        close, submit, className, invalid, groups, intl, posReportInterval, dispatch,
        sublocations, showMap, selectFloorplan, unSelectFloorplan, selectedFloorplan,
        stationaryLeafletPosition, stationaryPosition, isMapVisible,
        selectedArea, selectedSite, trackableCategory, uid, siteList, areaList,
        isBufferSelected, isStationary, relatedFloorplans,
        role, angle, maxHeight, trackables, t2tNoAlarmUid, customInfoLength,
        submitSucceeded, safe_distance, distance, alert_distance, delay, trackableType,
        categoryOptions, formState, pin,
        trackableTemplates,
    } = props;

    const isAdvancedOptions = formState && formState.values && formState.values.advanced_options;

    const leafletCursorPosition = null;

    const areaAutoSelection = (currentSiteUid) => {
        const autoSelectedArea = currentSiteUid && sublocations
            .find((sublocation) => sublocation.location_uid === currentSiteUid);
        if (autoSelectedArea) {
            dispatch(change(NAME_OF_FORMS.TRACKABLE_EDIT, 'area', autoSelectedArea.uid));
        }
    };

    const formDispatchChange = (field, value) => {
        dispatch(change(NAME_OF_FORMS.TRACKABLE_EDIT, field, value));
    };

    const isCategoryAsset = isAssetTag(trackableCategory);
    const isCategoryPerson = isPerson(trackableCategory);
    const isCategoryVehicle = isVehicle(trackableCategory);
    const isStationaryDisabled = isCategoryAsset || isCategoryPerson || isCategoryVehicle;

    const [submitClick, setSubmitClick] = React.useState(false);
    const [prevCategory, setPrevCategory] = React.useState(trackableCategory);
    const clearRoleField = () => formDispatchChange('role', '');
    const setUpdateIntervalToDefault = (cat) => formDispatchChange('position_update_interval', getUpdateIntervalValues(cat).default);
    const uncheckStationaryCheckbox = () => formDispatchChange('stationary', false);

    const groupsOnChange = (selectedGroups) => {
        const { low_latency_link_config: pwsConfig } = selectedGroups.find((group) => group.low_latency_link_config) || { low_latency_link_config: null };
        if (pwsConfig) {
            formDispatchChange('role', pwsConfig.role || '');
            formDispatchChange('safe_distance', pwsConfig.safe_distance || DEFAULT_VALUES.LLL_COVERAGE_SAFE_DISTANCE);
            formDispatchChange('distance', pwsConfig.distance || DEFAULT_VALUES.LLL_COVERAGE_DISTANCE);
            formDispatchChange('alert_distance', pwsConfig.alert_distance || DEFAULT_VALUES.LLL_COVERAGE_ALERT_DISTANCE);
            formDispatchChange('delay', pwsConfig.delay || DEFAULT_VALUES.LLL_DELAY);
            formDispatchChange('angle', pwsConfig.angle || DEFAULT_VALUES.LLL_COVERAGE_ANGLE);
            formDispatchChange('pin', pwsConfig.pin || DEFAULT_VALUES.LLL_TRACKABLE_PIN);
            formDispatchChange('haptic', pwsConfig.haptic || false);
            formDispatchChange('buzzer', pwsConfig.buzzer || false);
            formDispatchChange('alarm_back', pwsConfig.alarm_back || false);
        }
    };
    const templateProperties = TrackableTemplate.getTemplatePropertyByCategory(trackableCategory, trackableTemplates);

    const resetUserParameters = () => {
        dispatch(change(NAME_OF_FORMS.TRACKABLE_EDIT, 'user_params', {}));
    };

    return (
        <form className={className}>
            <div className="row" onMouseDown={onMouseDownScrollPrevent}>
                <div
                    className={isMapVisible ? 'col-md-5' : 'col-md-12'}
                    style={{ overflowY: 'auto', overflowX: 'hidden', maxHeight }}
                >
                    <Fragment key="name">
                        <div className="Form-Line row">
                            <Field
                                onChange={() => {
                                    dispatch(checkboxClick({}));
                                }}
                                className="Form-Input"
                                dataId="trackable-edit-name"
                                name="name"
                                component={renderField}
                                label={`${TRACKABLE_MODAL.EDIT.FORM.NAME_FUNC(intl)} *`}
                                type="text"
                                autoFocus={!uid}
                            />
                        </div>
                    </Fragment>
                    <UserTemplatesSection
                        templateProperties={templateProperties}
                    />

                    <Fragment key="category">
                        <div className="Form-Line row">
                            <div htmlFor="category" className="Form-Label col-md-4">
                                {TRACKABLE_MODAL.EDIT.FORM.TRACKABLE_CATEGORY_FUNC(intl)}
                                <RenderTooltipHelper
                                    tooltipText={TRACKABLE_MODAL.EDIT.FORM.TRACKABLE_CATEGORY_HELP_FUNC(intl)}
                                    placement="bottom"
                                />
                            </div>
                            <div className="col-md-8" data-id="trackable-edit-categories">
                                <Field
                                    onChange={
                                        (e, newVal) => {
                                            if (prevCategory !== newVal) {
                                                clearRoleField();
                                                setUpdateIntervalToDefault(newVal);
                                                if (!isUnspecified(newVal)) {
                                                    uncheckStationaryCheckbox();
                                                }
                                                resetUserParameters();
                                            }
                                            dispatch(checkboxClick({}));
                                            setPrevCategory(newVal);
                                        }
                                    }
                                    clearable={false}
                                    disabled={isCategoryAsset}
                                    className="Form-Input"
                                    name="category"
                                    component={SelectComponent}
                                    simpleValue
                                    placeholder={COMPONENTS.SELECT_COMPONENT_PLACEHOLDER(intl)}
                                    noResultsText={COMPONENTS.SELECT_COMPONENT_NO_RESULTS_TEXT}
                                    options={categoryOptions}
                                />
                            </div>
                        </div>
                    </Fragment>
                    <Fragment key="groups">
                        <div className="Form-Line row">
                            <div className="Form-Label col-md-4">{TRACKABLE_MODAL.EDIT.FORM.GROUPS}</div>
                            <div htmlFor="groupList" className="col-md-8" data-id="trackable-edit-group-list">
                                <Field
                                    onChange={groupsOnChange}
                                    name="groupList"
                                    type="text"
                                    label="Objective"
                                    options={groups}
                                    filterOptions={filterSelectBoxOptions}
                                    placeholder={COMPONENTS.SELECT_COMPONENT_PLACEHOLDER(intl)}
                                    noResultsText={COMPONENTS.SELECT_COMPONENT_NO_RESULTS_TEXT}
                                    component={MultiSelectComponent}
                                />
                            </div>
                        </div>
                    </Fragment>

                    <hr />

                    {renderBufferSection({
                        dispatch,
                        isBufferSelected,
                        uid,
                        intl,
                        trackableCategory,
                    })}

                    <hr />

                    {advancedOptionsSection({
                        dispatch,
                        isAdvancedOptions,
                        trackableType,
                        posReportInterval,
                        intl,
                        siteList,
                        areaList,
                        selectedArea,
                        selectedSite,
                        showMapClickTrackableEditModal,
                        showMap,
                        areaAutoSelection,
                        uid,
                        isStationary,
                        isCategoryAsset,
                        isStationaryDisabled,
                        trackableCategory,
                        clearRoleField,
                    })}
                    <hr />
                    <LowLatencyLinkComponent
                        safe_distance={safe_distance}
                        distance={distance}
                        alert_distance={alert_distance}
                        delay={delay}
                        angle={angle}
                        isAdvanced={isAdvancedOptions}
                        dispatch={dispatch}
                        intl={intl}
                        role={role}
                        pin={pin}
                        t2tNoAlarmUid={t2tNoAlarmUid}
                        trackables={trackables}
                        showAlarmDisableForField
                        groups={groups}
                        uid={uid}
                        formDispatchChange={formDispatchChange}
                        trackableCategory={trackableCategory}
                        isStationary={isStationary}
                    />
                    <CustomInfoFieldComponent
                        placeholder={LABELS.CUSTOM_INFO_PLACEHOLDER(intl)}
                        dataId="trackable-edit-custom-info"
                        customInfoLength={customInfoLength}
                    />
                </div>
                {isMapVisible && (
                    <div className="col-md-7">
                        {
                            !selectedFloorplan && relatedFloorplans.length > 1 && (
                                <FloorplanComponent
                                    relatedFloorplans={relatedFloorplans}
                                    selectFloorplan={selectFloorplan}
                                />
                            )
                        }
                        {
                            (selectedFloorplan || relatedFloorplans.length === 1) && (
                                <SelectedFloorplan
                                    selectedFloorplan={selectedFloorplan || relatedFloorplans[0]}
                                    unSelectFloorplan={unSelectFloorplan}
                                    stationaryLeafletPosition={stationaryLeafletPosition}
                                    leafletCursorPosition={leafletCursorPosition}
                                    dispatch={dispatch}
                                    intl={intl}
                                    saveStationaryUIPosition={saveStationaryUIPosition}
                                    stationaryPosition={stationaryPosition}
                                />
                            )
                        }
                    </div>
                )}
            </div>
            <div className="Form-Buttons">
                <button
                    type="button"
                    onClick={() => {
                        if (_Get(formState, 'syncWarnings.position_update_interval') && !submitClick) {
                            setSubmitClick(true);
                            submit();
                            dispatch(openStatusNotificationPopup({
                                title: '',
                                message: formState.syncWarnings.position_update_interval,
                            }));
                            setTimeout(() => {
                                dispatch(closeStatusNotificationPopup());
                            }, 2000);
                        } else {
                            submit();
                        }
                    }}
                    disabled={invalid || submitSucceeded || submitClick}
                    data-id="trackable-edit-update-save"
                >
                    {COMPONENTS.BUTTONS.SAVE}
                </button>
                <button type="button" data-id="trackable-edit-update-close" onClick={close}>
                    {COMPONENTS.BUTTONS.CANCEL}
                </button>
            </div>
        </form>
    );
};

TrackableEditForm.propTypes = {
    dispatch: PropTypes.func,
    close: PropTypes.func,
    submit: PropTypes.func,
    selectFloorplan: PropTypes.func,
    unSelectFloorplan: PropTypes.func,
    className: PropTypes.string,
    invalid: PropTypes.bool,
    showMap: PropTypes.bool,
    stationaryLeafletPosition: PropTypes.shape(),
    stationaryPosition: PropTypes.shape(),
    selectedFloorplan: PropTypes.shape(),
    isMapVisible: PropTypes.bool,
    groups: PropTypes.arrayOf(PropTypes.shape({
        value: PropTypes.string, // x.uid
        label: PropTypes.string, // x.name
    })),
    intl: PropTypes.shape({
        formatMessage: PropTypes.func,
    }),
    posReportInterval: PropTypes.arrayOf(PropTypes.shape({
        value: PropTypes.number,
        label: PropTypes.string,
        type: PropTypes.string,
    })),
    sublocations: PropTypes.arrayOf(PropTypes.shape()),
    uid: PropTypes.string,
    trackableType: PropTypes.string,
    trackableCategory: PropTypes.string,
    selectedArea: PropTypes.string,
    selectedSite: PropTypes.string,
    siteList: PropTypes.arrayOf(PropTypes.shape()),
    areaList: PropTypes.arrayOf(PropTypes.shape()),
    relatedFloorplans: PropTypes.arrayOf(PropTypes.shape()),
    isBufferSelected: PropTypes.bool,
    isAdvancedOptions: PropTypes.bool,
    isStationary: PropTypes.bool,
    role: PropTypes.string,
    angle: PropTypes.number,
    safe_distance: PropTypes.number,
    distance: PropTypes.number,
    alert_distance: PropTypes.number,
    delay: PropTypes.number,
    maxHeight: PropTypes.string,
    trackables: PropTypes.arrayOf(PropTypes.shape({})),
    t2tNoAlarmUid: PropTypes.string,
    customInfoLength: PropTypes.number,
    submitSucceeded: PropTypes.bool,
    categoryOptions: PropTypes.arrayOf(PropTypes.shape({})),
    formState: PropTypes.shape({}),
    haptic: PropTypes.bool,
    buzzer: PropTypes.bool,
    trackableTemplates: PropTypes.arrayOf(PropTypes.shape({
        category: PropTypes.string,
        variables: PropTypes.shape({
            label: PropTypes.string,
            name: PropTypes.string,
        }),
    })),
};

TrackableEditForm.defaultProps = {
    dispatch: () => {
    },
    close: () => {
    },
    submit: () => {
    },
    selectFloorplan: () => {
    },
    unSelectFloorplan: () => {
    },
    className: '',
    invalid: true,
    showMap: true,
    stationaryLeafletPosition: {},
    stationaryPosition: {},
    selectedFloorplan: {},
    isMapVisible: true,
    groups: [],
    intl: {
        formatMessage: () => {
        },
    },
    posReportInterval: [],
    sublocations: [],
    uid: '',
    trackableType: '',
    trackableCategory: '',
    selectedArea: '',
    selectedSite: '',
    siteList: [],
    areaList: [],
    relatedFloorplans: [],
    isBufferSelected: false,
    isAdvancedOptions: false,
    isStationary: false,
    role: '',
    angle: DEFAULT_VALUES.LLL_COVERAGE_ANGLE,
    safe_distance: DEFAULT_VALUES.LLL_COVERAGE_SAFE_DISTANCE,
    distance: DEFAULT_VALUES.LLL_COVERAGE_DISTANCE,
    alert_distance: DEFAULT_VALUES.LLL_COVERAGE_SAFE_DISTANCE,
    delay: DEFAULT_VALUES.LLL_DELAY,
    maxHeight: '60vh',
    trackables: [],
    t2tNoAlarmUid: '',
    customInfoLength: 0,
    submitSucceeded: false,
    haptic: false,
    buzzer: false,
    categoryOptions: [],
    formState: {},
    trackableTemplates: [],
};

TrackableEditForm = reduxForm({
    form: NAME_OF_FORMS.TRACKABLE_EDIT,
    validate,
    warn,
})(TrackableEditForm);

export const getValue = (data) => (data && data.value ? data.value : data);

const getCategoryOptions = (categories, isBuffer, role) => categories.map((cat) => {
    if (isAssetTag(cat.value)) {
        cat.disabled = isBuffer || !!role; // disabling category Asset is buffer or PWS enabled
    }
    return cat;
});

const mapStateToForm = (state) => {
    const parsedTrackable = getParsedTrackable(state);

    const initialValues = parsedTrackable.params || parsedTrackable;
    const selector = formValueSelector(NAME_OF_FORMS.TRACKABLE_EDIT);

    const selectedArea = selector(state, 'area');
    const selectedSite = selector(state, 'site');
    const { uid, category, type } = selector(state, 'uid', 'category', 'type');

    const trackableType = getValue(type);
    const trackableCategory = getValue(category);

    const selectedFieldsObject = selector(
        state,
        'buffer', 'buffer_radius', 'active_buffer', 'buffer_exceptions', 'zones', 'stationary', 'advanced_options',
        'tag_config', 'role', 'angle', 't2t_no_alarm_uid', 'custom_info', 'pin',
        'safe_distance', 'distance', 'alert_distance', 'delay', 'safe_distance_prevent', 'haptic', 'buzzer',
    );

    const trackables = sortByField(Trackable.getAllTrackablesArray(state), 'name')
        .reduce((r, tko) => {
            r.push({ ...tko, value: tko.uid, label: tko.name });
            return r;
        }, []);
    const locations = sortByField(Location.getAllLocations(state), 'name');
    const sublocations = sortByField(Sublocation.getAllSublocations(state), 'name');
    const floorplans = sortByField(Floorplan.getAllFloorplans(state), 'name');

    const siteList = locations.map((location) => ({
        value: location.uid,
        label: location.name,
    }));

    const areaList = (sublocations && sublocations.reduce((memo, sublocation) => {
        if (sublocation.location_uid === selectedSite) {
            memo.push({
                value: sublocation.uid,
                label: sublocation.name,
            });
        }
        return memo;
    }, [])) || [];

    const {
        role, safe_distance, distance, alert_distance, delay, angle, t2t_no_alarm_uid, custom_info, pin, haptic,
        buzzer } = selectedFieldsObject;

    const { isBufferSelected, isAdvancedOptions, isStationary } = isSelectedFields(initialValues.checkboxes || {}, selectedFieldsObject);

    initialValues.advanced_options = isAdvancedOptions;
    initialValues.haptic = haptic;
    initialValues.buzzer = buzzer;

    const relatedFloorplans = getFloorplansBySublocationUid(selectedArea, floorplans);

    const groups = sortByField(Group.getAllGroupsArray(state).map(x => x.addValueLabelToValue()), 'name');

    const categoryOptions = getCategoryOptions(CATEGORIES, isBufferSelected, role);

    return {
        uid,
        siteList,
        areaList,
        isStationary,
        sublocations,
        selectedArea,
        selectedSite,
        initialValues,
        trackableType,
        trackableCategory,
        trackables,
        t2tNoAlarmUid: t2t_no_alarm_uid,
        angle,
        safe_distance,
        distance,
        alert_distance,
        delay,
        haptic,
        buzzer,
        role,
        pin,
        isBufferSelected,
        formState: state.form[NAME_OF_FORMS.TRACKABLE_EDIT],
        relatedFloorplans,
        groups,
        customInfoLength: custom_info ? getLength(custom_info) : 0,
        posReportInterval: ALL_TYPE_VALUES,
        showMap: state.modals.trackableModals.trackableEdit.showMap,
        selectedFloorplan: state.modals.commonModals.activeModal.selectedFloorplan,
        stationaryPosition: state.modals.trackableModals.trackableEdit.stationaryPosition,
        categoryOptions,
        trackableTemplates: state.entities.trackableTemplates,
    };
};

TrackableEditForm = connect(mapStateToForm, { checkboxClick })(injectIntl(TrackableEditForm));

export default TrackableEditForm;
