import React from 'react';
import Modal from 'react-modal';
import PropTypes from 'prop-types';
import { connect } from 'react-redux';
import { Field, reduxForm, touch } from 'redux-form';
import { injectIntl } from 'react-intl';
import RenderSelectField from 'components/modals/RenderSelectField';
import { NAME_OF_FORMS, COMPONENTS, TIME_IN_MINUTES, USER_ROLES_DISPLAY, COMMON_LABELS, USER_ACCESS_CONTROL_MODAL, EXPIRATION_TIMEOUT_LIST, SESSION_TIMEOUT_LIST } from 'common';
import { makeCopy } from 'utils/simpleUtils';
import { closeUserAccessControlEditModal, openUserAccessControlWarningModal } from 'stores/actions';


const validate = (values, props) => {
    const errors = {};
    if (values.accountExpirationTimeout === '') {
        errors.accountExpirationTimeout = USER_ACCESS_CONTROL_MODAL.ERRORS.COULD_NOT_BE_EMPTY;
        props.dispatch(touch(NAME_OF_FORMS.USER_ROLE_EDIT, 'accountExpirationTimeout'));
    } else if (!EXPIRATION_TIMEOUT_LIST.includes(Number(values.accountExpirationTimeout))) {
        errors.accountExpirationTimeout = USER_ACCESS_CONTROL_MODAL.ERRORS.ACTUAL_MINUTES_FUNC(values.accountExpirationTimeout);
        props.dispatch(touch(NAME_OF_FORMS.USER_ROLE_EDIT, 'accountExpirationTimeout'));
    }

    if (values.sessionTimeout === '') {
        errors.sessionTimeout = USER_ACCESS_CONTROL_MODAL.ERRORS.COULD_NOT_BE_EMPTY;
        props.dispatch(touch(NAME_OF_FORMS.USER_ROLE_EDIT, 'sessionTimeout'));
    } else if (!SESSION_TIMEOUT_LIST.includes(Number(values.sessionTimeout))) {
        errors.sessionTimeout = USER_ACCESS_CONTROL_MODAL.ERRORS.ACTUAL_MINUTES_FUNC(values.sessionTimeout);
        props.dispatch(touch(NAME_OF_FORMS.USER_ROLE_EDIT, 'sessionTimeout'));
    }
    return errors;
};

let UserControlEditForm = (props) => {
    const { close, submit, invalid, className, intl } = props;

    return (
        <form className={className}>
            <div className="Form-Line row">
                <div htmlFor="name" className="Form-Label col-md-6">{COMMON_LABELS.USER_SESSION_TIMEOUT}</div>
                <div className="col-md-6">
                    <Field
                        className="form-control input-sm "
                        name="sessionTimeout"
                        dataId="user-role-edit-session-timeout"
                        style={{ fontSize: '14px' }}
                        component={RenderSelectField}
                    >
                        <option />
                        <option value={0}>{COMPONENTS.FIELDS.COMMON.NEVER_FUNC(intl)}</option>
                        <option value={TIME_IN_MINUTES.HALF_HOUR}>{COMMON_LABELS.MINS_30(intl)}</option>
                        <option value={TIME_IN_MINUTES.HOUR}>{COMMON_LABELS.HOUR_1(intl)}</option>
                        <option value={TIME_IN_MINUTES.TWO_HOURS}>{COMMON_LABELS.HOURS_2(intl)}</option>
                        <option value={TIME_IN_MINUTES.FOUR_HOURS}>{COMMON_LABELS.HOURS_4(intl)}</option>
                        <option value={TIME_IN_MINUTES.EIGHT_HOURS}>{COMMON_LABELS.HOURS_8(intl)}</option>
                        <option value={TIME_IN_MINUTES.SIXTEEN_HOURS}>{COMMON_LABELS.HOURS_16(intl)}</option>
                        <option value={TIME_IN_MINUTES.ONE_DAY}>{COMMON_LABELS.HOURS_24(intl)}</option>
                    </Field>
                </div>
            </div>

            <div className="Form-Line row">
                <div htmlFor="name" className="Form-Label col-md-6">{COMPONENTS.FIELDS.USER.ACCOUNT_EXPIRATION_TIMEOUT}</div>
                <div className="col-md-6">
                    <Field
                        className="form-control input-sm "
                        name="accountExpirationTimeout"
                        dataId="user-role-edit-account-expiration-timeout"
                        style={{ fontSize: '14px' }}
                        component={RenderSelectField}
                    >
                        <option />
                        <option value={0}>{COMPONENTS.FIELDS.COMMON.NEVER_FUNC(intl)}</option>
                        <option value={TIME_IN_MINUTES.ONE_DAY}>{COMMON_LABELS.DAY_1(intl)}</option>
                        <option value={TIME_IN_MINUTES.WEEK}>{COMMON_LABELS.DAYS_7(intl)}</option>
                        <option value={TIME_IN_MINUTES.MONTH}>{COMMON_LABELS.DAYS_30(intl)}</option>
                        <option value={TIME_IN_MINUTES.THREE_MONTHS}>{COMMON_LABELS.DAYS_90(intl)}</option>
                        <option value={TIME_IN_MINUTES.HALF_YEAR}>{COMMON_LABELS.DAYS_180(intl)}</option>
                        <option value={TIME_IN_MINUTES.YEAR}>{COMMON_LABELS.DAYS_365(intl)}</option>
                    </Field>
                </div>
            </div>
            
            <div className="Form-Buttons">
                <button type="button" data-id="user-role-edit-save" onClick={submit} disabled={invalid}>{COMPONENTS.BUTTONS.SAVE}</button>
                <button type="button" data-id="user-role-edit-cancel" onClick={close}>{COMPONENTS.BUTTONS.CANCEL}</button>
            </div>
        </form>
    );
};

UserControlEditForm.propTypes = {
    userRoleEdit: PropTypes.shape({
        params: PropTypes.shape({
            role: PropTypes.string,
            userSessionTimeout: PropTypes.number,
            accountExpirationTimeout: PropTypes.number,
        }),
    }),
    className: PropTypes.string,
    close: PropTypes.func,
    submit: PropTypes.func,
    formState: PropTypes.shape({}),
    intl: PropTypes.shape({
        formatMessage: PropTypes.func,
    }),
};

UserControlEditForm.defaultProps = {
    userRoleEdit: {
        params: {
            role: '',
            userSessionTimeout: null,
            accountExpirationTimeout: null,
        },
    },
    className: '',
    close: () => {},
    submit: () => {},
    formState: {},
    intl: {
        formatMessage: () => {},
    },
};

UserControlEditForm = reduxForm({
    form: NAME_OF_FORMS.USER_ROLE_EDIT,
    validate,
})(injectIntl(UserControlEditForm));

const mapStateToForm = (state) => {
    const initialValues = state.modals.userModals.userAccessControlEdit.params;

    return {
        initialValues,
        formState: state.form,
        userAccessControlEdit: state.modals.userModals.userAccessControlEdit,
    };
};

UserControlEditForm = connect(mapStateToForm)(injectIntl(UserControlEditForm));

class UserControlEditModal extends React.Component {
    static propTypes = {
        dispatch: PropTypes.func,
        userAccessControlEdit: PropTypes.shape({}),
        style: PropTypes.shape({}),
    };
    
    static defaultProps = {
        dispatch: () => {},
        userAccessControlEdit: {},
        style: {},
    };
    
    close = () => {
        this.props.dispatch(closeUserAccessControlEditModal());
    };
    
    submit = (values) => {
        this.props.dispatch(openUserAccessControlWarningModal(values));
    };
    
    render() {
        const { style, userAccessControlEdit } = this.props;
        const title = USER_ACCESS_CONTROL_MODAL.TITLE.EDIT_FUNC(USER_ROLES_DISPLAY[userAccessControlEdit?.params?.role]);
        const currentStyle = makeCopy(style);
        currentStyle.content.width = '600px';

        return (
            <Modal
                contentLabel="userControlEdit"
                isOpen={this.props.userAccessControlEdit.open}
                onRequestClose={this.close}
                style={currentStyle}
            >
                <div className="RPP-Modal">
                    <span className="Modal-Header multiple-lines-ellipsis" data-id="user-access-control-edit-title">
                        {title}
                    </span>
                    <UserControlEditForm
                        className="Modal-Form"
                        close={this.close}
                        onSubmit={this.submit}
                    />
                </div>
            </Modal>
        );
    }
}

const mapStateToProps = state => ({
    userAccessControlEdit: state.modals.userModals.userAccessControlEdit,
});

export default connect(mapStateToProps)(UserControlEditModal);
