import React from 'react';
import Modal from 'react-modal';
import PropTypes from 'prop-types';
import { connect } from 'react-redux';
import moment from 'moment';
import { COMMON_LABELS } from 'common/ui-labels';
import { makeCopy } from 'utils/simpleUtils';
import { closeUserSessionExpirationModal, openCommonMessageModal, userLogoutWithoutBrowserReload } from 'stores/actions';

const LOGOUT_TIME = 30; // seconds

class UserSessionExpirationModal extends React.Component {
    static propTypes = {
        dispatch: PropTypes.func,
        userSessionExpiration: PropTypes.shape({}),
        style: PropTypes.shape({}),
    };
    
    static defaultProps = {
        dispatch: () => {},
        userSessionExpiration: {},
        style: {},
    };

    constructor(props) {
        super(props);
        this.state = {
            timeRemaining: LOGOUT_TIME, // seconds
            timer: null,
        }
    }

    componentDidUpdate(prevProps, prevState, snapshot) {
        const modalHasOpened = !prevProps.userSessionExpiration.open && this.props.userSessionExpiration.open;
        const modalHasClosed = prevProps.userSessionExpiration.open && !this.props.userSessionExpiration.open;
        if (modalHasOpened) {
            this.handleOpenedModal();
        } else if (modalHasClosed) {
            this.handleClosedModal();
        }
    }

    handleOpenedModal = () => {
        this.setState({ timeRemaining: LOGOUT_TIME });
        this.state.timer = setInterval(() => {
            if (this.state.timeRemaining <= 0) {
                clearInterval(this.state.timer);
                this.autoLogout();
            } else {
                this.setState({ timeRemaining: this.state.timeRemaining - 1  });
            }
        }, 1000);
    };

    handleClosedModal = () => {
        clearInterval(this.state.timer);
    };

    stayIn = () => {
        this.props.dispatch(closeUserSessionExpirationModal());
    };

    logout = () => {
        this.props.dispatch(closeUserSessionExpirationModal());
        this.props.dispatch(userLogoutWithoutBrowserReload());
    };
    
    autoLogout = () => {
        this.logout();
        const infoTitle = COMMON_LABELS.SESSION_EXPIRED;
        const infoMessage = COMMON_LABELS.SESSION_EXPIRED_MSG(moment().format('YYYY-MM-DD HH:mm'));
        this.props.dispatch(openCommonMessageModal({ title: infoTitle, message: infoMessage, status: '' }));
    };

    render() {
        const { style, userSessionExpiration } = this.props;

        const currentStyle = makeCopy(style);
        currentStyle.content.width = '600px';

        return (
            <Modal
                contentLabel="userControlWarning"
                isOpen={userSessionExpiration.open}
                onRequestClose={this.close}
                style={currentStyle}
            >
                <div className="RPP-Modal">
                    <span className="Modal-Header multiple-lines-ellipsis" data-id="user-access-control-warning-title">
                        {COMMON_LABELS.SESSION_EXPIRATION_WARN}
                    </span>
                    <form className="Modal-Form">
                        <div className="Form-DeleteTitle">
                            {COMMON_LABELS.SESSION_EXPIRATION_MSG}
                        </div>
                        <div className="Form-DeleteTitle">
                            {COMMON_LABELS.TIME_REMAINING_MSG(this.state.timeRemaining)}
                        </div>
                        <div className="Form-Buttons">
                            <button type="button" data-id="user-role-warning-save" onClick={this.stayIn}>{COMMON_LABELS.STAY_LOGGED}</button>
                            <button type="button" data-id="user-role-warning-cancel" onClick={this.logout}>{COMMON_LABELS.LOGOUT_NOW}</button>
                        </div>
                    </form>
                </div>
            </Modal>
        );
    }
}

const mapStateToProps = state => ({
    userSessionExpiration: state.modals.userModals.userSessionExpiration,
});

export default connect(mapStateToProps)(UserSessionExpirationModal);
