import React, { Fragment } from 'react';
import { connect } from 'react-redux';
import PropTypes from 'prop-types';
import Draggable from 'react-draggable';
import _isEqual from 'lodash/isEqual';
import { injectIntl } from 'react-intl';
import {
    removeZoneVertexCoordinate, addZoneVertexCoordinate, closeZoneCoordinatesModeModal, openZoneAddConfigurationModal,
    setErrorZoneCoordinatesModeModal, exitDrawRegime, startLiveMode, deactivateZoneEditCoordinatesMode
} from 'stores/actions';
import { getVerticesLimit } from 'utils/zoneUtils';
import Polygon from 'stores/classes/Polygon';
import { ZONE_COORDINATE_ENTER, COMPONENTS } from 'common/ui-labels';
import VerticesList from './VerticesList';


const validate = (verticesMap, intl) => {
    const error = {};
    const verticesList = Object.values(verticesMap);
    if (verticesList.length < 3) {
        error.invalid = true;
    }
    const polygon = new Polygon(verticesList);
    if (polygon.selfIntersects) {
        error.message = ZONE_COORDINATE_ENTER.SELF_INTERSECTION(intl);
        error.invalid = true;
        error.geometryIntersects = true;
    }
    return error;
};

class ZoneCoordinatesEnterModal extends React.Component {
    constructor(props) {
        super(props);
        this.state = {
            modalPosX: 0,
            modalPosY: 0,
        };
    }
    
    componentDidUpdate(prevProps) {
        const prevVertices = Object.values(prevProps.vertices);
        const vertices = Object.values(this.props.vertices);
        if (!_isEqual(prevVertices, vertices)) {
            const error = validate(this.props.vertices, this.props.intl);
            this.props.setError(error);
        }
    }
    
    setPosY = (modalPosY) => {
        this.setState({ modalPosY });
    };
    
    setPosX = (modalPosX) => {
        this.setState({ modalPosX });
    };
    
    render() {
        const verticesLimit = getVerticesLimit(this.props.currentProject);
        
        if (!this.props.open) {
            return (<Fragment />);
        }
        
        return (
            <Draggable
                enableUserSelectHack={false}
                onStart={(e, data) => {
                    this.setPosX(this.state.modalPosX + data.x);
                    this.setPosY(this.state.modalPosY + data.y);
                }}
                onDrag={(e, data) => {
                    this.setPosX(this.state.modalPosX + data.x);
                    this.setPosY(this.state.modalPosY + data.y);
                }}
                onStop={(e, data) => {
                    this.setPosX(this.state.modalPosX + data.x);
                    this.setPosY(this.state.modalPosY + data.y);
                }}
            >
                <div
                    style={{
                        position: 'absolute',
                        bottom: '100px',
                        right: '100px',
                        zIndex: 99999,
                        backgroundColor: 'ghostwhite',
                        height: '520px',
                        width: '350px',
                        padding: '10px',
                        borderRadius: '10px',
                        border: '2px solid grey',
                        cursor: 'pointer',
                    }}
                    className="RPP-Modal"
                >
                    <div className="Modal-Header">{ZONE_COORDINATE_ENTER.CREATE_ZONE(this.props.intl)}</div>
                    <div className="Modal-Text">{ZONE_COORDINATE_ENTER.ENTER_COORDINATES(this.props.intl)}</div>
                    <div className="Modal-Form">
                        <div className="no-cursor" style={{ height: '320px', overflowY: 'auto', overflowX: 'hidden' }}>
                            <VerticesList
                                verticesLimit={verticesLimit}
                                addVertex={this.props.addVertex}
                                removeVertex={this.props.removeVertex}
                                sublocation={this.props.sublocation}
                            />
                        </div>
                        <div className="Form-Buttons-Red">
                            <button
                                data-cy="add-zone-coordinate-continue"
                                disabled={this.props.error.invalid}
                                onClick={() => {
                                    const positions = Object.values(this.props.vertices).map(pos => ({
                                        x: pos.x,
                                        y: pos.y,
                                    }));
                                    this.props.continue(positions, this.props.zone_type_name, this.props.sublocation.uid);
                                }}
                            >
                                {COMPONENTS.BUTTONS.CONTINUE}
                            </button>
                            <button
                                data-cy="add-zone-coordinate-cancel"
                                onClick={() => {
                                    this.props.closeModal();
                                }}
                            >
                                {COMPONENTS.BUTTONS.CANCEL}
                            </button>
                            {this.props.error.invalid && <div style={{ color: 'red' }}>{this.props.error.message}</div>}
                        </div>
                    </div>
                </div>
            </Draggable>
        );
    }
}

ZoneCoordinatesEnterModal.propTypes = {
    intl: PropTypes.shape({
        formatMessage: PropTypes.func,
    }),
    open: PropTypes.bool,
    vertices: PropTypes.shape({}),
    zone_type_name: PropTypes.string,
    sublocation: PropTypes.shape({
        offset_x: PropTypes.number,
        offset_y: PropTypes.number,
        uid: PropTypes.string,
    }),
    currentProject: PropTypes.shape({}),
    continue: PropTypes.func,
    closeModal: PropTypes.func,
    addVertex: PropTypes.func,
    removeVertex: PropTypes.func,
    setError: PropTypes.func,
    error: PropTypes.shape({
        invalid: PropTypes.bool,
        message: PropTypes.string,
    }),
};

ZoneCoordinatesEnterModal.defaultProps = {
    intl: {
        formatMessage: () => {
        },
    },
    open: false,
    zone_type_name: '',
    vertices: {},
    sublocation: {
        offset_x: 0,
        offset_y: 0,
        uid: '',
    },
    currentProject: null,
    continue: () => {},
    closeModal: () => {},
    addVertex: () => {},
    removeVertex: () => {},
    setError: () => {},
    error: {
        invalid: false,
        geometryIntersects: false,
        message: '',
    },
};

const mapDispatchToProps = dispatch => ({
    continue: (positions, zone_type_name, sublocation_uid) => {
        const zone = {
            sublocation_uid,
            zone_type_name,
            zone_geometry: {
                positions,
            },
        };
        dispatch(openZoneAddConfigurationModal(zone));
        dispatch(exitDrawRegime());
        dispatch(closeZoneCoordinatesModeModal());
        dispatch(deactivateZoneEditCoordinatesMode());
    },
    setError: (error) => {
        dispatch(setErrorZoneCoordinatesModeModal(error));
    },
    closeModal: () => {
        dispatch(startLiveMode());
        dispatch(closeZoneCoordinatesModeModal());
        dispatch(deactivateZoneEditCoordinatesMode());
        dispatch(exitDrawRegime());
    },
    addVertex: (position) => {
        dispatch(addZoneVertexCoordinate(position));
    },
    removeVertex: (index) => {
        dispatch(removeZoneVertexCoordinate(index));
    },
});

const mapStateToProps = (state) => {
    const { zone_type_name, open } = state.modals.zoneModals.zoneCoordinatesMode;
    const { error, vertices } = state.siteViewZoneEditCoordinates;

    return {
        sublocation: state.siteView.selectedSublocation,
        currentProject: state.appContext.project,
        vertices,
        zone_type_name,
        error,
        open,
    };
};

export default connect(mapStateToProps, mapDispatchToProps)(injectIntl(ZoneCoordinatesEnterModal));
