import React from 'react';
import connect from 'react-redux/es/connect/connect';
import PropTypes from 'prop-types';
import _isEmpty from 'lodash/isEmpty';
import { change, formValueSelector, reduxForm } from 'redux-form';
import { injectIntl } from 'react-intl';
import Tabs from '@material-ui/core/Tabs';
import Tab from '@material-ui/core/Tab';
import ScheduleForm from 'components/modals/forms/zones/ScheduleForm';
import TrackablesAndGroups from 'components/modals/forms/zones/TrackablesAndGroups';
import CustomZoneOptions from 'components/modals/forms/zones/CustomZoneOptions';
import { Floorplan, Location, Sublocation, ZonePrototype } from 'stores/classes';
import { getParsedGroups, getParsedTrackables, getParsedZones } from 'stores/getters';
import { DEFAULT_VALUES, ERROR_TEXT, NAME_OF_FORMS, ZONE_TYPE, GEOMETRY_RESTRICTION_TYPES } from 'common/constants';
import { COMPONENTS, ZONE_EDIT_MODAL, TAB_LABELS } from 'common/ui-labels';
import {
    geometryFromDegreesToMillimeters, getLength, getValuesFromZoneFormState, getZoneTypeName,
    isDirectionalRestriction, rotateGeometry, sortByField,
} from 'utils';
import CustomTabPanel from 'components/CustomTabPanel';
import { setZoneCreateAdminModal } from 'stores/actions/modalActions';
import Polygon from 'stores/classes/Polygon';
import SelectedFloorBody from './SelectedFloorBody';
import WidthControl from './WidthControl';
import Buttons from './Buttons';
import RotationControl from './RotationControl';
import BasicOptions from './BasicOptions';
import ActivityOptions from './ActivityOptions';
import NotSelectedFloorBody from './NotSelectedFloorBody';
import VerticesCoordinates from './VerticesCoordinates';
import VerticesControl from './VerticesControl';

const validateGeometry = (geometry, coordinateMode, intl) => {
    if (!coordinateMode) {
        return {};
    }
    const geometryError = {};
    const verticesMap = geometry.vertices || {};
    const verticesList = Object.values(verticesMap);
    const polygon = new Polygon(verticesList);

    if (verticesList.length < 3) {
        geometryError.verticesNumber = ZONE_EDIT_MODAL.ERRORS.GEOMETRY_REQUIRED_FUNC(intl);
    } else if (polygon.selfIntersects) {
        geometryError.geometryIntersects = ZONE_EDIT_MODAL.ERRORS.GEOMETRY_INTERSECTS_FUNC(intl);
    }
    return geometryError;
};

const validate = (values, props) => {
    const errors = {};
    const { zones, selectedSublocationUid } = props;
    const zoneWithTheSameName = zones.find(zone => (zone && zone.sublocation_uid === selectedSublocationUid && zone.name === values.name));

    if (!values.name) {
        errors.name = COMPONENTS.VALIDATION.REQUIRED;
    }
    if (zoneWithTheSameName) {
        errors.name = COMPONENTS.VALIDATION.NAME_EXIST;
    }
    if (!values.sublocation_uid) {
        errors.sublocation_uid = COMPONENTS.VALIDATION.REQUIRED;
    }
    if (!values.zone_type_name) {
        errors.zone_type_name = COMPONENTS.VALIDATION.REQUIRED;
    }
    if (!props.coordinateMode && (!values.geometry || !values.geometry.zoneGeometryInMillimeters || !values.geometry.zoneGeometryInMillimeters.length)) {
        errors.geometry = ZONE_EDIT_MODAL.ERRORS.GEOMETRY_REQUIRED;
    }
    if (!values.zone_alarm_level && values.zone_type_name !== ZONE_TYPE.INFO.name) {
        errors.zone_alarm_level = COMPONENTS.VALIDATION.REQUIRED;
    }
    if (values.custom_info && getLength(values.custom_info) > DEFAULT_VALUES.CUSTOM_INFO_LIMIT) {
        errors.custom_info = ERROR_TEXT.CUSTOM_INFO_ERROR();
    }

    return errors;
};

const activityHandler = (props, e) => {
    const activeValue = e.target.value === 'true' ? false : e.target.value === 'false';
    props.dispatch(change(NAME_OF_FORMS.ZONE_CREATE, 'active24', activeValue));
};

let ZoneCreateAdminPageForm = (props) => {
    const {
        close, submit, selectFloorplan, unSelectFloorplan, className, sublocations, floorplans, locations, selectedFloorplanUid,
        invalid, initialValues, boundsForScale, onMove, zoom, setLeafletToState, intl, formState, dispatch, geometry, trackables,
        groups, selectedSublocationUid, selectedLocationUid, zone_type_name, zonePrototype, customInfoLength, zonesPrototypes,
        coordinateMode, geometryError, submitSucceeded, zones, isPolygonDrawing, geometryRestriction,
    } = props;
    initialValues.default_allowed += '';

    const leafletClass = selectedFloorplanUid ? 'Zone-Details-Leaflet-Selected' : 'Zone-Create-Leaflet';
    const selectedFloorplan = floorplans.find(floorplan => floorplan.isFloorplanUidEqualTo(selectedFloorplanUid));
    const selectedSublocation = sublocations.find(sublocation => sublocation.uid === selectedSublocationUid);
    const selectedLocation = locations.find(location => location.isLocationUidEqualTo(selectedLocationUid));
    const createZoneFormState = formState[NAME_OF_FORMS.ZONE_CREATE] || {};
    const sublocation_name = selectedSublocation ? selectedSublocation.name : '';
    const location_name = selectedLocation ? selectedLocation.name : '';
    let selectedVerticesOptions = true;
    let rotation = 0;
    let width = 0;
    let isKeepOutZone = false;
    let isKeepInZone = false;
    let active24 = false;
    let rotatedInDegrees = [];
    
    const handleChangeGeometry = ({ zoneGeometryInDegrees, zoneGeometryInMillimeters }) => {
        dispatch(setZoneCreateAdminModal({
            ...createZoneFormState.values,
            geometry: { zoneGeometryInDegrees, zoneGeometryInMillimeters },
        }));
    };
    
    const handleChangeRotation = (inRotation) => {
        dispatch(setZoneCreateAdminModal({
            ...createZoneFormState.values,
            rotation: inRotation,
        }));
    };
    
    const handleChangeWidth = (newWidth) => {
        if (createZoneFormState && createZoneFormState.values) {
            createZoneFormState.values.width = newWidth;
        }
    };
    
    if (createZoneFormState && createZoneFormState.values) {
        if (createZoneFormState.values.zone_type_name) {
            isKeepInZone = zone_type_name === ZONE_TYPE.KEEP_IN.name;
            isKeepOutZone = zone_type_name === ZONE_TYPE.KEEP_OUT.name;
        }
        rotation = createZoneFormState.values.rotation;
        width = createZoneFormState.values.width;
        active24 = createZoneFormState.values.active24;
        rotatedInDegrees = rotateGeometry(geometry, rotation);
        createZoneFormState.values.rotated_in_degrees = rotatedInDegrees;
        createZoneFormState.values.geometry = geometry;
        selectedVerticesOptions = createZoneFormState.values.vertices_options;
        const geometryInObjects = rotatedInDegrees.map(point => ({ lng: point[1], lat: point[0] }));
        createZoneFormState.values.rotated_in_millimiters = geometryFromDegreesToMillimeters(geometryInObjects);
    }
    const isKeepInOutZone = (isKeepInZone || isKeepOutZone);
    const zoneDisplayName = getZoneTypeName(zonesPrototypes, zone_type_name, intl);
    
    const isDirectionalGeometry = zonePrototype && isDirectionalRestriction(zonePrototype.zone_geometry_restriction);
    
    const polygon = {
        dashArray: '1',
        dashColor: 'black',
        dashOpacity: 0.9,
        dashWeight: 2,
        fillColor: '#001cc9',
        positions: rotatedInDegrees,
    };

    const { activated } = (createZoneFormState && createZoneFormState.values)
        ? getValuesFromZoneFormState(createZoneFormState)
        : { activated: false };

    const [value, setValue] = React.useState(0);

    const handleChange = (event, newValue) => {
        setValue(newValue);
    };

    const isFinishButtonVisible = (geometryRestriction && Array.isArray(geometryRestriction))
        ? geometryRestriction.indexOf(GEOMETRY_RESTRICTION_TYPES.POLYGON) !== -1
        : false;

    const handleFinishClick = () => {
        const finishPolygonDraw = document.querySelector('[title="Finish drawing"]');
        finishPolygonDraw.click();
    };

    return (
        <form className={className}>
            <div className="row create-zone-form-row">
                <div className="col-md-4 col-sm-4">
                    <Tabs
                        className="with-border-bottom"
                        value={value}
                        onChange={handleChange}
                        indicatorColor="primary"
                        textColor="primary"
                        variant="fullWidth"
                        aria-label="fullWidthTabs"
                    >
                        <Tab label={TAB_LABELS.GENERAL(intl)} data-id="zone-create-admin-general" />
                        <Tab label={TAB_LABELS.ADVANCED(intl)} data-id="zone-create-admin-advanced" />
                    </Tabs>
                    <CustomTabPanel value={value} index={0}>
                        <BasicOptions
                            location_name={location_name}
                            zoneTypeName={zoneDisplayName}
                            sublocation_name={sublocation_name}
                            customInfoLength={customInfoLength}
                            intl={intl}
                        />
                        <RotationControl
                            geometry={geometry}
                            label={`${ZONE_EDIT_MODAL.LABEL_ROTATION(intl)}: `}
                        />
                        <WidthControl
                            show={isDirectionalGeometry}
                            geometry={geometry}
                        />
                        <ActivityOptions
                            isKeepInOutZone={isKeepInOutZone}
                            activityHandler={(event) => { activityHandler(props, event); }}
                        />
                        <CustomZoneOptions
                            zones={zones}
                            zone_type_name={zone_type_name}
                            zonePrototype={zonePrototype}
                            isBigField={false}
                            formState={initialValues}
                        />
                        <VerticesControl
                            coordinateMode={coordinateMode}
                        />
                        <VerticesCoordinates
                            vertices={geometry.vertices}
                            show={selectedVerticesOptions}
                            coordinateMode={coordinateMode}
                            sublocation={selectedSublocation}
                        />
                    </CustomTabPanel>
                    <CustomTabPanel value={value} index={1}>
                        <TrackablesAndGroups
                            trackables={trackables}
                            groups={groups}
                            intl={intl}
                        />
                        <ScheduleForm
                            active24={active24}
                            activated={activated}
                        />
                    </CustomTabPanel>
                </div>
                
                <div className="col-md-8 col-sm-8">
                    {
                        NotSelectedFloorBody({
                            intl,
                            dispatch,
                            floorplans,
                            leafletClass,
                            selectFloorplan,
                            unSelectFloorplan,
                            setLeafletToState,
                            selectedFloorplan,
                            createZoneFormState,
                            selectedSublocation,
                            selectedFloorplanUid,
                            selectedSublocationUid,
                        })
                    }
                    {selectedFloorplanUid &&
                    <SelectedFloorBody
                        intl={intl}
                        zoom={zoom}
                        width={width}
                        onMove={onMove}
                        polygon={polygon}
                        rotation={rotation}
                        geometry={geometry}
                        coordinateMode={coordinateMode}
                        error={geometryError}
                        dispatch={dispatch}
                        leafletClass={leafletClass}
                        zone_type_name={zone_type_name}
                        boundsForScale={boundsForScale}
                        unSelectFloorplan={unSelectFloorplan}
                        setLeafletToState={setLeafletToState}
                        selectedFloorplan={selectedFloorplan}
                        handleChangeWidth={handleChangeWidth}
                        handleChangeGeometry={handleChangeGeometry}
                        handleChangeRotation={handleChangeRotation}
                        createZoneFormState={createZoneFormState}
                        selectedSublocation={selectedSublocation}
                        isDirectionalGeometry={isDirectionalGeometry}
                    />
                    }
                    {!_isEmpty(geometryError) &&
                        <div>
                            {Object.keys(geometryError).map((key) => {
                                if (key !== 'verticesNumber') {
                                    return geometryError[key] && <span className="error">&#9888;{geometryError[key]}</span>;
                                }
                                return null;
                            })}
                        </div>
                    }
                    <Buttons
                        invalid={invalid || !_isEmpty(geometryError) || submitSucceeded}
                        submit={submit}
                        createZoneFormState={createZoneFormState}
                        close={close}
                        isFinishButtonVisible={isFinishButtonVisible}
                        isFinishButtonDisabled={!isPolygonDrawing}
                        handleFinishClick={handleFinishClick}
                    />
                </div>
            </div>
        </form>
    );
};

ZoneCreateAdminPageForm.propTypes = {
    close: PropTypes.func,
    submit: PropTypes.func,
    dispatch: PropTypes.func,
    onMove: PropTypes.func,
    setLeafletToState: PropTypes.func,
    boundsForScale: PropTypes.shape(),
    geometryError: PropTypes.shape(),
    initialValues: PropTypes.shape(),
    intl: PropTypes.shape({
        formatMessage: PropTypes.func,
    }),
    selectedFloorplanUid: PropTypes.string,
    selectedLocationUid: PropTypes.string,
    selectedSublocationUid: PropTypes.string,
    zone_type_name: PropTypes.string,
    zonePrototype: PropTypes.shape(),
    zonesPrototypes: PropTypes.arrayOf(PropTypes.shape()),
    selectFloorplan: PropTypes.func,
    unSelectFloorplan: PropTypes.func,
    className: PropTypes.string,
    zoom: PropTypes.number,
    invalid: PropTypes.bool,
    formState: PropTypes.shape(),
    locations: PropTypes.arrayOf(PropTypes.shape()),
    geometry: PropTypes.shape({
        zoneGeometryInDegrees: PropTypes.array,
        zoneGeometryInMillimeters: PropTypes.array,
        vertices: PropTypes.shape({}),
    }),
    floorplans: PropTypes.arrayOf(PropTypes.shape()),
    sublocations: PropTypes.arrayOf(PropTypes.shape()),
    trackables: PropTypes.arrayOf(PropTypes.shape()),
    zones: PropTypes.arrayOf(PropTypes.shape()),
    groups: PropTypes.arrayOf(PropTypes.shape()),
    customInfoLength: PropTypes.number,
    coordinateMode: PropTypes.bool,
    submitSucceeded: PropTypes.bool,
    isPolygonDrawing: PropTypes.bool,
    geometryRestriction: PropTypes.arrayOf(PropTypes.string),
};

ZoneCreateAdminPageForm.defaultProps = {
    close: () => {},
    submit: () => {},
    dispatch: () => {},
    selectFloorplan: () => {},
    onMove: () => {},
    setLeafletToState: () => {},
    unSelectFloorplan: () => {},
    boundsForScale: {},
    geometryError: {},
    intl: {
        formatMessage: () => {},
    },
    initialValues: {
        user_params: {},
        vertices: {},
    },
    invalid: true,
    selectedFloorplanUid: null,
    selectedSublocationUid: null,
    selectedLocationUid: null,
    className: '',
    zone_type_name: '',
    zonePrototype: {},
    zonesPrototypes: [],
    zoom: 0,
    formState: {},
    locations: [],
    floorplans: [],
    geometry: {
        zoneGeometryInDegrees: [],
        zoneGeometryInMillimeters: [],
        vertices: {},
    },
    sublocations: [],
    trackables: [],
    zones: [],
    groups: [],
    customInfoLength: 0,
    coordinateMode: false,
    submitSucceeded: false,
    isPolygonDrawing: false,
    geometryRestriction: [],
};

ZoneCreateAdminPageForm = reduxForm({
    form: NAME_OF_FORMS.ZONE_CREATE,
    validate,
    enableReinitialize: true,
    keepDirtyOnReinitialize: true,
    updateUnregisteredFields: true,
})(ZoneCreateAdminPageForm);

const mapStateToForm = (state, { intl }) => {
    const selector = formValueSelector(NAME_OF_FORMS.ZONE_CREATE);
    const custom_info = selector(state, 'custom_info');
    const initialValues = state.modals.zoneModals.zoneCreateAdminPage;
    const zonesPrototypes = ZonePrototype.getAllZonePrototypes(state);
    const { zone_type_name, geometry, coordinateMode } = initialValues;
    const zonePrototype = zonesPrototypes.find(zone_prototype => zone_prototype.isZoneTypeNameEqualTo(zone_type_name));
    const geometryError = validateGeometry(geometry, coordinateMode, intl);

    const isPolygonDrawing = state.modals.zoneModals.zoneChooseGeometry.polygon_create;
    const geometryRestriction = state.siteView.geometry.restriction;

    return {
        zonePrototype,
        initialValues,
        isPolygonDrawing,
        geometryRestriction,
        customInfoLength: custom_info ? getLength(custom_info) : 0,
        zone_type_name,
        formState: state.form,
        appContext: state.appContext,
        trackables: getParsedTrackables(state),
        zonesPrototypes,
        geometry: state.modals.zoneModals.zoneCreateAdminPage.geometry,
        groups: getParsedGroups(state),
        selectedLocationUid: state.modals.zoneModals.zoneCreateAdminPage.location_uid,
        selectedSublocationUid: state.modals.zoneModals.zoneCreateAdminPage.sublocation_uid,
        locations: Location.getAllLocations(state) || DEFAULT_VALUES.empty_array,
        zones: getParsedZones(state),
        floorplans: sortByField(Floorplan.getAllFloorplans(state), 'name') || DEFAULT_VALUES.empty_array,
        sublocations: Sublocation.getAllSublocations(state) || DEFAULT_VALUES.empty_array,
        coordinateMode,
        geometryError,
    };
};

export default ZoneCreateAdminPageForm = injectIntl(connect(mapStateToForm)(ZoneCreateAdminPageForm));
