import React from 'react';
import PropTypes from 'prop-types';
import Modal from 'react-modal';
import { connect } from 'react-redux';
import { reduxForm } from 'redux-form';
import { injectIntl } from 'react-intl';
import { NAME_OF_FORMS, COMPONENTS } from 'common';
import { closeZoneDetailsSitePageModal } from 'stores/actions';
import { getParsedGroups, getParsedTrackables } from 'stores/getters';
import { getZoneDetailsAndZoneTypeName, showActivePeriod } from 'utils';
import CustomZoneInfoOptions from 'components/modals/forms/zones/CustomZoneInfoOptions';
import { Zone, ZonePrototype } from 'stores/classes';
import { ZonePrototype as SelectorZonePrototype } from 'stores/classes/Selectors';
import BasicFields from './ZoneDetailsAdminPageModal/BasicFields';
import ApplyTo from './ZoneDetailsAdminPageModal/ApplyTo';


let ZoneDetailsSitePageForm = (props) => {
    const { close, params, className, groups, trackables, zonePrototype, intl, zones } = props;
    const { zone_exceptions, apply_to, zone_mode, schedule_start_time, schedule_stop_time, zone_exceptions_groups, user_params } = params;
    
    return (
        <form className={className}>
            <BasicFields
                zone={params}
                intl={intl}
            />
            <CustomZoneInfoOptions
                zonePrototype={zonePrototype}
                user_params={user_params}
                zones={zones}
            />
            <ApplyTo
                zone_display_name={params.zone_type ? params.zone_type.display_name : ''}
                apply_to={apply_to}
                zone_exceptions_uids={zone_exceptions.concat(zone_exceptions_groups)}
                groups={groups}
                trackables={trackables}
            />
            <div className="row Form-Line">
                <div className="col-md-5 Form-Label">{COMPONENTS.FIELDS.ZONE.ACTIVE_PERIOD}</div>
                <div
                    className="col-md-7 Form-Value"
                    data-id="zone-details-site-status"
                    data-cy="zone-details-site-active-period"
                >
                    {showActivePeriod(zone_mode, schedule_start_time, schedule_stop_time)}
                </div>
            </div>

            <div className="Form-Buttons OneBtn Btn-Close-Details-Zones">
                <button
                    type="button"
                    data-id="zone-details-close"
                    onClick={close}
                >
                    {COMPONENTS.BUTTONS.CLOSE}
                </button>
            </div>
        </form>
    );
};


ZoneDetailsSitePageForm.propTypes = {
    close: PropTypes.func,
    params: PropTypes.shape(),
    className: PropTypes.string,
    groups: PropTypes.arrayOf(PropTypes.shape({
        uid: PropTypes.string,
        name: PropTypes.string,
    })),
    zones: PropTypes.arrayOf(PropTypes.shape()),
    zonePrototype: PropTypes.shape(),
    trackables: PropTypes.arrayOf(PropTypes.shape({
        uid: PropTypes.string,
        name: PropTypes.string,
    })),
    intl: PropTypes.shape({
        formatMessage: PropTypes.func,
    }),
};

ZoneDetailsSitePageForm.defaultProps = {
    close: () => {
    },
    params: {
    },
    className: '',
    groups: [],
    zones: [],
    zonePrototype: {},
    trackables: [],
    intl: undefined,
};

ZoneDetailsSitePageForm = reduxForm({
    form: NAME_OF_FORMS.ZONE_DETAILS_SITE_PAGE,
})(ZoneDetailsSitePageForm);

const mapStateToForm = (state) => {
    const params = state.modals.zoneModals.zoneDetailsSitePage.params || {};
    const zone_type_id = params.zone_type && params.zone_type.id;
    const zonePrototype = SelectorZonePrototype.getZonePrototypeById(zone_type_id);
    
    return {
        zonePrototype,
        appContext: state.appContext,
        zones: Zone.getAllZonesArray(state),
        groups: getParsedGroups(state),
        trackables: getParsedTrackables(state),
        zone_type_name: zonePrototype ? zonePrototype.zone_type.name : '',
    };
};

ZoneDetailsSitePageForm = connect(mapStateToForm)(injectIntl(ZoneDetailsSitePageForm));

class ZoneDetailsSitePageModal extends React.Component {
    static propTypes = {
        dispatch: PropTypes.func,
        style: PropTypes.shape({
            overlay: PropTypes.object,
            content: PropTypes.object,
        }),
        zoneDetailsSitePage: PropTypes.shape({
            params: PropTypes.shape({
                name: PropTypes.string,
            }),
            open: PropTypes.bool,
        }),
    };
    
    static defaultProps = {
        dispatch: () => {
        },
        style: {
            overlay: {},
            content: {},
        },
        zoneDetailsSitePage: {
            params: {
                name: '',
            },
            open: false,
        },
    };
    
    close = () => {
        this.props.dispatch(closeZoneDetailsSitePageModal());
    };
    
    render() {
        const { style } = this.props;
        const zoneName = this.props.zoneDetailsSitePage.params ? this.props.zoneDetailsSitePage.params.name : '';
        return (
            <Modal
                contentLabel="userDetails"
                isOpen={this.props.zoneDetailsSitePage.open}
                onRequestClose={this.close}
                style={style}
            >

                <div className="RPP-Modal">
                    <span className="Modal-Header multiple-lines-ellipsis" data-cy="zone-details-site-page-title">
                        {COMPONENTS.FIELDS.COMMON.VIEW_INFORMATION_FUNC(zoneName)}
                    </span>
                    <ZoneDetailsSitePageForm
                        className="Modal-Form"
                        close={this.close}
                        params={this.props.zoneDetailsSitePage.params}
                    />
                </div>
                
            </Modal>
        );
    }
}

const mapStateToProps = (state) => {
    // todo: move to mapStateToForm
    const { details } = getZoneDetailsAndZoneTypeName(state.modals.zoneModals.zoneDetailsSitePage, ZonePrototype.getAllZonePrototypes(state));
    if (details.params.sublocation_uid) {
        const sublocation = state.entities.sublocations.find(sl => sl.uid === details.params.sublocation_uid);
        details.params.sublocation_name_display = sublocation ? sublocation.name : '';
    }
    if (details.params.location_uid) {
        const location = state.entities.locations.find(loc => loc.uid === details.params.location_uid);
        details.params.location_name_display = location ? location.name : '';
    }
    return {
        zoneDetailsSitePage: details,
    };
};

export default connect(mapStateToProps)(ZoneDetailsSitePageModal);
