import React from 'react';
import { connect } from 'react-redux';
import { change, formValueSelector, reduxForm } from 'redux-form';
import { injectIntl } from 'react-intl';
import PropTypes from 'prop-types';
import Tabs from '@material-ui/core/Tabs';
import Tab from '@material-ui/core/Tab';
import CustomTabPanel from 'components/CustomTabPanel';
import ScheduleForm from 'components/modals/forms/zones/ScheduleForm';
import SeverityOptions from 'components/modals/forms/zones/SeverityOptions';
import SubmitButtons from 'components/modals/forms/SubmitButtons';
import BasicOptions from 'components/modals/forms/zones/BasicOptions';
import TrackablesAndGroups from 'components/modals/forms/zones/TrackablesAndGroups';
import CustomZoneOptions from 'components/modals/forms/zones/CustomZoneOptions';
import FloorplansAll from 'components/modals/forms/FloorplansAll';
import FloorplanSelected from 'components/modals/forms/zones/FloorplanSelected';
import { Location, Sublocation, Floorplan, ZonePrototype } from 'stores/classes';
import { NAME_OF_FORMS, ZONE_TYPE, DEFAULT_VALUES, ERROR_TEXT } from 'common/constants';
import { COMPONENTS, TAB_LABELS } from 'common/ui-labels';
import {
    getInclusionsExclusionsObjects, getCronFromToStringValues, isInfoZone, sortByField,
    zonePositions, getZoneTypeName, getValuesFromZoneFormState, getLength,
} from 'utils';
import { getZoneColor } from 'utils/simpleUtils';
import { getParsedGroups, getParsedTrackables, getParsedZones } from 'stores/getters';


const findZoneSublocation = (zone, sublocations, trackables) => {
    let zoneSublocation = {};
    if (zone.sublocation_uid) {
        sublocations.forEach((sublocation) => {
            if (sublocation.uid === zone.sublocation_uid) {
                zoneSublocation = sublocation;
            }
        });
    } else if (zone.trackable_object_uid) {
        const trackable = trackables.find(tko => (tko.uid === zone.trackable_object_uid && tko.position && tko.position.sl_id));
        if (trackable) {
            zone.sublocation_uid = trackable.position.sl_id;
            zoneSublocation = sublocations.find(sublocation => (sublocation.uid === zone.sublocation_uid)) || {};
            zone.sublocation_name_display = zoneSublocation.name || '';
        }
    }
    return zoneSublocation;
};

const findFloorplansForZone = (zone, floorplans) => {
    let zoneFloorplans = [];
    if (zone.sublocation_uid) {
        zoneFloorplans = floorplans.filter(floorplan => floorplan.sublocation_uid === zone.sublocation_uid);
    }
    return zoneFloorplans;
};

const activityHandler = (props, e) => {
    const activeValue = e.target.value === 'true' ? false : e.target.value === 'false';
    props.dispatch(change(NAME_OF_FORMS.ZONE_EDIT_ADMIN_PAGE, 'active24', activeValue));
};

export const validate = (values, props) => {
    const errors = {};
    const resultError = 'Invalid time value. Time should be in 24-hour format';
    if (values) {
        const { fromHour, fromMinute, toHour, toMinute, name } = values;
        const { zones, uid, sublocation_uid } = props;
        let zoneWithTheSameName;
        if (zones && uid && sublocation_uid && name) {
            zoneWithTheSameName = zones.find(zone => (zone && zone.uid !== uid && zone.sublocation_uid === sublocation_uid && zone.name === name));
        }

        const validateHour = hour => Number(hour) > 23 || Number(hour) < 0;
        const validateMinute = minute => Number(minute) > 59 || Number(minute) < 0;
        const isMidnight = (hour, minute) => Number(hour) === 24 && Number(minute) === 0;

        if (!isMidnight(fromHour, fromMinute)) {
            if (validateHour(fromHour)) {
                errors.fromHour = resultError;
            }
            if (validateMinute(fromMinute)) {
                errors.fromMinute = resultError;
            }
        }
        if (!isMidnight(toHour, toMinute)) {
            if (validateHour(toHour)) {
                errors.toHour = resultError;
            }
            if (validateMinute(toMinute)) {
                errors.toMinute = resultError;
            }
        }
        if (!name || name.length === 0) {
            errors.name = COMPONENTS.VALIDATION.REQUIRED;
        }

        if (zoneWithTheSameName) {
            errors.name = COMPONENTS.VALIDATION.NAME_EXIST;
        }
    }

    if (!values || errors.fromHour || errors.fromMinute || errors.toHour || errors.toMinute) {
        errors.activePeriod = { _error: resultError };
    }

    if (values.custom_info && getLength(values.custom_info) > DEFAULT_VALUES.CUSTOM_INFO_LIMIT) {
        errors.custom_info = ERROR_TEXT.CUSTOM_INFO_ERROR();
    }

    return errors;
};

let ZoneEditAdminPageForm = (props) => {
    const {
        close, submit, formState, initialValues, intl, zone_type_name, zonePrototype,
        trackables, groups, sublocations, params, invalid, selectedFloorplan, direction,
        floorplans, selectFloorplan, unSelectFloorplan, setLeafletToState, onMove, boundsForScale, zoom, maxHeight,
        customInfoLength, zones_prototypes, zones, zone_id,
    } = props;

    initialValues.default_allowed += '';

    const editZoneFormState = formState[NAME_OF_FORMS.ZONE_EDIT_ADMIN_PAGE] || {};
    
    const {
        activated,
        location_name,
        sublocation_name,
        areAdvancedOptionsShown,
        active24,
    } = getValuesFromZoneFormState(editZoneFormState);

    const zoneSublocation = findZoneSublocation(params, sublocations, trackables);
    const floorplansList = selectedFloorplan ? [] : findFloorplansForZone(params, floorplans);

    const leafletClass = selectedFloorplan ? 'Zone-Details-Leaflet-Selected' : 'Zone-Details-Leaflet';
    const mapStyle = { height: '100%' };
    const zone = {
        dashOpacity: 0.3,
        dashArray: params.active ? '1' : '5 10',
        dashColor: 'black',
        dashWeight: 2,
        fillColor: getZoneColor(params),
        positions: zonePositions(params, zoneSublocation),
    };

    const isZoneTypeSelected = !!zone_type_name;
    const isKeepInType = zone_type_name === ZONE_TYPE.KEEP_IN.name;
    const isKeepOutType = zone_type_name === ZONE_TYPE.KEEP_OUT.name;
    const isKeepInOutType = (isKeepInType || isKeepOutType);
    const isInformationZone = isInfoZone({ zone_type_name });

    const zoneTypeName = getZoneTypeName(zones_prototypes, params.zone_type_name, intl);

    const [value, setValue] = React.useState(0);

    const handleChange = (event, newValue) => {
        setValue(newValue);
    };

    return (
        <form className="Modal-Form create-zone-form">
            <div className="row create-zone-form-row">
                <div className="col-md-4" style={{ overflowY: 'auto', maxHeight }}>
                    <Tabs
                        className="with-border-bottom"
                        value={value}
                        onChange={handleChange}
                        indicatorColor="primary"
                        textColor="primary"
                        variant="fullWidth"
                        aria-label="full width tabs example"
                    >
                        <Tab label={TAB_LABELS.GENERAL(intl)} data-id="zone-edit-admin-general" />
                        <Tab label={TAB_LABELS.ADVANCED(intl)} data-id="zone-edit-admin-advanced" />
                    </Tabs>
                    <CustomTabPanel value={value} index={0}>
                        <BasicOptions
                            activityHandler={(event) => { activityHandler(props, event); }}
                            customInfoLength={customInfoLength}
                            location_name={location_name}
                            sublocation_name={sublocation_name}
                            zoneTypeName={zoneTypeName}
                            isZoneTypeSelected={isZoneTypeSelected}
                            activated={activated}
                            intl={intl}
                        />

                        <SeverityOptions
                            show={isKeepInOutType}
                            isZoneTypeSelected={isZoneTypeSelected}
                            isInformationZone={isInformationZone}
                            intl={intl}
                        />
                        <CustomZoneOptions
                            zone_id={zone_id}
                            zones={zones}
                            zone_type_name={zone_type_name}
                            zonePrototype={zonePrototype}
                            formState={initialValues}
                        />
                    </CustomTabPanel>
                    <CustomTabPanel value={value} index={1}>
                        <TrackablesAndGroups
                            show={areAdvancedOptionsShown}
                            trackables={trackables}
                            groups={groups}
                            intl={intl}
                        />
                        <ScheduleForm
                            show={areAdvancedOptionsShown}
                            active24={active24}
                            activated={activated}
                        />
                    </CustomTabPanel>
                </div>

                <div className="col-md-8 col-sm-8">
                    <FloorplansAll
                        floorplansList={floorplansList}
                        selectedFloorplan={!!selectedFloorplan}
                        zone={zone}
                        selectFloorplan={selectFloorplan}
                        unSelectFloorplan={unSelectFloorplan}
                        leafletClass={leafletClass}
                        mapStyle={mapStyle}
                        intl={intl}
                    />
                    <FloorplanSelected
                        intl={intl}
                        zone={zone}
                        zoom={zoom}
                        onMove={onMove}
                        mapStyle={mapStyle}
                        direction={direction}
                        leafletClass={leafletClass}
                        zonePrototype={zonePrototype}
                        boundsForScale={boundsForScale}
                        zoneSublocation={zoneSublocation}
                        selectFloorplan={selectFloorplan}
                        selectedFloorplan={selectedFloorplan}
                        unSelectFloorplan={unSelectFloorplan}
                        setLeafletToState={setLeafletToState}
                    />
                    <SubmitButtons
                        className="Form-Buttons"
                        submit={submit}
                        invalid={invalid}
                        close={close}
                        saveDataId="zone-edit-admin-page-save"
                        cancelDataId="zone-edit-admin-page-cancel"
                    />
                </div>
            </div>
        </form>
    );
};

ZoneEditAdminPageForm = reduxForm({
    form: NAME_OF_FORMS.ZONE_EDIT_ADMIN_PAGE,
    validate,
})(ZoneEditAdminPageForm);

const mapStateToForm = (state) => {
    const selector = formValueSelector(NAME_OF_FORMS.ZONE_EDIT_ADMIN_PAGE);
    const custom_info = selector(state, 'custom_info');

    const modal = state.modals.zoneModals.zoneEditAdminPage;
    const params = modal.params;
    const {
        toHour,
        toMinute,
        fromHour,
        fromMinute,
    } = getCronFromToStringValues({ from: params.schedule_start_time, to: params.schedule_stop_time });

    const {
        uid,
        zone_id,
        apply_to,
        location_uid,
        zone_type_name,
        sublocation_uid,
        zone_exceptions,
        zone_type_id,
        zone_exceptions_groups,
        user_params: { direction },
    } = params;
    
    const trackables = getParsedTrackables(state);
    const groups = getParsedGroups(state);
    const trackables_and_groups = getInclusionsExclusionsObjects(zone_exceptions, zone_exceptions_groups, trackables, groups);
    const location = Location.getAllLocations(state).find(loc => loc.uid === location_uid);
    const zonesPrototypes = ZonePrototype.getAllZonePrototypes(state);

    let floorplans = Floorplan.getAllFloorplans(state) || [];
    
    floorplans = floorplans.map((floorplan) => {
        if (!floorplan.display_name) {
            const arrUrl = floorplan.url.split('/');
            floorplan.display_name = arrUrl[arrUrl.length - 1];
        }
        return floorplan;
    });

    const initialValues = {
        toHour,
        fromHour,
        toMinute,
        apply_to,
        fromMinute,
        ...params,
        advanced_options: true,
        trackables_and_groups,
        location_name: location ? location.name : '',
        sublocation_name: params.sublocation_name_display || '',
    };
    
    const zonePrototype = zonesPrototypes.find(zone_prototype => zone_prototype.isZoneTypeIdEqualTo(zone_type_id));

    return {
        uid,
        groups,
        params,
        zone_id,
        direction,
        trackables,
        floorplans: sortByField(floorplans, 'display_name'),
        initialValues,
        sublocation_uid,
        zonesPrototypes,
        zonePrototype,
        customInfoLength: custom_info ? getLength(custom_info) : 0,
        formState: state.form,
        appContext: state.appContext,
        zones_prototypes: state.entities.zonesPrototypes,
        sublocations: Sublocation.getAllSublocations(state),
        zones: getParsedZones(state),
        zone_type_name: params.zone_type ? params.zone_type.name : zone_type_name,
    };
};

ZoneEditAdminPageForm.propTypes = {
    close: PropTypes.func,
    submit: PropTypes.func,
    onMove: PropTypes.func,
    zone_id: PropTypes.number,
    selectFloorplan: PropTypes.func,
    unSelectFloorplan: PropTypes.func,
    setLeafletToState: PropTypes.func,
    intl: PropTypes.shape({
        formatMessage: PropTypes.func,
    }),
    groups: PropTypes.arrayOf(PropTypes.shape()),
    formState: PropTypes.shape(),
    trackables: PropTypes.arrayOf(PropTypes.shape()),
    zones: PropTypes.arrayOf(PropTypes.shape()),
    boundsForScale: PropTypes.shape(),
    floorplans: PropTypes.arrayOf(PropTypes.shape()),
    selectedFloorplan: PropTypes.shape(),
    initialValues: PropTypes.shape(),
    zone_type_name: PropTypes.string,
    zonePrototype: PropTypes.shape(),
    zoom: PropTypes.number,
    direction: PropTypes.number,
    customInfoLength: PropTypes.number,
    params: PropTypes.shape(),
    sublocations: PropTypes.arrayOf(PropTypes.shape()),
    invalid: PropTypes.bool,
    maxHeight: PropTypes.string,
    zones_prototypes: PropTypes.arrayOf(PropTypes.shape()),
};

ZoneEditAdminPageForm.defaultProps = {
    intl: {
        formatMessage: () => {},
    },
    groups: [],
    zone_id: 0,
    customInfoLength: 0,
    formState: {},
    trackables: [],
    zones: [],
    boundsForScale: {},
    floorplans: [{}],
    selectedFloorplan: null,
    close: () => {
    },
    submit: () => {
    },
    onMove: () => {
    },
    selectFloorplan: () => {
    },
    unSelectFloorplan: () => {
    },
    setLeafletToState: () => {
    },
    initialValues: {
        user_params: {},
    },
    zone_type_name: '',
    zonePrototype: {},
    zoom: DEFAULT_VALUES.MAP_ZOOM.DEFAULT,
    direction: 0,
    params: {},
    sublocations: [{}],
    invalid: true,
    maxHeight: '60px',
    zones_prototypes: [],
};

export default ZoneEditAdminPageForm = connect(mapStateToForm)(injectIntl(ZoneEditAdminPageForm));
