import React from 'react';
import { connect } from 'react-redux';
import { change, formValueSelector, reduxForm } from 'redux-form';
import { injectIntl } from 'react-intl';
import PropTypes from 'prop-types';
import BasicOptions from 'components/modals/forms/zones/BasicOptions';
import AdvancedOptionsControl from 'components/modals/forms/zones/AdvancedOptionsControl';
import CustomZoneOptions from 'components/modals/forms/zones/CustomZoneOptions';
import SubmitButtons from 'components/modals/forms/SubmitButtons';
import ScheduleForm from 'components/modals/forms/zones/ScheduleForm';
import SeverityOptions from 'components/modals/forms/zones/SeverityOptions';
import TrackablesAndGroups from 'components/modals/forms/zones/TrackablesAndGroups';
import { COMPONENTS, DEFAULT_VALUES, ERROR_TEXT, NAME_OF_FORMS, ZONE_TYPE } from 'common';
import { getInclusionsExclusionsObjects, getCronFromToStringValues, isInfoZone, getZoneTypeName, getValuesFromZoneFormState, getLength } from 'utils';
import { getParsedGroups, getParsedTrackables, getParsedZones } from 'stores/getters';


export const validate = (values, props) => {
    const errors = {};
    const resultError = 'Invalid time value. Time should be in 24-hour format';
    if (values) {
        const { fromHour, fromMinute, toHour, toMinute, name } = values;
        const { zones, uid, sublocation_uid } = props;
        let zoneWithTheSameName;
        if (zones && uid && sublocation_uid && name) {
            zoneWithTheSameName = zones.find(zone => (zone && zone.uid !== uid && zone.sublocation_uid === sublocation_uid && zone.name === name));
        }

        const validateHour = hour => Number(hour) > 23 || Number(hour) < 0;
        const validateMinute = minute => Number(minute) > 59 || Number(minute) < 0;
        const isMidnight = (hour, minute) => Number(hour) === 24 && Number(minute) === 0;

        if (!isMidnight(fromHour, fromMinute)) {
            if (validateHour(fromHour)) {
                errors.fromHour = resultError;
            }
            if (validateMinute(fromMinute)) {
                errors.fromMinute = resultError;
            }
        }
        if (!isMidnight(toHour, toMinute)) {
            if (validateHour(toHour)) {
                errors.toHour = resultError;
            }
            if (validateMinute(toMinute)) {
                errors.toMinute = resultError;
            }
        }
        if (!name || name.length === 0) {
            errors.name = COMPONENTS.VALIDATION.REQUIRED;
        }

        if (zoneWithTheSameName) {
            errors.name = COMPONENTS.VALIDATION.NAME_EXIST;
        }
    }

    if (!values || errors.fromHour || errors.fromMinute || errors.toHour || errors.toMinute) {
        errors.activePeriod = { _error: resultError };
    }

    if (values.custom_info && getLength(values.custom_info) > DEFAULT_VALUES.CUSTOM_INFO_LIMIT) {
        errors.custom_info = ERROR_TEXT.CUSTOM_INFO_ERROR();
    }

    return errors;
};

const activityHandler = (props, e) => {
    const activeValue = e.target.value === 'true' ? false : e.target.value === 'false';
    props.dispatch(change(NAME_OF_FORMS.ZONE_EDIT_SITE_PAGE, 'active24', activeValue));
};

let ZoneEditSitePageForm = (props) => {
    const {
        close, submit, className, formState, initialValues, intl, zone_type_name, zonePrototype,
        trackables, groups, invalid, maxHeight, customInfoLength, zonesPrototypes, zones, zone_id,
    } = props;
    
    initialValues.default_allowed += '';
    
    const editZoneFormState = formState[NAME_OF_FORMS.ZONE_EDIT_SITE_PAGE] || {};
    const {
        activated,
        location_name,
        sublocation_name,
        areAdvancedOptionsShown,
        active24,
    } = getValuesFromZoneFormState(editZoneFormState);

    const isZoneTypeSelected = !!zone_type_name;
    const isKeepInType = zone_type_name === ZONE_TYPE.KEEP_IN.name;
    const isKeepOutType = zone_type_name === ZONE_TYPE.KEEP_OUT.name;
    const isKeepInOutType = (isKeepInType || isKeepOutType);
    const isInformationZone = isInfoZone({ zone_type_name });
    const zoneTypeName = getZoneTypeName(zonesPrototypes, zone_type_name, intl);

    return (
        <form className={className}>
            <div
                style={{ overflowY: 'auto', overflowX: 'hidden', maxHeight, paddingRight: '10px' }}
            >
                <BasicOptions
                    activityHandler={(event) => { activityHandler(props, event); }}
                    customInfoLength={customInfoLength}
                    location_name={location_name}
                    sublocation_name={sublocation_name}
                    zoneTypeName={zoneTypeName}
                    isZoneTypeSelected={isZoneTypeSelected}
                    activated={activated}
                    intl={intl}
                />
                <SeverityOptions
                    show={isKeepInOutType}
                    isZoneTypeSelected={isZoneTypeSelected}
                    isInformationZone={isInformationZone}
                    intl={intl}
                />
                <CustomZoneOptions
                    zone_id={zone_id}
                    zones={zones}
                    zone_type_name={zone_type_name}
                    zonePrototype={zonePrototype}
                    formState={initialValues}
                />
                <AdvancedOptionsControl intl={intl} />
                <TrackablesAndGroups
                    show={areAdvancedOptionsShown}
                    trackables={trackables}
                    groups={groups}
                    intl={intl}
                />
                <ScheduleForm
                    show={areAdvancedOptionsShown}
                    active24={active24}
                    activated={activated}
                />
            </div>
            <SubmitButtons
                className="Form-Buttons"
                submit={submit}
                invalid={invalid}
                close={close}
                saveDataId="zone-add-configuration-save"
                cancelDataId="zone-add-configuration-cancel"
            />
        </form>
    );
};

ZoneEditSitePageForm = reduxForm({
    form: NAME_OF_FORMS.ZONE_EDIT_SITE_PAGE,
    validate,
})(ZoneEditSitePageForm);

const mapStateToForm = (state) => {
    const selector = formValueSelector(NAME_OF_FORMS.ZONE_EDIT_SITE_PAGE);
    const custom_info = selector(state, 'custom_info');

    const modalParams = state.modals.zoneModals.zoneEditSitePage.params || {};
    const {
        toHour,
        toMinute,
        fromHour,
        fromMinute,
    } = getCronFromToStringValues({ from: modalParams.schedule_start_time, to: modalParams.schedule_stop_time });
    
    const {
        uid,
        zone_id,
        apply_to,
        location_uid,
        zone_type_name,
        sublocation_uid,
        zone_exceptions,
        zone_type_id,
        zone_exceptions_groups,
    } = modalParams;
    
    const trackables = getParsedTrackables(state);
    const groups = getParsedGroups(state);
    const trackables_and_groups = getInclusionsExclusionsObjects(zone_exceptions, zone_exceptions_groups, trackables, groups);
    const location = state.entities.locations.find(loc => loc.uid === location_uid);
    const sublocation = state.entities.sublocations.find(sl => sl.uid === sublocation_uid);
    const zonesPrototypes = state.entities.zonesPrototypes;
    const zonePrototype = zonesPrototypes.find(zp => zp.isZoneTypeIdEqualTo(zone_type_id));
    const advanced_options = false;
    
    const initialValues = {
        toHour,
        fromHour,
        toMinute,
        apply_to,
        fromMinute,
        ...modalParams,
        advanced_options,
        trackables_and_groups,
        location_name: location ? location.name : '',
        sublocation_name: sublocation ? sublocation.name : '',
    };
    
    return {
        uid,
        groups,
        zone_id,
        trackables,
        zone_type_id,
        zonePrototype,
        initialValues,
        sublocation_uid,
        zonesPrototypes,
        formState: state.form,
        appContext: state.appContext,
        customInfoLength: custom_info ? getLength(custom_info) : 0,
        zones: getParsedZones(state),
        zone_type_name: zonePrototype ? zonePrototype.zone_type.name : zone_type_name,
    };
};

ZoneEditSitePageForm.propTypes = {
    close: PropTypes.func,
    submit: PropTypes.func,
    invalid: PropTypes.bool,
    intl: PropTypes.shape({
        formatMessage: PropTypes.func,
    }),
    groups: PropTypes.arrayOf(PropTypes.object),
    className: PropTypes.string,
    formState: PropTypes.shape(),
    zonePrototype: PropTypes.shape(),
    zonesPrototypes: PropTypes.arrayOf(PropTypes.shape()),
    trackables: PropTypes.arrayOf(PropTypes.shape()),
    zones: PropTypes.arrayOf(PropTypes.shape()),
    initialValues: PropTypes.shape(),
    zone_type_name: PropTypes.string,
    maxHeight: PropTypes.string,
    customInfoLength: PropTypes.number,
    zone_id: PropTypes.number,
};

ZoneEditSitePageForm.defaultProps = {
    intl: {
        formatMessage: () => {
        },
    },
    groups: [],
    invalid: false,
    className: '',
    formState: {},
    zonePrototype: {},
    zonesPrototypes: [],
    trackables: [],
    zones: [],
    close: () => {
    },
    submit: () => {
    },
    initialValues: {
        user_params: {},
    },
    zone_type_name: '',
    maxHeight: '60vh',
    customInfoLength: 0,
    zone_id: 0,
};

export default ZoneEditSitePageForm = connect(mapStateToForm)(injectIntl(ZoneEditSitePageForm));
