import React from 'react';
import { connect } from 'react-redux';
import PropTypes from 'prop-types';
import { injectIntl } from 'react-intl';

import { BootstrapTable, TableHeaderColumn, SearchField } from 'react-bootstrap-table';
import { DEFAULT_VALUES, NAME_OF_GEAR_BUTTONS, TABLES, BOOTSTRAP_TABLE_SORT, COMPONENTS, NAMES_OF_TABS } from 'common';

import {
    openCompanyDetailsModal, openCompanyEditModalSystemScope,
    openCompanyDeleteModal, changeSelectedItems, selectItems, unselectItems, userLoginAsCompanyAdmin,
} from 'stores/actions';

class CompaniesTab extends React.Component {
    static propTypes = {
        dispatch: PropTypes.func,
        intl: PropTypes.shape({
            formatMessage: PropTypes.func,
        }),
        companies: PropTypes.arrayOf(PropTypes.shape()),
        selectedItems: PropTypes.arrayOf(PropTypes.shape()),
    };
    
    static defaultProps = {
        dispatch: () => {
        },
        intl: {
            formatMessage: () => {
            },
        },
        companies: [],
        selectedItems: [],
    };
    
    getActionsIcons = (cell, row) => (
        <section data-info="actions-section">
            <i
                onClick={this.showCompanyInfo.bind(this, row)}
                data-info="actions-section"
                data-id="companies-actions-info"
                data-cy="company-table-info-button"
                title={COMPONENTS.SHOW_DETAILS(this.props.intl)}
                className="fa fa-info-circle inner-table-action"
                aria-hidden="true"
            />
            <i
                onClick={this.openEditModal.bind(this, row)}
                data-info="actions-section"
                data-id="companies-actions-edit"
                data-cy="company-table-edit-button"
                title={COMPONENTS.EDIT(this.props.intl)}
                className="fa fa-pencil-alt inner-table-action"
                aria-hidden="true"
            />
            <i
                onClick={this.openDeleteModal.bind(this, row)}
                data-info="actions-section"
                data-id="companies-actions-remove"
                data-cy="company-table-remove-button"
                title={COMPONENTS.DELETE(this.props.intl)}
                className="fa fa-times inner-table-action"
                aria-hidden="true"
            />
        </section>
    );
    
    openMultiDeleteModal = () => {
        const params = [];
        this.props.selectedItems.forEach((item) => {
            const company = this.props.companies.find(x => x.uid === item.uid);
            
            if (company) {
                params.push({
                    name: company.name,
                    uid: company.uid,
                });
            }
        });
        
        if (params.length > 0) {
            this.props.dispatch(openCompanyDeleteModal(params));
        }
    };
    
    showCompanyInfo = row => this.props.dispatch(openCompanyDetailsModal(row));
    
    openEditModal = row => this.props.dispatch(openCompanyEditModalSystemScope(row));
    
    openDeleteModal = row => this.props.dispatch(openCompanyDeleteModal(row));
    
    createAddDeleteBtn = () => (
        <section className="Table">
            <i
                className="fas fa-trash-alt fa-lg table-action-icon"
                data-id="companies-trash-button"
                data-cy="company-table-batch-remove-button"
                onClick={this.openMultiDeleteModal}
            />
            <i
                className="fa fa-plus fa-lg table-action-icon"
                data-id="companies-plus-button"
                data-cy="company-table-add-button"
                onClick={this.openEditModal.bind(this, { tag_allocation: true })}
            />
        </section>
    );
    
    handleSelectRow = (row, isSelected, e) => {
        if (e.target.getAttribute('data-info') === 'actions-section') {
            return false;
        }
        if (e.target.getAttribute('data-info') === 'company-button') {
            this.props.dispatch(userLoginAsCompanyAdmin(row));
            return false;
        }
        this.props.dispatch(changeSelectedItems(row));
        return undefined;
    };
    
    handleSelectAll = (isSelected) => {
        if (isSelected) {
            this.props.dispatch(selectItems(this.props.companies));
        } else {
            this.props.dispatch(unselectItems(this.props.companies));
        }
    };
    
    trClassFormat = () => 'Table-Row-CursorPointer';
    
    createCustomSearchField = intl => (
        <SearchField
            data-id="companies-search"
            placeholder={TABLES.SEARCH(intl)}
        />
    );
    
    render() {
        const { companies, intl } = this.props;
        const options = {
            btnGroup: this.createAddDeleteBtn,
            noDataText: (TABLES.NO_DATA_TO_DISPLAY),
            searchField: this.createCustomSearchField.bind(this, intl),
            beforeShowError: () => true,
            sortName: 'name',
            sortOrder: BOOTSTRAP_TABLE_SORT.ASC,
        };
        const headerStyle = {
            marginBottom: '0',
        };
        
        const selectRow = {
            mode: 'checkbox',
            clickToSelect: true,
            bgColor: DEFAULT_VALUES.SELECTED_ROW_COLOR,
            onSelect: this.handleSelectRow,
            onSelectAll: this.handleSelectAll,
        };
        
        return (
            <div className="AppLayout-Admin-Tab col-md-12">
                <h3 className="text-center AdminTab-Title">{NAMES_OF_TABS.COMPANIES}</h3>
                <h4
                    className="text-center AdminTab-Subtitle"
                    data-id="companies-total-count"
                    data-cy="company-table-total-count"
                >
                    {COMPONENTS.FIELDS.COMMON.TOTAL_COUNT_FUNC(companies.length)}
                </h4>
                <hr />
                <BootstrapTable
                    data={companies}
                    trClassName={this.trClassFormat}
                    options={options}
                    tableBodyClass="Table-Body automation-companies-table"
                    selectRow={selectRow}
                    headerStyle={headerStyle}
                    bodyContainerClass="Admin-BootstrapTable"
                    hover
                    condensed
                    striped
                    search
                >
                    <TableHeaderColumn
                        dataField="uid"
                        searchable={false}
                        hidden
                        isKey
                    >
                        {COMPONENTS.FIELDS.COMMON.UID}
                    </TableHeaderColumn>
                    <TableHeaderColumn
                        dataField="name"
                        tdAttr={{ 'data-id': 'company-table-name', 'data-info': 'company-button' }}
                        columnClassName="Table-Button"
                        headerAlign="center"
                        width="150"
                    >
                        {NAME_OF_GEAR_BUTTONS.COMPANY}
                    </TableHeaderColumn>
                    <TableHeaderColumn
                        dataField="phone"
                        tdAttr={{ 'data-id': 'company-table-phone', 'data-info': 'company-button' }}
                        width="170"
                        headerAlign="center"
                    >
                        {COMPONENTS.FIELDS.USER.PHONE}
                    </TableHeaderColumn>
                    <TableHeaderColumn
                        dataField="website"
                        tdAttr={{ 'data-id': 'company-table-website', 'data-info': 'company-button' }}
                        width="230"
                        headerAlign="center"
                    >
                        {COMPONENTS.FIELDS.COMPANY.WEBSITE}
                    </TableHeaderColumn>
                    <TableHeaderColumn
                        dataField="actions"
                        width="85"
                        headerAlign="center"
                        dataAlign="center"
                        expandable={false}
                        dataFormat={this.getActionsIcons}
                    >
                        {COMPONENTS.FIELDS.COMMON.ACTIONS}
                    </TableHeaderColumn>
                </BootstrapTable>
            </div>
        );
    }
}

const mapStateToProps = (state) => {
    const companies = state.entities.companies.map(company => company.getFormattedCompany());
    
    return {
        companies,
        selectedItems: state.adminPage.selectedItems,
    };
};

export default connect(mapStateToProps)(injectIntl(CompaniesTab));
