import React from 'react';
import { connect } from 'react-redux';
import PropTypes from 'prop-types';
import { injectIntl } from 'react-intl';
import _isEqual from 'lodash/isEqual';

import { BootstrapTable, TableHeaderColumn, SearchField } from 'react-bootstrap-table';
import { DEFAULT_VALUES, TABLES, BOOTSTRAP_TABLE_SORT, COMPONENTS, NAMES_OF_TABS } from 'common';
import { convertNumbersToWeekDays } from 'utils/formatters';
import { formatTimeToDisplay, getTimeZoneOfProject } from 'utils/timeUtils';
import { getShifts, getActiveShiftBySiteUid } from 'utils/utils';
import { changeSelectedItems, selectItems, unselectItems } from 'stores/actions';
import { openShiftEditModal, openShiftDeleteModal, openShiftDetailsModal } from 'stores/actions/modalActions/shiftModalActions';

class ShiftsTab extends React.Component {
    static propTypes = {
        dispatch: PropTypes.func,
        intl: PropTypes.shape({
            formatMessage: PropTypes.func,
        }),
        shifts: PropTypes.arrayOf(PropTypes.shape()),
        selectedItems: PropTypes.arrayOf(PropTypes.shape()),
        selectedGroup: PropTypes.shape({
            uid: PropTypes.string,
        }),
        currentShift: PropTypes.shape({}),
        canEditShift: PropTypes.bool,
    };

    static defaultProps = {
        dispatch: () => {
        },
        intl: {
            formatMessage: () => {
            },
        },
        shifts: [],
        selectedItems: [],
        selectedGroup: {},
        currentShift: {},
        canEditShift: true,
    };

    shouldComponentUpdate(nextProps) {
        return !_isEqual(nextProps, this.props);
    }

    getActionsIcons = (cell, row) => (
        <section data-info="actions-section">
            <i
                onClick={this.openDetailsModal.bind(this, row)}
                data-info="actions-section"
                data-id="shifts-actions-details"
                data-cy="shift-table-info-button"
                title={COMPONENTS.SHOW_DETAILS(this.props.intl)}
                className="fa fa-info-circle inner-table-action"
                aria-hidden="true"
            />
            <i
                onClick={this.openEditModal.bind(this, row)}
                data-info="actions-section"
                data-id="shifts-actions-edit"
                data-cy="shift-table-edit-button"
                data-enable={this.props.canEditShift}
                title={this.props.canEditShift ? COMPONENTS.EDIT(this.props.intl) : ''}
                className="fa fa-pencil-alt inner-table-action"
                aria-hidden="true"
            />
            <i
                onClick={this.openDeleteModal.bind(this, row)}
                data-info="actions-section"
                data-id="shifts-actions-remove"
                data-cy="shift-table-remove-button"
                data-enable={this.props.canEditShift}
                title={this.props.canEditShift ? COMPONENTS.DELETE(this.props.intl) : ''}
                className="fa fa-times inner-table-action"
                aria-hidden="true"
            />
        </section>
    );

    openMultiDeleteModal = () => {
        this.props.dispatch(openShiftDeleteModal(this.props.selectedItems));
    };

    openCreateModal = () => this.props.dispatch(openShiftEditModal());

    openEditModal = (row) => {
        if (this.props.canEditShift) {
            this.props.dispatch(openShiftEditModal(row));
        }
    };

    openDetailsModal = (row) => this.props.dispatch(openShiftDetailsModal(row));

    openDeleteModal = (row) => {
        if (this.props.canEditShift) {
            this.props.dispatch(openShiftDeleteModal(row));
        }
    };

    createAddDeleteBtn = () => (
        <section className="Table">
            <i
                className="fas fa-trash-alt fa-lg table-action-icon"
                data-id="shift-trash-button"
                data-cy="shift-table-batch-remove-button"
                onClick={this.openMultiDeleteModal}
            />
            <i
                className="fa fa-plus fa-lg table-action-icon"
                data-id="shift-plus-button"
                data-cy="shift-table-add-button"
                onClick={this.openCreateModal}
            />
        </section>
    );

    handleSelectRow = (row, isSelected, e) => {
        if (e.target.getAttribute('data-info') === 'actions-section') {
            return false;
        }
        this.props.dispatch(changeSelectedItems(row));
        
        return true;
    };

    handleSelectAll = (isSelected) => {
        const { shifts } = this.props;

        this.props.dispatch((isSelected) ? selectItems(shifts) : unselectItems(shifts));
    };

    trClassFormat = (row) => {
        const { selectedGroup } = this.props;
        const rowClass = [];

        if (selectedGroup && selectedGroup.uid === row.uid) {
            rowClass.push('Table-Assigned-Row-Selected');
        }

        return rowClass.join(' ');
    };

    createCustomSearchField = (intl) => (
        <SearchField
            data-id="shift-search"
            data-cy="shift-table-search"
            placeholder={TABLES.SEARCH(intl)}
        />
    );

    timeFormatter = (cell) => formatTimeToDisplay(cell);

    weekdaysFormatter = (cell) => convertNumbersToWeekDays(cell, this.props.intl).join(', ');
    
    statusFormatter = (cell) => (cell ? COMPONENTS.FIELDS.COMMON.ACTIVE : COMPONENTS.FIELDS.COMMON.INACTIVE);

    shiftNameFormatter = (cell, shift) => {
        const { currentShift } = this.props;

        return (currentShift && currentShift.uid === shift.uid)
            ? (<span className="marked-shift-name">{cell}</span>)
            : cell;
    }

    render() {
        const { shifts, intl } = this.props;
        const options = {
            btnGroup: this.createAddDeleteBtn,
            noDataText: (TABLES.NO_DATA_TO_DISPLAY),
            searchField: this.createCustomSearchField.bind(this, intl),
            beforeShowError: () => true,
            defaultSortName: 'name',
            defaultSortOrder: BOOTSTRAP_TABLE_SORT.ASC,
        };
        const headerStyle = {
            marginBottom: '0',
        };

        const selectRow = {
            mode: 'checkbox',
            clickToSelect: true,
            bgColor: DEFAULT_VALUES.SELECTED_ROW_COLOR,
            onSelect: this.handleSelectRow,
            onSelectAll: this.handleSelectAll,
        };

        return (
            <div className="AppLayout-Admin-Tab col-md-12">
                <h3 className="text-center AdminTab-Title">{NAMES_OF_TABS.SHIFTS}</h3>
                <h4
                    className="text-center AdminTab-Subtitle"
                    data-id="shifts-total-count"
                    data-cy="shift-table-total-count"
                >
                    {COMPONENTS.FIELDS.COMMON.TOTAL_COUNT_FUNC(shifts.length)}
                </h4>
                <hr />
                <BootstrapTable
                    data={shifts}
                    trClassName={this.trClassFormat}
                    options={options}
                    tableBodyClass="Table-Body automation-shifts-table"
                    selectRow={selectRow}
                    headerStyle={headerStyle}
                    bodyContainerClass="Admin-BootstrapTable"
                    hover
                    condensed
                    striped
                    search
                >
                    <TableHeaderColumn
                        dataField="uid"
                        searchable={false}
                        hidden
                        isKey
                    />
                    <TableHeaderColumn
                        dataField="name"
                        tdAttr={{ 'data-id': 'shift-name' }}
                        dataFormat={this.shiftNameFormatter}
                        headerAlign="center"
                        width="150"
                        dataSort
                    >
                        {COMPONENTS.FIELDS.COMMON.NAME}
                    </TableHeaderColumn>
                    <TableHeaderColumn
                        dataField="location_name"
                        tdAttr={{ 'data-id': 'shift-site-name' }}
                        headerAlign="center"
                        width="150"
                        dataSort
                    >
                        {COMPONENTS.FIELDS.SITE.SITE}
                    </TableHeaderColumn>
                    <TableHeaderColumn
                        dataField="start_time"
                        dataFormat={this.timeFormatter}
                        tdAttr={{ 'data-id': 'shift-start-time' }}
                        headerAlign="center"
                        dataAlign="center"
                        width="50"
                        dataSort
                    >
                        {COMPONENTS.FIELDS.SHIFT.START}
                    </TableHeaderColumn>
                    <TableHeaderColumn
                        dataField="end_time"
                        dataFormat={this.timeFormatter}
                        tdAttr={{ 'data-id': 'shift-end-time' }}
                        headerAlign="center"
                        dataAlign="center"
                        width="50"
                        dataSort
                    >
                        {COMPONENTS.FIELDS.SHIFT.END}
                    </TableHeaderColumn>
                    <TableHeaderColumn
                        dataField="weekdays"
                        dataFormat={this.weekdaysFormatter}
                        tdAttr={{ 'data-id': 'shift-weekdays' }}
                        headerAlign="center"
                        dataAlign="center"
                        width="80"
                    >
                        <span style={{ textTransform: 'none' }}>{COMPONENTS.FIELDS.SHIFT.WEEKDAYS}</span>
                    </TableHeaderColumn>
                    <TableHeaderColumn
                        dataField="active"
                        dataFormat={this.statusFormatter}
                        tdAttr={{ 'data-id': 'status' }}
                        headerAlign="center"
                        dataAlign="center"
                        width="80"
                    >
                        <span>{COMPONENTS.FIELDS.COMMON.STATUS}</span>
                    </TableHeaderColumn>
                    <TableHeaderColumn
                        dataField="actions"
                        width="40"
                        headerAlign="center"
                        dataAlign="center"
                        expandable={false}
                        dataFormat={this.getActionsIcons}
                    >
                        {COMPONENTS.FIELDS.COMMON.ACTIONS}
                    </TableHeaderColumn>
                </BootstrapTable>
            </div>
        );
    }
}

const mapStateToProps = (state) => {
    const { locations } = state.entities;
    const { selectedItems, canEditShift } = state.adminPage;
    const timeZone = getTimeZoneOfProject(state.appContext.project);
    const selectedSite = state.siteView.selectedLocation;
    const currentShift = selectedSite && getActiveShiftBySiteUid(selectedSite.uid, locations, timeZone);

    return {
        shifts: getShifts(locations),
        selectedItems,
        currentShift,
        canEditShift,
    };
};

export default connect(mapStateToProps)(injectIntl(ShiftsTab));
