import React from 'react';
import PropTypes from 'prop-types';
import { connect } from 'react-redux';
import { createSelector } from 'reselect';
import { injectIntl } from 'react-intl';
import { BootstrapTable, TableHeaderColumn, SearchField } from 'react-bootstrap-table';

import {
    NODE_TYPE,
    DEFAULT_VALUES,
    NODES_TABLE_COMPANY_SCOPE,
    TABLES,
    ADMIN_TABS_ROUTES,
    BOOTSTRAP_TABLE_SORT,
    COMPONENTS,
} from '../../../common';
import {
    openReleaseAssignModal, changeSelectedItems, openAddNodeModal, openImportNodeModal, openDetailsNodeModal, openDeleteNodeModal,
    selectItems, unselectItems, userLoginAsProjectSystemAdmin, changeLastAdminPath,
} from '../../../stores/actions';
import { getNodeSwVersion } from '../../../utils';
import { Node } from '../../../stores/classes';

class NodesTableCompanyScope extends React.Component {
    static propTypes = {
        nodes: PropTypes.arrayOf(PropTypes.shape()),
        projects: PropTypes.arrayOf(PropTypes.shape()),
        selectedItems: PropTypes.arrayOf(PropTypes.shape()),
        dispatch: PropTypes.func,
        intl: PropTypes.shape({
            formatMessage: PropTypes.func,
        }),
    };
    
    static defaultProps = {
        nodes: [],
        projects: [],
        selectedItems: [],
        dispatch: () => {
        },
        intl: {
            formatMessage: () => {
            },
        },
    };
    
    showNodeInfo = (row) => {
        this.props.dispatch(openDetailsNodeModal(row));
    };

    openImportModal = () => {
        this.props.dispatch(openImportNodeModal());
    };

    onRowDoubleClick = (row) => {
        const project = {
            uid: row.project_uid,
            name: row.project_name,
            user_params: row.project_user_params,
        };
        if (row.project_uid) {
            this.props.dispatch(userLoginAsProjectSystemAdmin(project));
            this.props.dispatch(changeLastAdminPath(ADMIN_TABS_ROUTES.NODES));
        }
    };

    getActionsIcons = (cell, row) => (
        <section>
            <i
                data-info="actions-section"
                data-id="tags-actions-info"
                data-cy="tag-table-info-button"
                onClick={this.showNodeInfo.bind(this, row)}
                title={COMPONENTS.SHOW_DETAILS(this.props.intl)}
                className="fa fa-info-circle inner-table-action"
                aria-hidden="true"
            />
            <i
                data-info="actions-section"
                data-id="tags-actions-remove"
                data-cy="tag-table-remove-button"
                onClick={this.openDeleteModal.bind(this, row)}
                title={COMPONENTS.DELETE(this.props.intl)}
                className="fa fa-times inner-table-action"
                aria-hidden="true"
            />
        </section>
    );

    openAddModal = () => {
        this.props.dispatch(openAddNodeModal());
    };
    
    openDeleteModal = (row) => {
        this.props.dispatch(openDeleteNodeModal(row));
    };
    
    openMultiDeleteModal = () => {
        const params = [];
        this.props.selectedItems.forEach((item) => {
            const node = this.props.nodes.find(x => (x.uid === item.uid));
            
            if (node) {
                params.push({
                    mac_address: node.mac_address,
                    uid: node.uid,
                });
            }
        });
        
        if (params.length > 0) {
            this.props.dispatch(openDeleteNodeModal(params));
        }
    };

    ownershipButton = (cell, row) => (
        <section>
            <button
                data-info="actions-section"
                onClick={this.changeOwnership.bind(this, row)}
                title={NODES_TABLE_COMPANY_SCOPE.OWNERSHIP(this.props.intl)}
            >
                {row.project_uid && row.company_uid ? COMPONENTS.BUTTONS.RELEASE : COMPONENTS.BUTTONS.ASSIGN}
            </button>
        </section>
    );
    
    changeOwnership = (row) => {
        this.props.dispatch(openReleaseAssignModal(row));
    };
    
    createAddDeleteBtn = () => (
        <section className="Table">
            <i
                className="fas fa-trash-alt fa-lg table-action-icon"
                data-id="tags-trash-button"
                data-cy="tag-table-remove-button"
                onClick={this.openMultiDeleteModal}
            />
            <i
                className="fas fa-file-upload fa-lg table-action-icon"
                data-id="tags-import-button"
                data-cy="tag-table-import-button"
                onClick={this.openImportModal}
            />
            <i
                className="fa fa-plus fa-lg table-action-icon"
                data-id="tags-plus-button"
                data-cy="tag-table-add-button"
                onClick={this.openAddModal}
            />
        </section>
    );
    
    handleSelectRow = (row, isSelected, e) => {
        if (e.target.getAttribute('data-info') === 'actions-section') {
            return false;
        }
        this.props.dispatch(changeSelectedItems(row));
        if (!isSelected) {
            this.forceUpdate();
        }
        return undefined;
    };
    
    handleSelectAll = (isSelected, nodes) => {
        if (isSelected) {
            this.props.dispatch(selectItems(nodes));
        } else {
            this.props.dispatch(unselectItems(nodes));
            this.forceUpdate();
        }
    };
    
    
    toolTip = cell => <div title={NODES_TABLE_COMPANY_SCOPE.DOUBLE_CLICK(this.props.intl)}>{cell}</div>
    
    trClassFormat = (row) => {
        if (!row.company_uid) {
            return 'Nodes-Unassigned';
        }
        const rowClass = 'Nodes-Table-';
        const projectUids = this.props.projects.map(project => project.uid);
        const colorNumber = projectUids.indexOf(row.project_uid) + 1;
        return row.project_uid ? rowClass + colorNumber : 'Nodes-Unassigned';
    };
    
    createCustomSearchField = intl => (
        <SearchField
            data-id="nodes-copany-search"
            data-cy="company-tag-table-search"
            placeholder={TABLES.SEARCH(intl)}
        />
    );
    
    render() {
        const options = {
            btnGroup: this.createAddDeleteBtn,
            noDataText: (TABLES.NO_DATA_TO_DISPLAY),
            sizePerPageList: [10],
            paginationSize: 3,
            onRowDoubleClick: this.onRowDoubleClick,
            searchField: this.createCustomSearchField.bind(this, this.props.intl),
            beforeShowError: () => true,
            defaultSortName: 'mac_address',
            defaultSortOrder: BOOTSTRAP_TABLE_SORT.ASC,
        };
        const headerStyle = {
            marginBottom: '0',
        };
        
        const selectRow = {
            mode: 'checkbox',
            clickToSelect: true,
            bgColor: DEFAULT_VALUES.SELECTED_ROW_COLOR,
            onSelect: this.handleSelectRow,
            onSelectAll: this.handleSelectAll,
        };
        
        return (
            <div className="AppLayout-Admin-Tab col-md-12">
                <h3 className="text-center AdminTab-Title"> {NODES_TABLE_COMPANY_SCOPE.TITLE} </h3>
                <h4
                    className="text-center AdminTab-Subtitle"
                    data-id="tags-total-count"
                >
                    {COMPONENTS.FIELDS.COMMON.TOTAL_COUNT_FUNC(this.props.nodes.length)}
                </h4>
                <hr />
                <BootstrapTable
                    ref="nodesTable"
                    trClassName={this.trClassFormat}
                    tableBodyClass="automation-nodes-company-table"
                    data={this.props.nodes}
                    options={options}
                    selectRow={selectRow}
                    headerStyle={headerStyle}
                    bodyContainerClass="Admin-BootstrapTable"
                    pagination
                    condensed
                    search
                    hover
                >
                    <TableHeaderColumn
                        dataField="uid"
                        searchable={false}
                        hidden
                        isKey
                    >
                        {COMPONENTS.FIELDS.COMMON.UID}
                    </TableHeaderColumn>
                    <TableHeaderColumn
                        dataField="mac_address"
                        tdAttr={{ 'data-id': 'tags-table-mac_address' }}
                        width="180"
                        headerAlign="center"
                        dataSort
                    >
                        {COMPONENTS.FIELDS.NODE.MAC_ADDRESS}
                    </TableHeaderColumn>
                    <TableHeaderColumn
                        dataField="project_name"
                        tdAttr={{ 'data-id': 'tags-table-project' }}
                        dataSort
                        dataFormat={this.toolTip}
                        dataAlign="center"
                        width="170"
                        headerAlign="center"
                    >
                        {COMPONENTS.FIELDS.COMMON.PROJECT}
                    </TableHeaderColumn>
                    <TableHeaderColumn
                        dataField="firmware_version"
                        tdAttr={{ 'data-id': 'tags-table-fw' }}
                        dataAlign="center"
                        headerAlign="center"
                        dataSort
                    >
                        {COMPONENTS.FIELDS.NODE.FIRMWARE_VERSION}
                    </TableHeaderColumn>
                    <TableHeaderColumn
                        dataField="ownership"
                        tdAttr={{ 'data-id': 'tags-table-ownership' }}
                        dataAlign="center"
                        dataFormat={this.ownershipButton}
                        width="100"
                        headerAlign="center"
                    >
                        {COMPONENTS.FIELDS.NODE.PROJECT_OWNERSHIP}
                    </TableHeaderColumn>
                    <TableHeaderColumn
                        dataField="actions"
                        width="100"
                        headerAlign="center"
                        dataAlign="center"
                        expandable={false}
                        dataFormat={this.getActionsIcons}
                    >
                        {COMPONENTS.FIELDS.COMMON.ACTIONS}
                    </TableHeaderColumn>
                </BootstrapTable>
            </div>
        );
    }
}

const formatNodes = (nodes = [], projects = [], currentCompany) => {
    const formattedNodes = nodes.map((_node) => {
        const node = _node.getProperties();
        node.mac_address = node.mac_address.replace(/-|\s/g, '').toUpperCase();
        node.firmware_version = getNodeSwVersion(node);
        
        if (!node.company_uid) {
            node.project_uid = null;
        }
        
        projects.forEach((project) => {
            if (project.uid === node.project_uid && node.company_uid) {
                node.project_name = project.name;
                node.project_user_params = project.user_params;
            }
        });
        return node;
    });
    
    return formattedNodes.filter(node => ((!node.project_uid && node.company_uid === currentCompany.uid) || node.node_type !== NODE_TYPE.WIN_ANCHOR));
};

const getProjects = state => state.entities.projects;
const getCompany = state => state.appContext.company;

const getFilteredNodes = createSelector(
    [
        Node.getAllNodesArray, getProjects, getCompany,
    ],
    (nodes, projects, company) => (formatNodes(nodes, projects, company)),
);

const mapStateToProps = state => (
    {
        nodes: getFilteredNodes(state),
        projects: state.entities.projects ? state.entities.projects : DEFAULT_VALUES.empty_array,
        selectedItems: state.adminPage.selectedItems,
    }
);

export default connect(mapStateToProps)(injectIntl(NodesTableCompanyScope));
