import React from 'react';
import PropTypes from 'prop-types';
import { injectIntl } from 'react-intl';
import { Column, Table } from 'react-virtualized';
import 'react-virtualized/styles.css';

import withSortSelect from 'components/HOC/withSortSelect';
import { COMPONENTS } from 'common/ui-labels';
import { isNodeUnresponsive } from 'utils/utils';
import { batteryStatus, virtualizedHeaderRenderer } from 'utils/htmlUtils';
import { openDetailsNodeModal, openReleaseAssignModal, openDeleteNodeModal } from 'stores/actions';
import TableButtons from './TableButtons';


class VirtualTable extends React.Component {
    static propTypes = {
        // Props from HOC
        sort: PropTypes.func,
        sortBy: PropTypes.string,
        trStyleFormat: PropTypes.func,
        setFilterItems: PropTypes.func,
        handleSelectRow: PropTypes.func,
        sortDisplayItems: PropTypes.func,
        sortDirection: PropTypes.string,
        filteredItems: PropTypes.arrayOf(PropTypes.shape()),
    
        // Props from parent
        allItems: PropTypes.shape(),
        totalCount: PropTypes.number,
        tableSize: PropTypes.shape(),
        selectedItems: PropTypes.arrayOf(PropTypes.shape()),
        dispatch: PropTypes.func,
        trClassFormat: PropTypes.func,
        checkboxButton: PropTypes.func,
        headerCheckbox: PropTypes.func,
        intl: PropTypes.shape({
            formatMessage: PropTypes.func,
        }),
    };
    static defaultProps = {
        // Props from HOC
        sort: () => { },
        sortBy: '',
        trStyleFormat: () => { },
        setFilterItems: () => { },
        handleSelectRow: () => { },
        sortDisplayItems: () => { },
        sortDirection: '',
        filteredItems: [],
    
        // Props from parent
        allItems: new Map(),
        totalCount: 0,
        tableSize: {},
        selectedItems: [],
        dispatch: () => { },
        trClassFormat: () => { },
        checkboxButton: () => { },
        headerCheckbox: () => { },
        intl: {
            formatMessage: () => { },
        },
    };
    
    constructor(props) {
        super(props);
        this.totalCount = props.totalCount;
        this.dispatch = props.dispatch;
        
        // HOC Methods
        this.trClassFormat = props.trClassFormat.bind(this);
        this.trStyleFormat = props.trStyleFormat.bind(this);
        this.handleSelectRow = props.handleSelectRow.bind(this);
        this.checkboxButton = props.checkboxButton.bind(this);
        this.headerCheckbox = props.headerCheckbox.bind(this);
        this.sort = props.sort.bind(this);
        this.sortDisplayItems = props.sortDisplayItems.bind(this);
        this.setFilterItems = props.setFilterItems.bind(this);
    }
    
    getActionsIcons = row => (
        <section data-info="actions-section">
            <i
                data-info="actions-section"
                data-id="tags-actions-info-button"
                onClick={this.showNodeInfo.bind(this, row)}
                title={COMPONENTS.SHOW_DETAILS(this.props.intl)}
                className="fa fa-info-circle inner-table-action"
                aria-hidden="true"
            />
            <i
                data-info="actions-section"
                data-id="tags-actions-remove-button"
                onClick={this.openDeleteModal.bind(this, row)}
                title={COMPONENTS.DELETE(this.props.intl)}
                className="fa fa-times inner-table-action"
                aria-hidden="true"
            />
        </section>
    );
    
    openDeleteModal = (row) => {
        this.dispatch(openDeleteNodeModal(row));
    };
    
    changeOwnership = (row, event) => {
        this.dispatch(openReleaseAssignModal(row));
        event.stopPropagation();
    };
    
    showNodeInfo = (row) => {
        this.dispatch(openDetailsNodeModal(row));
    };
    
    ownershipButton = (row) => {
        const claim = COMPONENTS.BUTTONS.CLAIM;
        const release = COMPONENTS.BUTTONS.RELEASE;
        return (
            <section>
                <button
                    type="button"
                    data-cy="tag-table-claim-release-button"
                    data-info="actions-section"
                    onClick={(event) => { this.changeOwnership(row, event); }}
                    title="ownership"
                >
                    {row.project_uid ? release : claim}
                </button>
            </section>
        );
    };
    
    sinceFormatter = (cell, row) => {
        const statusStyle = {
            color: isNodeUnresponsive(row) || cell.text === 'Never' ? 'red' : 'black',
        };
        
        return (
            <div style={statusStyle}>{cell.text}</div>
        );
    };
    
    render() {
        const {
            sortBy, sortDirection, tableSize, allItems, selectedItems, filteredItems, intl,
        } = this.props;
        const allItemsArray = [...allItems.values()];
        const displayItems = this.sortDisplayItems(filteredItems);
        const displayCount = displayItems.length;

        const allColumnsWidthExceptTrackableName = tableSize.width - 1100;

        return (
            <section style={{ width: tableSize.width }}>
                <TableButtons
                    items={allItemsArray}
                    width={tableSize.width}
                    setFilterItems={this.setFilterItems}
                    selectedItems={selectedItems}
                    searchFields={[
                        'mac_address',
                        'trackable_name',
                        'firmware_version',
                    ]}
                />
                <Table
                    sortBy={sortBy}
                    sort={this.sort}
                    sortDirection={sortDirection}
                    className="Virtual-Table-Big"
                    width={tableSize.width} // required
                    height={tableSize.height} // required
                    gridClassName="Virtual-Table-Grid"
                    rowHeight={tableSize.rowHeight} // required
                    headerHeight={tableSize.headerHeight} // required
                    headerClassName="Virtual-Table-Header react-bs-container-header"
                    rowClassName={({ index }) => this.trClassFormat(displayItems, { index })}
                    rowStyle={({ index }) => this.trStyleFormat(displayItems, { index })}
                    onRowClick={({ event, index, rowData }) => this.handleSelectRow(displayCount, { event, index, rowData })}
                    rowCount={displayCount} // required
                    rowGetter={({ index }) => displayItems[index]} // required
                >
                    <Column
                        label={<span title="">{this.headerCheckbox(displayItems)}</span>}
                        dataKey="checkbox"
                        cellRenderer={({ rowData }) => this.checkboxButton(rowData, displayCount)}
                        width={100}
                        disableSort
                        className="Virtual-Table-Column-Checkbox first-column"
                    />
                    <Column
                        label={<span title="">{COMPONENTS.FIELDS.NODE.MAC_ADDRESS}</span>}
                        headerRenderer={virtualizedHeaderRenderer}
                        dataKey="mac_address"
                        width={340}
                        className="Virtual-Table-Column-Left text-small"
                    />
                    <Column
                        label={<span title="">{COMPONENTS.FIELDS.NODE.TRACKABLE}</span>}
                        headerRenderer={virtualizedHeaderRenderer}
                        dataKey="trackable_name"
                        width={allColumnsWidthExceptTrackableName >= 360 ? allColumnsWidthExceptTrackableName : 360}
                        className="Virtual-Table-Column-Left"
                    />
                    <Column
                        width={560}
                        label={<span title="">{COMPONENTS.FIELDS.NODE.LAST_REPORTED}</span>}
                        headerRenderer={virtualizedHeaderRenderer}
                        cellRenderer={({ cellData }) => cellData}
                        cellDataGetter={({ rowData }) => this.sinceFormatter(rowData.since, rowData)}
                        dataKey="since"
                        className="Virtual-Table-Column-Center"
                    />
                    <Column
                        width={300}
                        label={<span title="">{COMPONENTS.FIELDS.NODE.BATTERY}</span>}
                        headerRenderer={virtualizedHeaderRenderer}
                        cellRenderer={({ cellData }) => cellData}
                        cellDataGetter={({ rowData }) => batteryStatus(rowData.battery, intl)}
                        dataKey="battery"
                        className="Virtual-Table-Column-Center"
                    />
                    <Column
                        width={300}
                        label={<span title="">{COMPONENTS.FIELDS.NODE.FIRMWARE_VERSION}</span>}
                        headerRenderer={virtualizedHeaderRenderer}
                        dataKey="firmware_version"
                        className="Virtual-Table-Column-Left"
                    />
                    <Column
                        width={300}
                        label={<span title="">{COMPONENTS.FIELDS.NODE.PROJECT_OWNERSHIP}</span>}
                        headerRenderer={virtualizedHeaderRenderer}
                        cellRenderer={({ cellData }) => cellData}
                        cellDataGetter={({ rowData }) => this.ownershipButton(rowData)}
                        dataKey="ownership"
                        className="Virtual-Table-Column-Center"
                    />
                    <Column
                        width={200}
                        label={<span title="">{COMPONENTS.FIELDS.COMMON.ACTIONS}</span>}
                        cellRenderer={({ cellData }) => cellData}
                        cellDataGetter={({ rowData }) => this.getActionsIcons(rowData)}
                        dataKey="actions"
                        disableSort
                        className="Virtual-Table-Column-Center"
                    />
                </Table>
            </section>
        );
    }
}

export default injectIntl(withSortSelect(VirtualTable));
