import React, { Component, Fragment } from 'react';
import PropTypes from 'prop-types';
import { connect } from 'react-redux';
import { injectIntl } from 'react-intl';
import { createSelector } from 'reselect';
import { SortDirection, AutoSizer } from 'react-virtualized';

import { Node as NodeSelector } from 'stores/classes/Selectors';
import { NODE_TYPE, DEFAULT_VALUES } from 'common/constants';
import { COMPONENTS } from 'common/ui-labels';
import { durationFormatting } from 'utils/formatters';
import { getTimeZoneOffsetByProject } from 'utils/timeUtils';
import TableTitle from './TableTitle';
import VirtualTable from './VirtualTable';


class NodesTableProjectScope extends Component {
    static propTypes = {
        dispatch: PropTypes.func,
        nodes: PropTypes.shape(),
        selectedItems: PropTypes.arrayOf(PropTypes.shape()),
    };

    static defaultProps = {
        dispatch: () => {
        },
        nodes: [],
        selectedItems: [],
    };

    constructor(props) {
        super(props);

        this.state = {
            containerWidth: 0,
            containerHeight: 0,
        };
    }

    render() {
        const { nodes, dispatch, selectedItems } = this.props;
        const { containerWidth, containerHeight } = this.state;
        const nodesCount = nodes.size;

        return (
            <div className="AppLayout-Admin-Tab tab-admin-tag">
                <AutoSizer
                    onResize={({ height, width }) => {
                        this.setState({
                            containerWidth: width,
                            containerHeight: height,
                        });
                    }}
                >
                    {({ height, width }) => (
                        <Fragment>
                            <TableTitle
                                totalCount={nodesCount}
                                width={containerWidth}
                            />
                            <VirtualTable
                                defaultSortName="since"
                                defaultSortOrder={SortDirection.ASC}
                                tableSize={{
                                    width: containerWidth,
                                    height: containerHeight,
                                    headerHeight: 40,
                                    rowHeight: 40,
                                }}
                                allItems={nodes}
                                totalCount={nodesCount}
                                selectedItems={selectedItems}
                                dispatch={dispatch}
                            />
                        </Fragment>
                    )}
                </AutoSizer>
            </div>
        );
    }
}

const neverReported = (last_heard) => {
    const last_heard_date = new Date(last_heard);
    return last_heard_date.getFullYear() === DEFAULT_VALUES.ZERO_TIME.FULL_YEAR || Number.isNaN(last_heard_date.getFullYear());
};

const nodeSince = (row, now, intl) => {
    const storeNode = NodeSelector.getNodesFromStoreByMac(row.mac_address);
    const lastHeard = storeNode && storeNode.last_heard;
    
    if (!lastHeard || (lastHeard && neverReported(lastHeard))) {
        return { text: COMPONENTS.FIELDS.COMMON.NEVER_FUNC(intl), duration: now };
    }
    
    const last_heard = new Date(lastHeard).getTime();
    const duration = now - last_heard;
    return durationFormatting(duration, intl);
};

const formatNodes = (nodes = [], associationsMap, trackablesMap, currentProject, currentCompany, intl) => {
    const currentProjectUid = currentProject.uid;
    const currentCompanyUid = currentCompany.uid;
    const isInCurrentProject = node => node.project_uid === currentProjectUid;
    const hasNoProjectInCurrentCompany = node => !node.project_uid && node.company_uid === currentCompanyUid;
    const isNotWinAnchor = node_type => node_type !== NODE_TYPE.WIN_ANCHOR;
    const now = new Date().getTime();
    const nodesMap = new Map();

    nodes.forEach((node) => {
        if ((isInCurrentProject(node) || hasNoProjectInCurrentCompany(node)) && isNotWinAnchor(node.node_type)) {
            const pushedNode = node.formattedNode(currentProject, associationsMap, trackablesMap);
            pushedNode.since = nodeSince(pushedNode, now, intl);
            nodesMap.set(pushedNode.uid, pushedNode);
        }
    });
    return nodesMap;
};

const getProject = state => state.appContext.project;
const getNodes = state => Object.values(state.entities.nodes.byMac);
const getAssociations = state => state.entities.associations;
const getTrackablesMap = (state) => {
    const formattedTrackableMap = {};
    const trackbalesMap = state.entities.trackables.byUid;
    for (const uid in trackbalesMap) {
        formattedTrackableMap[uid] = trackbalesMap[uid].formattedTrackablesUidName();
    }
    return formattedTrackableMap;
};

const getCompany = state => state.appContext.company;
const getIntl = (state, intl) => intl;

export const getFilteredNodes = createSelector(
    [
        getNodes,
        getAssociations,
        getTrackablesMap,
        getProject,
        getCompany,
        getIntl,
    ],
    (nodes, associationsMap, trackablesMap, project, company, intl) => (formatNodes(nodes, associationsMap, trackablesMap, project, company, intl)),
);

const mapStateToProps = (state, { intl }) => {
    const { project } = state.appContext;
    const timeZoneOffset = getTimeZoneOffsetByProject(project);
    return {
        timeZoneOffset,
        nodes: getFilteredNodes(state, intl),
        selectedItems: state.adminPage.selectedItems,
        associations: getAssociations(state),
    };
};

export default injectIntl(connect(mapStateToProps)(NodesTableProjectScope));
