import React from 'react';
import PropTypes from 'prop-types';
import _Get from 'lodash/get';
import { injectIntl } from 'react-intl';
import { Column, Table } from 'react-virtualized';
import 'react-virtualized/styles.css';

import withSortSelect from 'components/HOC/withSortSelect';
import { COMPONENTS } from 'common/ui-labels';
import { CUSTOM_LOCAL_STORE_KEYS } from 'common/constants';
import { batteryStatus, virtualizedHeaderRenderer } from 'utils/htmlUtils';
import { getLocalStorageValue } from 'utils/browserStorage';
import { stringToBoolean } from 'utils/utils';
import { openTagAssignModal, openTagUnAssignModal } from 'stores/actions';
import TableButtons from './TableButtons';

class VirtualTable extends React.Component {
    static propTypes = {
        // Props from HOC
        sort: PropTypes.func,
        sortBy: PropTypes.string,
        trStyleFormat: PropTypes.func,
        setFilterItems: PropTypes.func,
        handleSelectRow: PropTypes.func,
        sortDisplayItems: PropTypes.func,
        sortDirection: PropTypes.string,
        filteredItems: PropTypes.arrayOf(PropTypes.shape()),

        // Props from parent
        allItems: PropTypes.shape(),
        totalCount: PropTypes.number,
        tableSize: PropTypes.shape(),
        dispatch: PropTypes.func,
        intl: PropTypes.shape({
            formatMessage: PropTypes.func,
        }),
    };
    static defaultProps = {
        // Props from HOC
        sort: () => { },
        sortBy: '',
        trStyleFormat: () => { },
        setFilterItems: () => { },
        handleSelectRow: () => { },
        sortDisplayItems: () => { },
        sortDirection: '',
        filteredItems: [],
    
        // Props from parent
        allItems: new Map(),
        totalCount: 0,
        tableSize: {},
        dispatch: () => { },
        intl: {
            formatMessage: () => { },
        },
    };
    
    constructor(props) {
        super(props);
        this.totalCount = props.totalCount;
        this.dispatch = props.dispatch;
        
        // HOC Methods
        this.trStyleFormat = props.trStyleFormat.bind(this);
        this.handleSelectRow = props.handleSelectRow.bind(this);
        this.sort = props.sort.bind(this);
        this.sortDisplayItems = props.sortDisplayItems.bind(this);
        this.setFilterItems = props.setFilterItems.bind(this);
    }
    
    assignTag = (node) => {
        this.props.dispatch(openTagAssignModal(node));
    };
    
    unAssignTag = (node) => {
        this.props.dispatch(openTagUnAssignModal(node));
    };
    
    assignNodeFormatter = (trackable, row) => {
        if (trackable) {
            return (
                <div>
                    <button
                        data-id="trackable-nodes-table-unassign-button"
                        data-info="actions-section"
                        className="RPP-Assign-buttons"
                        onClick={this.unAssignTag.bind(this, row)}
                    >
                        {COMPONENTS.BUTTONS.UNASSIGN}
                    </button>
                </div>);
        }
        return (
            <div>
                <button
                    data-id="trackable-nodes-table-assign-button"
                    data-info="actions-section"
                    className="RPP-Assign-buttons"
                    onClick={this.assignTag.bind(this, row)}
                >
                    {COMPONENTS.BUTTONS.ASSIGN}
                </button>
            </div>);
    };
    
    trClassFormat = (items, { index }) => {
        const currentItem = items[index];
        if (index >= 0 && currentItem) {
            if (!currentItem.trackable) {
                return 'Table-Not-Assign-Row';
            }
            return 'Table-Assigned-Row';
        }
        return 'Virtual-Table-Header-Row';
    };
    
    render() {
        const {
            sortBy, sortDirection, tableSize, allItems, filteredItems, intl,
        } = this.props;
        const allItemsArray = [...allItems.values()];
        const displayItems = this.sortDisplayItems(filteredItems);
        const displayCount = displayItems.length;
        const showVersionField = stringToBoolean(getLocalStorageValue(CUSTOM_LOCAL_STORE_KEYS.SHOW_EXTENDS));

        return (
            <section style={{ width: tableSize.width }}>
                <TableButtons
                    items={allItemsArray}
                    width={tableSize.width}
                    setFilterItems={this.setFilterItems}
                    searchFields={['mac_address', 'trackableName']}
                />
                <Table
                    sortBy={sortBy}
                    sort={this.sort}
                    sortDirection={sortDirection}
                    className="Virtual-Table-Small"
                    width={tableSize.width} // required
                    height={tableSize.height} // required
                    gridClassName="Virtual-Table-Grid"
                    rowHeight={tableSize.rowHeight} // required
                    headerHeight={tableSize.headerHeight} // required
                    headerClassName="Virtual-Table-Header react-bs-container-header"
                    rowClassName={({ index }) => this.trClassFormat(displayItems, { index })}
                    rowStyle={({ index }) => this.trStyleFormat(displayItems, { index })}
                    onRowClick={({ event, index, rowData }) => this.handleSelectRow(displayCount, { event, index, rowData })}
                    rowCount={displayCount} // required
                    rowGetter={({ index }) => displayItems[index]} // required
                >
                    <Column
                        label={<span title="">{COMPONENTS.FIELDS.NODE.MAC_ADDRESS}</span>}
                        headerRenderer={virtualizedHeaderRenderer}
                        dataKey="mac_address"
                        id="trackable-nodes-table-mac"
                        width={300}
                        className="Virtual-Table-Column-Center first-column"
                    />
                    {showVersionField && <Column
                        label={<span title="">CBID</span>}
                        dataKey="hw_info"
                        cellDataGetter={({ rowData }) => (`${_Get(rowData, 'hw_info.cbid', '--')}`)}
                        headerRenderer={virtualizedHeaderRenderer}
                        width={130}
                        className="Virtual-Table-Column-Center"
                    />}
                    <Column
                        width={280}
                        label={<span title="">{COMPONENTS.FIELDS.NODE.TRACKABLE}</span>}
                        headerRenderer={virtualizedHeaderRenderer}
                        dataKey="trackableName"
                        id="trackable-nodes-table-trackable-name"
                        headerClassName="stretch-column"
                        className="Virtual-Table-Column-Center stretch-column"
                    />
                    <Column
                        width={220}
                        label={<span title="">{COMPONENTS.FIELDS.NODE.BATTERY}</span>}
                        cellRenderer={({ cellData }) => cellData}
                        cellDataGetter={({ rowData }) => batteryStatus(rowData.battery, intl)}
                        headerRenderer={virtualizedHeaderRenderer}
                        dataKey="battery"
                        headerClassName="fixed-width"
                        className="Virtual-Table-Column-Center fixed-width"
                    />
                    <Column
                        width={200}
                        label={<span title="">{COMPONENTS.BUTTONS.ASSIGN}</span>}
                        cellRenderer={({ cellData }) => cellData}
                        cellDataGetter={({ rowData }) => this.assignNodeFormatter(rowData.trackable, rowData)}
                        headerRenderer={virtualizedHeaderRenderer}
                        dataKey="trackable"
                        id="trackable-nodes-table-assign-column"
                        headerClassName="fixed-width"
                        className="Virtual-Table-Column-Center fixed-width"
                    />
                </Table>
            </section>
        );
    }
}

export default injectIntl(withSortSelect(VirtualTable));
