import React from 'react';
import PropTypes from 'prop-types';
import { connect } from 'react-redux';
import _Get from 'lodash/get';
import { BootstrapTable, TableHeaderColumn, SearchField } from 'react-bootstrap-table';
import { injectIntl } from 'react-intl';
import isEqual from 'lodash/isEqual';

import {
    getLocalStorageValue,
    stringToBoolean,
    trOddEvenClassFormat,
} from '../../../utils';
import {
    DEFAULT_VALUES,
    NAMES_OF_TABS,
    TABLES,
    NOTIFICATION_TYPE,
    COMPONENTS,
    BOOTSTRAP_TABLE_SORT, CUSTOM_LOCAL_STORE_KEYS,
} from './../../../common';

import {
    openNotificationEditModal,
    openNotificationDetailsModal,
    openNotificationDeleteModal,
    selectItems,
    unselectItems,
    changeSelectedItems,
} from '../../../stores/actions';
import Notification from '../../../stores/classes/Notification';

class NotificationsTable extends React.Component {
    static propTypes = {
        dispatch: PropTypes.func,
        selectedGroup: PropTypes.shape({
            uid: PropTypes.string,
        }),
        intl: PropTypes.shape({
            formatMessage: PropTypes.func,
        }),
        trackables: PropTypes.arrayOf(PropTypes.shape()),
        notifications: PropTypes.arrayOf(PropTypes.shape()),
        selectedItems: PropTypes.arrayOf(PropTypes.shape()),
    };
    
    static defaultProps = {
        dispatch: () => {
        },
        selectedGroup: {},
        intl: {
            formatMessage: () => {
            },
        },
        trackables: [],
        notifications: [],
        selectedItems: [],
    };
    
    shouldComponentUpdate(nextProps) {
        return !isEqual(nextProps, this.props);
    }
    
    onRowClick = (row, columnNumber) => {
        if (isNaN(columnNumber) || columnNumber === 4) {
            return;
        }
        this.props.dispatch(unselectItems());
        if (!this.props.selectedGroup || this.props.selectedGroup.uid !== row.uid) {
            this.props.dispatch(unselectItems(this.props.trackables));
            this.props.dispatch(selectItems(row));
        }
    };
    
    getActionsIcons = (cell, row) => (
        <section data-info="actions-section">
            <i
                data-info="actions-section"
                data-id="notifications-actions-info"
                onClick={this.showInfo.bind(this, row)}
                title={COMPONENTS.SHOW_DETAILS(this.props.intl)}
                className="fa fa-info-circle inner-table-action"
                aria-hidden="true"
            />
            <i
                data-info="actions-section"
                data-id="notifications-actions-edit"
                onClick={this.openEditModal.bind(this, row)}
                title={COMPONENTS.EDIT(this.props.intl)}
                className="fa fa-pencil-alt inner-table-action"
                aria-hidden="true"
            />
            <i
                data-info="actions-section"
                data-id="notifications-actions-remove"
                onClick={this.openDeleteModal.bind(this, row)}
                title={COMPONENTS.DELETE(this.props.intl)}
                className="fa fa-times inner-table-action"
                aria-hidden="true"
            />
        </section>
    );

    getExtendsInfo = (cell, row) => {
        const sendSms = _Get(row, 'types.phone', false);
        const sendEmail = _Get(row, 'types.email', false);
        const notifyZoneViolation = !!row.alarm_type_list.find(x => x.uid === 'zone_violation');
        const notifyBufferViolation = !!row.alarm_type_list.find(x => x.uid === 'buffer_violation');
        const notifyNodeLowBattery = !!row.alarm_type_list.find(x => x.uid === 'node_low_battery');

        return (<section>
            {(sendSms || sendEmail) && 'Send' }
            {sendSms && <i
                title="Send Sms"
                className="fa fa-mobile inner-table-action"
            />}
            {sendEmail && <i
                title="Send Email"
                className="fa fa-envelope-open inner-table-action"
            />}
            {(notifyZoneViolation || notifyBufferViolation || notifyNodeLowBattery) && 'Notify' }
            {notifyZoneViolation && <i
                title={`Zone violation, Severity: ${row.zone_severity_level}`}
                className="fa fa-stop inner-table-action"
            />}
            {notifyBufferViolation && <i
                title="Buffer"
                className="fa fa-genderless inner-table-action"
            />}
            {notifyNodeLowBattery && <i
                title="Low battery"
                className="fa fa-battery-quarter inner-table-action"
            />}
        </section>);
    };

    openMultiDeleteModal = () => {
        const params = [];
        this.props.selectedItems.forEach((item) => {
            const notification = this.props.notifications.find(x => x.uid === item.uid);
            
            if (notification) {
                params.push({
                    notification_type: notification.notification_type,
                    name: notification.name,
                    uid: notification.uid,
                    email: notification.email,
                });
            }
        });
        
        if (params.length > 0) {
            this.props.dispatch(openNotificationDeleteModal(params));
        }
    };
    
    createAddDeleteBtn = () => (
        <section className="Table">
            <i
                className="fas fa-trash-alt fa-lg table-action-icon"
                data-id="notifications-button-trash"
                onClick={this.openMultiDeleteModal}
            />
            <i
                className="fa fa-plus fa-lg table-action-icon"
                data-id="notifications-button-plus"
                onClick={this.openCreateModal}
            />
        </section>
    );
    
    handleSelectRow = (row, isSelected, e) => {
        if (e.target.getAttribute('data-info') === 'actions-section') {
            return false;
        }
        this.props.dispatch(changeSelectedItems(row));
        if (!isSelected) {
            this.forceUpdate();
        }
        return true;
    };
    
    handleSelectAll = (isSelected, nodes) => {
        if (isSelected) {
            this.props.dispatch(selectItems(nodes));
        } else {
            this.props.dispatch(unselectItems(nodes));
            this.forceUpdate();
        }
    };
    
    showInfo = (row) => {
        this.props.dispatch(openNotificationDetailsModal(row));
    };
    
    openCreateModal = () => {
        const defaultNotificationEntity = {
            active: false,
            email: false,
            name: '',
            phone: false,
            zone_severity_level: '',
        };
        this.props.dispatch(openNotificationEditModal(defaultNotificationEntity));
    };
    
    openEditModal = (row) => {
        this.props.dispatch(openNotificationEditModal(row));
    };
    
    openDeleteModal = (row) => {
        this.props.dispatch(openNotificationDeleteModal(row));
    };
    
    trClassFormat = (row, rowIndex) => {
        if (this.props.selectedGroup && this.props.selectedGroup.uid === row.uid) {
            return 'Table-Assigned-Row-Selected';
        }
        return trOddEvenClassFormat(row, rowIndex);
    };
    
    createCustomSearchField = intl => (
        <SearchField
            data-id="notifications-search"
            placeholder={TABLES.SEARCH(intl)}
        />
    );

    render() {
        const { intl, notifications } = this.props;
        
        const options = {
            btnGroup: this.createAddDeleteBtn,
            noDataText: (TABLES.NO_DATA_TO_DISPLAY),
            searchField: this.createCustomSearchField.bind(this, intl),
            beforeShowError: () => true,
            defaultSortName: 'name',
            defaultSortOrder: BOOTSTRAP_TABLE_SORT.ASC,
        };
        const headerStyle = {
            marginBottom: '0',
        };
        
        const selectRow = {
            mode: 'checkbox',
            bgColor: DEFAULT_VALUES.SELECTED_ROW_COLOR,
            clickToSelect: true, // click to select, default is false
            onSelect: this.handleSelectRow,
            onSelectAll: this.handleSelectAll,
        };

        const showExtends = stringToBoolean(getLocalStorageValue(CUSTOM_LOCAL_STORE_KEYS.SHOW_EXTENDS));
        return (
            <div className="AppLayout-Admin-Tab admin-alarm-notification">
                <h3 className="text-center AdminTab-Title">{NAMES_OF_TABS.NOTIFICATIONS}</h3>
                <h4
                    className="text-center AdminTab-Subtitle"
                    data-id="notifications-total-count"
                >
                    {COMPONENTS.FIELDS.COMMON.TOTAL_COUNT_FUNC(notifications.length)}
                </h4>
                <hr />
                <BootstrapTable
                    ref="notificationsTable"
                    condensed
                    trClassName={this.trClassFormat}
                    tableBodyClass="automation-notifications-table"
                    data={notifications}
                    options={options}
                    hover
                    selectRow={selectRow}
                    headerStyle={headerStyle}
                    bodyContainerClass="Admin-BootstrapTable"
                    search
                >
                    <TableHeaderColumn
                        dataField="uid"
                        hidden
                        searchable={false}
                        isKey
                    >
                        {COMPONENTS.FIELDS.COMMON.UID}
                    </TableHeaderColumn>
                    <TableHeaderColumn
                        dataField="name"
                        tdAttr={{ 'data-id': 'notifications-name' }}
                        dataSort
                        headerAlign="center"
                    >
                        {COMPONENTS.FIELDS.COMMON.NAME}
                    </TableHeaderColumn>
                    <TableHeaderColumn
                        dataField="activeText"
                        tdAttr={{ 'data-id': 'notifications-active' }}
                        dataSort
                        headerAlign="center"
                    >
                        {COMPONENTS.FIELDS.COMMON.ACTIVE}
                    </TableHeaderColumn>
                    {showExtends && <TableHeaderColumn
                        dataAlign="center"
                        width="200"
                        headerAlign="center"
                        dataFormat={this.getExtendsInfo}
                    />}
                    <TableHeaderColumn
                        width="90"
                        headerAlign="center"
                        dataAlign="center"
                        expandable={false}
                        dataFormat={this.getActionsIcons}
                    >
                        {COMPONENTS.FIELDS.COMMON.ACTIONS}
                    </TableHeaderColumn>
                </BootstrapTable>
            </div>
        );
    }
}

const mapStateToProps = (state, { type = NOTIFICATION_TYPE.ALARM, intl }) => {
    const notifications = Notification.getAllNotifications(state)
        .reduce((memo, notify) => {
            if (notify.notification_type === type) {
                memo.push(notify.formattedNotification(state, intl));
            }
            return memo;
        }, []);

    const { selectedItems } = state.adminPage;
    
    return {
        notifications,
        selectedItems,
    };
};

export default injectIntl(connect(mapStateToProps)(injectIntl(NotificationsTable)));
