import React from 'react';
import PropTypes from 'prop-types';
import { injectIntl } from 'react-intl';
import { Column, Table } from 'react-virtualized';
import 'react-virtualized/styles.css';

import withSortSelect from 'components/HOC/withSortSelect';
import { virtualizedHeaderRenderer, getRoleIcon, getPaired, getCategoryText } from 'utils/htmlUtils';
import { stringToBoolean } from 'utils/utils';
import { getLocalStorageValue } from 'utils/browserStorage';
import { COMPONENTS } from 'common/ui-labels';
import { CUSTOM_LOCAL_STORE_KEYS, DEFAULT_VALUES } from 'common/constants';
import {
    openTrackableDetailsModal, openTrackableEditModal, openTrackableDeleteModal,
    openTrackableAssignModal, openTrackableUnAssignModal,
} from 'stores/actions';
import { Trackable, TrackableDetails } from 'stores/classes';
import TableButtons from './TableButtons';

class VirtualTable extends React.Component {
    static propTypes = {
        // Props from HOC
        sort: PropTypes.func,
        sortBy: PropTypes.string,
        trStyleFormat: PropTypes.func,
        checkboxButton: PropTypes.func,
        headerCheckbox: PropTypes.func,
        setFilterItems: PropTypes.func,
        handleSelectRow: PropTypes.func,
        sortDisplayItems: PropTypes.func,
        sortDirection: PropTypes.string,
        filteredItems: PropTypes.arrayOf(PropTypes.shape({})),
        
        // Props from parent
        allItems: PropTypes.shape({}),
        totalCount: PropTypes.number,
        isReadOnly: PropTypes.bool,
        tableSize: PropTypes.shape({}),
        selectedItems: PropTypes.arrayOf(PropTypes.shape({})),
        dispatch: PropTypes.func,
        intl: PropTypes.shape({
            formatMessage: PropTypes.func,
        }),
        templateProperties: PropTypes.arrayOf(PropTypes.shape({})),
    };

    static defaultProps = {
        // Props from HOC
        sort: () => { },
        sortBy: '',
        trStyleFormat: () => { },
        checkboxButton: () => { },
        headerCheckbox: () => { },
        setFilterItems: () => { },
        handleSelectRow: () => { },
        sortDisplayItems: () => { },
        sortDirection: '',
        filteredItems: [],
    
        // Props from parent
        allItems: new Map(),
        totalCount: 0,
        isReadOnly: false,
        tableSize: {},
        selectedItems: [],
        dispatch: () => { },
        intl: {
            formatMessage: () => { },
        },
        templateProperties: [],
    };
    
    constructor(props) {
        super(props);
        this.totalCount = props.totalCount;
        this.dispatch = props.dispatch;
        
        // HOC Methods
        // this.trClassFormat = props.trClassFormat.bind(this);
        this.trStyleFormat = props.trStyleFormat.bind(this);
        this.handleSelectRow = props.handleSelectRow.bind(this);
        this.checkboxButton = props.checkboxButton.bind(this);
        this.headerCheckbox = props.headerCheckbox.bind(this);
        this.sort = props.sort.bind(this);
        this.sortDisplayItems = props.sortDisplayItems.bind(this);
        this.setFilterItems = props.setFilterItems.bind(this);
    }
    
    getActionsIcons = (row) => {
        if (this.props.isReadOnly) {
            return (
                <section data-info="actions-section">
                    <i
                        data-info="actions-section"
                        key="trackables-actions-info"
                        data-id="trackables-actions-info"
                        onClick={this.showInfo.bind(this, row)}
                        title={COMPONENTS.SHOW_DETAILS(this.props.intl)}
                        className="fa fa-info-circle inner-table-action"
                        aria-hidden="true"
                    />
                </section>
            );
        }
        return (
            <section data-info="actions-section">
                <i
                    data-info="actions-section"
                    key="trackables-actions-info"
                    data-id="trackables-actions-info"
                    onClick={this.showInfo.bind(this, row)}
                    title={COMPONENTS.SHOW_DETAILS(this.props.intl)}
                    className="fa fa-info-circle inner-table-action"
                    aria-hidden="true"
                />
                <i
                    data-info="actions-section"
                    key="trackables-actions-edit"
                    data-id="trackables-actions-edit"
                    onClick={this.openEditModal.bind(this, row)}
                    title={COMPONENTS.EDIT(this.props.intl)}
                    className="fa fa-pencil-alt inner-table-action"
                    aria-hidden="true"
                />
                <i
                    data-info="actions-section"
                    key="trackables-actions-remove"
                    data-id="trackables-actions-remove"
                    onClick={this.openDeleteModal.bind(this, row)}
                    title={COMPONENTS.DELETE(this.props.intl)}
                    className="fa fa-times inner-table-action"
                    aria-hidden="true"
                />
            </section>
        );
    };
    
    trClassFormat = (items, { index }) => {
        const currentItem = items[index];
        if (index >= 0 && currentItem) {
            if (!items[index].node) {
                return 'Table-Not-Assign-Row';
            }
            return 'Table-Assigned-Row';
        }
        return 'Virtual-Table-Header-Row';
    };
    
    openEditModal = (row) => {
        this.props.dispatch(openTrackableEditModal(new Trackable({
            default_height: DEFAULT_VALUES.trackable_default_height,
            type: DEFAULT_VALUES.trackable_type,
            category: DEFAULT_VALUES.trackable_category,
            buffer_radius: DEFAULT_VALUES.trackable_buffer_radius,
            ...row,
        })));
    };
    
    showInfo = (row) => {
        this.props.dispatch(openTrackableDetailsModal(new TrackableDetails(row)));
    };
    
    openDeleteModal = (row) => {
        this.props.dispatch(openTrackableDeleteModal(row));
    };
    
    assignFormatter = (node, row) => {
        if (node) {
            return (
                <div data-info="actions-section">
                    <button
                        data-info="actions-section"
                        data-id="trackables-table-unassign-button"
                        className="RPP-Assign-buttons"
                        type="button"
                        onClick={this.unassignTrackable.bind(this, row)}
                    >
                        {COMPONENTS.BUTTONS.UNASSIGN}
                    </button>
                </div>
            );
        }
        return (
            <div>
                <button
                    data-info="actions-section"
                    data-id="trackables-table-assign-button"
                    className="RPP-Assign-buttons"
                    type="button"
                    onClick={this.assignTrackable.bind(this, row)}
                >
                    {COMPONENTS.BUTTONS.ASSIGN}
                </button>
            </div>
        );
    };
    
    assignTrackable = (trackable) => {
        this.props.dispatch(openTrackableAssignModal(trackable));
    };
    
    unassignTrackable = (trackable) => {
        this.props.dispatch(openTrackableUnAssignModal(trackable));
    };

    _noRowsRenderer = () => (
        <table className="table table-bordered table-hover table-condensed table-no-data">
            <tbody>
                <tr>
                    <td className="react-bs-table-no-data">
                        <span>There is no data to display</span>
                    </td>
                </tr>
            </tbody>
        </table>
    );
    
    render() {
        const {
            sortBy, sortDirection, tableSize, allItems, selectedItems, filteredItems, isReadOnly, templateProperties,
        } = this.props;
        const allItemsArray = [...allItems.values()];
        const displayItems = this.sortDisplayItems(filteredItems);
        const displayCount = displayItems.length;
        const showRolePaired = stringToBoolean(getLocalStorageValue(CUSTOM_LOCAL_STORE_KEYS.SHOW_EXTENDS));

        return (
            <section style={{ width: tableSize.width }}>
                <TableButtons
                    isReadOnly={isReadOnly}
                    items={allItemsArray}
                    width={tableSize.width}
                    setFilterItems={this.setFilterItems}
                    selectedItems={selectedItems}
                    searchFields={['name', 'groupNames', 'mac_address', 'category']}
                />
                <Table
                    noRowsRenderer={this._noRowsRenderer}
                    sortBy={sortBy}
                    sort={this.sort}
                    sortDirection={sortDirection}
                    className="Virtual-Table-Medium"
                    width={tableSize.width} // required
                    height={tableSize.height} // required
                    gridClassName="Virtual-Table-Grid"
                    rowHeight={tableSize.rowHeight} // required
                    headerHeight={tableSize.headerHeight} // required
                    headerClassName="Virtual-Table-Header react-bs-container-header"
                    rowClassName={({ index }) => this.trClassFormat(displayItems, { index })}
                    rowStyle={({ index }) => this.trStyleFormat(displayItems, { index })}
                    onRowClick={({ event, index, rowData }) => this.handleSelectRow(displayCount, { event, index, rowData })}
                    rowCount={displayCount} // required
                    rowGetter={({ index }) => displayItems[index]} // required
                >
                    {!isReadOnly && <Column
                        label={<span title="">{this.headerCheckbox(displayItems)}</span>}
                        dataKey="checkbox"
                        cellRenderer={({ rowData }) => this.checkboxButton(rowData, displayCount)}
                        width={100}
                        disableSort
                        className="Virtual-Table-Column-Checkbox first-column"
                    />}
                    <Column
                        label={<span title="">{COMPONENTS.FIELDS.COMMON.NAME}</span>}
                        headerRenderer={virtualizedHeaderRenderer}
                        dataKey="name"
                        width={360}
                        id="trackables-table-name"
                        className="Virtual-Table-Column-Left"
                    />
                    {templateProperties.map((x, idx) => (
                        <Column
                            key={idx}
                            label={<span title="">{x.label}</span>}
                            headerRenderer={virtualizedHeaderRenderer}
                            cellRenderer={({ rowData }) => (rowData?.user_params ? rowData.user_params[x.name] : '')}
                            dataKey="serial_number"
                            width={360}
                            id="trackables-table-name-serial-number"
                            className="Virtual-Table-Column-Left"
                        />
                    ))}
                    <Column
                        label={<span title="">{COMPONENTS.FIELDS.NODE.CATEGORY}</span>}
                        cellDataGetter={({ rowData }) => getCategoryText(rowData.category, this.props.intl)}
                        headerRenderer={virtualizedHeaderRenderer}
                        dataKey="category"
                        width={250}
                        id="trackables-table-category"
                        className="Virtual-Table-Column-Center"
                    />
                    <Column
                        width={360}
                        label={<span title="">{COMPONENTS.FIELDS.NODE.GROUP}</span>}
                        headerRenderer={virtualizedHeaderRenderer}
                        dataKey="groupNames"
                        id="trackables-table-groups"
                        className="Virtual-Table-Column-Left"
                    />
                    <Column
                        width={300}
                        label={<span title="">{COMPONENTS.FIELDS.NODE.TAG}</span>}
                        headerRenderer={virtualizedHeaderRenderer}
                        dataKey="mac_address"
                        id="trackables-table-mac"
                        className="Virtual-Table-Column-Left text-small"
                    />
                    {!isReadOnly && <Column
                        width={360}
                        label={<span title="">{COMPONENTS.BUTTONS.ASSIGN}</span>}
                        cellRenderer={({cellData}) => cellData}
                        cellDataGetter={({rowData}) => this.assignFormatter(rowData.node, rowData)}
                        headerRenderer={virtualizedHeaderRenderer}
                        dataKey="node"
                        id="trackables-table-assign-column"
                        className="Virtual-Table-Column-Center"
                    />}
                    {showRolePaired &&
                    <Column
                        cellRenderer={({ cellData }) => cellData}
                        cellDataGetter={({ rowData }) => (<section>{getRoleIcon(rowData)}{getPaired(rowData)}</section>)}
                        headerRenderer={virtualizedHeaderRenderer}
                        width={100}
                        className="Virtual-Table-Column-Center"
                    />}
                    <Column
                        width={280}
                        label={<span title="">{COMPONENTS.FIELDS.COMMON.ACTIONS}</span>}
                        cellRenderer={({ cellData }) => cellData}
                        cellDataGetter={({ rowData }) => this.getActionsIcons(rowData)}
                        dataKey="actions"
                        disableSort
                        className="Virtual-Table-Column-Center"
                    />
                </Table>
            </section>
        );
    }
}

export default injectIntl(withSortSelect(VirtualTable));
