import React, { Fragment, Component } from 'react';
import PropTypes from 'prop-types';
import { connect } from 'react-redux';
import { injectIntl } from 'react-intl';
import { createSelectorCreator, defaultMemoize } from 'reselect';
import isEqual from 'lodash/isEqual';
import { SortDirection, AutoSizer } from 'react-virtualized';
import { getAdminTrackables, getNodes } from 'stores/getters';
import { Zone, Group, Association, TrackableTemplate } from 'stores/classes';

import TableTitle from './TableTitle';
import VirtualTable from './VirtualTable';

class TrackablesTable extends Component {
    static propTypes = {
        dispatch: PropTypes.func,
        isReadOnly: PropTypes.bool,
        selectedItems: PropTypes.arrayOf(PropTypes.shape({})),
        templateProperties: PropTypes.arrayOf(PropTypes.shape({})),
        trackables: PropTypes.shape({}),
    };

    static defaultProps = {
        dispatch: () => {
        },
        isReadOnly: false,
        selectedItems: [],
        templateProperties: [],
        trackables: [],
    };

    constructor(props) {
        super(props);

        this.state = {
            containerWidth: 0,
            containerHeight: 0,
        };
    }

    shouldComponentUpdate(nextProps, nextState) {
        const { containerWidth, containerHeight } = this.state;
        const { selectedItems, trackables } = this.props;

        return (containerWidth !== nextState.containerWidth)
            || (containerHeight !== nextState.containerHeight)
            || !isEqual(selectedItems, nextProps.selectedItems)
            || !isEqual(trackables, nextProps.trackables);
    }

    render() {
        const { trackables, dispatch, selectedItems, isReadOnly, templateProperties } = this.props;
        const { containerWidth, containerHeight } = this.state;
        const trackablesCount = trackables.size;

        return (
            <div className="AppLayout-Admin-Tab tab-admin-trackables">
                <AutoSizer
                    onResize={({ height, width }) => {
                        this.setState({
                            containerWidth: width,
                            containerHeight: height,
                        });
                    }}
                >
                    {({ height, width }) => (
                        <Fragment>
                            <TableTitle
                                totalCount={trackablesCount}
                                width={containerWidth}
                            />
                            <VirtualTable
                                isReadOnly={isReadOnly}
                                defaultSortName="name"
                                defaultSortOrder={SortDirection.ASC}
                                tableSize={{
                                    width: containerWidth,
                                    height: containerHeight,
                                    headerHeight: 40,
                                    rowHeight: 40,
                                }}
                                allItems={trackables}
                                totalCount={trackablesCount}
                                templateProperties={templateProperties}
                                selectedItems={selectedItems}
                                dispatch={dispatch}
                            />
                        </Fragment>
                    )}
                </AutoSizer>
            </div>
        );
    }
}

const createDeepEqualSelector = createSelectorCreator(
    defaultMemoize,
    isEqual,
);

const getFormattedTrackables = createDeepEqualSelector(
    [
        getAdminTrackables,
        getNodes,
        Association.getAllAssociations,
        Group.getAllGroupsArray,
        Zone.getAllZonesArray,
    ],
    (trackables) => trackables,
);

const mapStateToProps = (state, props) => ({
    isReadOnly: props.isReadOnly,
    selectedItems: state.adminPage.selectedItems,
    templateProperties: TrackableTemplate.getTemplateProperty(state.entities.trackableTemplates),
    trackables: getFormattedTrackables(state),
});

export default connect(mapStateToProps)(injectIntl(TrackablesTable));
