import React from 'react';
import PropTypes from 'prop-types';
import { connect } from 'react-redux';
import { injectIntl } from 'react-intl';
import { BootstrapTable, TableHeaderColumn, SearchField } from 'react-bootstrap-table';
import { TABLES, BOOTSTRAP_TABLE_SORT, COMPONENTS, USER_ROLES_DISPLAY } from 'common';
import { openUserAccessControlEditModal } from 'stores/actions';
import { durationToDaysHumanize, durationToHoursHumanize } from 'utils/timeUtils';
import { userHasPrivileges } from 'utils';


class UsersAccessControlTable extends React.Component {
    static propTypes = {
        dispatch: PropTypes.func,
        currentUser: PropTypes.shape({}),
        usersAccessControl: PropTypes.arrayOf(PropTypes.shape({})),
        intl: PropTypes.shape({
            formatMessage: PropTypes.func,
        }),
    };

    static defaultProps = {
        dispatch: () => {},
        currentUser: {},
        usersAccessControl: [],
        intl: {
            formatMessage: () => {},
        },
    };

    getActionsIcons = (cell, row) => {
        const currentUserRole = this.props.currentUser.role || this.props.currentUser?.access?.role;
        const hasPrivileges = userHasPrivileges(currentUserRole, row?.role);

        return (
            <section data-info="actions-section">
                <i
                    onClick={this.openEditModal.bind(this, row)}
                    data-info="actions-section"
                    data-id="super-admin-users-actions-edit"
                    data-cy="super-admin-user-table-edit-button"
                    title={COMPONENTS.EDIT(this.props.intl)}
                    className={`fa fa-pencil-alt inner-table-action ${hasPrivileges ? '' : 'disable-click'}`}
                    aria-hidden="true"
                />
            </section>
        );
    }

    getUserRole = (cell) => {
        return <span title={USER_ROLES_DISPLAY[cell]} className="text-ellipsis">{USER_ROLES_DISPLAY[cell]}</span>;
    }

    getSessionTimeout = (intl, cell) => {
        if (!cell) return COMPONENTS.FIELDS.COMMON.NEVER_FUNC(intl);
        return durationToHoursHumanize(cell, 'minutes')
    }

    getExpirationTimeout = (intl, cell) => {
        if (!cell) return COMPONENTS.FIELDS.COMMON.NEVER_FUNC(intl);
        return durationToDaysHumanize(cell, 'minutes');
    }

    openEditModal = (row) => {
        this.props.dispatch(openUserAccessControlEditModal(row));
    };

    createCustomSearchField = intl => (
        <SearchField
            data-id="super-admin-users-search"
            data-cy="super-admin-user-table-search"
            placeholder={TABLES.SEARCH(intl)}
        />
    );

    render() {
        const options = {
            btnGroup: this.createAddDeleteBtn,
            noDataText: (TABLES.NO_DATA_TO_DISPLAY),
            searchField: this.createCustomSearchField.bind(this, this.props.intl),
            beforeShowError: () => true,
            sortOrder: BOOTSTRAP_TABLE_SORT.ASC,
        };
        const headerStyle = {
            marginBottom: '0',
        };

        const { usersAccessControl } = this.props;

        return (
            <div className="AppLayout-Admin-Tab col-md-12">
                <h3 className="text-center AdminTab-Title">{COMPONENTS.FIELDS.COMMON.USERS_ACCESS_CONTROL}</h3>
                <h4
                    className="text-center AdminTab-Subtitle"
                    data-id="super-admin-users-total-count"
                    data-cy="super-admin-user-table-total-count"
                >
                    {COMPONENTS.FIELDS.COMMON.TOTAL_COUNT_FUNC(usersAccessControl.length)}
                </h4>
                <hr />
                <BootstrapTable
                    data={usersAccessControl}
                    tableBodyClass="automation-super-admin-users-table"
                    options={options}
                    headerStyle={headerStyle}
                    bodyContainerClass="Admin-BootstrapTable"
                    hover
                    condensed
                    striped
                    search
                >
                    <TableHeaderColumn
                        dataField="role"
                        dataFormat={this.getUserRole}
                        searchable={false}
                        width="30%"
                        headerAlign="center"
                        isKey
                    >
                        {COMPONENTS.FIELDS.USER.USER_TYPE}
                    </TableHeaderColumn>
                    <TableHeaderColumn
                        dataField="sessionTimeout"
                        dataAlign="center"
                        tdAttr={{ 'data-id': 'users-roles-table-session-timeout' }}
                        dataFormat={(cell) => this.getSessionTimeout(this.props.intl, cell) }
                        width="25%"
                        headerAlign="center"
                    >
                        {COMPONENTS.FIELDS.USER.SESSION_TIMEOUT}
                    </TableHeaderColumn>
                    <TableHeaderColumn
                        dataField="accountExpirationTimeout"
                        dataAlign="center"
                        width="35%"
                        tdAttr={{ 'data-id': 'users-roles-table-expiration-timeout' }}
                        dataFormat={(cell) => this.getExpirationTimeout(this.props.intl, cell) }
                        headerAlign="center"
                    >
                        {COMPONENTS.FIELDS.USER.ACCOUNT_EXPIRATION_TIMEOUT}
                    </TableHeaderColumn>
                    <TableHeaderColumn
                        dataField="actions"
                        width="85"
                        headerAlign="center"
                        dataAlign="center"
                        expandable={false}
                        dataFormat={this.getActionsIcons}
                    >
                        {COMPONENTS.FIELDS.COMMON.ACTIONS}
                    </TableHeaderColumn>
                </BootstrapTable>
            </div>
        );
    }
}

const mapStateToProps = (state) => {
    return {
        usersAccessControl: state.entities.usersAccessControl,
        currentUser: state.user,
    };
};

export default connect(mapStateToProps)(injectIntl(UsersAccessControlTable));
