import React from 'react';
import PropTypes from 'prop-types';
import { connect } from 'react-redux';
import { injectIntl } from 'react-intl';
import { BootstrapTable, TableHeaderColumn, SearchField } from 'react-bootstrap-table';
import { openUserDetailsModal, openUserEditModal, openUserDeleteModal, changeSelectedItems, selectItems, unselectItems, editUser } from 'stores/actions';
import { trOddEvenClassFormat, userHasPrivileges, getRolesToUpdateInProject, getRolesToRenderInProject } from 'utils';
import { DEFAULT_VALUES, NAMES_OF_TABS, TABLES, BOOTSTRAP_TABLE_SORT, USER_ROLES, COMPONENTS } from 'common';
import User from 'stores/classes/User';
import { ProjectUsersActions } from './ProjectUsersActions';
import { ProjectUserRoleSelect } from 'components/ProjectUserRoleSelect';


class UsersTableProjectScope extends React.Component {
    static propTypes = {
        dispatch: PropTypes.func,
        users: PropTypes.arrayOf(PropTypes.shape({})),
        unselectableUsers: PropTypes.arrayOf(PropTypes.string),
        selectedItems: PropTypes.arrayOf(PropTypes.shape({})),
        currentUser: PropTypes.shape({
            email: PropTypes.string,
            access: PropTypes.object,
        }),
        intl: PropTypes.shape({
            formatMessage: PropTypes.func,
        }),
        appContext: PropTypes.shape({
            project: PropTypes.shape({
                uid: PropTypes.string,
            }),
            company: PropTypes.shape({
                uid: PropTypes.string,
            }),
        }),
    };
    
    static defaultProps = {
        dispatch: () => {
        },
        users: [],
        unselectableUsers: [],
        selectedItems: [],
        currentUser: {
            email: '',
        },
        intl: {
            formatMessage: () => {
            },
        },
        appContext: {
            project: {
                uid: '',
            },
            company: {
                uid: '',
                
            },
        },
    };
    
    handleSelectRow = (row, isSelected, e) => {
        if (e.target.getAttribute('data-info') === 'actions-section') {
            return false;
        }
        this.props.dispatch(changeSelectedItems(row));
    };
    
    handleSelectAll = (isSelected, _users) => {
        const users = _users.filter(user => !this.props.unselectableUsers.includes(user.uid));
        if (isSelected) {
            this.props.dispatch(selectItems(users));
        } else {
            this.props.dispatch(unselectItems(users));
        }
    };
    
    showUserInfo = (row) => {
        this.props.dispatch(openUserDetailsModal(row));
    };
    
    openEditModal = (row) => {
        this.props.dispatch(openUserEditModal(row));
    };
    
    openAddUserModal = () => {
        this.props.dispatch(openUserEditModal({}));
    };
    
    openDeleteModal = (row) => {
        this.props.dispatch(openUserDeleteModal(row));
    };
    
    openMultiDeleteModal = () => {
        const params = [];
        this.props.selectedItems.forEach((item) => {
            const user = this.props.users.find(x => x.uid === item.uid);
            
            if (user) {
                params.push({ ...user });
            }
        });
        
        if (params.length > 0) {
            this.props.dispatch(openUserDeleteModal(params));
        }
    };

    getUserRoleOptions = (intl, cell, row) => {
        const currentUserRole = this.props.currentUser.role || this.props.currentUser?.access?.currentRole;
        const hasPriv = userHasPrivileges(currentUserRole, row.role);
        const disabled = this.props.currentUser.email === row.email || row.role === USER_ROLES.COMPANY_ADMIN || !hasPriv;
        const roles = hasPriv ? getRolesToUpdateInProject(currentUserRole, row.role) : getRolesToRenderInProject();

        return (
            <ProjectUserRoleSelect
                uid={row.uid}
                onChange={e => this.updateUserRole(e, row)}
                intl={intl}
                role={row.role}
                roles={roles}
                disabled={disabled}
            />
        )
    };

    updateUserRole = (e, row) => {
        row.role = e.target.value;
        this.props.dispatch(editUser(row));
    };
    
    createAddDeleteBtn = () => (
        <section className="Table">
            <i
                data-id="users-project-trash-button"
                data-info="actions-section"
                onClick={this.openMultiDeleteModal}
                className="fas fa-trash-alt fa-lg table-action-icon"
            />
            <i
                data-id="users-project-plus-button"
                data-info="actions-section"
                onClick={this.openAddUserModal}
                className="fa fa-plus fa-lg table-action-icon"
            />
        </section>);
    
    createCustomSearchField = intl => (
        <SearchField
            data-id="users-project-search"
            placeholder={TABLES.SEARCH(intl)}
        />
    );
    
    render() {
        const options = {
            btnGroup: this.createAddDeleteBtn,
            noDataText: (TABLES.NO_DATA_TO_DISPLAY),
            searchField: this.createCustomSearchField.bind(this, this.props.intl),
            beforeShowError: () => true,
            defaultSortName: 'name',
            defaultSortOrder: BOOTSTRAP_TABLE_SORT.ASC,
        };
        const headerStyle = {
            marginBottom: '0',
        };
        const selectRow = {
            mode: 'checkbox',
            clickToSelect: true, // click to select, default is false
            unselectable: [...this.props.unselectableUsers],
            bgColor: DEFAULT_VALUES.SELECTED_ROW_COLOR,
            onSelect: this.handleSelectRow,
            onSelectAll: this.handleSelectAll,
            columnWidth: '30px',
        };
        
        const { users, currentUser } = this.props;

        return (
            <section className="AppLayout-Admin-Tab col-md-12">
                <h3 className="text-center AdminTab-Title">{NAMES_OF_TABS.USERS_PROJECT_SCOPE}</h3>
                <h4
                    className="text-center AdminTab-Subtitle"
                    data-id="users-project-total-count"
                >
                    {COMPONENTS.FIELDS.COMMON.TOTAL_COUNT_FUNC(users.length)}
                </h4>
                <hr />
                <BootstrapTable
                    condensed
                    data={users}
                    options={options}
                    hover
                    selectRow={selectRow}
                    expandComponent={this.expandComponent}
                    trClassName={trOddEvenClassFormat}
                    tableBodyClass="automation-users-project-table"
                    headerStyle={headerStyle}
                    bodyContainerClass="Admin-BootstrapTable"
                    search
                    className="middle-align"
                >
                    <TableHeaderColumn
                        dataField="uid"
                        hidden
                        searchable={false}
                        isKey
                        width="40"
                    >
                        {COMPONENTS.FIELDS.COMMON.UID}
                    </TableHeaderColumn>
                    <TableHeaderColumn
                        width="140"
                        dataField="name"
                        tdAttr={{ 'data-id': 'users-project-table-name' }}
                        dataAlign="left"
                        dataSort
                        headerAlign="center"
                    >
                        {COMPONENTS.FIELDS.COMMON.NAME}
                    </TableHeaderColumn>
                    <TableHeaderColumn
                        width="180"
                        dataField="email"
                        tdAttr={{ 'data-id': 'users-project-table-email' }}
                        dataAlign="left"
                        dataSort
                        headerAlign="center"
                        columnTitle
                    >
                        {COMPONENTS.FIELDS.USER.EMAIL}
                    </TableHeaderColumn>
                    <TableHeaderColumn
                        width="130"
                        dataField="phone_number"
                        tdAttr={{ 'data-id': 'users-project-table-phone' }}
                        dataAlign="center"
                        headerAlign="center"
                    >
                        {COMPONENTS.FIELDS.USER.PHONE}
                    </TableHeaderColumn>
                    <TableHeaderColumn
                        width="180"
                        dataField="role"
                        tdAttr={{ 'data-id': 'users-project-table-type' }}
                        dataFormat={(cell, row) => this.getUserRoleOptions(this.props.intl, cell, row)}
                        dataSort
                        dataAlign="center"
                        headerAlign="center"
                        expandable={false}
                    >
                        {COMPONENTS.FIELDS.USER.USER_TYPE}
                    </TableHeaderColumn>
                    <TableHeaderColumn
                        dataField="actions"
                        width="80"
                        headerAlign="center"
                        dataAlign="center"
                        dataFormat={(cell, row) => (
                            <ProjectUsersActions
                                currentUser={currentUser}
                                tableUser={row}
                                showUserInfo={this.showUserInfo}
                                openDeleteModal={this.openDeleteModal}
                                openEditModal={this.openEditModal}
                                intl={this.props.intl}
                            />
                        )}
                        expandable={false}
                    >
                        {COMPONENTS.FIELDS.COMMON.ACTIONS}
                    </TableHeaderColumn>
                </BootstrapTable>
            </section>
        );
    }
}


const mapStateToProps = (state) => {
    const selectedProject = state.appContext.project;
    const projectsMap = state.entities.projects.reduce((memo, project) => {memo[project.uid] = project; return memo;}, {});
    const unselectableUsers = [];
    const users = User.getAllUsers(state).reduce((memo, user) => {
        const formattedUser = user.getFormattedUser(projectsMap[user.project_uid]);
        if (!selectedProject || user.belongToProject(selectedProject)) {
            memo.push(formattedUser);
        } else if (formattedUser.role === USER_ROLES.COMPANY_ADMIN) {
            memo.push(formattedUser);
            unselectableUsers.push(formattedUser.uid);
        }
        return memo;
    }, []);
    return {
        users,
        unselectableUsers,
        appContext: state.appContext,
        selectedItems: state.adminPage.selectedItems,
        currentUser: state.user,
    };
};

export default connect(mapStateToProps)(injectIntl(UsersTableProjectScope));
