import React from 'react';
import PropTypes from 'prop-types';
import { connect } from 'react-redux';
import { injectIntl } from 'react-intl';
import { BootstrapTable, TableHeaderColumn, SearchField } from 'react-bootstrap-table';
import { DEFAULT_VALUES, TABLES, BOOTSTRAP_TABLE_SORT, COMPONENTS, NAMES_OF_TABS, USER_ROLES } from 'common';
import { openUserDetailsModal, openUserEditModalSystemScope, openUserDeleteModal, changeSelectedItems, selectItems, unselectItems } from 'stores/actions';
import User from 'stores/classes/User';


class UsersTableSystemScope extends React.Component {
    static propTypes = {
        dispatch: PropTypes.func,
        selectedItems: PropTypes.arrayOf(PropTypes.shape({})),
        users: PropTypes.arrayOf(PropTypes.shape({})),
        intl: PropTypes.shape({
            formatMessage: PropTypes.func,
        }),
    };

    static defaultProps = {
        dispatch: () => {
        },
        selectedItems: [],
        users: [],
        intl: {
            formatMessage: () => {
            },
        },
    };

    getActionsIcons = (cell, row) => (
        <section data-info="actions-section">
            <i
                onClick={this.showCompanyInfo.bind(this, row)}
                data-info="actions-section"
                data-id="super-admin-users-actions-info"
                data-cy="super-admin-user-table-info-button"
                title={COMPONENTS.SHOW_DETAILS(this.props.intl)}
                className="fa fa-info-circle inner-table-action"
                aria-hidden="true"
            />
            <i
                onClick={this.openEditModal.bind(this, row)}
                data-info="actions-section"
                data-id="super-admin-users-actions-edit"
                data-cy="super-admin-user-table-edit-button"
                title={COMPONENTS.EDIT(this.props.intl)}
                className="fa fa-pencil-alt inner-table-action"
                aria-hidden="true"
            />
            <i
                onClick={this.openDeleteModal.bind(this, row)}
                data-info="actions-section"
                data-id="super-admin-users-actions-remove"
                data-cy="super-admin-user-table-remove-button"
                title={COMPONENTS.DELETE(this.props.intl)}
                className="fa fa-times inner-table-action"
                aria-hidden="true"
            />
        </section>
    );

    showCompanyInfo = (row) => {
        this.props.dispatch(openUserDetailsModal(row));
    };
    openEditModal = (row) => {
        this.props.dispatch(openUserEditModalSystemScope(row));
    };
    openDeleteModal = (row) => {
        this.props.dispatch(openUserDeleteModal(row));
    };
    openMultiDeleteModal = () => {
        const params = [];
        this.props.selectedItems.forEach((item) => {
            const user = this.props.users.find(x => x.uid === item.uid);

            if (user) {
                params.push({
                    name: user.name,
                    uid: user.uid,
                    email: user.email,
                });
            }
        });

        if (params.length > 0) {
            this.props.dispatch(openUserDeleteModal(params));
        }
    };

    createAddDeleteBtn = () => (
        <section className="Table">
            <i
                className="fas fa-trash-alt fa-lg table-action-icon"
                data-id="super-admin-users-trash-button"
                data-cy="super-admin-user-table-batch-remove-button"
                onClick={this.openMultiDeleteModal}
            />
            <i
                className="fa fa-plus fa-lg table-action-icon"
                data-id="super-admin-users-plus-button"
                data-cy="super-admin-user-table-add-button"
                onClick={this.openEditModal.bind(this, {})}
            />
        </section>
    );

    handleSelectRow = (row, isSelected, e) => {
        if (e.target.getAttribute('data-info') === 'actions-section') {
            return false;
        }
        this.props.dispatch(changeSelectedItems(row));
        if (!isSelected) {
            this.forceUpdate();
        }
        return undefined;
    };

    handleSelectAll = (isSelected) => {
        if (isSelected) {
            this.props.dispatch(selectItems(this.props.users));
        } else {
            this.props.dispatch(unselectItems(this.props.users));
        }
    };

    createCustomSearchField = intl => (
        <SearchField
            data-id="super-admin-users-search"
            data-cy="super-admin-user-table-search"
            placeholder={TABLES.SEARCH(intl)}
        />
    );

    render() {
        const options = {
            btnGroup: this.createAddDeleteBtn,
            noDataText: (TABLES.NO_DATA_TO_DISPLAY),
            searchField: this.createCustomSearchField.bind(this, this.props.intl),
            beforeShowError: () => true,
            sortName: 'name',
            sortOrder: BOOTSTRAP_TABLE_SORT.ASC,
        };
        const headerStyle = {
            marginBottom: '0',
        };

        const selectRow = {
            mode: 'checkbox',
            clickToSelect: true,
            bgColor: DEFAULT_VALUES.SELECTED_ROW_COLOR,
            onSelect: this.handleSelectRow,
            onSelectAll: this.handleSelectAll,
        };

        const { users } = this.props;

        return (
            <div className="AppLayout-Admin-Tab col-md-12">
                <h3 className="text-center AdminTab-Title">{NAMES_OF_TABS.SUPER_ADMIN_USERS}</h3>
                <h4
                    className="text-center AdminTab-Subtitle"
                    data-id="super-admin-users-total-count"
                    data-cy="super-admin-user-table-total-count"
                >
                    {COMPONENTS.FIELDS.COMMON.TOTAL_COUNT_FUNC(users.length)}
                </h4>
                <hr />
                <BootstrapTable
                    data={users}
                    tableBodyClass="automation-super-admin-users-table"
                    options={options}
                    selectRow={selectRow}
                    headerStyle={headerStyle}
                    bodyContainerClass="Admin-BootstrapTable"
                    hover
                    condensed
                    striped
                    search
                >
                    <TableHeaderColumn
                        dataField="uid"
                        searchable={false}
                        hidden
                        isKey
                    >
                        {COMPONENTS.FIELDS.COMMON.UID}
                    </TableHeaderColumn>
                    <TableHeaderColumn
                        dataField="name"
                        tdAttr={{ 'data-id': 'super-users-table-name' }}
                        headerAlign="center"
                    >
                        {COMPONENTS.FIELDS.COMMON.NAME}
                    </TableHeaderColumn>
                    <TableHeaderColumn
                        dataField="email"
                        width="50%"
                        tdAttr={{ 'data-id': 'super-users-table-email' }}
                        headerAlign="center"
                    >
                        {COMPONENTS.FIELDS.USER.EMAIL}
                    </TableHeaderColumn>
                    <TableHeaderColumn
                        dataField="actions"
                        width="85"
                        headerAlign="center"
                        dataAlign="center"
                        expandable={false}
                        dataFormat={this.getActionsIcons}
                    >
                        {COMPONENTS.FIELDS.COMMON.ACTIONS}
                    </TableHeaderColumn>
                </BootstrapTable>
            </div>
        );
    }
}

const mapStateToProps = (state) => {
    let users = User.getAllUsers(state);
    users = users.reduce((memo, user) => {
        if (user.role === USER_ROLES.SYSTEM_ADMIN) {
            memo.push(user.getFormattedSuperAdminUser());
        }
        return memo;
    }, []);

    return {
        users,
        selectedItems: state.adminPage.selectedItems,
    };
};

export default connect(mapStateToProps)(injectIntl(UsersTableSystemScope));
