import React, { Component, Fragment } from 'react';
import PropTypes from 'prop-types';
import { connect } from 'react-redux';
import { injectIntl } from 'react-intl';
import { createSelectorCreator, defaultMemoize } from 'reselect';
import isEqual from 'lodash/isEqual';
import { SortDirection, AutoSizer } from 'react-virtualized';

import TableTitle from './TableTitle';
import VirtualTable from './VirtualTable';
import { Zone, Location, Sublocation, ZonePrototype } from '../../../../stores/classes';

class ZonesTable extends Component {
    static propTypes = {
        dispatch: PropTypes.func,
        zones: PropTypes.shape(),
        selectedZones: PropTypes.arrayOf(PropTypes.shape()),
    };

    static defaultProps = {
        dispatch: () => { },
        zones: [],
        selectedZones: [],
    };

    constructor(props) {
        super(props);

        this.state = {
            containerWidth: 0,
            containerHeight: 0,
        };
    }

    render() {
        const { containerWidth, containerHeight } = this.state;
        const { zones, dispatch, selectedZones } = this.props;
        const zonesCount = zones.size;

        return (
            <div className="AppLayout-Admin-Tab tab-admin-zones">
                <AutoSizer
                    onResize={({ height, width }) => {
                        this.setState({
                            containerWidth: width,
                            containerHeight: height,
                        });
                    }}
                >
                    {({ height, width }) => (
                        <Fragment>
                            <TableTitle
                                totalCount={zonesCount}
                                width={containerWidth}
                            />
                            <VirtualTable
                                defaultSortName="name"
                                defaultSortOrder={SortDirection.ASC}
                                tableSize={{
                                    width: containerWidth,
                                    height: containerHeight,
                                    headerHeight: 40,
                                    rowHeight: 40,
                                }}
                                allItems={zones}
                                totalCount={zonesCount}
                                selectedItems={selectedZones}
                                dispatch={dispatch}
                            />
                        </Fragment>
                    )}
                </AutoSizer>
            </div>
        );
    }
}

const createDeepEqualSelector = createSelectorCreator(
    defaultMemoize,
    isEqual,
);

const getFilteredZones = createDeepEqualSelector(
    [Zone.getAllZonesArray, Sublocation.getAllSublocations, Location.getAllLocations, ZonePrototype.getAllZonePrototypes],
    (zones, sublocations, locations, zonesPrototypes) => {
        const zonesMap = new Map();
        zones.forEach((zone) => {
            if (!zone.dynamic) {
                zonesMap.set(zone.uid, zone.formattedZone({ locations, sublocations, zonesPrototypes }));
            }
        });
        return zonesMap;
    },
);

const mapStateToProps = (state, { intl }) => ({
    zones: getFilteredZones(state, intl),
    selectedZones: state.adminPage.selectedItems.map(zone => new Zone(zone)),
});

export default connect(mapStateToProps)(injectIntl(ZonesTable));
