import React, { Component } from 'react';
import ScrollBar from 'react-scrollbar';
import { connect } from 'react-redux';
import PropTypes from 'prop-types';
import { injectIntl } from 'react-intl';
import { SHIFTS_MODAL, TABLES } from 'common/ui-labels';
import CurrentShiftSummaryTableData from 'stores/classes/CurrentShiftSummaryTableData';
import { minToMs } from 'utils/timeUtils';
import * as api from 'api';
import { errorHandlerGetRequest } from 'stores/actions/errorHandlerActions';
import { ACTIONS } from 'common/constants';
import Header from './Header';
import SummaryTable from './SummaryTable';


class ShiftSummaryDashboard extends Component {
    static propTypes = {
        locationTableData: PropTypes.shape({}),
        intl: PropTypes.shape({
            formatMessage: PropTypes.func,
        }),
        widget: PropTypes.shape({}),
        dispatch: PropTypes.func,
        selectedLocation: PropTypes.shape({}),
        currentActiveShift: PropTypes.shape({}),
    };

    static defaultProps = {
        locationTableData: {},
        intl: {
            formatMessage: () => {
            },
        },
        widget: {},
        dispatch: () => {},
        selectedLocation: {},
        currentActiveShift: null,
    };

    constructor() {
        super();
        this.state = {
            updatedAt: null,
            loading: false,
        };
        this.interval = null;
        this.now = Date.now();
    }

    componentDidMount() {
        this.getZonesInfo();

        clearInterval(this.interval);
        this.interval = setInterval(() => {
            this.getZonesInfo();
            this.now = Date.now();
        }, minToMs(3));
    }

    componentWillUnmount() {
        clearInterval(this.interval);
    }

    getZonesInfo() {
        this.setState({ loading: true });
        api.getZones()
            .then((zones) => {
                this.props.dispatch({ type: ACTIONS.ZONES_SET, params: zones });
                this.setState({ updatedAt: new Date().toISOString(), loading: false });
            })
            .catch((err) => {
                this.setState({ loading: false });
                errorHandlerGetRequest(err, this.props.dispatch);
            });
    }

    render() {
        const {
            locationTableData,
            intl,
            selectedLocation,
            currentActiveShift,
            widget: { isExpanded },
        } = this.props;

        const tableData = selectedLocation && locationTableData[selectedLocation.uid];

        if (tableData && currentActiveShift) {
            tableData.people_expected = currentActiveShift.expected_person || SHIFTS_MODAL.NOT_SPECIFIED_TXT(intl);
            tableData.vehicle_total = currentActiveShift.expected_vehicles || SHIFTS_MODAL.NOT_SPECIFIED_TXT(intl);
            tableData.others_expected = currentActiveShift.expected_others || SHIFTS_MODAL.NOT_SPECIFIED_TXT(intl);
        }

        if (!currentActiveShift) {
            return (
                <div
                    className="text-center"
                    data-cy="current-shift-no-data"
                >
                    {TABLES.NO_DATA_TO_DISPLAY}
                </div>
            );
        }

        return (
            <ScrollBar
                className={`DashboardWidget ${isExpanded ? 'expanded' : ''}`}
                contentClassName="content"
                smoothScrolling
                horizontal={false}
            >
                <Header
                    people={tableData.people_active}
                    others={tableData.others_actual}
                    vehicles={tableData.vehicle_active}
                    isExpanded={isExpanded}
                    loading={this.state.loading}
                    updatedAt={this.state.updatedAt}
                />
                {
                    isExpanded && <SummaryTable tableData={[tableData]} />
                }
            </ScrollBar>
        );
    }
}

const mapStateToProps = (state) => {
    const { locations, sublocations } = state.entities;
    const trackables = state.entities.trackables.byUid;
    const zones = state.entities.zones.byUid;
    const currentShiftSummaryTableData = new CurrentShiftSummaryTableData(locations, sublocations, trackables, zones);
    const { selectedLocation } = state.siteView;

    return {
        selectedLocation,
        locationTableData: currentShiftSummaryTableData.getSummaryTableDataArray(),
    };
};

export default connect(mapStateToProps)(injectIntl(ShiftSummaryDashboard));
