import React, { Fragment, useEffect, useState } from 'react';
import PropTypes from 'prop-types';
import { injectIntl } from 'react-intl';
import { durationFormatting } from 'utils/formatters';
import { TIMELINE_ACTIONS, COMPONENTS } from 'common/ui-labels';

import TotalCountLabelValue from '../TotalCountLabelValue';


function LastUpdated(props) {
    const { lastUpdated, loading } = props;
    if (loading) return (<i className="fa fa-spinner fa-spin" style={{ color: 'grey' }} />);
    return <span style={{ fontWeight: 'bold' }}>{lastUpdated || '-'}</span>;
}

LastUpdated.propTypes = {
    lastUpdated: PropTypes.string,
    loading: PropTypes.bool,
};

LastUpdated.defaultProps = {
    lastUpdated: '',
    loading: false,
};

function CountInfo(props) {
    const { active, idle, loading, isExpanded, titleLabel } = props;
    if (loading) return (<i className="fa fa-spinner fa-spin" style={{ color: 'grey' }} />);
    return (<Fragment>
        <div className="d-flex va-middle ha-end col-md-5 font-big">{titleLabel}:</div>
        <div className="d-flex va-middle ha-center col-md-5 widget-counters-bracket">
            <TotalCountLabelValue
                dividerClass="with-dash"
                label={TIMELINE_ACTIONS.ACTIVE}
                isColumnDirection={!isExpanded}
                dataCy={`dashboard-person-activity-widget-${titleLabel.toLowerCase()}-active`}
                value={active}
            />
            <div style={{ marginRight: '20px' }}>{ isExpanded && ', '}</div>
            <TotalCountLabelValue
                dividerClass="with-dash"
                isColumnDirection={!isExpanded}
                label={TIMELINE_ACTIONS.IDLE}
                dataCy={`dashboard-person-activity-widget-${titleLabel.toLowerCase()}-idle`}
                value={idle}
            />
        </div>
    </Fragment>);
}

CountInfo.propTypes = {
    titleLabel: PropTypes.string,
    active: PropTypes.number,
    idle: PropTypes.number,
    loading: PropTypes.bool,
    isExpanded: PropTypes.bool,
};

CountInfo.defaultProps = {
    titleLabel: '',
    active: 0,
    idle: 0,
    loading: false,
    isExpanded: false,
};

function Header(props) {
    const { updatedAt, drivers, nonDrivers, loading, intl, isExpanded } = props;

    const { LAST_UPDATED } = COMPONENTS.FIELDS.NODE;

    const [lastUpdated, setLastUpdated] = useState(null);
    const getDuration = () => (updatedAt ? durationFormatting(new Date() - new Date(updatedAt), intl).text : null);

    useEffect(() => {
        setLastUpdated(getDuration());
        const interval = setInterval(() => {
            if (updatedAt) {
                setLastUpdated(getDuration());
            } else {
                setLastUpdated(null);
            }
        }, 10000);
        return function cleanup() {
            clearInterval(interval);
        };
    }, [updatedAt, loading]);

    return (
        <Fragment>
            <div className="widget-body-last-updated">
                <span className="capitalizedText">{LAST_UPDATED}: </span>
                <LastUpdated loading={loading} lastUpdated={lastUpdated} />
            </div>
            <div className="widget-counters row">
                <div className="d-flex va-middle ha-end col-md-6 row">
                    <CountInfo
                        titleLabel={'Drivers'}
                        loading={loading}
                        isExpanded={isExpanded}
                        active={drivers.active}
                        idle={drivers.idle}
                    />
                </div>
                <div className="d-flex va-middle ha-start col-md-6 row">
                    <CountInfo
                        titleLabel={'Non-Drivers'}
                        loading={loading}
                        isExpanded={isExpanded}
                        active={nonDrivers.active}
                        idle={nonDrivers.idle}
                    />
                </div>
            </div>
        </Fragment>
    );
}

Header.propTypes = {
    updatedAt: PropTypes.string,
    drivers: PropTypes.shape({
        active: PropTypes.number,
        idle: PropTypes.number,
    }),
    nonDrivers: PropTypes.shape({
        active: PropTypes.number,
        idle: PropTypes.number,
    }),
    intl: PropTypes.shape({
        formatMessage: PropTypes.func,
    }),
    loading: PropTypes.bool,
    isExpanded: PropTypes.bool,
};

Header.defaultProps = {
    updatedAt: '',
    drivers: {
        active: 0,
        idle: 0,
    },
    nonDrivers: PropTypes.shape({
        active: 0,
        idle: 0,
    }),
    intl: {
        formatMessage: () => {
        },
    },
    loading: false,
    isExpanded: false,
};

export default injectIntl(Header);
