import React, { Component } from 'react';
import { connect } from 'react-redux';
import { injectIntl } from 'react-intl';
import ScrollBar from 'react-scrollbar';
import PropTypes from 'prop-types';
import { DASHBOARD, WIN_ANCHOR_TYPE } from 'common/constants';
import { batteryStatus, formatInactivity, isAnchorOrBridge } from 'utils';
import { Trackable, WinAnchor, Association, Node } from 'stores/classes';
import { Node as NodeSelector } from 'stores/classes/Selectors';
import AnchorsTable from './AnchorsTable';
import TagsTable from './TagsTable';
import Header from './Header';


class WINStatusWidget extends Component {
    static propTypes = {
        dispatch: PropTypes.func,
        winWidget: PropTypes.shape({
            color: PropTypes.string,
        }),
        nodes: PropTypes.arrayOf(PropTypes.object),
        winAnchors: PropTypes.arrayOf(PropTypes.object),
        countOfLowBattery: PropTypes.number,
        anchorsAndBridges: PropTypes.number,
        inactiveAnchorsAndBridges: PropTypes.number,
        intl: PropTypes.shape({
            formatMessage: PropTypes.func,
        }),
        widget: PropTypes.shape({}),
    };
    
    static defaultProps = {
        dispatch: () => {
        },
        winWidget: {
            color: '#bebebe',
        },
        nodes: [{}],
        winAnchors: [{}],
        countOfLowBattery: 0,
        anchorsAndBridges: 0,
        inactiveAnchorsAndBridges: 0,
        intl: {
            formatMessage: () => { },
        },
        widget: {},
    };

    formatBattery = (battery, node) => {
        const { intl } = this.props;
        return (node.responsive) ? batteryStatus(battery, intl) : '-';
    };
    
    render() {
        const { winWidget, winAnchors, nodes, anchorsAndBridges, inactiveAnchorsAndBridges, inactiveTags, countOfLowBattery, intl, widget: { isExpanded } } = this.props;
        
        return (
            <ScrollBar
                className="DashboardWidget"
                contentClassName="content"
                smoothScrolling
                horizontal={false}
            >
                <div className="WINWidget WINWidget-Tabs">
                    <Header
                        anchorsAndBridges={anchorsAndBridges}
                        inactiveAnchorsAndBridges={inactiveAnchorsAndBridges}
                        tags={nodes.length}
                        inactiveTags={inactiveTags}
                        lowBattery={countOfLowBattery}
                        isExpanded={isExpanded}
                    />
                    {isExpanded && <div className="d-flex">
                        <AnchorsTable
                            winAnchors={winAnchors}
                            color={winWidget.color}
                            formatInactivity={data => formatInactivity(data, intl)}
                        />
                        <TagsTable
                            nodes={nodes}
                            color={winWidget.color}
                            formatInactivity={formatInactivity}
                            formatBattery={this.formatBattery}
                        />
                    </div>}
                </div>
            </ScrollBar>
        );
    }
}

const mapStateToProps = (state) => {
    const project = state.appContext.project || {};
    let anchorsAndBridges = 0;
    let inactiveAnchorsAndBridges = 0;
    let inactiveTags = 0;
    let countOfLowBattery = 0;
    const allTrackables = Trackable.getAllTrackablesArray(state);
    const allAssociations = Association.getAllAssociations(state);

    const winAnchors = WinAnchor.getAllWinAnchorsArray(state)
        .reduce((memo, anchor) => {
            const anchorInProject = anchor.project_uid === project.uid;
            if (anchorInProject) {
                const node = anchor.node && anchor.node.mac_address ? NodeSelector.getNodeByMacAddress(anchor.node.mac_address) : null;

                if (node) {
                    const props = anchor.getProperties();
                    props.last_heard = node.last_heard;
                    props.battery = node.battery;
                    memo.push(new WinAnchor(props));
                    anchorsAndBridges += isAnchorOrBridge(anchor.type) ? 1 : 0;
                    inactiveAnchorsAndBridges += isAnchorOrBridge(anchor.type) && !anchor.node.responsive ? 1 : 0;
                }
            }
            return memo;
        }, []);

    // only active nodes associated with trackables
    const nodes = Node.getAllNodesArray(state)
        .reduce((memo, node) => {
            const nodeInProject = node.project_uid === project.uid;
            const nodeWithAssociation = node.mac_address && !!allAssociations.byMac[node.mac_address.toUpperCase()];

            if (nodeInProject && nodeWithAssociation) {
                const trackable = allTrackables.find(to => to.node && to.node.mac_address && to.node.mac_address === node.mac_address);

                if (trackable) {
                    const props = node.getProperties();
                    props.trackable_name = trackable.name || '';
                    props.responsive = trackable.isResponsive(state.playback.isOpenPlayBackControl);
                    if (props.battery && props.battery.pr < 10 && props.responsive) {
                        countOfLowBattery += 1;
                    }

                    inactiveTags += !props.responsive ? 1 : 0;

                    memo.push(props);
                }
            }
            return memo;
        }, []);

    const winWidget = state.dashboard.widgets.find(widget => widget.name === DASHBOARD.WIN_STATUS_WIDGET);
    
    return {
        anchorsAndBridges,
        inactiveAnchorsAndBridges,
        countOfLowBattery,
        winAnchors,
        nodes,
        inactiveTags,
        winWidget,
    };
};

export default connect(mapStateToProps)(injectIntl(WINStatusWidget));
