import React from 'react';
import PropTypes from 'prop-types';
import { injectIntl } from 'react-intl';
import { Column, Table } from 'react-virtualized';
import 'react-virtualized/styles.css';
import withSortSelect from 'components/HOC/withSortSelect';
import { virtualizedHeaderRenderer } from 'utils/htmlUtils';
import { REPORT_TABLE, COMPONENTS } from 'common/ui-labels';
import { getZoneTypeName } from 'utils/zoneUtils';
import { zoneSeverityColor } from 'utils/simpleUtils';
import { durationHumanize } from 'utils/timeUtils';
import { ZONE_SEVERITY } from 'common/constants';


class VirtualTable extends React.Component {
    static propTypes = {
        // Props from HOC
        sort: PropTypes.func,
        sortBy: PropTypes.string,
        trStyleFormat: PropTypes.func,
        sortDisplayItems: PropTypes.func,
        sortDirection: PropTypes.string,
        
        // Props from parent
        allItems: PropTypes.shape(),
        totalCount: PropTypes.number,
        tableSize: PropTypes.shape(),
        dispatch: PropTypes.func,
        trClassFormat: PropTypes.func,
        intl: PropTypes.shape({
            formatMessage: PropTypes.func,
        }),
    };
    
    static defaultProps = {
        // Props from HOC
        sort: () => { },
        sortBy: '',
        trStyleFormat: () => { },
        sortDisplayItems: () => { },
        sortDirection: '',
        
        // Props from parent
        allItems: new Map(),
        totalCount: 0,
        tableSize: {},
        dispatch: () => { },
        trClassFormat: () => { },
        intl: { formatMessage: () => { } },
    };
    
    constructor(props) {
        super(props);
        this.totalCount = props.totalCount;
        this.dispatch = props.dispatch;
        
        // HOC Methods
        this.trClassFormat = props.trClassFormat.bind(this);
        this.trStyleFormat = props.trStyleFormat.bind(this);
        this.sort = props.sort.bind(this);
        this.sortDisplayItems = props.sortDisplayItems.bind(this);
    }
    
    formatZoneAlarmLevel = row => (
        <div style={{
            backgroundColor: zoneSeverityColor(row.severity),
            textAlign: 'center',
            width: '90%',
            color: row.severity === ZONE_SEVERITY.GUARDED ? 'black' : 'white',
        }}
        >
            {row.severity}
        </div>
    );
    
    render() {
        const { sortBy, sortDirection, tableSize, allItems } = this.props;
        const displayItems = this.sortDisplayItems([...allItems.values()]);
        const displayCount = displayItems.length;
        
        return (
            <section style={{ width: tableSize.width }}>
                <Table
                    sortBy={sortBy}
                    sort={this.sort}
                    sortDirection={sortDirection}
                    className="Virtual-Table-Big"
                    width={tableSize.width} // required
                    height={tableSize.height} // required
                    gridClassName="Virtual-Table-Grid"
                    rowHeight={tableSize.rowHeight} // required
                    headerHeight={tableSize.headerHeight} // required
                    headerClassName="Virtual-Table-Header react-bs-container-header"
                    rowClassName={({ index }) => this.trClassFormat(displayItems, { index })}
                    rowStyle={({ index }) => this.trStyleFormat(displayItems, { index })}
                    rowCount={displayCount} // required
                    rowGetter={({ index }) => displayItems[index]} // required
                >
                    <Column
                        label={<span title="">#</span>}
                        headerRenderer={virtualizedHeaderRenderer}
                        dataKey="number"
                        cellRenderer={({ rowIndex }) => <div>{rowIndex + 1}</div>}
                        width={30}
                        disableSort
                        className="Virtual-Table-Column-Left text-small"
                    />
                    <Column
                        label={<span title="" className="text-capitalize">{REPORT_TABLE.ZONE_NAME}</span>}
                        headerRenderer={virtualizedHeaderRenderer}
                        dataKey="zone_name"
                        width={200}
                        className="Virtual-Table-Column-Left text-small"
                    />
                    <Column
                        label={<span title="">{REPORT_TABLE.ALARMS_TABLE.HEADER_SEVERITY}</span>}
                        headerRenderer={virtualizedHeaderRenderer}
                        cellRenderer={({ rowData }) => this.formatZoneAlarmLevel(rowData)}
                        dataKey="severity"
                        width={200}
                        className="Virtual-Table-Column-Center"
                    />
                    <Column
                        label={<span title="">{COMPONENTS.FIELDS.COMMON.TYPE}</span>}
                        headerRenderer={virtualizedHeaderRenderer}
                        dataKey="zone_type"
                        cellRenderer={({ rowData, cellData }) => {
                            const localizeZoneType = getZoneTypeName([], rowData.zone_type, this.props.intl);
                            return localizeZoneType || cellData;
                        }}
                        width={200}
                        className="Virtual-Table-Column-Center"
                    />
                    <Column
                        label={<span title="">{REPORT_TABLE.ALARMS_TABLE.HEADER_ALARMS_COUNT}</span>}
                        headerRenderer={virtualizedHeaderRenderer}
                        dataKey="count"
                        width={200}
                        className="Virtual-Table-Column-Center"
                    />
                    <Column
                        label={<span title="">{REPORT_TABLE.HEADER_DURATION}</span>}
                        headerRenderer={virtualizedHeaderRenderer}
                        cellRenderer={({ rowData }) => durationHumanize(rowData.accumulated_duration)}
                        dataKey="accumulated_duration"
                        width={tableSize.width - 800}
                        className="Virtual-Table-Column-Center"
                    />
                </Table>
            </section>
        );
    }
}

export default injectIntl(withSortSelect(VirtualTable));
