import React from 'react';
import PropTypes from 'prop-types';
import { connect } from 'react-redux';
import { injectIntl } from 'react-intl';
import _Has from 'lodash/has';
import { BootstrapTable, TableHeaderColumn, SearchField } from 'react-bootstrap-table';
import { DEFAULT_VALUES, TRACKABLE_CATEGORY } from 'common/constants';
import { TABLES, COMPONENTS, REPORTS_SIDE_BAR, NAMES_OF_TABS } from 'common/ui-labels';
import { trOddEvenClassFormat } from 'utils/htmlUtils';
import ExpandableTable from './ExpandableTable';


class DistanceTable extends React.Component {
    static propTypes = {
        data: PropTypes.arrayOf(PropTypes.shape({})),
        intl: PropTypes.shape({
            formatMessage: PropTypes.func,
        }),
        status: PropTypes.number,
        timezone: PropTypes.string,
        options: PropTypes.shape({
            ranges: PropTypes.arrayOf(PropTypes.number),
            from_iso: PropTypes.string,
            shift_name: PropTypes.string,
            from: PropTypes.string,
            to: PropTypes.string,
        }),
    };
    
    static defaultProps = {
        data: [],
        intl: {
            formatMessage: () => {},
        },
        status: 0,
        timezone: 'Etc/GMT',
        options: {
            ranges: [],
            from_iso: '',
            shift_name: '',
            from: '',
            to: '',
        },
    };
    
    isExpandableRow = () => true;

    createAddDeleteBtn = () => {
        if (!this.props.options || this.props.status === 0) return <section />;
        const trackables = this.props.data.length;
        return (
            <div className="row" style={{ marginLeft: '5px' }}>
                <div className="row">
                    <div className="col-md-6"><b>{NAMES_OF_TABS.TRACKABLE_OBJECTS}:</b> {trackables}</div>
                </div>
            </div>);
    };
    
    categoryFormatter = (cell) => {
        if (!cell) {
            return TRACKABLE_CATEGORY.UNSPECIFIED;
        }
        return cell;
    }

    expandComponent = row => this.props.options && <ExpandableTable
        startTime={_Has(this.props.options, 'from_iso') ? new Date(this.props.options.from_iso).getTime() : 0}
        ranges={this.props.options ? this.props.options.ranges : []}
        distances={row.distances}
        timezone={this.props.timezone}
    />;
    
    trClassFormat = () => 'Table-Row-CursorPointer';
    
    createCustomSearchField = intl => (
        <SearchField
            data-id="projects-search"
            data-cy="distance-table-search"
            placeholder={TABLES.SEARCH(intl)}
        />
    );
    
    render() {
        const { data } = this.props;
        const options = {
            onlyOneExpanding: true,
            noDataText: (TABLES.NO_DATA_TO_DISPLAY),
            expandBy: 'column',
            expandRowBgColor: 'white',
            searchField: this.createCustomSearchField.bind(this, this.props.intl),
            btnGroup: this.createAddDeleteBtn,
            beforeShowError: () => true,
        };
        const headerStyle = {
            marginBottom: '0',
        };
        const selectRow = {
            mode: 'radio',
            clickToSelect: true,
            hideSelectColumn: true,
            bgColor: DEFAULT_VALUES.SELECTED_ROW_COLOR,
            clickToExpand: true,
        };
        
        return (
            <section>
                <BootstrapTable
                    trClassName={(row, rowIndex) => (`${trOddEvenClassFormat(row, rowIndex)} ${this.trClassFormat()}`)}
                    tableBodyClass="automation-companies-table"
                    data={data}
                    options={options}
                    selectRow={selectRow}
                    headerStyle={headerStyle}
                    bodyStyle={{
                        maxHeight: window.innerHeight * 0.5,
                        overflow: 'scroll',
                        overflowX: 'auto',
                        overflowY: 'auto',
                    }}
                    expandableRow={this.isExpandableRow}
                    expandComponent={this.expandComponent}
                    bodyContainerClass="Admin-BootstrapTable"
                    expandColumnOptions={{
                        expandColumnVisible: false,
                        columnWidth: 30,
                    }}
                    hover
                    condensed
                    search
                    className="middle-align"
                >
                    <TableHeaderColumn
                        dataField="uid"
                        searchable={false}
                        hidden
                        isKey
                    >
                        Uid
                    </TableHeaderColumn>
                    <TableHeaderColumn
                        dataField="id"
                        tdAttr={{ 'data-id': 'id' }}
                        dataFormat={(cell, row, enumObject, index) => index + 1}
                        headerAlign="center"
                        dataAlign="center"
                        width="20"
                    >
                        #
                    </TableHeaderColumn>
                    <TableHeaderColumn
                        dataField="name"
                        tdAttr={{ 'data-id': 'trackable-name' }}
                        dataSort
                        headerAlign="center"
                        dataAlign="center"
                        width="140"
                        searchable
                    >
                        {COMPONENTS.FIELDS.COMMON.NAME}
                    </TableHeaderColumn>
                    <TableHeaderColumn
                        dataField="category"
                        tdAttr={{ 'data-id': 'trackable-category' }}
                        dataSort
                        headerAlign="center"
                        dataAlign="center"
                        dataFormat={this.categoryFormatter}
                        width="140"
                        searchable
                    >
                        {COMPONENTS.FIELDS.NODE.CATEGORY}
                    </TableHeaderColumn>
                    <TableHeaderColumn
                        dataField="formattedTotalDistance"
                        tdAttr={{ 'data-id': 'trackable-distance' }}
                        dataSort
                        headerAlign="center"
                        dataAlign="center"
                        width="140"
                    >
                        {REPORTS_SIDE_BAR.TRAVEL_DISTANCE}
                    </TableHeaderColumn>
                </BootstrapTable>
            </section>
        );
    }
}

const mapStateToProps = state => ({
    data: state.reports.trackableDistanceReport.tableData,
    options: state.reports.trackableDistanceReport.report,
});

export default connect(mapStateToProps)(injectIntl(DistanceTable));
