import React from 'react';
import PropTypes from 'prop-types';
import { injectIntl } from 'react-intl';
import { Column, Table } from 'react-virtualized';
import 'react-virtualized/styles.css';
import withSortSelect from 'components/HOC/withSortSelect';
import { virtualizedHeaderRenderer } from 'utils/htmlUtils';
import { zoneSeverityColor } from 'utils/simpleUtils';
import { humanizeDistance } from 'utils/formatters';
import { ZONE_SEVERITY } from 'common/constants';
import { REPORT_TABLE } from 'common/ui-labels';


class VirtualTable extends React.Component {
    static propTypes = {
        // Props from HOC
        sort: PropTypes.func,
        sortBy: PropTypes.string,
        trStyleFormat: PropTypes.func,
        sortDisplayItems: PropTypes.func,
        sortDirection: PropTypes.string,
        
        // Props from parent
        allItems: PropTypes.arrayOf(PropTypes.shape({})),
        totalCount: PropTypes.number,
        tableSize: PropTypes.shape(),
        dispatch: PropTypes.func,
        trClassFormat: PropTypes.func,
        timezone: PropTypes.string,
    };
    
    static defaultProps = {
        // Props from HOC
        sort: () => { },
        sortBy: '',
        trStyleFormat: () => { },
        sortDisplayItems: () => { },
        sortDirection: '',
        
        // Props from parent
        allItems: [],
        totalCount: 0,
        tableSize: {},
        dispatch: () => { },
        trClassFormat: () => { },
        timezone: 'Etc/GMT',
    };
    
    constructor(props) {
        super(props);
        this.totalCount = props.totalCount;
        this.dispatch = props.dispatch;
        
        // HOC Methods
        this.trClassFormat = props.trClassFormat.bind(this);
        this.trStyleFormat = props.trStyleFormat.bind(this);
        this.sort = props.sort.bind(this);
        this.sortDisplayItems = props.sortDisplayItems.bind(this);
    }
    
    formatZoneAlarmLevel = row => (
        <div style={{
            backgroundColor: zoneSeverityColor(row.severity),
            textAlign: 'center',
            width: '90%',
            color: row.severity === ZONE_SEVERITY.GUARDED ? 'black' : 'white',
        }}
        >
            {row.severity}
        </div>
    );
    
    render() {
        const { sortBy, sortDirection, tableSize, allItems } = this.props;
        const displayItems = this.sortDisplayItems(allItems);
        const displayCount = displayItems.length;
        
        return (
            <section style={{ width: tableSize.width }}>
                <Table
                    sortBy={sortBy}
                    sort={this.sort}
                    sortDirection={sortDirection}
                    className="Virtual-Table-Big"
                    width={tableSize.width} // required
                    height={tableSize.height} // required
                    gridClassName="Virtual-Table-Grid"
                    rowHeight={tableSize.rowHeight} // required
                    headerHeight={tableSize.headerHeight} // required
                    headerClassName="Virtual-Table-Header react-bs-container-header"
                    rowClassName={({ index }) => this.trClassFormat(displayItems, { index })}
                    rowStyle={({ index }) => this.trStyleFormat(displayItems, { index })}
                    rowCount={displayCount} // required
                    rowGetter={({ index }) => displayItems[index]} // required
                >
                    <Column
                        label={<span title="">{REPORT_TABLE.DISTANCE_TABLE.HEADER_TIME_PERIOD}</span>}
                        headerRenderer={virtualizedHeaderRenderer}
                        dataKey="range"
                        width={tableSize.width / 2}
                        className="Virtual-Table-Column-Center text-small"
                    />
                    <Column
                        label={<span title="">{REPORT_TABLE.DISTANCE_TABLE.HEADER_DISTANCE}</span>}
                        headerRenderer={virtualizedHeaderRenderer}
                        dataKey="distance"
                        cellRenderer={({ cellData }) => humanizeDistance(cellData, 'm', 'ft')}
                        width={tableSize.width / 2}
                        className="Virtual-Table-Column-Center text-small"
                    />
                </Table>
            </section>
        );
    }
}

export default injectIntl(withSortSelect(VirtualTable));
