import React from 'react';
import PropTypes from 'prop-types';
import { connect } from 'react-redux';
import { injectIntl } from 'react-intl';
import { BootstrapTable, TableHeaderColumn, SearchField } from 'react-bootstrap-table';
import { DEFAULT_VALUES, REPORT_TABLE, TABLES, DB_NAMES } from 'common';
import { trOddEvenClassFormat } from 'utils/htmlUtils';
import { durationHumanize } from 'utils/timeUtils';
import ExpandableTable from './ExpandableTable';


class PwsTable extends React.Component {
    static propTypes = {
        data: PropTypes.arrayOf(PropTypes.shape({})),
        intl: PropTypes.shape({
            formatMessage: PropTypes.func,
        }),
        status: PropTypes.number,
        report: PropTypes.shape({
            objectName: PropTypes.string,
            start: PropTypes.string,
            end: PropTypes.string,
        }),
        contactNames: PropTypes.shape(),
        totalDuration: PropTypes.number,
    };
    
    static defaultProps = {
        data: [],
        intl: {
            formatMessage: () => {},
        },
        status: 0,
        report: {
            objectName: '',
            start: '',
            end: '',
        },
        contactNames: {},
        totalDuration: 0,
    };
    
    isExpandableRow = () => true;

    createAddDeleteBtn = () => {
        if (!this.props.report || this.props.status === 0) return <section />;
        const duration = durationHumanize(this.props.totalDuration) || 0;
        const contacts = this.props.data.length;
        return (
            <div className="row" style={{ marginLeft: '5px' }}>
                <div className="row">
                    <div className="col-md-6"><b>{DB_NAMES.PERSON(this.props.intl)}:</b> {this.props.report.objectName || '-'}</div>
                    <div className="col-md-6"><b>{REPORT_TABLE.PWS_TABLE.CONTACTS}:</b> {contacts}</div>
                </div>
                <div className="row">
                    <div className="col-md-6"><b>{REPORT_TABLE.DATE_RANGE}:</b> {this.props.report.start}-{this.props.report.end}</div>
                    <div className="col-md-6"><b>{REPORT_TABLE.PWS_TABLE.TOTAL_DURATION}:</b> {duration}</div>
                </div>
            </div>);
    };
    
    nameFormatter = (cell, row) => {
        if (this.props.contactNames[cell] > 1) {
            return `${cell}(${row.uid})`;
        }
        return cell;
    }
    
    expandComponent = row => <ExpandableTable data={row.subTable} />
    
    trClassFormat = () => 'Table-Row-CursorPointer';
    
    createCustomSearchField = intl => (
        <SearchField
            data-id="projects-search"
            data-cy="pwd-table-search"
            placeholder={TABLES.SEARCH(intl)}
        />
    );
    
    render() {
        const { data } = this.props;
        const options = {
            onlyOneExpanding: true,
            noDataText: (TABLES.NO_DATA_TO_DISPLAY),
            expandBy: 'column',
            expandRowBgColor: 'white',
            searchField: this.createCustomSearchField.bind(this, this.props.intl),
            btnGroup: this.createAddDeleteBtn,
            beforeShowError: () => true,
        };
        const headerStyle = {
            marginBottom: '0',
        };
        const selectRow = {
            mode: 'radio',
            clickToSelect: true,
            hideSelectColumn: true,
            bgColor: DEFAULT_VALUES.SELECTED_ROW_COLOR,
            clickToExpand: true,
        };
        
        return (
            <section>
                <BootstrapTable
                    trClassName={(row, rowIndex) => (`${trOddEvenClassFormat(row, rowIndex)} ${this.trClassFormat()}`)}
                    tableBodyClass="automation-companies-table"
                    data={data}
                    options={options}
                    selectRow={selectRow}
                    headerStyle={headerStyle}
                    bodyStyle={{
                        maxHeight: window.innerHeight * 0.5,
                        overflow: 'scroll',
                        overflowX: 'auto',
                        overflowY: 'auto',
                    }}
                    expandableRow={this.isExpandableRow}
                    expandComponent={this.expandComponent}
                    bodyContainerClass="Admin-BootstrapTable"
                    expandColumnOptions={{
                        expandColumnVisible: false,
                        columnWidth: 30,
                    }}
                    hover
                    condensed
                    search
                    className="middle-align"
                >
                    <TableHeaderColumn
                        dataField="uid"
                        searchable={false}
                        hidden
                        isKey
                    >
                        Uid
                    </TableHeaderColumn>
                    <TableHeaderColumn
                        dataField="id"
                        tdAttr={{ 'data-id': 'contact-trackable-name' }}
                        headerAlign="center"
                        dataAlign="center"
                        width="20"
                    >
                        #
                    </TableHeaderColumn>
                    <TableHeaderColumn
                        dataField="contact_name"
                        tdAttr={{ 'data-id': 'contact-trackable-name' }}
                        dataFormat={this.nameFormatter}
                        dataSort
                        headerAlign="center"
                        dataAlign="center"
                        width="140"
                        searchable
                    >
                        {REPORT_TABLE.PWS_TABLE.HEADER_NAME}
                    </TableHeaderColumn>
                    <TableHeaderColumn
                        dataField="formattedDuration"
                        tdAttr={{ 'data-id': 'company-table-name' }}
                        dataSort
                        headerAlign="center"
                        dataAlign="center"
                        width="140"
                    >
                        {REPORT_TABLE.HEADER_DURATION}
                    </TableHeaderColumn>
                    <TableHeaderColumn
                        dataField="count"
                        tdAttr={{ 'data-id': 'company-table-name' }}
                        dataSort
                        headerAlign="center"
                        dataAlign="center"
                        width="140"
                    >
                        {REPORT_TABLE.PWS_TABLE.CONTACTS}
                    </TableHeaderColumn>
                </BootstrapTable>
            </section>
        );
    }
}

const mapStateToProps = (state) => {
    const report = state.reports.pwsReport.report || {};
    return ({
        data: state.reports.pwsReport.tableData,
        report,
        contactNames: report.contactNames,
        totalDuration: report.totalDuration,
    });
};

export default connect(mapStateToProps)(injectIntl(PwsTable));
