import React, { Component } from 'react';
import PropTypes from 'prop-types';
import { connect } from 'react-redux';
import { injectIntl } from 'react-intl';
import { CLIENT_REPORT_TYPES, COMPONENTS, SELECT_A_TXT_FUNC } from 'common';
import { Trackable, Location } from 'stores/classes';
import { getReportParams } from 'utils/utils';
import { getPwsTrackablesForPeriod, loadingTrackables } from 'stores/actions/reportsActions/pwsReportActions';
import AdvancedSelect from './AdvancedSelect';


class Filter extends Component {
    static propTypes = {
        reportType: PropTypes.string,
        from: PropTypes.string,
        to: PropTypes.string,
        updateTrackablesList: PropTypes.func,
    };
    
    static defaultProps = {
        reportType: '',
        from: '',
        to: '',
        updateTrackablesList: () => {},
    };
    
    componentDidUpdate(prevProps) {
        const fromChanged = this.props.from !== prevProps.from;
        const toChanged = this.props.to !== prevProps.to;
        const hasDates = this.props.from && this.props.to;
        
        if (this.props.reportType === CLIENT_REPORT_TYPES.PWS_REPORT && hasDates) {
            this.handlePwsPropsUpdate(fromChanged, toChanged);
        }
    }

    getPwsReportFilters = ({ trackables, intl, isLoading }) => (
        <AdvancedSelect
            singleSelect
            isLoading={isLoading}
            param_name="trackables"
            options={trackables}
            placeholder={SELECT_A_TXT_FUNC(intl, COMPONENTS.FIELDS.NODE.TRACKABLE_FUNC(intl))}
            reportType={this.props.reportType}
        />
    );
    
    getAttendanceReportFilters = ({ locations, intl }) => (
        <AdvancedSelect
            singleSelect
            param_name="locations"
            options={locations}
            placeholder={SELECT_A_TXT_FUNC(intl, COMPONENTS.FIELDS.SITE.SITE_FUNC(intl))}
            reportType={this.props.reportType}
        />
    );

    handlePwsPropsUpdate = (fromChanged, toChanged) => {
        if (fromChanged || toChanged) {
            this.props.updateTrackablesList(this.props.from, this.props.to);
        }
    }

    render() {
        const { reportType } = this.props;
        
        switch (reportType) {
            case CLIENT_REPORT_TYPES.PWS_REPORT: {
                return this.getPwsReportFilters(this.props);
            }
            case CLIENT_REPORT_TYPES.TRACKABLE_DISTANCE_REPORT:
            case CLIENT_REPORT_TYPES.ZONE_ACTIVITY_REPORT:
            case CLIENT_REPORT_TYPES.ALARMS_REPORT:
            case CLIENT_REPORT_TYPES.ATTENDANCE: {
                return this.getAttendanceReportFilters(this.props);
            }
            default: {
                return null;
            }
        }
    }
}

const configureProps = (state, reportType) => {
    const reportParams = getReportParams(state, reportType);
    
    switch (reportType) {
        case CLIENT_REPORT_TYPES.PWS_REPORT: {
            const trackables = reportParams.filter.periodTrackables || Trackable.getAllTrackablesArray(state)
                .map(trackable => ({ label: trackable.name, value: trackable.uid }));
            const { from, to, isLoading } = reportParams.filter;
            
            return { trackables, from, to, isLoading };
        }
        case CLIENT_REPORT_TYPES.TRACKABLE_DISTANCE_REPORT:
        case CLIENT_REPORT_TYPES.ZONE_ACTIVITY_REPORT:
        case CLIENT_REPORT_TYPES.ALARMS_REPORT:
        case CLIENT_REPORT_TYPES.ATTENDANCE: {
            const locations = Location.getAllLocations(state)
                .map(location => ({ label: location.name, value: location.uid }));
        
            return { locations };
        }
        default: {
            return {
            
            };
        }
    }
};

const mapDispatchToProps = dispatch => ({
    updateTrackablesList: (start, end) => {
        dispatch(loadingTrackables());
        dispatch(getPwsTrackablesForPeriod({ start, end }));
    },
});

const mapStateToProps = (state, { reportType, disableSeverity }) => {
    const options = configureProps(state, reportType, disableSeverity);

    return {
        ...options,
    };
};

export default injectIntl(connect(mapStateToProps, mapDispatchToProps)(Filter));
