import React from 'react';
import { connect } from 'react-redux';
import PropTypes from 'prop-types';
import { EDIT_ZONE_MODE, SITE_VIEW_MODE } from 'common/ui-labels';
import { exitDrawRegime, finishDrawingPolygon, openZoneAddConfigurationModal, rotateZoneGeometry, setZoneWidth, startLiveMode } from 'stores/actions';
import NotificationMessage from 'components/NotificationMessage';
import { GEOMETRY_RESTRICTION_TYPES } from 'common/constants';
import { geometryFromDegreesToMillimeters, rotateGeometry, setWidthToGeometry } from 'utils/zoneUtils';
import Buttons from './Buttons';
import WidthControl from './WidthControl';
import RotationControl from './RotationControl';


const formatCreatedZone = (newZoneLatLng, rotation, sublocation_uid) => {
    const positions = geometryFromDegreesToMillimeters(newZoneLatLng);
    return {
        sublocation_uid,
        rotation,
        zone_geometry: {
            positions,
        },
    };
};


function SiteViewEditZoneMode(props) {
    const {
        rotation, width, showWidthControl, creatingZone, type, showControls, sublocation, isPolygonDrawing,
    } = props;

    const { restriction } = creatingZone;

    const isFinishButtonVisible = (restriction && Array.isArray(restriction))
        ? restriction.indexOf(GEOMETRY_RESTRICTION_TYPES.POLYGON) !== -1
        : false;

    return (
        <div className="RPP-Edit-Zone-Mode">
            <div className="Edit-Zone-Dialog">
                <div className="Title-Name">{EDIT_ZONE_MODE.EDIT_ZONE}</div>
            
                <RotationControl
                    rotation={rotation}
                    show={showControls}
                    handleZoneRotate={props.handleZoneRotate}
                />
            
                <WidthControl
                    width={width}
                    show={showControls && showWidthControl}
                    handleZoneWithChange={props.handleZoneWithChange}
                />
                <Buttons
                    width={width}
                    creatingZone={creatingZone}
                    isFinishButtonVisible={isFinishButtonVisible}
                    isFinishButtonDisabled={!isPolygonDrawing}
                    handleSubmitClick={() => {
                        props.handleSubmitEditContinue(creatingZone, rotation, width, type, sublocation);
                    }}
                    handleCancelClick={() => {
                        props.handleCancelEditContinue();
                    }}
                    handleFinishClick={() => {
                        props.handleFinishClick();
                    }}
                />
            </div>
        
            <NotificationMessage
                show
                message={SITE_VIEW_MODE.NOTIFY_MESSAGE}
            />
        </div>
    );
}

const mapDispatchToProps = dispatch => ({
    handleZoneWithChange: (width) => {
        dispatch(setZoneWidth(width));
    },
    handleZoneRotate: (angle) => {
        dispatch(rotateZoneGeometry(angle));
    },
    handleSubmitEditContinue: (creatingZone, rotation, width, type, sublocation) => {
        const zoneGeometry = {
            creatingZone,
        };

        if (type === GEOMETRY_RESTRICTION_TYPES.DIRECTIONAL) {
            const { geometry } = setWidthToGeometry(creatingZone, width);
            zoneGeometry.creatingZone = { zoneGeometryInDegrees: geometry };
        }
        
        const geometryAsObjects = rotateGeometry(zoneGeometry.creatingZone, rotation).map(point => ({ lng: point[1], lat: point[0] }));
        const createdZone = formatCreatedZone(geometryAsObjects, rotation, sublocation.uid);
        createdZone.zone_type_name = creatingZone.zone_type_name;

        dispatch(openZoneAddConfigurationModal(createdZone));
        dispatch(exitDrawRegime());
    },
    handleCancelEditContinue: () => {
        dispatch(finishDrawingPolygon());
        dispatch(exitDrawRegime());
        dispatch(startLiveMode());
    },
    handleFinishClick: () => {
        const finishPolygonDraw = document.querySelector('[title="Finish drawing"]');
        finishPolygonDraw.click();
    },
});

const mapStateToProps = state => ({
    isPolygonDrawing: state.modals.zoneModals.zoneChooseGeometry.polygon_create,
});

export default connect(mapStateToProps, mapDispatchToProps)(SiteViewEditZoneMode);


SiteViewEditZoneMode.propTypes = {
    rotation: PropTypes.number,
    width: PropTypes.number,
    showWidthControl: PropTypes.bool,
    creatingZone: PropTypes.shape({}),
    sublocation: PropTypes.shape({}),
    type: PropTypes.string,
    showControls: PropTypes.bool,
    handleZoneWithChange: PropTypes.func,
    handleZoneRotate: PropTypes.func,
    handleSubmitEditContinue: PropTypes.func,
    handleCancelEditContinue: PropTypes.func,
    handleFinishClick: PropTypes.func,
    isPolygonDrawing: PropTypes.bool,
};

SiteViewEditZoneMode.defaultProps = {
    rotation: 0,
    width: 0,
    showWidthControl: false,
    creatingZone: {},
    sublocation: {},
    type: '',
    showControls: false,
    handleZoneWithChange: () => {},
    handleZoneRotate: () => {},
    handleSubmitEditContinue: () => {},
    handleCancelEditContinue: () => {},
    handleFinishClick: () => {},
    isPolygonDrawing: false,
};
