import React, { PureComponent } from 'react';
import { connect } from 'react-redux';
import PropTypes from 'prop-types';
import { Map, LayersControl } from 'react-leaflet';
import { injectIntl } from 'react-intl';
import Control from 'react-leaflet-control';
import _isEmpty from 'lodash/isEmpty';
import _isEqual from 'lodash/isEqual';

import LeafletHeatmapContainer from 'pages/site/LeafletHeatmapContainer';
import SiteViewFilter from 'components/leaflet/SiteViewFilter';
import FullscreenControl from 'components/leaflet/FullscreenControl';
import LeafletLayerFilter from 'components/leaflet/LeafletLayerFilter';
import LeafletFloorplan from 'components/leaflet/LeafletFloorplan';
import LeafletZoneLayer from 'components/leaflet/LeafletZoneLayer';
import LeafletBufferLayer from 'components/leaflet/LeafletBufferLayer';
import LeafletEditModeZoneLayer from 'components/leaflet/LeafletEditModeZoneLayer';
import LeafletSpaghettiMarkers from 'components/leaflet/LeafletSpaghettiMarkers';
import LeafletSpaghettiDiagram from 'components/leaflet/LeafletSpaghettiDiagram';
import LeafletSpaghettiIdleTimeMarkers from 'components/leaflet/LeafletSpaghettiIdleTimeMarkers';
import LeafletNearMiss from 'components/leaflet/LeafletNearMiss';

import LeafletScaling from 'components/leaflet/LeafletScaling';
import LeafletTrackableLayer from 'components/leaflet/LeafletTrackableLayer';
import LeafletClusterLayer from 'components/leaflet/LeafletClusterLayer';
import LeafletAnchorsLayer from 'components/leaflet/LeafletAnchorsLayer';
import ZoneDrawingControl from 'components/leaflet/ZoneDrawingControl';
import dispatch from 'stores/dispatch';
import { changeParamsOnLeaflet, changeSpaghettiViewMode, selectLeafletItemForPopup } from 'stores/actions';
import { MAP_BOUNDS, handleLeafletDeltaZoom, userHasPrivileges, getZonesByPoint } from 'utils';
import { DEFAULT_VALUES, SITE_VIEW_LEAFLET, USER_ROLES } from 'common';
import { WinAnchor } from 'stores/classes/Selectors';

const { Overlay } = LayersControl;
const ZOOM_DELTA_BUTTONS = 0.25;


class SiteViewLeaflet extends PureComponent {
    static propTypes = {
        sublocation: PropTypes.shape(),
        spaghetti: PropTypes.shape(),
        floorplan: PropTypes.shape(),
        intl: PropTypes.shape({
            formatMessage: PropTypes.func,
        }),
        center: PropTypes.shape(),
        mode: PropTypes.shape(),
        creatingZone: PropTypes.shape(),
        nearMissReport: PropTypes.shape(),
        width: PropTypes.number,
        zoom: PropTypes.number,
        rotation: PropTypes.number,
        currentRole: PropTypes.string,
        timeZone: PropTypes.string,
        isOpenGroupFilter: PropTypes.bool,
        coordinateMode: PropTypes.bool,
        handleSiteViewZoneSelection: PropTypes.func,
        visibleTrackables: PropTypes.arrayOf(PropTypes.shape({})),
        visibleZones: PropTypes.arrayOf(PropTypes.shape({})),
    };

    static defaultProps = {
        sublocation: null,
        spaghetti: null,
        floorplan: null,
        intl: {
            formatMessage: () => {
            },
        },
        center: {},
        mode: {},
        creatingZone: {},
        nearMissReport: {},
        width: 2,
        zoom: 18,
        rotation: 0,
        currentRole: '',
        timeZone: 'Etc/GMT',
        isOpenGroupFilter: false,
        coordinateMode: false,
        handleSiteViewZoneSelection: () => { },
        visibleTrackables: [],
        visibleZones: [],
    };

    constructor(props) {
        super(props);
        this.state = { clusterTrackables: {}, mouseClickPosition: null };
        this.timer = null;
    }

    componentDidMount() {
        if (this.props.floorplan) {
            this.showFloorPlan(this.props.floorplan);
            // todo remove: need for update trackableLayer related to cluster
            this.timer = setTimeout(() => {
                this.showFloorPlan(this.props.floorplan);
            }, 5000);
        }
    }

    componentDidUpdate(prevProps) {
        const { mode: prevMode } = prevProps;
        const { mode: currentMode } = this.props;

        // if a mode is changed - then update leaflet to get correct map bounds
        if (!_isEqual(prevMode, currentMode)) {
            this.invalidateMapSize();
        }

        if (this.props.floorplan !== prevProps.floorplan && this.props.floorplan) {
            this.showFloorPlan(prevProps.floorplan);

            if (this.leaflet) {
                this.saveZoom(this.leaflet.leafletElement._zoom);
            }

            setTimeout(() => {
                this.showFloorPlan(prevProps.floorplan);
            }, 100);
        }
    }

    componentWillUnmount() {
        if (this.timer) {
            clearTimeout(this.timer);
        }
    }

    onRefLeaflet = (leaflet, intl) => {
        if (!leaflet) return;

        this.leaflet = handleLeafletDeltaZoom(leaflet);
        if (this.leaflet && this.leaflet.leafletElement && this.leaflet.leafletElement.zoomControl && this.leaflet.leafletElement.zoomControl._zoomInButton) {
            this.leaflet.leafletElement.zoomControl._zoomInButton.title = SITE_VIEW_LEAFLET.TITLES.ZOOM_IN(intl);
            this.leaflet.leafletElement.zoomControl._zoomOutButton.title = SITE_VIEW_LEAFLET.TITLES.ZOOM_OUT(intl);
        }
    };

    setClusterTrackables = (clusterTrackables) => {
        this.setState({ clusterTrackables });
    }

    showFloorPlan = (floorplan) => {
        if (floorplan && this.props.floorplan && this.props.floorplan.uid === floorplan.uid &&
            !_isEmpty(this.props.center) && this.props.zoom !== null
        ) {
            this.leaflet.leafletElement.setView(this.props.center, this.props.zoom, {});
        } else {
            this.handleFitToScreen(floorplan);
        }
    };

    handleFitToScreen = (floorplan) => {
        if (this.leaflet && this.props.sublocation && (this.props.floorplan || floorplan)) {
            const anchors = WinAnchor.getAnchorsBySublocation(this.props.sublocation);
            this.leaflet.leafletElement.fitBounds(MAP_BOUNDS(this.props.sublocation, this.props.floorplan || floorplan, anchors));
        }
    };

    saveZoom = (zoom) => {
        dispatch(changeParamsOnLeaflet({ zoom }));
    };

    handleMove = (e) => {
        dispatch(changeParamsOnLeaflet({
            zoom: e.target._zoom,
            bounds: this.leaflet.leafletElement.getBounds(),
            center: this.leaflet.leafletElement.getCenter(),
        }));
    };

    handleMapClick = (e) => {
        this.setState({ mouseClickPosition: e.latlng })
        const selectedZones = getZonesByPoint(this.props.visibleZones, this.props.sublocation, e.latlng);
        if (Array.isArray(selectedZones) && !this.props.coordinateMode) {
            dispatch(selectLeafletItemForPopup(selectedZones));
        }
    };

    showPolylineMarker = (marker) => {
        dispatch(changeSpaghettiViewMode({ marker }));
    };

    invalidateMapSize = () => {
        this.leaflet.leafletElement.invalidateSize();
    };

    render() {
        if (!this.props.sublocation) {
            return null;
        }

        const {
            sublocation,
            floorplan,
            intl,
            mode,
            width,
            rotation,
            creatingZone,
            isOpenGroupFilter,
            handleSiteViewZoneSelection,
            nearMissReport,
            spaghetti,
            timeZone,
            currentRole,
            visibleTrackables,
            visibleZones,
            coordinateMode,
        } = this.props;

        const { isSpaghettiDiagramShowing, marker, idleTimeMarkers, spaghettiId, byId, trackables } = spaghetti;

        const timeframe = (byId[spaghettiId] && byId[spaghettiId].timeframe) || [];
        const filteredTrailUids = byId[spaghettiId] && byId[spaghettiId].filteredTrailUids;
        const { byUid, reportUid } = nearMissReport;

        const isNearMissReady = byUid[reportUid] && byUid[reportUid].ready;
        const isSpaghettiShowing = byId[spaghettiId] && byId[spaghettiId].ready && isSpaghettiDiagramShowing;

        const isLiveModeOn = sublocation && !isSpaghettiShowing && !isNearMissReady;

        return (
            <div className="Leaflet-Wrap">
                <Map
                    center={[0, 0]}
                    ref={(leaflet) => {
                        this.onRefLeaflet(leaflet, intl);
                    }}
                    className="Leaflet"
                    zoomAnimation={false}
                    zoomSnap={0}
                    zoomDelta={ZOOM_DELTA_BUTTONS} // zoomDelta use only by zoomControl buttons
                    minZoom={DEFAULT_VALUES.MAP_ZOOM.MIN}
                    maxZoom={DEFAULT_VALUES.MAP_ZOOM.MAX}
                    maxBoundsViscosity={1}
                    zoom={DEFAULT_VALUES.MAP_ZOOM.DEFAULT}
                    scrollWheelZoom={!isOpenGroupFilter ? 'center' : false}
                    onMoveend={this.handleMove}
                    onClick={this.handleMapClick}
                >
                    <LayersControl ref={(layers) => {
                        this.layers = layers;
                    }}
                    >
                        {floorplan && <Overlay checked name="Floorplan">
                            <LeafletFloorplan floorplan={floorplan} />
                        </Overlay>}

                        <FullscreenControl
                            title={SITE_VIEW_LEAFLET.TITLES.FULL_SCREEN(intl)}
                            position="bottomright"
                        />

                        <LeafletLayerFilter position="topright" intl={intl} />

                        <SiteViewFilter position="topleft" intl={intl} isSpaghettiShowing={isSpaghettiShowing} />

                        <Control position="bottomright">
                            <button
                                title={SITE_VIEW_LEAFLET.TITLES.FIT_FUNC(intl)}
                                className="leaflet-control-zoom-fit"
                                data-description="Fit to screen"
                                onClick={this.handleFitToScreen}
                            >
                                <i className="fa fa-crosshairs" aria-hidden="true" />
                            </button>
                        </Control>

                        <Overlay checked name="Heatmaps">
                            <LeafletHeatmapContainer />
                        </Overlay>

                        {floorplan && <Overlay checked name="Scaling">
                            <LeafletScaling floorplan={floorplan} />
                        </Overlay>}

                        {isLiveModeOn && <Overlay checked name="Zones">
                            <LeafletZoneLayer
                                sublocation={sublocation}
                                visibleZones={visibleZones}
                                mouseClickPosition={this.state.mouseClickPosition}
                                handleSiteViewZoneSelection={handleSiteViewZoneSelection}
                            />
                            <LeafletBufferLayer
                                sublocation={sublocation}
                                visibleTrackables={visibleTrackables}
                            />
                        </Overlay>}

                        {sublocation && <Overlay checked name="Zones">
                            <LeafletEditModeZoneLayer
                                coordinateMode={coordinateMode}
                                sublocation={sublocation}
                                mode={mode}
                                width={width}
                                rotation={rotation}
                                creatingZone={creatingZone}
                            />
                        </Overlay>}

                        {isLiveModeOn && userHasPrivileges(currentRole, USER_ROLES.PROJECT_MAINTAINER) &&
                        <Overlay show="true" checked name="Anchors">
                            <LeafletAnchorsLayer sublocation={sublocation} />
                        </Overlay>}

                        {isLiveModeOn && <Overlay checked name="Trackables">
                            <LeafletTrackableLayer
                                sublocation={sublocation}
                                visibleTrackables={visibleTrackables}
                                clusterTrackables={this.state.clusterTrackables}
                            />
                        </Overlay>}

                        {isLiveModeOn && <Overlay checked name="Trackables">
                            <LeafletClusterLayer
                                sublocation={sublocation}
                                visibleTrackables={visibleTrackables}
                                setClusterTrackables={this.setClusterTrackables}
                                clusterTrackables={this.state.clusterTrackables}
                            />
                        </Overlay>}

                        {
                            isSpaghettiShowing && <LeafletSpaghettiDiagram
                                trackables={trackables}
                                timeframe={timeframe}
                                timeZone={timeZone}
                                showPolylineMarker={this.showPolylineMarker}
                                filteredTrailUids={filteredTrailUids}
                            />
                        }
                        {
                            marker && isSpaghettiDiagramShowing && <Overlay checked name="marker">
                                <LeafletSpaghettiMarkers
                                    marker={marker}
                                    dispatch={dispatch}
                                    timeframe={timeframe}
                                />
                            </Overlay>
                        }
                        {
                            isSpaghettiShowing && <Overlay checked name="idleTimeMarkers">
                                <LeafletSpaghettiIdleTimeMarkers
                                    trackables={trackables}
                                    idleTimeMarkers={idleTimeMarkers}
                                    dispatch={dispatch}
                                    filteredTrailUids={filteredTrailUids}
                                />
                            </Overlay>
                        }

                        {
                            isNearMissReady && <Overlay checked name="nearMissEventMarkers">
                                <LeafletNearMiss
                                    nearMissReport={nearMissReport}
                                />
                            </Overlay>
                        }
                    </LayersControl>

                    {sublocation && <ZoneDrawingControl sublocation={sublocation} />}
                </Map>
            </div>
        );
    }
}

export default connect()(injectIntl(SiteViewLeaflet));
