import React, { Fragment } from 'react';
import PropTypes from 'prop-types';
import { connect } from 'react-redux';
import { STORAGE_KEY, GEOMETRY_RESTRICTION_TYPES } from 'common/constants';
import { SITE_ANALYTICS_LABELS } from 'common/ui-labels';
import { setFilteredTrailUids, siteViewClearMode, exitDrawRegime, closeZoneCoordinatesModeModal } from 'stores/actions';
import { openErrorModal } from 'stores/actions/modalActions/commonModalActions';
import store from 'stores/LocationStore';
import { getTimeByFormatAndTimezone, getTimeZoneOfProject } from 'utils/timeUtils';
import { isZoneEditing, isSpaghettiMode, isNearMissMode, isPlaybackMode, isHeatmapMode } from 'utils/simpleUtils';
import SpaghettiSummaryInfo from './SpaghettiSummaryInfo';
import SiteViewEditZoneMode from './SiteViewEditZoneMode';


class SiteViewMode extends React.PureComponent {
    static propTypes = {
        dispatch: PropTypes.func,
        isCoordinateMode: PropTypes.bool,
        isModeLoading: PropTypes.bool,
        timeZone: PropTypes.string,
        showControls: PropTypes.bool,
        rotation: PropTypes.number,
        width: PropTypes.number,
        spaghetti: PropTypes.shape({
            byId: PropTypes.shape({}),
            spaghettiId: PropTypes.string,
        }),
        creatingZone: PropTypes.shape({
            zoneGeometryInDegrees: PropTypes.array,
            zoneGeometryInMillimeters: PropTypes.array,
        }),
        floorplan: PropTypes.shape(),
        sublocation: PropTypes.shape(),
        heatmap: PropTypes.shape({
            currentHeatmaps: PropTypes.array,
        }),
        mode: PropTypes.shape({
            name: PropTypes.string,
            onReturn: PropTypes.func,
        }),
        playback: PropTypes.shape({
            playSpeed: PropTypes.number,
        }),
        nearMissReport: PropTypes.shape({
            error: PropTypes.shape({
                message: PropTypes.string,
                status: PropTypes.number,
            }),
        }),
    };
    
    static defaultProps = {
        dispatch: () => {},
        isCoordinateMode: false,
        isModeLoading: false,
        timeZone: 'Etc/GMT',
        showControls: false,
        rotation: 0,
        width: 0,
        spaghetti: {
            byId: {},
            spaghettiId: '',
        },
        creatingZone: {
            zoneGeometryInDegrees: [],
            zoneGeometryInMillimeters: [],
        },
        floorplan: {},
        sublocation: {},
        heatmap: {
            currentHeatmaps: [],
        },
        mode: {
            name: '',
            onReturn: () => {},
        },
        playback: {
            playSpeed: 1,
        },
        nearMissReport: {
            error: null,
        },
    };

    componentWillUnmount() {
        const { dispatch, mode, isCoordinateMode } = this.props;

        if (mode) {
            if (isZoneEditing(mode.name)) {
                if (isCoordinateMode) dispatch(closeZoneCoordinatesModeModal());
                dispatch(exitDrawRegime());
            } else {
                mode.onReturn && mode.onReturn();
            }
        }

        dispatch(siteViewClearMode());
    }

    handleHeatmapMode = (isModeLoading, heatmaps, timeZone, customStyle) => {
        let additionalInfo;
    
        if (isModeLoading) {
            return null;
        } else if (heatmaps.length > 0) {
            const uids = new Map();
            const names = new Map();
        
            heatmaps.forEach((heatmap) => {
                heatmap.input.uids.forEach((uid) => {
                    uids.set(uid, uid);
                });
            
                heatmap.input.names.forEach((name) => {
                    names.set(name, name);
                });
            });
        
            const namesOfTrackables = [...names.values()].join(', ');
        
            const minStart = Math.min(...heatmaps.map(heatmap => heatmap.input.start * 1000));
            const maxFinish = Math.max(...heatmaps.map(heatmap => heatmap.input.end * 1000));
            
            const fromTime = getTimeByFormatAndTimezone(minStart, 'MM-DD-YYYY HH:mm', timeZone);
            const toTime = getTimeByFormatAndTimezone(maxFinish, 'MM-DD-YYYY HH:mm', timeZone);
            
            additionalInfo = `${namesOfTrackables} ${fromTime} - ${toTime}`;
            customStyle.fontSize = '1.3em';
            customStyle.display = 'inline-block';
            customStyle.maxWidth = '50%';
            customStyle.maxHeight = '60px';
            customStyle.overflow = 'auto';
        }
        return additionalInfo;
    };

    filterSpaghettiTrail = (filteredTrailUids) => {
        const { spaghetti: { spaghettiId } } = this.props;

        this.props.dispatch(setFilteredTrailUids({ filteredTrailUids, spaghettiId }));
    };

    isSpaghettiTrailFilterSelected = (uid) => {
        const { spaghetti: { byId, spaghettiId } } = this.props;
        const filteredTrailUids = (byId[spaghettiId] && byId[spaghettiId].filteredTrailUids) || [];

        return filteredTrailUids.includes(uid);
    };

    handleSpaghettiMode = (trackablesArr) => {
        const { isModeLoading, timeZone, spaghetti: { selectedTrailUid, byId, spaghettiId, error } } = this.props;

        if (isModeLoading) {
            return null;
        } else if (error) {
            this.props.dispatch(openErrorModal({ message: error.message, title: 'Spaghetti Generation' }));
        } else if (trackablesArr.length > 0) {
            const isAnyTrailSelected = trackablesArr.some(trackable => selectedTrailUid === trackable.uid);
            const timeframe = (byId[spaghettiId] && byId[spaghettiId].timeframe) || {};

            return (
                <Fragment>
                    <div className={`spaghetti-diagram-legend${isAnyTrailSelected ? ' trail-selected' : ''}`}>
                        <h4 className="legend-title">{SITE_ANALYTICS_LABELS.TRACE_FILTER}</h4>
                        {
                            trackablesArr.map((trackable, index) =>
                                (<div key={index} className={`legend-row${selectedTrailUid === trackable.uid ? ' selected' : ''}`}>
                                    <div
                                        className="legend-trackable-color"
                                        style={{ backgroundColor: trackable.color }}
                                    />
                                    <input
                                        type="checkbox"
                                        name={`trail-checkbox${trackable.uid}`}
                                        checked={this.isSpaghettiTrailFilterSelected(trackable.uid)}
                                        onChange={() => this.filterSpaghettiTrail(trackable.uid)}
                                    />
                                    <div className="legend-trackable-name text-ellipsis" title={trackable.name}>
                                        { trackable.name }
                                    </div>
                                </div>))
                        }
                    </div>
                    <SpaghettiSummaryInfo
                        timeZone={timeZone}
                        trackables={trackablesArr}
                        timeframe={timeframe}
                    />
                </Fragment>
            );
        }
    };
    
    handleNearMissMode = (customStyle) => {
        customStyle.padding = 0;
        const { error } = this.props.nearMissReport;
        if (error) {
            this.props.dispatch(openErrorModal({ message: error.message, title: 'Near Miss Events Generation' }));
        } else if (this.props.isModeLoading) {
            return null;
        }
        return (<Fragment />);
    };
    
    handlePlaybackMode = (isModeLoading, playback, customStyle) => {
        if (isModeLoading) {
            return null;
        }
        
        const additionalInfo = `x${playback.playSpeed}`;
        customStyle.fontSize = '2.5em';
        
        return additionalInfo;
    };
    
    render() {
        const {
            isModeLoading, timeZone, mode, floorplan, playback, heatmap: { currentHeatmaps },
            creatingZone, rotation, width, showControls, spaghetti, sublocation, isCoordinateMode,
        } = this.props;

        const { trackables } = spaghetti;

        const trackablesArr = Object.values(trackables);

        if (!mode || !floorplan) {
            return null;
        }
        
        const { name, type } = mode;
        const showWidthControl = type === GEOMETRY_RESTRICTION_TYPES.DIRECTIONAL;
        let additionalInfo;
        const customStyle = {};

        if (isSpaghettiMode(name)) {
            additionalInfo = this.handleSpaghettiMode(trackablesArr);
        }
    
        if (isNearMissMode(name)) {
            additionalInfo = this.handleNearMissMode(customStyle);
        }

        if (isPlaybackMode(name)) {
            additionalInfo = this.handlePlaybackMode(isModeLoading, playback, customStyle);
        }
        
        if (isHeatmapMode(name)) {
            additionalInfo = this.handleHeatmapMode(isModeLoading, currentHeatmaps, timeZone, customStyle);
        }
        
        if (isZoneEditing(name) && !isCoordinateMode) {
            return (
                <SiteViewEditZoneMode
                    sublocation={sublocation}
                    rotation={rotation}
                    width={width}
                    showWidthControl={showWidthControl}
                    creatingZone={creatingZone}
                    type={type}
                    showControls={showControls}
                />
            );
        }

        return (
            <div className="RPP-Mode">
                {isModeLoading && <div className="Loader-Position"><div className="loader" /></div>}
                <div className="Mode-AdditionalInfo" style={customStyle}>{additionalInfo}</div>
            </div>
        );
    }
}

const mapStateToProps = (state) => {
    const { appContext, playback, heatmap, spaghetti, nearMissReport, modals } = state;
    const { geometry, rotation, width, selectedFloorplan, selectedSublocation } = state.siteView;

    const project = appContext.project;
    const timeZone = getTimeZoneOfProject(project);

    const showControls = geometry.zoneGeometryInDegrees.length !== 0;
    const isCoordinateMode = state.siteViewZoneEditCoordinates.active;

    return {
        isCoordinateMode,
        showControls,
        timeZone,
        rotation,
        width: store.getFromLocalStorage(STORAGE_KEY.LAST_DIRECTIONAL_ZONE_WIDTH) || width,
        creatingZone: geometry,
        playback,
        heatmap,
        floorplan: selectedFloorplan,
        sublocation: selectedSublocation,
        spaghetti,
        nearMissReport,
    };
};

export default connect(mapStateToProps)(SiteViewMode);
