import React, { Component } from 'react';
import PropTypes from 'prop-types';
import { connect } from 'react-redux';
import { reduxForm, Field } from 'redux-form';

import RenderRangeSlider from 'components/modals/RenderRangeSlider';
import { getTimeZoneOfProject, getTimeZoneOffsetByProject, getTimeByFormatAndTimezone } from 'utils/timeUtils';
import { changeNearMissDisplayTimeFrame, getNearMissEventsForTimeframe, openNearMissEventPopup } from 'stores/actions/nearMissEventsActions';


class SiteViewNearMissControls extends Component {
    static propTypes = {
        changeTimeFrame: PropTypes.func,
        selectMarker: PropTypes.func,
        showControl: PropTypes.bool,
        reportUid: PropTypes.string,
    };
    
    static defaultProps = {
        changeTimeFrame: () => {},
        selectMarker: () => {},
        showControl: false,
        reportUid: '',
    };
    
    constructor(props) {
        super(props);
    
        this.from = null;
        this.to = null;
    }

    onTimeframeDragEnd = ([displayFrom, displayTo]) => {
        this.props.changeTimeFrame({ displayFrom, displayTo }, this.props.reportUid);
    };

    render() {
        if (!this.props.showControl) {
            return null;
        }

        return (
            <SiteViewNearMissControlsForm
                onTimeframeDragEnd={this.onTimeframeDragEnd}
                selectMarker={this.props.selectMarker}
                onSubmit={this.handleSubmit}
            />
        );
    }
}

const mapDispatchToProps = dispatch => ({
    changeTimeFrame: (timeframe, reportUid) => {
        const { displayFrom, displayTo } = timeframe;
        dispatch(changeNearMissDisplayTimeFrame(timeframe));
        getNearMissEventsForTimeframe({ dateFromUtc: displayFrom, dateToUtc: displayTo, reportUid });
    },
    selectMarker: (event) => {
        dispatch(openNearMissEventPopup(event));
    },
});

const mapStateToProps = (state) => {
    const { nearMissReport: { byUid, reportUid } } = state;

    return {
        reportUid,
        showControl: byUid && byUid[reportUid] && byUid[reportUid].ready,
    };
};

function TimeLineMarker(props) {
    return (
        <div
            className={`near-miss-timeline-mark${props.isSelected ? '-selected' : ''}`}
            data-id="marker"
            onClick={() => {
                props.onSelectMarker(props.event);
            }}
        />
    );
}

TimeLineMarker.propTypes = {
    onSelectMarker: PropTypes.func,
    isSelected: PropTypes.bool,
    event: PropTypes.shape(),
};

TimeLineMarker.defaultProps = {
    onSelectMarker: () => {},
    isSelected: false,
    event: null,
};


let SiteViewNearMissControlsForm = (props) => {
    const {
        handleSubmit, timeframe, onTimeframeDragEnd, timeZone, timeLineMarkers, selectMarker,
        events, selectedNearMissMarker,
    } = props;

    const displayTimeFormat = 'MM/DD h:mm:ss a';
    const getDisplayTime = date => getTimeByFormatAndTimezone(date, displayTimeFormat, timeZone);

    return (
        <form onSubmit={handleSubmit}>
            <div className="slider-wrapper timeframe-slider">
                <Field
                    name="timeframe"
                    component={RenderRangeSlider}
                    min={timeframe.dateFrom}
                    max={timeframe.dateTo}
                    marks={timeLineMarkers.map((marker) => {
                        const event = events.find(_event => _event.id === marker.id);
                        marker.label = (<TimeLineMarker
                            id={marker.id}
                            onSelectMarker={selectMarker}
                            isSelected={selectedNearMissMarker === marker.id}
                            event={event}
                        />);
                        return marker;
                    })
                    }
                    valueLabelFormat={value => getTimeByFormatAndTimezone(value, displayTimeFormat, timeZone)}
                    onTimeframeDragEnd={onTimeframeDragEnd}
                />
                <div className="ProgressBarWithTime-Time">
                    <div className="ProgressBarWithTime-Time-elem ProgressBarWithTime-Time-start">
                        {getDisplayTime(timeframe.dateFrom)}
                    </div>
                
                    <div className="ProgressBarWithTime-Time-elem ProgressBarWithTime-Time-finish">
                        {getDisplayTime(timeframe.dateTo)}
                    </div>
                </div>
            </div>
        </form>
    );
};

SiteViewNearMissControlsForm.propTypes = {
    handleSubmit: PropTypes.func,
    selectMarker: PropTypes.func,
    timeframe: PropTypes.shape(),
    onTimeframeDragEnd: PropTypes.func,
    timeZone: PropTypes.string,
    timeZoneOffset: PropTypes.number,
    selectedNearMissMarker: PropTypes.number,
    timeLineMarkers: PropTypes.arrayOf(PropTypes.shape({})),
    events: PropTypes.arrayOf(PropTypes.shape({})),
};

SiteViewNearMissControlsForm.defaultProps = {
    handleSubmit: () => {},
    selectMarker: () => {},
    timeframe: {},
    onTimeframeDragEnd: () => {},
    timeZone: 'Etc/GMT',
    timeZoneOffset: 0,
    selectedNearMissMarker: null,
    timeLineMarkers: [],
    events: [],
};

SiteViewNearMissControlsForm = reduxForm({
    form: 'SiteViewNearMissControls',
    enableReinitialize: true,
})(SiteViewNearMissControlsForm);

const mapStateToForm = (state) => {
    const { nearMissReport: { byUid, reportUid, displayFrom, displayTo, selectedNearMissMarker } } = state;
    const { timeframe, ready, timeLineMarkers, events } = byUid[reportUid];
    const { project } = state.appContext;
    const timeZone = getTimeZoneOfProject(project);
    const timeZoneOffset = getTimeZoneOffsetByProject(project);

    return (timeframe) ? {
        initialValues: {
            timeframe: [displayFrom, displayTo],
        },
        timeframe,
        timeZone,
        timeZoneOffset,
        isNearMissReportReady: ready,
        selectedNearMissMarker,
        timeLineMarkers,
        events,
    } : {};
};

SiteViewNearMissControlsForm = connect(mapStateToForm)(SiteViewNearMissControlsForm);

export default connect(mapStateToProps, mapDispatchToProps)(SiteViewNearMissControls);
