import React, { Fragment, Component } from 'react';
import { connect } from 'react-redux';
import { injectIntl } from 'react-intl';
import PropTypes from 'prop-types';
import { createSelectorCreator, defaultMemoize } from 'reselect';
import isEqual from 'lodash/isEqual';
import { LIVE_STATE, DEFAULT_VALUES, USER_ROLES } from 'common';
import { userHasRole, getTimeZoneOfProject } from 'utils';
import { Zone, Alarm } from 'stores/classes';
import { SortDirection, AutoSizer } from 'react-virtualized';
import VirtualTable from './VirtualTable';

class AlarmsTable extends Component {
    static propTypes = {
        selectedItems: PropTypes.arrayOf(PropTypes.shape()),
        timeZone: PropTypes.string,
        alarms: PropTypes.shape({}),
        intl: PropTypes.shape({
            formatMessage: PropTypes.func,
        }),
        isUserProjectAdmin: PropTypes.bool,
        height: PropTypes.number,
        dispatch: PropTypes.func,
    };

    static defaultProps = {
        selectedItems: [],
        timeZone: 'Etc/GMT',
        alarms: new Map(),
        isUserProjectAdmin: false,
        intl: {
            formatMessage: () => {
            },
        },
        height: 50,
        dispatch: () => {
        },
    };

    constructor(props) {
        super(props);

        this.state = {
            containerWidth: 0,
            containerHeight: 0,
        };
    }

    render() {
        const { selectedItems, timeZone, isUserProjectAdmin, alarms, dispatch } = this.props;
        const { containerWidth, containerHeight } = this.state;

        return (
            <div style={{ height: this.props.height }}>
                <AutoSizer
                    onResize={({ height, width }) => {
                        this.setState({
                            containerWidth: width,
                            containerHeight: height,
                        });
                    }}
                >
                    {({ height, width }) => (
                        <Fragment>
                            <VirtualTable
                                defaultSortName=""
                                defaultSortOrder={SortDirection.ASC}
                                tableSize={{
                                    width: containerWidth,
                                    height: containerHeight,
                                    headerHeight: 28,
                                    rowHeight: 32,
                                }}
                                allItems={alarms}
                                totalCount={alarms.size}
                                selectedItems={selectedItems}
                                dispatch={dispatch}
                                timeZone={timeZone}
                                isUserProjectAdmin={isUserProjectAdmin}
                            />
                        </Fragment>
                    )}
                </AutoSizer>
            </div>
        );
    }
}

const createDeepEqualSelector = createSelectorCreator(
    defaultMemoize,
    isEqual,
);

const getSelectedSublocation = state => state.siteView.selectedSublocation;
const getTimeZone = state => getTimeZoneOfProject(state.appContext.project);
const getTrackablesAlarm = (state) => {
    if (state.playback.isOpenPlayBackControl) {
        return state.playback.trackables.reduce((memo, tko) => {
            memo[tko.uid] = tko;
            return memo;
        }, {});
    }
    return state.entities.trackables.byUid;
};

const getIntl = (state, intl) => intl;

function showAlarm(alarm, sublocation, trackables, zones) {
    if (alarm.acknowledged) return false;
    if (sublocation.uid === alarm.sublocation_uid) return true;

    const trackable = trackables[alarm.obj_uid];
    if (trackable && trackable.position &&
        ((trackable.position.sl_uid === sublocation.uid)
            || (trackable.position.sl_id === sublocation.uid))) {
        return true;
    }

    const zone = zones[alarm.zone_uid];
    if (zone && sublocation.uid === zone.sublocation_uid) {
        return true;
    }
    return false;
}

const getAlarmsByZones = (alarms, selectedSublocation, zones, trackables, intl, timeZone) => {
    if (selectedSublocation) {
        return alarms
            .reduce((memo, alarm) => {
                if (showAlarm(alarm, selectedSublocation, trackables, zones)) {
                    const trackable = trackables[alarm.obj_uid];
                    alarm.trackableName = trackable ? trackable.name : '';
                    alarm.alarmInfo = `${LIVE_STATE.ALARM(intl)}: ${alarm.trackableName}${alarm.getAlarmInfo(zones, timeZone, intl)}`;
                    if (!memo.get(alarm.uid)) {
                        memo.set(alarm.uid, alarm);
                    }
                }
                return memo;
            }, new Map());
    }
    return DEFAULT_VALUES.empty_array;
};

const getAlarms = createDeepEqualSelector(
    [Alarm.getAllAlarms, getSelectedSublocation, Zone.getAllZones, getTrackablesAlarm, getIntl, getTimeZone],
    (alarms, selectedSublocation, zones, trackables, intl, timeZone) => getAlarmsByZones(alarms, selectedSublocation, zones, trackables, intl, timeZone),
);

const mapStateToProps = (state, { intl }) => {
    const user = state.user;
    const isUserProjectAdmin = user && user.access && (userHasRole(user.access, USER_ROLES.PROJECT_OPERATIONS_ADMIN) ||
        userHasRole(user.access, USER_ROLES.PROJECT_SYSTEM_ADMIN));
    const timeZone = getTimeZoneOfProject(state.appContext.project);
    const alarms = getAlarms(state, intl);

    return {
        height: state.siteView.height,
        selectedItems: state.siteView.selectedItems,
        timeZone,
        isUserProjectAdmin,
        alarms,
    };
};

export default injectIntl(connect(mapStateToProps)(injectIntl(AlarmsTable)));
