import React from 'react';
import PropTypes from 'prop-types';
import { injectIntl } from 'react-intl';
import { Column, Table } from 'react-virtualized';
import 'react-virtualized/styles.css';
import withSortSelect from 'components/HOC/withSortSelect';
import { virtualizedHeaderRenderer } from 'utils/htmlUtils';
import { COMPONENTS, TABLES } from 'common/ui-labels';
import { openTrackableDetailsModal, selectTrackables, unselectTrackables } from 'stores/actions';
import TableButtons from './TableButtons';

class VirtualTable extends React.Component {
    static propTypes = {
        // Props from HOC
        sort: PropTypes.func,
        sortBy: PropTypes.string,
        trStyleFormat: PropTypes.func,
        setFilterItems: PropTypes.func,
        sortDisplayItems: PropTypes.func,
        sortDirection: PropTypes.string,
        filteredItems: PropTypes.arrayOf(PropTypes.shape()),
        
        // Props from parent
        allItems: PropTypes.shape(),
        totalCount: PropTypes.number,
        showUnresponsive: PropTypes.bool,
        isPlaybackOpen: PropTypes.bool,
        tableSize: PropTypes.shape(),
        selectedItems: PropTypes.arrayOf(PropTypes.string),
        dispatch: PropTypes.func,
        intl: PropTypes.shape({ formatMessage: PropTypes.func }),
    };
    static defaultProps = {
        // Props from HOC
        sort: () => { },
        sortBy: '',
        trStyleFormat: () => { },
        setFilterItems: () => { },
        sortDisplayItems: () => { },
        sortDirection: '',
        filteredItems: [],
    
        // Props from parent
        allItems: new Map(),
        totalCount: 0,
        showUnresponsive: true,
        isPlaybackOpen: false,
        tableSize: {},
        selectedItems: [],
        dispatch: () => { },
        intl: { formatMessage: () => { } },
    };
    
    constructor(props) {
        super(props);
        this.totalCount = props.totalCount;
        this.dispatch = props.dispatch;
        this.myRef = React.createRef();
        
        // HOC Methods
        this.trStyleFormat = props.trStyleFormat.bind(this);
        this.sort = props.sort.bind(this);
        this.sortDisplayItems = props.sortDisplayItems.bind(this);
        this.setFilterItems = props.setFilterItems.bind(this);
    }

    componentDidUpdate() {
        const checkboxRef = this.myRef && this.myRef.current;
        const selectedCount = this.props.selectedItems.length;
        const itemsCount = this.props.totalCount;

        if (checkboxRef && selectedCount !== itemsCount) {
            this.myRef.current.indeterminate = selectedCount > 0;
        } else if (checkboxRef && selectedCount === itemsCount) {
            this.myRef.current.indeterminate = false;
        }
    }

    getActionsIcons = row => (
        <section data-info="actions-section">
            <i
                data-info="actions-section"
                data-id="site-view-tab-zones-actions-info"
                data-cy="trackable-table-info-button"
                onClick={this.showInfo.bind(this, row)}
                title={COMPONENTS.SHOW_DETAILS(this.props.intl)}
                className="fa fa-info-circle inner-table-action"
                aria-hidden="true"
            />
        </section>
    );
    
    trClassFormat = (items, { index }) => {
        if (index === -1) return '';
        const currentItem = items[index];
        return (currentItem && currentItem.isResponsive(this.props.isPlaybackOpen) ? '' : 'redText');
    };

    _handleHeaderCheckboxClick = (filteredItems, selectedCount) => {
        if (!selectedCount) {
            this.props.dispatch(selectTrackables(filteredItems.map(item => item.uid)));
        } else {
            this.props.dispatch(unselectTrackables(filteredItems.map(item => item.uid)));
        }
    }

    headerCheckbox = (filteredItems, selectedCount, totalCount) => (
        <input
            type="checkbox"
            ref={this.myRef}
            checked={selectedCount === totalCount}
            onChange={() => {}}
            onClick={() => { this._handleHeaderCheckboxClick(filteredItems, selectedCount); }}
        />
    );

    checkboxButton = (rowData) => {
        const isChecked = this.props.selectedItems.indexOf(rowData.uid) !== -1;
        return (
            <section>
                <input type="checkbox" checked={isChecked} onChange={() => {}} />
            </section>
        );
    };

    handleSelectRow = (selectedItems, { event, rowData }) => {
        if (event.target.getAttribute('data-info') === 'actions-section') {
            return false;
        }

        const isSelect = selectedItems.indexOf(rowData.uid) === -1;

        if (isSelect) {
            this.props.dispatch(selectTrackables([rowData.uid]));
        } else {
            this.props.dispatch(unselectTrackables([rowData.uid]));
        }
        return true;
    };
    
    showInfo = (row) => {
        this.props.dispatch(openTrackableDetailsModal(row));
    };
    
    _noRowsRenderer = () => (
        <table className="table table-bordered table-hover table-condensed table-no-data">
            <tbody>
                <tr>
                    <td className="react-bs-table-no-data">
                        <span>{TABLES.NO_DATA_TO_DISPLAY}</span>
                    </td>
                </tr>
            </tbody>
        </table>
    );
    
    render() {
        const { sortBy, sortDirection, tableSize, totalCount, allItems, selectedItems, filteredItems, showUnresponsive } = this.props;
        const allItemsArray = [...allItems.values()];
        const displayFilteredItems = this.sortDisplayItems(filteredItems);
        const displayFilteredCount = displayFilteredItems.length;
        const selectedCount = selectedItems.length;
        return (
            <section style={{ width: tableSize.width }}>
                <TableButtons
                    items={allItemsArray}
                    width={tableSize.width}
                    setFilterItems={this.setFilterItems}
                    showUnresponsive={showUnresponsive}
                    searchFields={['name', 'displayGroups']}
                />
                <Table
                    noRowsRenderer={this._noRowsRenderer}
                    sortBy={sortBy}
                    sort={this.sort}
                    sortDirection={sortDirection}
                    className="Virtual-Table-Medium"
                    width={tableSize.width} // required
                    height={tableSize.height} // required
                    gridClassName="Virtual-Table-Grid"
                    rowHeight={tableSize.rowHeight} // required
                    headerHeight={tableSize.headerHeight} // required
                    headerClassName="Virtual-Table-Header"
                    headerStyle={{ height: tableSize.headerHeight, borderTop: '1px solid #d3d3d3' }}
                    rowStyle={({ index }) => this.trStyleFormat(displayFilteredItems, { index })}
                    rowClassName={({ index }) => this.trClassFormat(displayFilteredItems, { index })}
                    onRowClick={({ event, index, rowData }) => this.handleSelectRow(selectedItems, { event, index, rowData })}
                    rowCount={displayFilteredCount} // required
                    rowGetter={({ index }) => displayFilteredItems[index]} // required
                >
                    <Column
                        label={<span title="">{this.headerCheckbox(displayFilteredItems, selectedCount, totalCount)}</span>}
                        dataKey="checkbox"
                        width={33}
                        cellRenderer={({ rowData }) => this.checkboxButton(rowData)}
                        disableSort
                        className="Virtual-Table-Column-Checkbox first-column"
                    />
                    <Column
                        label={<span title="">{COMPONENTS.FIELDS.COMMON.NAME}</span>}
                        headerRenderer={virtualizedHeaderRenderer}
                        width={125}
                        cellRenderer={({ cellData, rowData }) => (<div title={rowData.title}>{cellData}</div>)}
                        dataKey="name"
                        id="trackables-table-name"
                        className="Virtual-Table-Column-Left"
                    />
                    <Column
                        width={93}
                        label={<span title="">{COMPONENTS.FIELDS.NODE.GROUP}</span>}
                        headerRenderer={virtualizedHeaderRenderer}
                        cellRenderer={({ cellData }) => <div title={cellData}>{cellData}</div>}
                        dataKey="displayGroups"
                        id="trackables-table-groups"
                        className="Virtual-Table-Column-Left"
                    />
                    <Column
                        width={40}
                        label=""
                        cellRenderer={({ cellData }) => cellData}
                        cellDataGetter={({ rowData }) => this.getActionsIcons(rowData)}
                        dataKey="actions"
                        disableSort
                        className="Virtual-Table-Column-Center"
                    />
                </Table>
            </section>
        );
    }
}

export default injectIntl(withSortSelect(VirtualTable));
