import React, { Fragment } from 'react';
import PropTypes from 'prop-types';
import { connect } from 'react-redux';
import { injectIntl } from 'react-intl';
import { createSelectorCreator, defaultMemoize } from 'reselect';
import isEqual from 'lodash/isEqual';
import { SortDirection, AutoSizer } from 'react-virtualized';
import { Zone } from 'stores/classes';
import ZonesPrototypes from 'stores/classes/ZonePrototype';
import VirtualTable from './VirtualTable';

class ZonesTable extends React.Component {
    static propTypes = {
        zones: PropTypes.shape(),
        height: PropTypes.number,
        intl: PropTypes.shape({ formatMessage: PropTypes.func }),
        dispatch: PropTypes.func,
        selectedItems: PropTypes.arrayOf(PropTypes.shape({ uid: PropTypes.string })),
        currentUser: PropTypes.shape(),
        isOpenPlayBackControl: PropTypes.bool,
        isZoneCreationDisabled: PropTypes.bool,
    };

    static defaultProps = {
        zones: new Map(),
        selectedItems: [],
        height: 0,
        intl: { formatMessage: () => {} },
        dispatch: () => {},
        currentUser: {},
        isOpenPlayBackControl: false,
        isZoneCreationDisabled: false,
    };

    constructor(props) {
        super(props);

        this.state = {
            containerWidth: 0,
            containerHeight: 0,
        };
    }

    render() {
        const { isOpenPlayBackControl, zones, selectedItems, currentUser, isZoneCreationDisabled, dispatch } = this.props;
        const { containerWidth, containerHeight } = this.state;

        return (
            <div style={{ height: this.props.height }}>
                <AutoSizer
                    onResize={({ height, width }) => {
                        this.setState({
                            containerWidth: width,
                            containerHeight: height,
                        });
                    }}
                >
                    {({ height, width }) => (
                        <Fragment>
                            <VirtualTable
                                defaultSortName="name"
                                defaultSortOrder={SortDirection.ASC}
                                tableSize={{
                                    width: containerWidth,
                                    height: containerHeight,
                                    headerHeight: 28,
                                    rowHeight: 32,
                                }}
                                allItems={zones}
                                totalCount={zones.size}
                                selectedItems={selectedItems}
                                currentUser={currentUser}
                                isOpenPlayBackControl={isOpenPlayBackControl}
                                isZoneCreationDisabled={isZoneCreationDisabled}
                                dispatch={dispatch}
                            />
                        </Fragment>
                    )}
                </AutoSizer>
            </div>
        );
    }
}

const createDeepEqualSelector = createSelectorCreator(
    defaultMemoize,
    isEqual,
);

const getIntl = (state, intl) => intl;
const getSelectedSublocation = state => state.siteView.selectedSublocation;
const getPlaybackZones = state => state.playback.zones;
const getPlaybackCurrentTs = state => state.playback.currentTs;
const getIsOpenPlayBackControl = state => state.playback.isOpenPlayBackControl;

const handleZone = (intl, zonesPrototypes, zone) => (zone.formattedZone({ intl, zonesPrototypes }));

const getZonesFromStore = (zones, zonesPrototypes, selectedSublocation, intl, playbackZones, isOpenPlayBackControl, currentTs) => {
    if (isOpenPlayBackControl) {
        const currentTsDate = new Date(currentTs).toISOString();

        return playbackZones.reduce((memo, zone) => {
            const zoneInSublocation = selectedSublocation && zone.sublocation_uid === selectedSublocation.uid;
            const zoneCurrentlyExist = zone.created_at < currentTsDate && currentTsDate < zone.removed_at;

            if (zoneInSublocation && zoneCurrentlyExist) {
                memo.push(handleZone.apply(this, [intl, zonesPrototypes, zone]));
            }
            return memo;
        }, []);
    }

    return zones.reduce((memo, zone) => {
        if (selectedSublocation && (zone.sublocation_uid === selectedSublocation.uid)) {
            memo.set(zone.uid, handleZone.apply(this, [intl, zonesPrototypes, zone]));
        }
        return memo;
    }, new Map());
};

const getFilteredZones = createDeepEqualSelector(
    [Zone.getAllZonesArray, ZonesPrototypes.getAllZonePrototypes, getSelectedSublocation, getIntl, getPlaybackZones, getIsOpenPlayBackControl, getPlaybackCurrentTs],
    (zones, zonesPrototypes, selectedSublocation, intl, playbackZones, isOpenPlayBackControl, currentTs) =>
        getZonesFromStore(zones, zonesPrototypes, selectedSublocation, intl, playbackZones, isOpenPlayBackControl, currentTs),
);

const mapStateToProps = (state, { intl }) => {
    const { selectedItems, height, mode } = state.siteView;
    const { isOpenPlayBackControl } = state.playback;

    return {
        selectedItems,
        height,
        zones: getFilteredZones(state, intl),
        isOpenPlayBackControl,
        currentUser: state.user,
        isZoneCreationDisabled: !!mode,
    };
};

export default injectIntl(connect(mapStateToProps)(injectIntl(ZonesTable)));
