import React, { Fragment } from 'react';
import PropTypes from 'prop-types';
import { connect } from 'react-redux';
import { SortDirection, AutoSizer } from 'react-virtualized';
import { createSelectorCreator, defaultMemoize } from 'reselect';
import isEqual from 'lodash/isEqual';
import _CloneDeep from 'lodash/cloneDeep';

import { getNodeSwVersion, getTimeZoneOfProject } from 'utils';
import { WinAnchor } from 'stores/classes';
import { Node, Sublocation } from 'stores/classes/Selectors';

import VirtualTable from './VirtualTable';
import TableTitle from './TableTitle';

let containerWidth = 1300;
let containerHeight = 600;

function SHMWinAnchorsTable(props) {
    const { anchors, timeZone, dispatch } = props;
    const anchorsCount = anchors.size;

    return (
        <div className="AppLayout-System-Tab tab-system-anchors">
            <AutoSizer
                onResize={({ height, width }) => {
                    containerWidth = width;
                    containerHeight = height;
                }}
            >
                {({ height, width }) => (
                    <Fragment>
                        <TableTitle
                            totalCount={anchorsCount}
                            width={containerWidth}
                        />
                        <VirtualTable
                            defaultSortName="name"
                            defaultSortOrder={SortDirection.ASC}
                            timeZone={timeZone}
                            tableSize={{
                                width: containerWidth,
                                height: containerHeight,
                                headerHeight: 40,
                                rowHeight: 40,
                            }}
                            allItems={anchors}
                            totalCount={anchorsCount}
                            dispatch={dispatch}
                        />
                    </Fragment>
                )}
            </AutoSizer>
        </div>
    );
}

SHMWinAnchorsTable.propTypes = {
    dispatch: PropTypes.func,
    anchors: PropTypes.shape(),
    timeZone: PropTypes.string,
};

SHMWinAnchorsTable.defaultProps = {
    dispatch: () => {
    },
    anchors: new Map(),
    timeZone: 'Etc/GMT',
};

const getAnchors = (anchors) => {
    const anchorsMap = new Map();
    anchors.forEach((anchor) => {
        if (anchor.node && anchor.node.mac_address) {
            const sublocation = anchor.position && (anchor.position.sublocation_uid || anchor.position.sl_uid) ?
                Sublocation.getSublocationByUid(anchor.position.sublocation_uid) : {};
            const node = Node.getNodeByMacAddress(anchor.node.mac_address);
            if (node) {
                const formattedAnchor = {
                    ...anchor,
                    sublocation_name: sublocation ? sublocation.name : '',
                    mac_address: node.mac_address,
                    last_heard: node.last_heard,
                    firmware_version: getNodeSwVersion(node),
                };

                anchorsMap.set(anchor.uid, formattedAnchor);
            } else {
                console.warn('Not found node for win_anchor');
                anchorsMap.set(anchor.uid, anchor);
            }
        }
    });
    return anchorsMap;
};

const getTimeZone = state => getTimeZoneOfProject(state.appContext.project);

const getNodesWithoutLastHeard = state => (Object.values(state.entities.nodes.byMac).map((_node) => {
    const node = _CloneDeep(_node);
    delete node.last_heard;
    return node;
}));

const createDeepEqualSelector = createSelectorCreator(
    defaultMemoize,
    isEqual,
);
const getFilteredAnchors = createDeepEqualSelector(
    [WinAnchor.getAllWinAnchorsArray, getNodesWithoutLastHeard],
    (anchors, nodes) => (getAnchors(anchors, nodes)),
);

const mapStateToProps = state => ({
    anchors: getFilteredAnchors(state),
    timeZone: getTimeZone(state),
});

export default connect(mapStateToProps)(SHMWinAnchorsTable);
