import { store } from 'stores/configureStore';
import * as storage from 'utils/browserStorage';
import { STORAGE_KEY } from 'common/constants';
import { getNodeByMacAddress, userHasRoleCompanyAdmin, isProjectScopeUser } from 'utils/utils';
import { tryParse, getHostName, isUserImpersonate } from 'utils/simpleUtils';


class LocationStore {
    static projectStorageKeys = [
        STORAGE_KEY.LAST_LOCATION, STORAGE_KEY.LAST_SUBLOCATION, STORAGE_KEY.LAST_FLOORPLAN,
        STORAGE_KEY.LAST_INDEX_OF_SITE_VIEW_TABS, STORAGE_KEY.LAST_STATE_OF_RIGHT_SIDE_BAR,
        STORAGE_KEY.LAST_STATE_OF_OBJECT_FILTER, STORAGE_KEY.LAST_DASHBOARD_WIDGETS,
        STORAGE_KEY.LAST_STATE_OF_VIEW_FILTER, STORAGE_KEY.LAST_CREATED_HEATMAPS,
        STORAGE_KEY.LAST_CONTEXT_OF_SHM,
    ];
    
    // APP
    static getHostName = () => {
        let { hostName } = store.getState().appContext;
        
        if (!hostName) {
            hostName = storage.getLocalStorageValue(STORAGE_KEY.HOST_NAME);
        }
        
        if (!hostName) {
            hostName = getHostName();
            storage.setLocalStorageKeyValue(STORAGE_KEY.HOST_NAME, hostName);
        }
        
        return hostName;
    };
    
    static getlocalStorageKey = (key) => {
        const email = LocationStore.getUserEmail();
        const projectUid = LocationStore.getCurrentProjectUid();
        const isValidKey = LocationStore.projectStorageKeys.includes(key);

        const emailBasedKey = `${key}:${email}`;
        return (isValidKey && projectUid) ? `${emailBasedKey}:${projectUid}` : emailBasedKey;
    };

    // NEW CONTEXT
    static setToLocalStorage = (key, value) => {
        storage.setLocalStorageKeyValue(LocationStore.getlocalStorageKey(key), JSON.stringify(value));
    };
    
    static getFromLocalStorage = (key) => {
        const storageKey = LocationStore.getlocalStorageKey(key);
        return tryParse(storage.getLocalStorageValue(storageKey));
    };
    
    // COMPANY
    static isCompanyScope = () => !store.getState().appContext.project;
    
    static getCompanyByName = name => store.getState().entities.companies.find(company => company.name === name);
    
    // PROJECT
    static getCurrentProjectUid = () => {
        const currentProject = LocationStore.getCurrentProject();
        return (currentProject && currentProject.uid) || null;
    };
    
    static getCurrentProject = () => store.getState().appContext.project;
    
    static getProjectByName = name => store.getState().entities.projects.find(project => project.name === name);
    
    static setUserEmail = (email) => {
        storage.setSessionStorageKeyValue(STORAGE_KEY.USER_EMAIL, email);
    };
    
    static getUserEmail = () => store.getState().user.email || storage.getSessionStorageValue(STORAGE_KEY.USER_EMAIL);

    static isSystemUser = user => user && user.access && user.access.isSystemUser;
    
    static isCompanyAdmin = user => user && user.access && user.access.isCompanyAdmin;
    
    static getCurrentUser = () => store.getState().user;
    
    static getCurrentUserData = () => {
        const currentUser = LocationStore.getCurrentUser();

        if (isUserImpersonate(currentUser)) {
            return currentUser.onBehalf.labUserData;
        }
        const { email } = currentUser;
        return LocationStore.getCompanyUsers().find(user => user.email === email);
    };
    
    static getCompanyUsers = () => Object.values(store.getState().entities.users.byUid);
    
    static isCompanyAdminAsProjectSystemAdmin = () => {
        const currentUser = LocationStore.getCurrentUser();
        return isProjectScopeUser(currentUser.access.currentRole) && userHasRoleCompanyAdmin(currentUser.access);
    };
    
    static isOnBehalfOfSystemUser = () => !!LocationStore.getCurrentUser().onBehalf;
    
    // TRACKABLES
    static getTrackablesByGroup = groupUid => LocationStore.getTrackables()
        .filter((trackable) => {
            if (trackable && trackable.groups && groupUid) {
                return trackable.groups.indexOf(groupUid) !== -1;
            }
            return false;
        });
    
    static getNodesByGroup = groupUid => LocationStore.getTrackablesByGroup(groupUid)
        .map((trackable) => {
            if (trackable && trackable.node && trackable.node.mac_address) {
                return getNodeByMacAddress(store, trackable.node.mac_address);
            }
            return undefined;
        })
        .filter(x => x);
    
    static getAlarms = () => {
        const isPlaybackOpen = (store.getState().playback && store.getState().playback.isOpenPlayBackControl);
        if (isPlaybackOpen) {
            return store.getState().playback.alarms;
        }
        return store.getState().entities.alarms;
    };
    
    // used in classes
    static getNotifications = () => store.getState().entities.notifications;
    static getPlayback = () => store.getState().playback;
    static getLocations = () => store.getState().entities.locations;
    static getSublocations = () => store.getState().entities.sublocations;
    static getAnchors = () => Object.values(store.getState().entities.winAnchors.byUid);
    static getAssociations = () => store.getState().entities.associations;
    static getNodeByMacAddress = mac_address => mac_address && store.getState().entities.nodes.byMac[mac_address.toUpperCase()];
    static getNodesObject = () => store.getState().entities.nodes.byMac;
    static getTrackables = () => Object.values(store.getState().entities.trackables.byUid);
    static getTrackablesObject = () => store.getState().entities.trackables.byUid;
    static getZones = () => store.getState().entities.zones.byUid;
    static getZonesArray = () => Object.values(store.getState().entities.zones.byUid);
    static getGroups = () => Object.values(store.getState().entities.groups.byUid);
    static getGroupsObject = () => store.getState().entities.groups.byUid;
    static getZonesPrototypes = () => store.getState().entities.zonesPrototypes;
}

export default LocationStore;
