import * as api from 'api';
import { ACTIONS, MODAL_ACTIONS, STORAGE_KEY } from 'common/constants';
import { isUserImpersonate } from 'utils/simpleUtils';
import { USER_ROLES } from 'common/user-roles';
import { updateLabUserData } from 'stores/actions/userActions';
import { errorHandler, errorHandlerGetRequest } from 'stores/actions/errorHandlerActions';
import * as storage from 'utils/browserStorage';

export const loadHelperLink = () => async (dispatch, getState) => {
    const { appContext } = getState();
    try {
        const helperLink = await api.getHelperLink();
        const hostName = storage._getHostName(appContext);
        dispatch({
            type: ACTIONS.HELPER_LINK_SET,
            params: hostName + helperLink,
        });
    } catch (err) {
        console.error(err);
        errorHandlerGetRequest(err, dispatch);
    }
};

export const loadMonitorLink = () => async (dispatch) => {
    try {
        const siteMonitorLink = await api.getSiteMonitorLink();
        dispatch({
            type: ACTIONS.SITE_MONITOR_LINK_SET,
            params: siteMonitorLink,
        });
    } catch (err) {
        console.error(err);
        errorHandlerGetRequest(err, dispatch);
    }
};

export const loadUsers = () => (dispatch) => new Promise((resolve) => {
    api.getUsers()
        .then((users) => {
            dispatch({
                type: ACTIONS.USERS_SET,
                params: users.filter((user) => !isUserImpersonate(user)),
            });
            resolve();
        })
        .catch((err) => {
            resolve();
            console.warn(err);
            errorHandlerGetRequest(err, dispatch);
        });
});

export const loadUserRoles = () => async (dispatch) => {
    try {
        const userRoles = await api.getUserAccessControl();
        dispatch({
            type: ACTIONS.USERS_ACCESS_CONTROL_SET,
            params: userRoles,
        });
    } catch (err) {
        console.warn(err);
        errorHandlerGetRequest(err, dispatch);
    }
};

/**
 * @function
 * @param {Object} data
 * @property {String} data.projectUid - user project UID
 * @property {String} data.userUid - user UID
 * @property {String} user.userRole - user role
 * @property {Function} user.dispatch - redux dispatch function
 * @property {Function} user.getState - redux get state function
 * @return {Promise}
 * */
async function updateUserAccessInProject(data) {
    const { projectUid, userUid, userRole, dispatch, getState } = data;
    const project = getState().appContext.project;
    if (!projectUid) return Promise.resolve(1);

    return api.updateUserAccess(projectUid, { uid: userUid, role: userRole })
        .then((user) => {
            dispatch({
                type: ACTIONS.USERS_UPDATE_PROJECT_ACCESS,
                params: {
                    ...user,
                    project_uid: projectUid,
                    isCurrentProject: project ? project.uid === projectUid : false,
                },
            });
        })
        .catch((err) => {
            Promise.resolve(1);
            errorHandler(err, dispatch);
        });
}

/**
 * @function
 * @param {Object} userData
 * @property {String} user.uid - user UID
 * @property {String} user.role - user role
 * @property {String} user.project_uid - user project UID
 * @return {Function}
 * */
export const updateUserAccess = (userData) => async (dispatch, getState) => updateUserAccessInProject({
    projectUid: userData.project_uid,
    userUid: userData.uid,
    userRole: userData.role,
    dispatch,
    getState,
});

/**
 * @function
 * @param {Object} user
 * @property {String} user.uid - user uid
 * @property {String} user.email - user email
 * @property {String} user.first_name - user first name
 * @property {String} user.last_name - user last name
 * @property {Object} user.user_details - user details
 * @property {String} user.role - user role
 * @return {Promise<Function>}
 * */
export const deleteUser = (user) => async (dispatch, getState) => {
    try {
        if (user.uid) {
            await api.deleteUser(user.uid);
        } else {
            for (let i = 0; i < user.length; i += 1) {
                // eslint-disable-next-line no-await-in-loop
                await api.deleteUser(user[i].uid);
            }
        }
        dispatch({
            type: ACTIONS.USERS_DELETE,
            params: user,
        });
    } catch (err) {
        console.error(err);
        errorHandler(err, dispatch);
    }
};

/**
 * @function
 * @param {Object} user
 * @property {String} user.email - user email
 * @property {String} user.first_name - user first name
 * @property {String} user.last_name - user last name
 * @property {Object} user.user_details - user details
 * @property {String} user.role - user role
 * @return {Promise<Function>}
 * */
export const editUser = (user) => async (dispatch, getState) => {
    try {
        const userToUpdate = {
            email: user.email,
            first_name: user.first_name,
            last_name: user.last_name,
            user_details: { phone_number: user.phone_number ? user.phone_number : '' },
            role: user.role,
        };

        const userData = await api.updateUser(userToUpdate, user.uid);

        const stateUser = getState().user;
        if (isUserImpersonate(stateUser) && stateUser?.onBehalf?.labUserData?.email === userData.email) {
            dispatch(updateLabUserData(userData));
            return;
        }

        if (stateUser.email === user.email) {
            dispatch({
                type: ACTIONS.USERS_UPDATE,
                params: userData,
            });
            dispatch({
                type: ACTIONS.USER_LOGIN,
                params: userData,
            });
            api.setParamsToHeader({ email: userData.email }, storage.getLocalStorageValue(STORAGE_KEY.SYSTEM_COMPANY_UID));
        } else if (!getState().worker.isWebSocketOn) {
            dispatch({
                type: ACTIONS.USERS_UPDATE,
                params: userData,
            });
        }

        if (user.role !== USER_ROLES.COMPANY_ADMIN) {
            await updateUserAccessInProject({
                projectUid: user.project_uid,
                userUid: user.uid,
                userRole: user.role,
                dispatch,
                getState,
            });
        }
    } catch (err) {
        console.error(err);
        errorHandler(err, dispatch);
    }
};

/**
 * @function
 * @param {Object} params
 * @return {Promise<Function>}
 * */
export const editUserAccessControl = (params) => async (dispatch) => {
    try {
        const data = await api.updateUserAccessControl(params);
        dispatch({
            type: ACTIONS.USERS_ACCESS_CONTROL_UPDATE,
            params: data,
        });
    } catch (err) {
        console.error(err);
        errorHandler(err, dispatch);
    }
};

/**
 * @function
 * @param {Object} params
 * @property {String} params.project_uid - project UID
 * @property {String} params.email - user email
 * @property {String} params.password - user password
 * @property {String} params.first_name - user first name
 * @property {String} params.last_name - user last name
 * @property {Object} params.user_details - user details
 * @property {Boolean} params.send_notification - flag to send notification
 * @property {String} params.role - user role
 * @return {Function}
* */
export const createUser = (params) => (dispatch, getState) => {
    const projectUid = params.project_uid;

    const user = {
        email: params.email,
        password: params.password,
        first_name: params.first_name,
        last_name: params.last_name,
        user_details: {
            phone_number: params.phone_number ? params.phone_number : '',
        },
        send_notification: !!params.send_notification,
        role: params.role,
    };

    api.createUser(user)
        .then(async (userData) => {
            dispatch({
                type: ACTIONS.USERS_CREATE,
                params: userData,
            });
            // todo: project roles function
            if (user.role !== USER_ROLES.COMPANY_ADMIN) {
                await updateUserAccessInProject({
                    projectUid,
                    userUid: userData.uid,
                    userRole: user.role,
                    dispatch,
                    getState,
                });
            }
        })
        .catch((err) => {
            Promise.resolve(1);
            console.error(err);
            errorHandler(err, dispatch);
        });
};

/**
 * @function
 * @param {String} email - user email
 * @return {Function}
 * */
export const restoreUserPassword = (email) => (dispatch) => {
    dispatch({
        type: MODAL_ACTIONS.MODAL_SEND_NOTIFICATION_USER_EDIT,
    });

    return new Promise((resolve, reject) => {
        api.restorePassword(email)
            .then((user) => {
                dispatch({
                    type: MODAL_ACTIONS.MODAL_GET_NOTIFICATION_USER_EDIT,
                });

                resolve(user);
            }, (error) => reject(error))
            .catch((err) => {
                resolve();
                console.error(err);
                errorHandler(err, dispatch);
            });
    });
};

/**
 * @function
 * @param {Object} params
 * @property {String} params.project_uid - project UID
 * @property {String} params.email - user email
 * @property {String} params.password - user password
 * @property {String} params.first_name - user first name
 * @property {String} params.last_name - user last name
 * @property {Object} params.user_details - user details
 * @property {Boolean} params.send_notification - flag to send notification
 * @property {String} params.role - user role
 * @return {Function}
 * */
export const createUserSystemScope = (params) => (dispatch) => {
    const user = {
        email: params.email,
        password: params.password,
        first_name: params.first_name,
        last_name: params.last_name,
        user_details: {
            phone_number: params.phone_number ? params.phone_number : '',
        },
        send_notification: !!params.send_notification,
        role: params.role,
    };
    api.createUser(user)
        .then((userData) => {
            dispatch({
                type: ACTIONS.USERS_CREATE,
                params: userData,
            });

            return userData;
        })
        .catch((err) => {
            Promise.resolve(1);
            console.error(err);
            errorHandler(err, dispatch);
        });
};

/**
 * @function
 * @param {Object} params
 * @property {String} params.email - user email
 * @property {String} params.first_name - user first name
 * @property {String} params.last_name - user last name
 * @property {Object} params.user_details - user details
 * @property {String} params.role - user role
 * @return {Promise<Function>}
 * */
export const editUserSystemScope = (params) => async (dispatch, getState) => {
    try {
        const user = {
            email: params.email,
            first_name: params.first_name,
            last_name: params.last_name,
            role: params.role,
            user_details: {
                phone_number: params.phone_number ? params.phone_number : '',
            },
        };
        const userData = await api.updateUser(user, params.uid);
        const stateUser = getState().user;

        if (stateUser.email === params.email) {
            dispatch({
                type: ACTIONS.USERS_UPDATE,
                params: userData,
            });
            dispatch({
                type: ACTIONS.USER_LOGIN,
                params: userData,
            });
            api.setParamsToHeader({ email: userData.email }, storage.getLocalStorageValue(STORAGE_KEY.SYSTEM_COMPANY_UID));
        } else if (!getState().worker.isWebSocketOn) {
            dispatch({
                type: ACTIONS.USERS_UPDATE,
                params: userData,
            });
        }
    } catch (err) {
        console.error(err);
        errorHandler(err, dispatch);
    }
};

/**
 * @function
 * @param {Object} params
 * @property {String} params.uid - user UID
 * @property {String} params.oldPassword - user old password
 * @property {String} params.newPassword - user new password
 * @return {Promise<Function>}
 * */
export const changePassword = (params) => async (dispatch) => {
    try {
        await api.changePassword(params);
        storage.LocationStoreMethods.removeLocalStorageKeyValue(STORAGE_KEY.FORCE_TO_CHANGE_PASSWORD, params.decodedToken.user.email, null);
    } catch (err) {
        console.error(err);
        errorHandler(err, dispatch);
    }
};
