import _isEmpty from 'lodash/isEmpty';
import moment from 'moment-timezone';
import { getReportsList, getReportData } from 'api';
import { ACTIONS, TRACKABLE_CATEGORY } from 'common/constants';
import { errorHandlerGetRequest } from 'stores/actions/errorHandlerActions';
import { handleTrackableTimeline, getTimeZoneOfProject, getTimeByFormatAndTimezone, durationHumanize } from 'utils/timeUtils';
import { download } from 'utils/htmlUtils';


const formatData = (data) => {
    data.formattedTotalOnSite = durationHumanize(data.totalOnSite);
    data.formattedTotalUnresponsive = durationHumanize(data.totalUnresponsive);
    data.formattedTotalIdle = durationHumanize(data.totalIdle);
    data.formattedTotalActive = durationHumanize(data.totalActive);

    return data;
};

function handleReportData(data, from, to, timezone) {
    if (!data.trackableObjects) return [];
    
    const trackables = [];
    for (const uid in data.trackableObjects) {
        if (data.trackableObjects[uid] && data.trackableObjects[uid].category === TRACKABLE_CATEGORY.PERSON) {
            const trackable = handleTrackableTimeline(uid, data.trackableObjects[uid], from, to, timezone);
            trackables.push(formatData(trackable));
        }
    }
    return trackables;
}

export const getAttendanceReport = params => async (dispatch, getState) => {
    try {
        const { appContext: { project } } = getState();
        const timezone = getTimeZoneOfProject(project);
        const report = await getReportData(params, true);
        if (_isEmpty(report)) return;
        if (typeof report.shift === 'string') {
            report.shift = JSON.parse(report.shift);
        }
        if (typeof report.data === 'string') {
            report.data = JSON.parse(report.data);
        }

        const from = moment.tz(report.start, 'Etc/GMT').valueOf();
        const to = moment.tz(report.end, 'Etc/GMT').valueOf();
        const data = handleReportData(report.data, from, to, timezone);
        
        dispatch({
            type: ACTIONS.SET_ATTENDANCE_REPORT,
            params: {
                status: 100,
                data,
                from,
                to,
                shift_name: report.shift && report.shift.name,
                uid: report.uid,
                site_name: report.site_name,
                project_name: report.project_name,
            },
        });
    } catch (err) {
        dispatch({
            type: ACTIONS.SET_ATTENDANCE_REPORT_ERROR,
            params: { message: 'Error during report requesting', title: 'Report error' },
        });
        dispatch(errorHandlerGetRequest(err, dispatch));
    }
};

export const getAttendanceReportsListAction = (params, type) => async (dispatch) => {
    try {
        const { reports } = await getReportsList(params, type);
        
        dispatch({
            type: ACTIONS.SET_ATTENDANCE_REPORT_LIST,
            params: reports,
        });
    } catch (err) {
        dispatch({
            type: ACTIONS.SET_ATTENDANCE_REPORT_ERROR,
            params: { message: 'Error during report requesting', title: 'Report error' },
        });
        dispatch(errorHandlerGetRequest(err, dispatch));
    }
};

export const getAttendanceReportCsv = (data, report, project, companyName, userEmail) => {
    const timezone = getTimeZoneOfProject(project);
    const fields = ['Name', 'Status', 'Start Time', 'End Time'];
    const reportNameRow = '#Attendance report\n';
    const companyRow = `#Company: ${companyName}\n`;
    const projectRow = `#Project: ${report.project_name}\n`;
    const siteRow = `#Site: ${report.site_name}\n`;
    const shiftName = `#Shift Name: ${report.shift_name || 'Auto-created'}\n`;
    const shiftStart = `#Shift Start: ${new Date(report.from).toISOString()}\n`;
    const shiftEnd = `#Shift End: ${new Date(report.to).toISOString()}\n`;
    const peoples = `#People involved: ${data.length}\n`;
    const downloadedBy = `#Downloaded by ${userEmail}\n`;
    function getType(type) {
        switch (type) {
            case 'active': return 'Active';
            case 'idle': return 'Idle';
            case 'unresponsive': return 'Unresponsive';
            default: return '';
        }
    }
    let result = `${reportNameRow}${companyRow}${projectRow}${siteRow}${shiftName}${shiftStart}${shiftEnd}${peoples}${downloadedBy}\n${fields.join(',')}\n`;
    for (let i = 0; i < data.length; i += 1) {
        const trackable = data[i];
        if (trackable.timeline) {
            result += trackable.timeline.reduce((memo, action, index) => {
                if (index === 0 && action.type && trackable.timeline.length > 1) {
                    memo += `${trackable.name},Check-in,${action.from_iso},-\n`;
                    memo += `${trackable.name},${getType(action.type)},${action.from_iso},${action.to_iso}\n`;
                } else if (index === 0 && action.type) {
                    memo += `${trackable.name},Check-in,${action.from_iso},-\n`;
                    memo += `${trackable.name},${getType(action.type)},${action.from_iso},${action.to_iso}\n`;
                    memo += `${trackable.name},Check-out,-,${action.to_iso}\n`;
                } else if (index === 0 && !action.type) {
                    memo += `${trackable.name},Check-in,${action.to_iso},-\n`;
                } else if (index < trackable.timeline.length - 1 && !action.type) {
                    memo += `${trackable.name},Check-out,-,${action.from_iso}\n`;
                    memo += `${trackable.name},Check-in,${action.to_iso},-\n`;
                } else if (action.type && index === trackable.timeline.length - 1) {
                    memo += `${trackable.name},${getType(action.type)},${action.from_iso},${action.to_iso}\n`;
                    memo += `${trackable.name},Check-out,-,${action.to_iso}\n`;
                } else if (!action.type) {
                    memo += `${trackable.name},Check-out,-,${action.from_iso}\n`;
                } else {
                    memo += `${trackable.name},${getType(action.type)},${action.from_iso},${action.to_iso}\n`;
                }
                return memo;
            }, '');
        }
    }
    const start = getTimeByFormatAndTimezone(report.from, 'YYYY-MM-DD', timezone);
    const file_name = `${report.shift_name || 'day'}_${start}_attendance.csv`;

    download(file_name, result);
};
