import * as api from 'api';
import { push } from 'connected-react-router';
import { ACTIONS, STORAGE_KEY } from 'common/constants';
import { runWorker } from 'stores/actions/workerActions';
import { openChangePassword, openEditProfile } from 'stores/actions/modalActions/authActions';
import { openCompanyEditModalCompanyScope } from 'stores/actions/modalActions/companyModalActions';
import { setRolesToUser, userHasRole, decodeToken, isTemporaryPassword, getCurrentContextRole } from 'utils/utils';
import { getPagesForScope, getNavButtonsForScope, getUserMenuItemsForScope } from 'utils/scopeUtils';
import {
    _getHostName,
    getLocalStorageValue,
    LocationStoreMethods,
    setLocalStorageKeyValue,
    setSessionStorageKeyValue,
    reloadBrowser,
    getQueryParam,
} from 'utils/browserStorage';
import { makeCopy, isUserImpersonate, isUserSuperUser } from 'utils/simpleUtils';
import { USER_ROLES } from 'common/user-roles';
import { setAppContext } from 'stores/actions/appContextActions';
import { errorHandler } from 'stores/actions/errorHandlerActions';
import { clearStore } from 'stores/actions/commonActions';

/**
 *
 * @param params
 * @returns {{
 *      userParams: {
 *          access,
 *          project,
 *          onBehalf,
 *          email: string,
 *          token
 *      },
 *      appContextParams: {
 *          project,
 *          company
 *      }
 *  }}
 */
export const fillHeader = (params = LocationStoreMethods.getParamsFromStorage()) => {
    const { email, token, access, onBehalf, project, company } = params;
    const userParams = { email, token, access, onBehalf, project };
    const appContextParams = { company, project };

    if (!company || getLocalStorageValue(STORAGE_KEY.SYSTEM_COMPANY_UID) === company?.uid) {
        delete appContextParams.company;
    }

    appContextParams.hostName = _getHostName(undefined);

    api.setParamsToHeader({ ...appContextParams, ...userParams }, getLocalStorageValue(STORAGE_KEY.SYSTEM_COMPANY_UID));

    return {
        appContextParams,
        userParams,
    };
};

/**
 * @function
 * @params {Object} params
 * @returns {Promise<Function>}
 */
export const settingUp = (params) => async (dispatch) => {
    const { appContextParams, userParams } = fillHeader(params);

    dispatch(setAppContext(appContextParams));

    if (userParams.email && userParams.token) {
        dispatch(userLoginByToken(userParams));
    }
};

const getCurrentUserData = (state) => {
    const currentUser = state.user;
    
    if (isUserImpersonate(currentUser)) {
        return currentUser.onBehalf.labUserData;
    }
    return Object.values(state.entities.users.byUid).find((user) => user.email === currentUser.email);
};

const saveUserInfoAndReloadContext = (params) => {
    const userInfo = params;
    if (!userInfo.access) {
        const roles = setRolesToUser(userInfo);
        userInfo.access = {
            roles,
            currentRole: getCurrentContextRole(userInfo.role),
            role: userInfo.role,
            email: userInfo.email,
            isCompanyAdmin: userInfo.isCompanyAdmin,
            isSystemUser: userInfo.isSystemUser,
            isHighChartsLicensed: userInfo.isHighChartsLicensed,
        };
    }

    if (userInfo.company) {
        userInfo.company = {
            ...userInfo.company,
            ...userInfo.company.user_params,
        };
    } else if (!userInfo.company && userInfo.company !== null) {
        userInfo.company = LocationStoreMethods.getParamsFromStorage().company;
    }

    if (userInfo.project) {
        userInfo.project = {
            ...userInfo.project,
            ...userInfo.project.user_params,
        };
    } else if (!userInfo.project && userInfo.project !== null) {
        userInfo.project = LocationStoreMethods.getParamsFromStorage().project;
    }

    LocationStoreMethods.setParamsToStorage(userInfo);

    return userInfo;
};

export const userLogout = (currentRole) => (dispatch) => {
    userLogoutWithoutBrowserReload(currentRole)(dispatch);
    reloadBrowser();
};

export const userLogoutWithoutBrowserReload = (currentRole) => (dispatch, getState) => {
    const { email, onBehalf } = LocationStoreMethods.getParamsFromStorage();
    if (onBehalf) {
        api.logout({ email: onBehalf.email });
    } else {
        api.logout({ email });
    }

    LocationStoreMethods.clearUserParamsFromStorage(currentRole);
    fillHeader();

    dispatch(clearStore());

    dispatch({
        type: ACTIONS.USER_LOGOUT,
    });

    const targetServer = getQueryParam('target_server');
    const search = targetServer ? `?target_server=${targetServer}` : '';

    dispatch(push(`/${search}`));
};

export const userLoginByPassword = (params) => async (dispatch, getState) => {
    const targetServer = getQueryParam('target_server');
    const search = targetServer ? `?target_server=${targetServer}` : '';

    dispatch(push(`/${search}`)); // clear url if we already have some to avoid problems with components which render by this some url

    const data = await api.auth(params);
    const decodedToken = decodeToken(data.token);

    if (data.isSystemUser) {
        setLocalStorageKeyValue(STORAGE_KEY.SYSTEM_COMPANY_UID, data.company.uid);
        delete data.company;
    }

    const userInfo = saveUserInfoAndReloadContext(data);

    // need to show first modal(recommendation to change password and welcome)
    // after loading all data, this params removed after loadEntities
    getState().user.password = params.password;
    getState().user.sign_in_count = data.sign_in_count;

    return {
        user: {
            ...data,
            uid: decodedToken.user.uid,
            isTemporaryPassword: isTemporaryPassword(params.password),
        },
        userInfo,
    };
};

/**
 *
 * @param params.email
 * @param params.token
 * @param params.access
 * @param params.scope
 * @returns {Function}
 */
export const userLoginByToken = (params) => async (dispatch, getState) => {
    const targetServer = getQueryParam('target_server');
    const search = targetServer ? `?target_server=${targetServer}` : '';

    dispatch(push(`/${search}`)); // clear url if we already have some to avoid problems with components which render by this some url

    params.scope = createScope(params.access, getState);
    const { email, token, access, scope, onBehalf } = params;
    await dispatch({
        type: ACTIONS.USER_LOGIN,
        params: {
            email, token, access, scope, onBehalf,
        },
    });
    await dispatch(push(`/loading${search}`));
    const onBehalfToken = onBehalf ? onBehalf.token : null;
    try {
        const { status, needUpdatePassword } = await api.validateToken({ email, token, onBehalfToken });
        if (!status) {
            fillHeader();
        }
        dispatch(runWorker());

        if (needUpdatePassword && !isUserSuperUser(params)) {
            const decodedToken = decodeToken(onBehalfToken || token);
            dispatch(openChangePassword({
                uid: decodedToken.user.uid,
                email,
                token: onBehalfToken || token,
                force: true,
            }));
        }
    } catch (err) {
        errorHandler(err, dispatch);
    }
};

const createScope = (userAccess, getState) => {
    const { appContext: { company } } = getState();

    return {
        pages: getPagesForScope(userAccess),
        navButtons: getNavButtonsForScope(userAccess),
        userMenu: (currentUserData) => getUserMenuItemsForScope({
            userAccess,
            isCompanyContext: !!company,
            currentUserData,
            onCompanyEditClick: () => openCompanyEditModalCompanyScope(company),
            onCompanyReturnClick: () => userLogoutFromProjectSystemAdmin(),
            onUserEditClick: () => openEditProfile(currentUserData),
            onUserSignOutClick: () => userLogout(userAccess.currentRole),
            onUserGuideClick: () => {
                const { appContext: { helperLink } } = getState();

                const targetServer = getQueryParam('target_server');
                const search = targetServer ? `?target_server=${targetServer}` : '';

                window.open(`${helperLink}${search}`, '_blank');
            },
        }),
    };
};

export const updateLabUserData = (params) => (dispatch, getState) => {
    dispatch({
        type: ACTIONS.UPDATE_CURRENT_USER,
        params: {
            onBehalf: { ...getState().user.onBehalf, labUserData: params },
        },
    });
};

/**
 *
 * @param params
 * @param getState
 * @param labUserData
 * @returns {Object}
 * @example {
 *      email,
 *      token,
 *      company,
 *      isSuperAdmin,
 *      isSystemUser,
 *      isCompanyAdmin,
 *      sign_in_count,
 *      isHighChartsLicensed,
 *      needUpdatePassword,
 *      onBehalf: {
 *          logged,
 *          email,
 *          token,
 *          access: {
 *              roles string[],
 *              currentRole,
 *              email,
 *              isCompanyAdmin,
 *              isSystemUser,
 *              isHighChartsLicensed
 *          }
 *      }
 * }
 */
const getCompanyOnBehalfUser = (params, getState, labUserData) => {
    const data = {
        ...params,
        onBehalf: {
            ...getState().user,
            labUserData,
        },
    };
    delete data.onBehalf.company;
    return data;
};

/**
 * Logging system user create by script in company scope using onBehalf mechanism
 * @param company
 * @param company.uid
 * @param company.name
 * @param company.projects
 * @param company.projects[].id
 * @param company.projects[].name
 * @param company.projects[].taggings_count
 * @param company.projects[].user_params
 * @param company.projects[].user_params.timezone
 * @param company.projects[].user_params.offset
 * @param company.projects[].user_params.manager
 * @param company.projects[].uid
 * @param company.projects[].scope
 * @param company.projects[].role_uid
 * @param company.projects[].company_uid
 * @param company.projects[].privileges
 * @param company.projects[].company_name
 * @param company.projects[].company_role_uid
 * @param company.user_params
 * @param company.user_params.tag_allocation
 * @param company.user_params.color
 * @param company.users
 * @param company.users[].uid
 * @param company.users[].admin
 * @param company.users[].email
 * @param company.users[].first_name
 * @param company.users[].last_name
 * @param company.users[].created_at
 * @param company.users[].updated_at
 * @param company.color
 * @param company.address
 * @param company.tag_allocation
 * @returns {void}
 */
export const userLoginAsCompanyAdmin = (company) => (dispatch, getState) => { // change context
    const labUserData = getCurrentUserData(getState());

    api.getCompanyAccess(company.uid)
        .then((params) => {
            const onBehalfUser = getCompanyOnBehalfUser(params, getState, labUserData);
            const userInfo = saveUserInfoAndReloadContext(onBehalfUser);
            dispatch(settingUp(userInfo));
        })
        .catch((err) => { errorHandler(err, dispatch); });
};

export const userLogoutFromCompanyAdmin = () => (dispatch, getState) => {
    delete getState().user.onBehalf.labUserData;
    const onBehalf = makeCopy(getState().user.onBehalf);

    setSessionStorageKeyValue(STORAGE_KEY.CURRENT_COMPANY, null);

    dispatch(clearStore());

    const userInfo = saveUserInfoAndReloadContext(onBehalf);
    dispatch(settingUp(userInfo));
};

export const userLoginAsProjectSystemAdmin = (project) => (dispatch, getState) => {
    const currentUser = makeCopy(getState().user);

    if (!userHasRole(currentUser.access, USER_ROLES.PROJECT_SYSTEM_ADMIN)) {
        return;
    }

    currentUser.access.currentRole = USER_ROLES.PROJECT_SYSTEM_ADMIN;
    currentUser.project = project;

    const userInfo = saveUserInfoAndReloadContext(currentUser);
    dispatch(settingUp(userInfo));
};

export const userLogoutFromProjectSystemAdmin = () => (dispatch, getState) => {
    const currentUser = makeCopy(getState().user);

    if (!userHasRole(currentUser.access, USER_ROLES.COMPANY_ADMIN)) {
        return;
    }

    currentUser.access.currentRole = USER_ROLES.COMPANY_ADMIN;
    currentUser.project = null;

    dispatch(clearStore());

    setSessionStorageKeyValue(STORAGE_KEY.CURRENT_PROJECT, null);

    const userInfo = saveUserInfoAndReloadContext(currentUser);
    dispatch(settingUp(userInfo));
};
