import _Get from 'lodash/get';
import { WORKER_METHODS, WS_ACTIONS, NOTIFY_ACTIONS } from 'worker';
import * as api from 'api';
import { loadAlarms } from 'stores/actions/entityActions/alarmActions';
import RawPosition from 'stores/classes/RawPosition';
import RawHeartbeat from 'stores/classes/RawHeartbeat';
import * as storage from 'utils/browserStorage';
import { STORAGE_KEY } from 'common/constants';
import { errorHandlerGetRequest } from 'stores/actions/errorHandlerActions';
import { openProjectDeletionPopupModal } from 'stores/actions/modalActions/authActions';
import Worker from 'worker/ws.worker.js'; // handled by plugin worker-loader in webpack config

const isWorkerAlreadyExist = (getState) => !!getState().worker.worker;

export const rerunWorker = () => (dispatch, getState) => {
    const { worker, user, appContext } = getState();
    worker && worker.worker && worker.worker.postMessage({
        method: WORKER_METHODS.UPDATE_CONNECTION,
        params: {
            userEmail: user.email || storage.getSessionStorageValue(STORAGE_KEY.USER_EMAIL),
            userToken: user.token || storage.getSessionStorageValue(STORAGE_KEY.USER_TOKEN),
            companyUid: (appContext.company && appContext.company.uid) || storage.getLocalStorageValue(STORAGE_KEY.SYSTEM_COMPANY_UID),
            projectUid: (appContext.project && appContext.project.uid) || null,
        },
    });
};

function handlePositions(message, dispatch, worker) {
    const { type, params } = message.data;
    const positions = params.map((posValues) => new RawPosition(posValues));
    dispatch({ type, params: positions });
    worker.postMessage({ method: WORKER_METHODS.CLIENT_RECEIVED_POS });
}

function handleHeartbeats(message, dispatch) {
    const { type, params } = message.data;
    const heartbeats = params.map((hbValues) => new RawHeartbeat(hbValues));
    dispatch({ type, params: heartbeats });
}

function handleUserProjectDelete(message, dispatch, user, appContext) {
    const { type, params } = message.data;
    const currentProjectUid = appContext.project ? appContext.project.uid : null;
    if ((params.deletedUserEmail === user.email || currentProjectUid) && currentProjectUid === params.uid) {
        dispatch(openProjectDeletionPopupModal());
    } else {
        dispatch({ type, params });
    }
}

function handleOthers(message, dispatch) {
    const { type, params } = message.data;
    dispatch({ type, params });
}

export const runWorker = function () {
    return (dispatch, getState) => {
        if (isWorkerAlreadyExist(getState)) {
            dispatch(rerunWorker());
            return;
        }

        const worker = new Worker();
        const { user, appContext, router } = getState();
        const projectUid = _Get(appContext, 'project.uid', null);
        api.getWsHost(storage._getHostName(appContext))
            .then((wsHost) => worker.postMessage({
                method: WORKER_METHODS.INIT,
                params: {
                    userEmail: user.email || storage.getSessionStorageValue(STORAGE_KEY.USER_EMAIL),
                    userToken: user.token || storage.getSessionStorageValue(STORAGE_KEY.USER_TOKEN),
                    companyUid: (appContext.company && appContext.company.uid) || storage.getLocalStorageValue(STORAGE_KEY.SYSTEM_COMPANY_UID),
                    projectUid: (appContext.project && appContext.project.uid) || null,
                    wsHost: wsHost + (router.location.search || ''),
                },
            }))
            .catch((err) => {
                errorHandlerGetRequest(err, dispatch);
            });

        worker.onmessage = function (message) {
            const { type } = message.data;

            if (type === WS_ACTIONS.POSITIONS) {
                handlePositions(message, dispatch, worker);
            } else if (type === NOTIFY_ACTIONS.HBS) {
                handleHeartbeats(message, dispatch);
            } else if ((type === NOTIFY_ACTIONS.USER_DELETED || type === NOTIFY_ACTIONS.PROJECT_DELETED) && projectUid) {
                handleUserProjectDelete(message, dispatch, user, appContext);
            } else {
                handleOthers(message, dispatch);
            }
        };

        dispatch({
            type: WS_ACTIONS.WORKER_RUN,
            params: {
                worker,
            },
        });
    };
};

export const stopLiveMode = () => (dispatch, getState) => {
    const state = getState();
    state && state.worker && state.worker.worker && state.worker.worker.postMessage({
        method: WORKER_METHODS.TURN_OFF_LIVE_MODE,
    });

    dispatch({
        type: WS_ACTIONS.CHANGE_WORKER_PARAMS,
        params: {
            isRealTimeOn: false,
        },
    });
};

export const startLiveMode = () => (dispatch, getState) => {
    const state = getState();
    state && state.worker && state.worker.worker && state.worker.worker.postMessage({
        method: WORKER_METHODS.TURN_ON_LIVE_MODE,
    });

    dispatch(loadAlarms());

    dispatch({
        type: WS_ACTIONS.CHANGE_WORKER_PARAMS,
        params: {
            isRealTimeOn: true,
        },
    });
};

export const wsUnsubscribe = (channels) => (dispatch, getState) => {
    const state = getState();
    state && state.worker && state.worker.worker && state.worker.worker.postMessage({
        method: WORKER_METHODS.UNSUBSCRIBE,
        params: channels,
    });
};

export const wsSubscribe = (channels) => (dispatch, getState) => {
    const state = getState();
    state && state.worker && state.worker.worker && state.worker.worker.postMessage({
        method: WORKER_METHODS.SUBSCRIBE,
        params: channels,
    });
};
