import { channelList, parseHexToChannels } from 'utils/utils';
import { DEFAULT_VALUES, FILTER, OBJECT_TYPES } from 'common/constants';
import { RIGHT_SIDE_BAR_MESSAGING } from 'common/ui-labels';


export default class Group {
    constructor(group = []) {
        this.uid = group.uid;
        this.name = group.name;
        this.project_uid = group.project_uid;
        this.company_uid = group.company_uid;
        this.created_at = group.created_at;
        this.updated_at = group.updated_at;
        this.removed_at = group.removed_at;
        this.custom_info = group.custom_info;
        this.color = group.color;

        // default values by validation schema
        this.sensor_config = group.sensor_config || { sensors: [] };
        this.user_params = group.user_params || {};
    }

    getProperties() {
        return { ...this };
    }


    _getGroupTrackables(trackablesMap) {
        const trackables = Object.values(trackablesMap.byUid);
        return trackables.filter(to => to.groups && to.groups.some(uid => uid === this.uid));
    }

    _getCustomParams() {
        return this.user_params.customParams;
    }

    _getColor() {
        return this.color || '#000000';
    }

    _getZoneWithExceptions(zones) {
        return zones.filter(zone => zone.zone_exceptions_groups && zone.zone_exceptions_groups.indexOf(this.uid) > -1);
    }

    _parseZoneWithExceptions(zones) {
        const inclusionsForZones = [];
        const exclusionsForZones = [];
        zones.forEach((zone) => {
            const result = zone.getZoneSpecialCases(this.uid);
            if (result !== null) {
                if (result) {
                    inclusionsForZones.push(zone.name);
                } else {
                    exclusionsForZones.push(zone.name);
                }
            }
        });
        return { inclusionsForZones, exclusionsForZones };
    }

    addValueLabelToValue() {
        const group = new Group(this);

        // todo fix for add value label and keep methods
        // see trackableHasProperLowLatencyLinkOptions() TEM/i
        group.value = this.uid;
        group.label = this.name;
        group._type = OBJECT_TYPES.GROUP;
        return group;
    }

    getFormattedGroupForSendingMessage(intl) {
        return {
            value: this.uid,
            label: `${RIGHT_SIDE_BAR_MESSAGING.BUTTONS.GROUP_START(intl)}${this.name}`,
            uid: this.uid,
            name: this.name,
        };
    }

    getFormattedGroup(trackablesMap, zones) {
        const formattedGroup = { ...this };
        const trackables = this._getGroupTrackables(trackablesMap);

        formattedGroup.trackable_count = trackables.length;
        formattedGroup.trackable_statinary_count = trackables.filter(x => x.stationary).length;
        formattedGroup.color = this._getColor();
        formattedGroup.customParams = this._getCustomParams();

        if (this.sensor_config.sensors.length > 0) {
            const sensorInterface = parseHexToChannels(this.sensor_config.sensors.filter(x => x.channels));
            if (sensorInterface && sensorInterface.length) {
                formattedGroup.sensorInterface = channelList.map((x) => {
                    const found = sensorInterface.find(y => y.channels === x);
                    if (found) {
                        if (found.op_mode === 'on-request' || found.op_mode === 'periodic') {
                            found.enabled = true;
                        }
                        return found;
                    }
                    return { channels: x, type: FILTER.NA };
                });
            }

            const sensorSpecialCases = this.sensor_config.sensors.filter(x => !x.channels);
            if (sensorSpecialCases && sensorSpecialCases.length > 0) {
                formattedGroup.sensorSpecialCases = sensorSpecialCases && sensorSpecialCases.length > 0 ? sensorSpecialCases.reduce((r, x) => {
                    r[x.type] = x.enabled !== undefined ? x.enabled : false;
                    return r;
                }, {}) : [];
            }
        } else {
            formattedGroup.sensorSpecialCases = {};
            formattedGroup.sensorInterface = [];
        }

        formattedGroup.zoneExceptionsGroups = this._getZoneWithExceptions(zones);
        const { inclusionsForZones, exclusionsForZones } = this._parseZoneWithExceptions(formattedGroup.zoneExceptionsGroups);
        formattedGroup.inclusionsForZones = inclusionsForZones;
        formattedGroup.exclusionsForZones = exclusionsForZones;
        return formattedGroup;
    }

    static getAllGroups(state) {
        return state.entities.groups.byUid;
    }

    static getAllGroupsArray(state) {
        return Object.values(Group.getAllGroups(state));
    }

    static getColorOfGroup(uid, groups) {
        const group = groups.byUid[uid];
        return (group && group.color) || DEFAULT_VALUES.TRACKABLE_COLOR;
    }
}
