import _Has from 'lodash/has';
import _Get from 'lodash/get';
import { T2T_TYPE, OBJECT_TYPES, TRACKABLE_CATEGORY } from 'common/constants';
import { SITE_TRACKABLES_TABLE, RIGHT_SIDE_BAR_MESSAGING } from 'common/ui-labels';
import { hasProperVersionForLowLatencyLink } from 'utils/utils';
import { translateT2TLowLatencyConfigForTrackable } from 'utils/pwsConfigUtils';

export default class Trackable {
    constructor(trackable) {
        this.acc_config = trackable.acc_config || null;
        this.attached_zones = trackable.attached_zones || null;
        this.bt_config = trackable.bt_config || null;
        this.company_uid = trackable.company_uid || '';
        this.created_at = trackable.created_at || '';
        this.default_height = trackable.default_height || null;
        this.display_name = trackable.display_name || null;
        this.groups = trackable.groups || null;
        this.name = trackable.name || '';
        this.tag_config = trackable.tag_config || {};
        this.node = trackable.node || null;
        this.stationary = _Get(trackable, 'tag_config.stationary', false);
        this.position = trackable.position || null;
        this.pos_report_frequency = trackable.pos_report_frequency || '';
        this.position_update_interval = !isNaN(trackable.position_update_interval) ? trackable.position_update_interval : null;
        this.project_uid = trackable.project_uid || '';
        this.removed_at = trackable.removed_at || '';
        this.self_zone_uid = trackable.self_zone_uid || null;
        this.sensor_config = trackable.sensor_config || {};
        this.type = trackable.type || null;
        this.category = trackable.category || TRACKABLE_CATEGORY.UNSPECIFIED;
        this.paired = trackable.paired || null;
        this.pairedGroups = trackable.pairedGroups || null;
        this.uid = trackable.uid;
        this.custom_info = trackable.custom_info;
        this.updated_at = trackable.updated_at || '';
        if (trackable.low_latency_link_config && !_Has(trackable, 'tag_config.t2t_cfg')) {
            this.low_latency_link_config = trackable.low_latency_link_config || {};
        } else if (_Has(trackable, 'tag_config.t2t_cfg')) {
            this.low_latency_link_config = translateT2TLowLatencyConfigForTrackable(_Get(trackable, 'tag_config.t2t_cfg'));
        }
        if (Array.isArray(trackable.t2t_no_alarm_uid)) {
            this.t2t_no_alarm_uid = trackable.t2t_no_alarm_uid ? trackable.t2t_no_alarm_uid.join(',') : '';
        } else {
            this.t2t_no_alarm_uid = trackable.t2t_no_alarm_uid;
        }
        if (trackable.user_params) {
            this.user_params = trackable.user_params;
        }
    }

    addValueLabelToValue() {
        const props = this.getProperties();
        const trackable = new Trackable(props);
        trackable.value = props.uid;
        trackable.label = props.name;
        trackable._type = OBJECT_TYPES.TRACKABLE;
        return trackable;
    }

    getProperties() {
        return { ...this };
    }

    formattedTrackableForAdminPage(zones, associations, nodes, groups) {
        const trackable = this.getProperties();
        const assoc = associations.byObjUid[this.uid];
        if (assoc) {
            const node = nodes[assoc.mac_address.toUpperCase()];
            trackable.mac_address = node ? node.mac_address.toUpperCase() : null;
        } else {
            trackable.node = null;
            trackable.mac_address = '';
        }
        if (trackable.groups && trackable.groups.length > 0) {
            const names = trackable.groups.map(uid => groups[uid] && groups[uid].name);
            trackable.groupNames = names.join(', ');
        } else {
            trackable.groupNames = '';
        }
        const zoneExceptions = this.parseZoneWithExceptions(zones);
        const { inclusionsForZones, exclusionsForZones } = this.parseZoneExceptions(zoneExceptions);
        trackable.inclusionsForZones = inclusionsForZones;
        trackable.exclusionsForZones = exclusionsForZones;
        trackable.stationary = trackable.tag_config.stationary;

        return trackable;
    }

    formattedTrackableForSitePage(intl, zones, groups, nodes) {
        const props = this.getProperties();
        if (this.node && this.node.mac_address) {
            const node = nodes[this.node.mac_address.toUpperCase()];
            if (node) {
                props.battery = node.battery;
            }
        }

        if (this.groups) {
            const displayGroups = [];
            props.groupList = this.groups.reduce((memo, uid) => {
                if (groups[uid]) {
                    memo.push(groups[uid]);
                    displayGroups.push(groups[uid].name);
                }
                return memo;
            }, []);
            props.displayGroups = displayGroups.join(',');
        }

        props.isZoneSpecial = zones.filter((zone) => {
            if (zone.zone_exceptions) {
                return zone.zone_exceptions.some(uid => uid === this.uid);
            }
            return false;
        }).length > 0;

        props.title = Trackable.makeTitle(props, intl);
        const zoneExceptions = this.parseZoneWithExceptions(zones);
        if (zoneExceptions) {
            const { inclusionsForZones, exclusionsForZones } = this.parseZoneExceptions(zoneExceptions);
            props.inclusionsForZones = inclusionsForZones;
            props.exclusionsForZones = exclusionsForZones;
        } else {
            props.inclusionsForZones = [];
            props.exclusionsForZones = [];
        }
        return props;
    }

    /**
     * @private
     * @param zones
     * @returns {*}
     */
    parseZoneWithExceptions(zones) {
        return zones.filter(zone => zone.zone_exceptions.indexOf(this.uid) > -1);
    }

    /**
     * @private
     * @param zones
     * @returns {{exclusionsForZones: *, inclusionsForZones: *}}
     */
    parseZoneExceptions(zones) {
        const inclusionsForZones = [];
        const exclusionsForZones = [];
        zones.forEach((zone) => {
            const result = zone.getZoneSpecialCases(this.uid);
            if (result !== null) {
                if (result) {
                    inclusionsForZones.push(zone.name);
                } else {
                    exclusionsForZones.push(zone.name);
                }
            }
        });
        return { inclusionsForZones, exclusionsForZones };
    }

    formattedTrackableUidNameNode(intl) {
        return {
            ...this.formattedTrackablesValueLabel(),
            label: `${RIGHT_SIDE_BAR_MESSAGING.BUTTONS.TRACKABLE_START(intl)}${this.name}`,
            groups: this.groups,
            node: this.node ? this.node : null,
        };
    }

    formattedTrackablesUidNameGroup() {
        return {
            ...this.formattedTrackablesUidName(),
            groups: this.groups ? this.groups : [],
        };
    }

    formattedTrackablesUidName() {
        return {
            uid: this.uid,
            name: this.name,
        };
    }

    formattedTrackablesValueLabel() {
        return {
            value: this.uid,
            label: this.name,
            ...this.formattedTrackablesUidName(),
        };
    }

    formattedTrackablesAddUidName() {
        return {
            ...this.getProperties(),
            ...this.formattedTrackablesValueLabel(),
        };
    }

    formattedTrackableWithoutPosition() {
        const props = this.getProperties();
        delete props.position;
        const tko = new Trackable(props);
        delete tko.isUnresponsive;
        return tko;
    }

    formattedTrackableForWidget(state) {
        const sublocation_uid = this.position.sl_id;
        const sublocation = state.entities.sublocations.find(_sublocation => _sublocation.uid === sublocation_uid);
        const location_uid = sublocation ? sublocation.location_uid : null;

        return {
            ...this.getProperties(),
            sublocation_uid,
            location_uid,
            y: this.position.y,
            x: this.position.x,
        };
    }

    hasProperVersionForLowLatencyLink(node) {
        return hasProperVersionForLowLatencyLink(node, this.low_latency_link_config);
    }

    static makeTitle(row, intl) {
        if (row) {
            const groupList = row.groupList ? row.groupList.map(g => g.name).join(', ') : '';
            const nodeMacAddress = row.node && row.node.mac_address ? row.node.mac_address : '';
            const haveAttachedZones = row.attached_zones && row.attached_zones.length > 0
                ? SITE_TRACKABLES_TABLE.TEXT.YES_FUNC(intl)
                : SITE_TRACKABLES_TABLE.TEXT.NO_FUNC(intl);
            const haveSpecialCase = row.isZoneSpecial ? SITE_TRACKABLES_TABLE.TEXT.YES_FUNC(intl) : SITE_TRACKABLES_TABLE.TEXT.NO_FUNC(intl);

            return `${SITE_TRACKABLES_TABLE.TEXT.GROUPS_FUNC(intl)}: ${groupList}\n` +
                `${SITE_TRACKABLES_TABLE.TEXT.MAC_FUNC(intl)}: ${nodeMacAddress}\n` +
                `${SITE_TRACKABLES_TABLE.TEXT.BUFFER_FUNC(intl)}: ${haveAttachedZones}\n` +
                `${SITE_TRACKABLES_TABLE.TEXT.SPECIAL_CASE_FUNC(intl)}: ${haveSpecialCase}`;
        }
        return '';
    }

    isResponsive = (isPlaybackOpen) => isPlaybackOpen || (this.node && this.node.responsive);

    hasLowLatencyConfigurations() {
        const role = _Get(this, 'tag_config.t2t_cfg.role', false);
        return role && (role === T2T_TYPE.MONITOR || role === T2T_TYPE.ANNOUNCER);
    }

    hasAssociationAndPosition(state) {
        const playback = state.playback;
        let association;
        const isPlaybackOpen = playback && playback.isOpenPlayBackControl;
    
        if (!isPlaybackOpen) {
            association = state.entities.associations.byObjUid[this.uid];
        } else {
            association = playback.associations.find(assoc => assoc.trob_uid === this.uid);
        }
        
        return this.position && this.position.sl_id && association;
    }

    getLeafletTitle() {
        const serialNumber = this.user_params?.serial_number || '';
        if (serialNumber) {
            return `${this.name}, ${serialNumber}`;
        }
        return this.name;
    }

    /**
     *
     * @param mac
     * @param trackablesMap
     * @returns {Trackable}
     */
    static getTrackableByMacAddress = (mac, trackablesMap) => Object.values(trackablesMap).find((trackable) => {
        if (trackable.node) {
            return trackable.node.mac_address.toLowerCase() === mac.toLowerCase();
        }
        return false;
    });

    /**
     *
     * @param state
     * @returns {array<Trackable>}
     */
    static getAllTrackables(state) {
        return state.entities.trackables.byUid;
    }

    static getAllTrackablesArray(state) {
        return Object.values(Trackable.getAllTrackables(state));
    }

    static getNotAssociatedTrackables(state) {
        const trackables = Trackable.getAllTrackablesArray(state);
        const associations = state.entities.associations;
        return trackables.filter(trackable => !associations.byObjUid[trackable.uid]);
    }

    static getAssociatedTrackables(state) {
        const trackables = Trackable.getAllTrackablesArray(state);
        const associations = state.entities.associations;
        return trackables.filter(trackable => associations.byObjUid[trackable.uid]);
    }
}
