import { WS_ACTIONS } from 'worker';
import { ACTIONS, MODAL_ACTIONS, ALARM_STATES } from 'common/constants';
import { makeCopy } from 'utils/simpleUtils';
import PlaybackTrackable from 'stores/classes/PlaybackTrackable';
import Association from 'stores/classes/Association';
import Zone from 'stores/classes/Zone';
import Alarm from 'stores/classes/Alarm';


const initialState = {
    isPlaying: false,
    isOpenPlayBackControl: false,
    pending_request: false,
    zones: [],
    trackables: [],
    alarms: [],
    associations: [],
    playSpeed: 1,
    playSpeeds: [16, 8, 4, 2, 1, 0.5, 0.25, 0.125],
    
    popUpSettings: {
        show: false,
        time: null,
    },
    coords: {
        x: null,
        y: null,
    },
};

const alarmSet = (alarms, elem) => {
    if (alarms.find(x => x.uid === elem.uid)) {
        return alarms;
    }
    return [...alarms, new Alarm(elem)];
};

const alarmClear = (alarms, elem) => {
    const nextState = [];
    alarms.forEach((alarm) => {
        if (alarm.uid !== elem.uid) {
            nextState.push(alarm);
        }
    });
    return nextState;
};

const mergeTrackablesAndPositions = (trackables, positions) => {
    if (!Array.isArray(trackables)) {
        return [];
    } else if (!Array.isArray(positions)) {
        return trackables;
    }
    return trackables.map((trackable) => {
        const position = positions.find(p => trackable.uid === p.dev_id);
        if (position) {
            return new PlaybackTrackable({
                ...trackable,
                position,
            });
        }
        return trackable;
    });
};

const playback = (state = initialState, action) => {
    switch (action.type) {
        case WS_ACTIONS.PLAYBACK_CHANGE_PARAMS:
            return {
                ...state,
                ...action.params,
            };
        case ACTIONS.ZONES_INCLUDE_REMOVED_SET: {
            let zones = makeCopy(action.params);
            zones = zones.map(_zone => new Zone(_zone));
            
            return { ...state, zones };
        }
        case WS_ACTIONS.PLAYBACK_INITIAL_POSITIONS: {
            const trackablesWithInitPositions = state.trackables.reduce((memo, trackable) => {
                const position = action.params.find(_pos => _pos.dev_id === trackable.uid);
                if (position) {
                    memo.push(new PlaybackTrackable({
                        ...trackable,
                        position: { ...position, mac: position.mac_address },
                    }));
                } else {
                    memo.push(trackable);
                }
                return memo;
            }, []);
            return { ...state, trackables: trackablesWithInitPositions };
        }
        case ACTIONS.TRACKABLES_ASSOCIATION_PLAYBACK_SET:
            return { ...state, associations: action.params.map(to => new Association(to)) };
        case MODAL_ACTIONS.PLAYBACK_SELECT_TRACKABLES_CHANGE: {
            if (!action.params) {
                return { ...initialState };
            }
            const trackables = action.params.selectedTrackables ? action.params.selectedTrackables : action.params;
            return { ...state, trackables: trackables.map(to => new PlaybackTrackable(to)) };
        }
        case WS_ACTIONS.PLAYBACK_POSITIONS: {
            const positions = action.params.map(position => ({
                ...position,
                mac: position.mac_address,
            }));
            
            return {
                ...state,
                trackables: mergeTrackablesAndPositions(state.trackables, positions),
            };
        }
        case WS_ACTIONS.PLAYBACK_PENDING_REQUEST: {
            return {
                ...state,
                pending_request: true,
            };
        }
        case WS_ACTIONS.PLAYBACK_RECEIVED_REQUEST: {
            return {
                ...state,
                pending_request: false,
            };
        }
        case WS_ACTIONS.PLAYBACK_ALARMS: {
            let newAlarms = [...state.alarms];
            action.params
                .sort((a, b) => a.timestamp - b.timestamp)
                .forEach((elem) => {
                    if (elem.state && elem.state.toLowerCase() === ALARM_STATES.CLEAR) {
                        newAlarms = alarmClear(newAlarms, elem);
                    } else if (elem.state && elem.state.toLowerCase() === ALARM_STATES.SET) {
                        newAlarms = alarmSet(newAlarms, elem);
                    }
                });
            return { ...state, alarms: newAlarms };
        }
        case ACTIONS.ALARMS_PLAYBACK_CLEAR: {
            return { ...state, alarms: [] };
        }
        case WS_ACTIONS.PLAYBACK_CLEAR:
        case WS_ACTIONS.PLAYBACK_ERROR:
            return { ...initialState };
        default:
            return state;
    }
};

export default playback;
