import { all, fork, call, put, takeEvery, select } from 'redux-saga/effects';
import { push } from 'connected-react-router';
import _Get from 'lodash/get';
import { ACTIONS, STORAGE_KEY, SAGA_ACTIONS } from 'common/constants';
import { userLoginAsProjectSystemAdmin } from 'stores/actions/userActions';
import * as api from 'api';
import { getCurrentUserScope, getDefaultPageForScope, pageInDefaultScope } from 'common/scopes';
import { getLocalStorageValue, getSessionStorageValue, LocationStoreMethods } from 'utils/browserStorage';
import { getDashboardWidgets } from 'utils/scopeUtils';
import {
    findFloorplanBySelectedFloorplan, findLocationBySublocation, findLocationIfExisted, findSublocationByFloorplan,
    getFloorplan, getSublocation,
} from 'utils/utils';
import { initialState } from 'stores/reducers/siteViewFilter';
import errorHandler from 'stores/sagas/commonSaga';
import { applicationContext, sublocationsList, locationsList, floorplansList, currentUser } from './selectors';

function getFilterObject(userEmail, projectUid) {
    const lastStateOfObjectFilter = LocationStoreMethods.getFromLocalStorage(STORAGE_KEY.LAST_STATE_OF_OBJECT_FILTER, userEmail, projectUid);
    if (!lastStateOfObjectFilter) {
        LocationStoreMethods.setToLocalStorage(STORAGE_KEY.LAST_STATE_OF_OBJECT_FILTER, initialState.objectFilter, userEmail, projectUid);
    }
    return lastStateOfObjectFilter;
}

const getCustomZonesFilter = (customZonesPrototypes) => customZonesPrototypes.reduce((memo, customZone) => {
    const zoneTypeName = customZone.zone_type.name;

    const CUSTOM_ZONES_FILTER_DEFAULT_STATE = true;

    memo[`isShowing_${zoneTypeName}`] = CUSTOM_ZONES_FILTER_DEFAULT_STATE;
    memo[`isShowingActive_${zoneTypeName}`] = CUSTOM_ZONES_FILTER_DEFAULT_STATE;
    memo[`isShowingInactive_${zoneTypeName}`] = CUSTOM_ZONES_FILTER_DEFAULT_STATE;

    return memo;
}, {});

// needed to combine initial layerFilter with custom zones initial filter
function* setInitialFilter() {
    try {
        const customZonesPrototypes = yield call(api.getCustomZonesPrototypes);
    
        const customZonesFilter = getCustomZonesFilter(customZonesPrototypes);
        const initialFilter = { ...initialState.layerFilter, ...customZonesFilter };
    
        yield put({
            type: ACTIONS.SITE_VIEW_LAYER_FILTER,
            params: initialFilter,
        });
    } catch (err) {
        yield call(errorHandler, err);
    }
}

function getLayerFilter(userEmail, projectUid) {
    const lastStateOfLayerFilter = LocationStoreMethods.getFromLocalStorage(STORAGE_KEY.LAST_STATE_OF_VIEW_FILTER, userEmail, projectUid);
    if (!lastStateOfLayerFilter) {
        LocationStoreMethods.setToLocalStorage(STORAGE_KEY.LAST_STATE_OF_VIEW_FILTER, initialState.layerFilter, userEmail, projectUid);
    }
    return lastStateOfLayerFilter;
}

function getLeafletState(userEmail, projectUid) {
    const lastStateOfLeaflet = LocationStoreMethods.getFromLocalStorage(STORAGE_KEY.LAST_STATE_OF_LEAFLET, userEmail, projectUid);
    if (!lastStateOfLeaflet) {
        LocationStoreMethods.setToLocalStorage(STORAGE_KEY.LAST_STATE_OF_LEAFLET, initialState.leaflet, userEmail, projectUid);
    }
    
    return lastStateOfLeaflet;
}

function* getUserContextData() {
    const appContext = yield select(applicationContext);
    const user = yield select(currentUser);
    const userEmail = user.email || getSessionStorageValue(STORAGE_KEY.USER_EMAIL);
    const projectUid = (appContext.project && appContext.project.uid) || null;
    
    return {
        userEmail,
        projectUid,
        user,
    };
}

function getScopeContextData(access, userEmail, projectUid) {
    const widgets = (LocationStoreMethods.getFromLocalStorage(STORAGE_KEY.LAST_DASHBOARD_WIDGETS, userEmail, projectUid) || getDashboardWidgets(access));
    const isOpenSideBarFromContext = LocationStoreMethods.getFromLocalStorage(STORAGE_KEY.LAST_STATE_OF_RIGHT_SIDE_BAR, userEmail, projectUid);
    const selectedTab = LocationStoreMethods.getFromLocalStorage(STORAGE_KEY.LAST_INDEX_OF_SITE_VIEW_TABS, userEmail, projectUid);
    const lastPath = LocationStoreMethods.getFromLocalStorage(STORAGE_KEY.LAST_LOGGER_PATH, userEmail, projectUid);
    const completedHeatmaps = LocationStoreMethods.getFromLocalStorage(STORAGE_KEY.LAST_CREATED_HEATMAPS, userEmail, projectUid);
    const context = LocationStoreMethods.getFromLocalStorage(STORAGE_KEY.LAST_CONTEXT_OF_SHM, userEmail, projectUid);
    
    return {
        widgets,
        isOpenSideBarFromContext,
        selectedTab,
        lastPath,
        completedHeatmaps,
        context,
    };
}

function* getRouter() {
    return yield select((state) => state.router.location.search);
}

function* getBuildingInfrostructure(userEmail, projectUid) {
    const locations = yield select(locationsList);
    const sublocations = yield select(sublocationsList);
    const floorplans = yield select(floorplansList);
    
    let selectedLocation = findLocationIfExisted(LocationStoreMethods.getFromLocalStorage(STORAGE_KEY.LAST_LOCATION, userEmail, projectUid), locations);
    let selectedSublocation = LocationStoreMethods.getFromLocalStorage(STORAGE_KEY.LAST_SUBLOCATION, userEmail, projectUid);
    let selectedFloorplan = LocationStoreMethods.getFromLocalStorage(STORAGE_KEY.LAST_FLOORPLAN, userEmail, projectUid);
    if (selectedLocation) {
        selectedSublocation = getSublocation(selectedSublocation, selectedLocation, sublocations, userEmail, projectUid);
        selectedFloorplan = getFloorplan(selectedFloorplan, selectedSublocation, floorplans, userEmail, projectUid);
    } else {
        selectedFloorplan = findFloorplanBySelectedFloorplan(selectedFloorplan, floorplans, userEmail, projectUid);
        selectedSublocation = findSublocationByFloorplan(selectedSublocation, selectedFloorplan, sublocations, userEmail, projectUid);
        selectedLocation = findLocationBySublocation(selectedLocation, selectedSublocation, locations, userEmail, projectUid);
    }
    
    return {
        selectedFloorplan,
        selectedSublocation,
        selectedLocation,
    };
}

function* handleRedirect(currentScope) {
    const storedLastPage = yield call(getLocalStorageValue, STORAGE_KEY.LAST_PAGE);
    const lastPage = pageInDefaultScope(storedLastPage, currentScope) ? storedLastPage : null;
    const redirectPath = lastPage || getDefaultPageForScope(currentScope);
    const search = yield call(getRouter);
    yield put(push(redirectPath + search));
}

function isSystemUserAsCompanyAdmin(user) {
    const onBehalfIsSystemUser = _Get(user, 'onBehalf.access.isSystemUser', false);
    const userIsCompanyAdmin = _Get(user, 'access.isCompanyAdmin', false);

    return onBehalfIsSystemUser && userIsCompanyAdmin;
}

function* reloadLastContext(action) {
    try {
        const { access } = action.params;
        const { projectUid, user, userEmail } = yield call(getUserContextData);
        const appContext = yield select(applicationContext);
        const lastProject = LocationStoreMethods.getFromLocalStorage(STORAGE_KEY.LAST_PROJECT, userEmail, projectUid);
        if (lastProject && !appContext.project && !isSystemUserAsCompanyAdmin(user)) {
            yield put(userLoginAsProjectSystemAdmin(lastProject));
            return;
        }
        const currentScope = yield call(getCurrentUserScope, access.currentRole);
        const project = projectUid ? (yield call(api.getProjects))[0] : null;

        const {
            widgets, isOpenSideBarFromContext, selectedTab, lastPath, completedHeatmaps, context,
        } = yield call(getScopeContextData, access, userEmail, projectUid);

        const { selectedLocation, selectedSublocation, selectedFloorplan } = yield call(getBuildingInfrostructure, userEmail, projectUid);
        const lastStateOfObjectFilter = yield call(getFilterObject, userEmail, projectUid);

        yield call(setInitialFilter);

        const lastStateOfLayerFilter = yield call(getLayerFilter, userEmail, projectUid);
        const lastStateOfLeaflet = yield call(getLeafletState, userEmail, projectUid);

        yield put({
            type: ACTIONS.RELOAD_CONTEXT,
            params: {
                project,
                context,
                lastPath,
                widgets,
                selectedLocation,
                selectedSublocation,
                selectedFloorplan,
                completedHeatmaps,
                lastStateOfObjectFilter,
                lastStateOfLayerFilter,
                lastStateOfLeaflet,
                selectedTab,
                isOpenSideBar: typeof isOpenSideBarFromContext === 'boolean' && isOpenSideBarFromContext,
            },
        });

        yield call(handleRedirect, currentScope);
    } catch (err) {
        yield call(errorHandler, err);
    }
}

function* watchReloadLastContext() {
    yield takeEvery(SAGA_ACTIONS.SAGA_RELOAD_LAST_CONTEXT, reloadLastContext);
}

export default function* generalSagas() {
    return yield all([
        fork(watchReloadLastContext),
    ]);
}
