import { all, call, fork, put, select, takeEvery } from 'redux-saga/effects';
import _get from 'lodash/get';
import _set from 'lodash/set';
import { DEFAULT_VALUES, SAGA_ACTIONS, T2T_TYPE, ZONE_MODE } from 'common/constants';
import { TRACKABLE_BULK_EDIT } from 'common/ui-labels';
import { formatAttachedZone, getZoneExceptions } from 'common/formatters';
import { closeTrackableBulkEditModal, submitTrackableBulkEditModal, openCommonMessageModal } from 'stores/actions/modalActions';
import { isAssetTag, valueNaN } from 'utils/utils';
import { convertMetersToMillimeters } from 'utils/formatters';
import { calculateAllowDefault } from 'utils/zoneUtils';
import { formatPwsConfig } from 'utils/pwsConfigUtils';
import * as api from 'api';

import { trackableByUid } from '../selectors';
import errorHandler from '../commonSaga';
import { handleAttachedZone, handleGroupTrackable } from './trackableCreateSaga';

function* handleMessageModal(trackablesCount, errors) {
    let message = '';
    let title = '';
    let status = '';
    const updatedTrackables = trackablesCount - errors.length;
    
    if (errors.length > 0) {
        message = TRACKABLE_BULK_EDIT.WARN_MESSAGE(updatedTrackables, trackablesCount);
        title = TRACKABLE_BULK_EDIT.INFO_TITLE;
        status = DEFAULT_VALUES.WARNING_STATUS;
    } else {
        message = TRACKABLE_BULK_EDIT.INFO_MESSAGE(trackablesCount, trackablesCount);
        title = TRACKABLE_BULK_EDIT.INFO_TITLE;
        status = DEFAULT_VALUES.SUCCESS_STATUS;
    }
    yield put(openCommonMessageModal({ title, message, status }));
}

const formatTrackable = (params, { storeTrackable, uid }) => {
    const trackable = {
        uid,
        sensor_config: {
            sensors: DEFAULT_VALUES.empty_array,
        },
    };

    if (params.buffer_checkbox) {
        if (params.buffer_radius) { trackable.buffer_radius = params.buffer_radius; }
        if (params.active_buffer) { trackable.active_buffer = params.active_buffer; }
        if (storeTrackable.self_zone_uid) { trackable.self_zone_uid = storeTrackable.self_zone_uid; }
        if (params.trackables_and_groups) {
            trackable.trackables_and_groups = (params.trackables_and_groups || []).filter((trackable_group) => trackable_group.uid !== uid);
        }
    }

    if (params.category_checkbox && params.category) {
        trackable.category = params.category.value ? params.category.value : params.category;
    }
    if (params.custom_info_checkbox) {
        trackable.custom_info = params.custom_info || '';
    }

    if (params.advanced_checkbox) {
        if (params.defaultHeight !== undefined) {
            trackable.default_height = convertMetersToMillimeters(params.defaultHeight);
        }
        if (params.position_update_interval && !isAssetTag(storeTrackable.category)) {
            const interval = params.position_update_interval;
            const position_update_interval = (interval && valueNaN(interval.value, interval)) || 0;
            if (!Number.isNaN(position_update_interval)) {
                trackable.position_update_interval = position_update_interval;
            }
        }
    }

    if (params.pws_checkbox) {
        if (params.role === T2T_TYPE.MONITOR) {
            trackable.t2t_no_alarm_uid = params.t2t_no_alarm_uid ? params.t2t_no_alarm_uid.split(',') : [];
        }
        const t2t_cfg = formatPwsConfig(params, params.category);
        if (!trackable.tag_config) {
            trackable.tag_config = {};
        }
        trackable.tag_config.t2t_cfg = t2t_cfg || null;
    }

    return trackable;
};

const getAttachedZones = (params, storeTrackable = {}) => {
    let create_attached_zone;
    let update_attached_zone;
    if (!params.self_zone_uid) {
        create_attached_zone = formatAttachedZone({ ...params, name: storeTrackable.name }, storeTrackable);
        if (create_attached_zone) {
            create_attached_zone.zone_mode = params.active_buffer ? ZONE_MODE.ALWAYS_ACTIVE : ZONE_MODE.DEACTIVATED;
            const { zone_exceptions, zone_exceptions_groups } = getZoneExceptions(params.trackables_and_groups);
            create_attached_zone.zone_exceptions = zone_exceptions;
            create_attached_zone.zone_exceptions_groups = zone_exceptions_groups;
        }
    } else {
        update_attached_zone = { uid: params.self_zone_uid, dynamic: true, shape: 'circle' };
        update_attached_zone.default_allowed = calculateAllowDefault({ apply_to: params.apply_to });
        update_attached_zone.zone_mode = params.active_buffer ? ZONE_MODE.ALWAYS_ACTIVE : ZONE_MODE.DEACTIVATED;

        const { zone_exceptions, zone_exceptions_groups } = getZoneExceptions(params.trackables_and_groups);

        update_attached_zone.zone_exceptions = zone_exceptions;
        update_attached_zone.zone_exceptions_groups = zone_exceptions_groups;
        update_attached_zone.zone_geometry = { radius: params.buffer_radius }; // * 1000*/;
    }

    return {
        create_attached_zone,
        update_attached_zone,
    };
};

const handlePwsConfig = (trackable, storeTrackable) => {
    const newPws = _get(trackable, 'tag_config.t2t_cfg', false);

    if (trackable.category === storeTrackable.category) {
        // 1. Category in Modal == Trackable Category AND PWS in Modal NOT CHANGE  => TKO  has old category and new PWS
        // 2. Category in Modal == Trackable Category AND PWS in Modal CHANGED     => TKO  has old category and new PWS
        if (newPws) { _set(trackable, 'tag_config.t2t_cfg', newPws); }
    } else {
        // 3. Category in Modal != Trackable Category AND PWS in Modal NOT CHANGED  => TKO has new category and default PWS
        // 4. Category in Modal != Trackable Category AND PWS in Modal CHANGED      => TKO has new category and new PWS
        if (newPws) { _set(trackable, 'tag_config.t2t_cfg', newPws); }
    }
};

function* updateTrackable(trackable, formValues) {
    const result = {};
    try {
        const storeTrackable = yield select(trackableByUid, trackable.uid);
        if (formValues.buffer_checkbox) {
            const { create_attached_zone, update_attached_zone } = yield call(getAttachedZones, trackable, storeTrackable);

            if (create_attached_zone) {
                create_attached_zone.trackable_object_uid = storeTrackable.uid;
            }
            if (update_attached_zone) {
                update_attached_zone.trackable_object_uid = storeTrackable.uid;
            }

            const zone = yield call(handleAttachedZone, create_attached_zone, update_attached_zone);
            if (zone && zone.dynamic && !zone.sublocation_uid && zone.uid) {
                trackable.self_zone_uid = zone.uid;
            }
        } else if (trackable.category && isAssetTag(trackable.category) && storeTrackable.self_zone_uid) {
            // remove buffer
            yield call(handleAttachedZone, undefined, undefined, storeTrackable.self_zone_uid);
            trackable.self_zone_uid = null;
        }

        if (formValues.groups_checkbox) {
            const groupUids = formValues.groupList ? formValues.groupList.map((x) => x.value) : [];
            if (groupUids) {
                yield call(handleGroupTrackable, trackable, groupUids);
            }
        }

        if (formValues.pws_checkbox) {
            yield call(handlePwsConfig, trackable, storeTrackable);
        } else if (storeTrackable.category && trackable.category && storeTrackable.category !== trackable.category) {
            // on change category and no PWD => reset PWS
            _set(trackable, 'tag_config.t2t_cfg', null);
        }

        yield call(api.updateTrackable, trackable, trackable.uid);
    } catch (err) {
        result.error = { uid: trackable.uid, name: trackable.name };
        console.error(err);
        yield call(errorHandler, err);
    }
    return result;
}

function* editSingleTrackable(values, uid) {
    const storeTrackable = yield select(trackableByUid, uid);
    const trackableValues = { ...values };

    const trackable = yield call(formatTrackable, trackableValues, { storeTrackable, uid });
    return yield call(updateTrackable, trackable, values);
}

function* bulkEditTrackable(action) {
    try {
        yield put(submitTrackableBulkEditModal(true));
        const trackables_uids = action.params.editedTrackables.map((tko) => tko.value);
        const trackablesCount = trackables_uids.length;
        const trackableValues = { ...action.params };
        delete trackableValues.editedTrackables;
        const errors = [];

        for (let i = 0; i < trackables_uids.length; i += 1) {
            const result = yield call(editSingleTrackable, trackableValues, trackables_uids[i]);
            if (result.error) {
                errors.push(result.error);
            }
        }
        
        yield put(submitTrackableBulkEditModal(false));
        yield put(closeTrackableBulkEditModal(action.params));
    
        yield call(handleMessageModal, trackablesCount, errors);
    } catch (err) {
        yield put(submitTrackableBulkEditModal(false));
        yield put(closeTrackableBulkEditModal(action.params));
        yield call(errorHandler, err);
    }
}

/*
 Rules:
    PWS checkbox uncheck:
     * category change => reset PWS (tag_config.t2t_cfg = null)
    PWS checkbox check:
     * category not changed => save PWS from modal
     * category changed and
    Buffer:
     * on change category from non ASSET to asset => remove buffer
     * on buffer checkbox uncheck => no actions with buffer (tko edit, remove buffer on uncheck)
    position_update_interval:
     * on change category from ASSET to non ASSET position_update_interval ignore
*/

function* watchBulkEditTrackable() {
    yield takeEvery(SAGA_ACTIONS.SAGA_TRACKABLES_BULK_EDIT, bulkEditTrackable);
}

export default function* trackableBulkEditSaga() {
    return yield all([
        fork(watchBulkEditTrackable),
    ]);
}
