import { select, fork, takeEvery, call, put, all } from 'redux-saga/effects';
import { change } from 'redux-form';
import _IsEqual from 'lodash/isEqual';
import _Get from 'lodash/get';
import * as api from 'api';

import { HTTP_STATUS, NAME_OF_FORMS, SAGA_ACTIONS } from 'common/constants';
import { convertMillimetersToMeters, convertMetersToMillimeters } from 'utils/formatters';
import { changeScaleZoneRadius } from 'utils/zoneUtils';
import { getLLLConfig, getTrackableLLLConfig, valueNaN } from 'utils/utils';
import { makeCopy, lengthToDegree } from 'utils/simpleUtils';
import {
    closeTrackableCollisionValidationModal, closeTrackableEditModal, openTrackableCollisionValidationModal,
    saveStationaryPosition,
} from 'stores/actions/modalActions/trackableModalActions';
import errorHandler from 'stores/sagas/commonSaga';
import { groupsList, trackablesList, trackableByUid, trackablesMap, nodesMap, sublocationByUid } from 'stores/sagas/selectors';
import { formatCreatedTrackable, formatTrackable, getValidationInfo, getZoneExceptions } from 'common/formatters/trackableFormatters';
import { openErrorModal } from 'stores/actions/modalActions/commonModalActions';


export function* handleAttachedZone(create_attached_zone, update_attached_zone, delete_attached_zone_uid) {
    try {
        if (create_attached_zone) {
            const zoneCopy = changeScaleZoneRadius(create_attached_zone);
            return yield call(api.createZone, zoneCopy);
        }
        if (update_attached_zone) {
            const zoneCopy = changeScaleZoneRadius(update_attached_zone);
            return yield call(api.updateZone, zoneCopy.uid, zoneCopy);
        }
        if (delete_attached_zone_uid) {
            return yield call(api.deleteZone, delete_attached_zone_uid);
        }
    } catch (err) {
        console.error(err);
        yield call(errorHandler, err);
    }
    return undefined;
}

export function* handleGroupTrackable(trackable, groups) {
    try {
        yield call(api.replaceTrackableGroups, { uid: trackable.uid }, groups, true);
    } catch (error) {
        const status = error.response && error.response.status;
        if (status === HTTP_STATUS.UNPROCESSABLE && error.response.data) {
            yield put(openErrorModal({ message: error.response.data, title: 'Error assigning groups to trackable' }));
        }
        console.error(error);
        yield call(errorHandler, error);
    }
}

export function* updateTrackable(params) {
    const result = {};
    try {
        const oldTo = yield select(trackableByUid, params.uid);
        const {
            trackable, groups, create_attached_zone, update_attached_zone, delete_attached_zone_uid,
        } = yield call(formatTrackable, params, oldTo);

        if (create_attached_zone) {
            create_attached_zone.trackable_object_uid = oldTo.uid;
        }
        if (update_attached_zone) {
            update_attached_zone.trackable_object_uid = oldTo.uid;
        }

        const trackableCopy = makeCopy(trackable);
        const zone = yield call(handleAttachedZone, create_attached_zone, update_attached_zone, delete_attached_zone_uid);
        if (zone && zone.dynamic && !zone.sublocation_uid && zone.uid) {
            trackableCopy.self_zone_uid = zone.uid;
        }
        if (groups) {
            yield call(handleGroupTrackable, trackableCopy, groups);
        }

        yield call(api.updateTrackable, trackableCopy, params.uid);
    } catch (err) {
        result.error = { uid: params.uid, name: params.name };
        console.error(err);
        yield call(errorHandler, err);
    }
    return result;
}

export function* createTrackable(params) {
    try {
        const {
            trackable, groups, create_attached_zone, update_attached_zone,
        } = yield call(formatTrackable, params);
        const newTrackable = yield call(api.createTrackable, trackable);
        if (create_attached_zone) {
            create_attached_zone.trackable_object_uid = newTrackable.uid;

            const { zone_exceptions, zone_exceptions_groups } = yield call(getZoneExceptions, trackable.trackables_and_groups);
            create_attached_zone.zone_exceptions = zone_exceptions;
            create_attached_zone.zone_exceptions_groups = zone_exceptions_groups;
        }

        trackable.uid = newTrackable.uid;
        const zone = yield call(handleAttachedZone, create_attached_zone, update_attached_zone);
        if (groups && groups.length > 0) {
            yield call(handleGroupTrackable, trackable, groups);
        }
        const updatedTrackable = { ...trackable };
        if (zone) {
            updatedTrackable.self_zone_uid = zone.uid;
        }
        if ((groups && groups.length > 0) || zone) { // update for apply groups on ui
            yield call(api.updateTrackable, updatedTrackable, updatedTrackable.uid);
        }
    } catch (err) {
        console.error(err);
        yield call(errorHandler, err);
    }
    return undefined;
}

export function* createEditTrackable(action) {
    try {
        const { trackableValues, trackableEdit } = action.params;

        const sublocation = yield select(sublocationByUid, trackableValues.area);
        const groups = yield select(groupsList);
        const trackables = yield select(trackablesList);
        const trackableMap = yield select(trackablesMap);
        const nodeMap = yield select(nodesMap);
        const trackable = yield call(formatCreatedTrackable, trackableValues, { trackables, groups });
    
        const newLLL = yield call(getLLLConfig, trackable);
        const oldLLL = yield call(getTrackableLLLConfig, trackableValues, trackableMap);
        const isEqualLLL = _IsEqual(newLLL, oldLLL);
    
        const groupList = trackable.groupList;
        const interval = trackable.position_update_interval;
        trackable.position_update_interval = (interval && valueNaN(interval.value, interval)) || 0;
        trackable.type = trackable.type.value ? trackable.type.value : trackable.type;
        trackable.category = trackable.category.value ? trackable.category.value : trackable.category;

        const x = _Get(trackable, 'pos_x');
        const y = _Get(trackable, 'pos_y');
        if (!isNaN(x) && !isNaN(y) && !isNaN(sublocation.offset_x) && !isNaN(sublocation.offset_y)) {
            const pos_x = yield call(convertMetersToMillimeters, x);
            const pos_y = yield call(convertMetersToMillimeters, y);
            trackable.x = pos_x + sublocation.offset_x;
            trackable.y = pos_y + sublocation.offset_y;
        }
        const { hasProperVersion, hasNode, node } = yield call(getValidationInfo, trackable, groupList, trackableEdit.params, isEqualLLL, nodeMap);
    
        if (!hasProperVersion) {
            yield put(openTrackableCollisionValidationModal(trackable, trackableEdit.params, hasNode, node, groupList));
        } else {
            if (trackableEdit.params && trackableEdit.params.uid) {
                yield call(updateTrackable, trackable);
            } else {
                yield call(createTrackable, trackable);
            }
            yield put(closeTrackableEditModal());
        }
    } catch (err) {
        yield call(errorHandler, err);
    }
}

function* closeCreateEditLLLTrackable() {
    yield put(closeTrackableCollisionValidationModal());
    yield put(closeTrackableEditModal());
}

function* createEditLLLTrackable(action) {
    try {
        const values = action.params.values ? action.params.values : {};
        const trackable = action.params.trackable ? action.params.trackable : {};
    
        if (trackable && trackable.uid) {
            yield call(updateTrackable, values);
        } else {
            yield call(createTrackable, values);
        }
        yield call(closeCreateEditLLLTrackable);
    } catch (err) {
        yield call(errorHandler, err);
    }
}

function* handleTrackablePosition(action) {
    try {
        if (action.params) {
            const { trackableUid, sublocationUid } = action.params;
            const data = yield call(api.getTrackablePosition, trackableUid);
            const sublocation = yield select(sublocationByUid, sublocationUid);
            const { location_uid, sublocation_uid, position } = data || {};
            if (position) {
                const { x, y, z } = position;
                if (!isNaN(x) && !isNaN(y)) {
                    yield put(change(NAME_OF_FORMS.TRACKABLE_EDIT, 'site', location_uid));
                    yield put(change(NAME_OF_FORMS.TRACKABLE_EDIT, 'area', sublocation_uid));
                    const pos_x = yield call(convertMillimetersToMeters, x - sublocation.offset_x);
                    const pos_y = yield call(convertMillimetersToMeters, y - sublocation.offset_y);
                    yield put(change(NAME_OF_FORMS.TRACKABLE_EDIT, 'pos_x', pos_x));
                    yield put(change(NAME_OF_FORMS.TRACKABLE_EDIT, 'pos_y', pos_y));
                    yield put(change(NAME_OF_FORMS.TRACKABLE_EDIT, 'pos_z', convertMillimetersToMeters(z)));
                    yield put(saveStationaryPosition({
                        stationaryPosition: {
                            lng: lengthToDegree(pos_x) * 10,
                            lat: lengthToDegree(pos_y) * 10,
                        },
                    }));
                }
            }
        }
    } catch (err) {
        yield call(errorHandler, err);
    }
}


function* watchTrackablePositionEdit() {
    yield takeEvery(SAGA_ACTIONS.SAGA_TRACKABLES_EDIT_POSITION, handleTrackablePosition);
}

function* watchCreateEditTrackable() {
    yield takeEvery(SAGA_ACTIONS.SAGA_TRACKABLES_CREATE, createEditTrackable);
}

function* watchCreateEditTrackableLLL() {
    yield takeEvery(SAGA_ACTIONS.SAGA_TRACKABLES_CREATE_LLL, createEditLLLTrackable);
}

function* watchCloseCreateEditTrackableLLL() {
    yield takeEvery(SAGA_ACTIONS.SAGA_TRACKABLES_CREATE_LLL_CLOSE, closeCreateEditLLLTrackable);
}

export default function* trackableCreateSaga() {
    return yield all([
        fork(watchCreateEditTrackable),
        fork(watchCreateEditTrackableLLL),
        fork(watchCloseCreateEditTrackableLLL),
        fork(watchTrackablePositionEdit),
    ]);
}
