import { all, fork, call, put, takeEvery, select } from 'redux-saga/effects';

import { MODAL_ACTIONS, SAGA_ACTIONS, SITE_MODE, GEOMETRY_RESTRICTION_TYPES, ZONE_TYPE } from 'common/constants';
import { closeZoneChooseGeometryModal, openZoneChooseCreationModeModal, openZoneChooseGeometryModal } from 'stores/actions/modalActions/zoneModalActions';
import { setZoneTypeName, changeMode, clearCreateZoneMode, setZoneGeometryRestriction } from 'stores/actions/siteViewActions';
import { zonePrototypeList } from 'stores/sagas/selectors';
import { isDirectionalRestriction, isRectangleRestriction } from 'utils/zoneUtils';
import {
    clickOnLeafletCancelDrawButton, clickOnLeafletClearLayerDrawButton, clickOnLeafletRemoveLayersButton,
    clickLeafletDrawRectangleButton, clickLeafletDrawPolygonButton, clickLeafletDrawPolylineButton,
} from 'utils/htmlUtils';
import _Get from 'lodash/get';
import errorHandler from 'stores/sagas/commonSaga';

function* exitDrawRegime() {
    try {
        yield call(clickOnLeafletRemoveLayersButton);
        yield call(clickOnLeafletClearLayerDrawButton);
        yield call(clickOnLeafletCancelDrawButton);
        yield put(clearCreateZoneMode());
    } catch (err) {
        yield call(errorHandler, err);
    }
}

function* handleDrawRegime(modeConfig, geometry_restriction) {
    try {
        yield put(changeMode(modeConfig));
        yield put(setZoneGeometryRestriction(geometry_restriction));
        yield put(closeZoneChooseGeometryModal());
    } catch (err) {
        yield call(errorHandler, err);
    }
}

function* handleRectangleRegime(action) {
    try {
        const { geometry_restriction } = action.params;
        const modeConfig = { name: SITE_MODE.EDIT_ZONE };
        yield call(handleDrawRegime, modeConfig, geometry_restriction);
        yield call(clickLeafletDrawRectangleButton);
    } catch (err) {
        yield call(errorHandler, err);
    }
}

function* handleDirectionalRegime(action) {
    try {
        const { geometry_restriction } = action.params;
        const modeConfig = { name: SITE_MODE.EDIT_ZONE, type: GEOMETRY_RESTRICTION_TYPES.DIRECTIONAL };
        yield call(handleDrawRegime, modeConfig, geometry_restriction);
        yield put({ type: MODAL_ACTIONS.MODAL_ZONE_DIRECTION_START_CREATE });
        yield call(clickLeafletDrawPolylineButton);
    } catch (err) {
        yield call(errorHandler, err);
    }
}

function* handlePolygonRegime(action) {
    try {
        const { geometry_restriction } = action.params;
        const modeConfig = { name: SITE_MODE.EDIT_ZONE };
        yield call(handleDrawRegime, modeConfig, geometry_restriction);
        yield put({ type: MODAL_ACTIONS.MODAL_ZONE_POLYGON_START_CREATE });
        yield call(clickLeafletDrawPolygonButton);
    } catch (err) {
        yield call(errorHandler, err);
    }
}


function* handleChooseZoneType(action) {
    try {
        const { zone_type_name } = action.params;
        const zonesPrototypes = yield select(zonePrototypeList);
        const zonePrototype = zonesPrototypes.find(_zonePrototype => _zonePrototype.isZoneTypeNameEqualTo(zone_type_name));

        const modeChooseList = [ZONE_TYPE.INFO.name, ZONE_TYPE.KEEP_IN.name, ZONE_TYPE.KEEP_OUT.name];
        if (modeChooseList.includes(zone_type_name) || (zonePrototype && zonePrototype.allowCoordinateMode())) {
            yield put(setZoneTypeName(zone_type_name));
            yield put(openZoneChooseCreationModeModal({ zone_type_name }));
            return;
        }

        const geometryRestriction = _Get(zonePrototype, 'zone_geometry_restriction', undefined);

        yield put(setZoneTypeName(zone_type_name));

        if (isRectangleRestriction(geometryRestriction) && !isDirectionalRestriction(geometryRestriction)) {
            yield call(handleRectangleRegime, { params: { geometry_restriction: geometryRestriction } });
        } else if (isDirectionalRestriction(geometryRestriction)) {
            yield call(handleDirectionalRegime, { params: { geometry_restriction: geometryRestriction } });
        } else {
            yield put(openZoneChooseGeometryModal(action.params));
        }
    } catch (err) {
        yield call(errorHandler, err);
    }
}


function* watchCreateSitePageZone() {
    yield takeEvery(SAGA_ACTIONS.SAGA_ZONES_CHOOSE_TYPE_ON_CREATE, handleChooseZoneType);
}


function* watchReactangleRegime() {
    yield takeEvery(SAGA_ACTIONS.SAGA_ZONES_CREATE_RECTANGLE_REGIME, handleRectangleRegime);
}

function* watchPolygoneRegime() {
    yield takeEvery(SAGA_ACTIONS.SAGA_ZONES_CREATE_POLYGON_REGIME, handlePolygonRegime);
}

function* watchCancelDrawing() {
    yield takeEvery(SAGA_ACTIONS.SAGA_ZONES_EXIT_DRAW_REGIME, exitDrawRegime);
}

export default function* zoneChooseGeometrySaga() {
    return yield all([
        fork(watchCreateSitePageZone),
        fork(watchReactangleRegime),
        fork(watchPolygoneRegime),
        fork(watchCancelDrawing),
    ]);
}
