import { STORAGE_KEY } from 'common/constants';
import { getHostName, tryParse } from 'utils/simpleUtils';
import { USER_ROLES } from 'common/user-roles';

const formatServerKey = (targetServer, key) => `${targetServer}:${key}`;

const isKeyBelongToServer = (key, targetServer) => key.startsWith(targetServer);

export function getQueryParam(param) {
    try {
        const queryString = document?.location?.href.split('?')[1];
        if (!queryString) return null;
        const params = new URLSearchParams(queryString);
        return params.get(param);
    } catch (error) {
        console.error('Error parsing URL:', error);
        return null;
    }
}

/* sessionStorage */
export const setSessionStorageKeyValue = (key, value) => {
    const targetServer = getQueryParam('target_server');
    if (!targetServer) {
        sessionStorage.setItem(key, value);
    } else {
        sessionStorage.setItem(formatServerKey(targetServer, key), value);
    }
};

export const getSessionStorageValue = (key) => {
    const targetServer = getQueryParam('target_server');
    if (!targetServer) {
        return sessionStorage.getItem(key);
    } 
    return sessionStorage.getItem(formatServerKey(targetServer, key));
};

export const removeSessionStorageKeyValue = (key) => {
    const targetServer = getQueryParam('target_server');
    if (!targetServer) {
        sessionStorage.removeItem(key);
    } else {
        sessionStorage.removeItem(formatServerKey(targetServer, key));
    }
};

export const clearSessionStorage = () => {
    const targetServer = getQueryParam('target_server');
    if (!targetServer) {
        sessionStorage.clear();
        return;
    }

    let i = 0;
    while (sessionStorage.key(i)) {
        const key = sessionStorage.key(i);
        if (isKeyBelongToServer(key, targetServer)) {
            // when key removed next key become same `i` number
            sessionStorage.removeItem(key);
        } else {
            // move to next key number
            i += 1;
        }
    }
};

/* localStorage */

export const setLocalStorageKeyValue = (key, value) => {
    const targetServer = getQueryParam('target_server');
    if (!targetServer) {
        localStorage.setItem(key, value);
    } 
    localStorage.setItem(formatServerKey(targetServer, key), value);
};

export const removeLocalStorageKey = (key) => {
    const targetServer = getQueryParam('target_server');
    if (!targetServer) {
        localStorage.removeItem(key);
    }
    localStorage.removeItem(formatServerKey(targetServer, key));
};

export const clearLocalStorage = () => {
    const targetServer = getQueryParam('target_server');
    if (!targetServer) {
        localStorage.clear();
        return;
    }

    let i = 0;
    while (localStorage.key(i)) {
        const key = localStorage.key(i);
        if (isKeyBelongToServer(key, targetServer)) {
            // when key removed next key become same `i` number
            localStorage.removeItem(key);
        } else {
            // move to next key number
            i += 1;
        }
    }
};

export const getLocalStorageValue = (key) => {
    const targetServer = getQueryParam('target_server');
    if (!targetServer) {
        return localStorage.getItem(key);
    } 
    return localStorage.getItem(formatServerKey(targetServer, key));
};

export class LocationStoreMethods {
    static projectStorageKeys = [
        STORAGE_KEY.LAST_LOCATION, STORAGE_KEY.LAST_SUBLOCATION, STORAGE_KEY.LAST_FLOORPLAN,
        STORAGE_KEY.LAST_INDEX_OF_SITE_VIEW_TABS, STORAGE_KEY.LAST_STATE_OF_RIGHT_SIDE_BAR,
        STORAGE_KEY.LAST_STATE_OF_OBJECT_FILTER, STORAGE_KEY.LAST_DASHBOARD_WIDGETS,
        STORAGE_KEY.LAST_STATE_OF_VIEW_FILTER, STORAGE_KEY.LAST_CREATED_HEATMAPS,
        STORAGE_KEY.LAST_CONTEXT_OF_SHM,
    ];
    
    static getlocalStorageKey = (key, userEmail, projectUid) => {
        const isValidKey = LocationStoreMethods.projectStorageKeys.includes(key);
        
        const emailBasedKey = `${key}:${userEmail}`;
        return (isValidKey && projectUid) ? `${emailBasedKey}:${projectUid}` : emailBasedKey;
    };
    
    // NEW CONTEXT
    static setToLocalStorage = (key, value, userEmail, projectUid) => {
        setLocalStorageKeyValue(LocationStoreMethods.getlocalStorageKey(key, userEmail, projectUid), JSON.stringify(value));
    };
    
    static getFromLocalStorage = (key, userEmail, projectUid) => {
        const storageKey = LocationStoreMethods.getlocalStorageKey(key, userEmail, projectUid);
        return tryParse(getLocalStorageValue(storageKey));
    };
    
    static removeLocalStorageKeyValue = (key, userEmail, projectUid) => {
        const storageKey = LocationStoreMethods.getlocalStorageKey(key, userEmail, projectUid);
        removeLocalStorageKey(storageKey);
    };

    static getParamsFromStorage = () => {
        const email = getSessionStorageValue(STORAGE_KEY.USER_EMAIL);
        const token = getSessionStorageValue(STORAGE_KEY.USER_TOKEN);
        const access = tryParse(getSessionStorageValue(STORAGE_KEY.USER_ACCESS));
        const company = tryParse(getSessionStorageValue(STORAGE_KEY.CURRENT_COMPANY));
        const project = tryParse(getSessionStorageValue(STORAGE_KEY.CURRENT_PROJECT));
        const hostname = getLocalStorageValue(STORAGE_KEY.HOST_NAME);
        const onBehalf = tryParse(getSessionStorageValue(STORAGE_KEY.USER_ON_BEHALF));
        
        return {
            email,
            token,
            access,
            company,
            project,
            hostname,
            onBehalf,
        };
    };
    
    static clearUserParamsFromStorage = (currentRole) => {
        // redirecting company admin to the default page
        if (currentRole === USER_ROLES.COMPANY_ADMIN) {
            setLocalStorageKeyValue(STORAGE_KEY.LAST_PAGE, '');
        }
        clearSessionStorage();
    };
    
    static setParamsToStorage = (params) => {
        const { email, token, access, company, project, onBehalf } = params;
        
        if (email) {
            setSessionStorageKeyValue(STORAGE_KEY.USER_EMAIL, email);
        }
        if (token) {
            setSessionStorageKeyValue(STORAGE_KEY.USER_TOKEN, token);
        }
        if (access) {
            setSessionStorageKeyValue(STORAGE_KEY.USER_ACCESS, JSON.stringify(access));
        }
        
        if (company && getLocalStorageValue(STORAGE_KEY.SYSTEM_COMPANY_UID) !== company.uid) {
            setSessionStorageKeyValue(STORAGE_KEY.CURRENT_COMPANY, JSON.stringify(company));
        } else {
            removeSessionStorageKeyValue(STORAGE_KEY.CURRENT_COMPANY);
        }
        
        if (project) { // maybe add to localStorage like context
            setSessionStorageKeyValue(STORAGE_KEY.CURRENT_PROJECT, JSON.stringify(project));
        } else {
            removeSessionStorageKeyValue(STORAGE_KEY.CURRENT_PROJECT);
        }
        
        if (onBehalf) { // maybe add to localStorage like context
            setSessionStorageKeyValue(STORAGE_KEY.USER_ON_BEHALF, JSON.stringify(onBehalf));
        } else {
            removeSessionStorageKeyValue(STORAGE_KEY.USER_ON_BEHALF);
        }
    };
}

export const _getHostName = (appContext = {}) => {
    let { hostName } = appContext;
    
    if (!hostName) {
        hostName = getLocalStorageValue(STORAGE_KEY.HOST_NAME);
    }
    
    if (!hostName) {
        hostName = getHostName();
        setLocalStorageKeyValue(STORAGE_KEY.HOST_NAME, hostName);
    }
    
    return hostName;
};

export const reloadBrowser = () => {
    location.reload(true);
};

export const isBetaVersion = () => getLocalStorageValue(STORAGE_KEY.BETA_PATH);
