import React, { Fragment } from 'react';
import randomColor from 'randomcolor';
import _isNil from 'lodash/isNil';
import { SortDirection } from 'react-virtualized';
import _Get from 'lodash/get';
import { DB_NAMES, LABELS, TRACKABLE_MODAL } from 'common/ui-labels';
import { UNAVAILABLE, T2T_TYPE, TRACKABLE_CATEGORY } from 'common/constants';
import { capitalize } from 'utils/formatters';


export const colorFormatter = (color) => {
    const style = {
        backgroundColor: color || 'white',
        display: 'inline-block',
        minWidth: '20px',
        width: '20px',
        height: '20px',
        border: 'solid black 1px',
        margin: 'auto',
    };
    return (<span style={style} />);
};

export const formatDbNameToHumanReadable = (intl, text = '', defaultValue = '') => {
    if (DB_NAMES[text.toUpperCase()]) {
        return DB_NAMES[text.toUpperCase()](intl);
    }
    return defaultValue ? capitalize(defaultValue) : capitalize(text.split('_').join(''));
};

export const formatAdditionalInfoText = (intl, type, name, level) => {
    if (type === 'in' && DB_NAMES.ALARM_KEEP_IN_ADDITIONAL_INFO_TEXT_FUNC) {
        return DB_NAMES.ALARM_KEEP_IN_ADDITIONAL_INFO_TEXT_FUNC(intl, name, level);
    }
    if (type === 'out' && DB_NAMES.ALARM_KEEP_OUT_ADDITIONAL_INFO_TEXT_FUNC) {
        return DB_NAMES.ALARM_KEEP_OUT_ADDITIONAL_INFO_TEXT_FUNC(intl, name, level);
    }
    return type[0].toUpperCase() + type.replace('_', ' ').slice(1);
};

export const getUiConstForAdditionalInfoText = (intl, str) => {
    if (str) {
        const reg = /Keep (out|in) zone ([a-zA-Z0-9_\- ()]+) with zone alarm level ([1-5])/g;
        const exec_array = reg.exec(str);
        
        if (exec_array && exec_array.length === 4) {
            const type = exec_array[1];
            const name = exec_array[2];
            const level = exec_array[3];
            return formatAdditionalInfoText(intl, type, name, level);
        }
    }
    return str;
};

export const generateRandomColor = () => (randomColor());

export const trOddEvenClassFormat = (row, rowIndex) => (rowIndex % 2 === 0 ? 'Table-Odd-Row' : 'Table-Even-Row');

const isBatteryAvailable = battery => ((battery && battery.pr !== null) && Number.isInteger(battery.pr));
const isValidBatteryLevel = batteryLevel => (batteryLevel >= 0 && batteryLevel <= 100);

const batteryIconHtmlTemplate = (batIconInfo, parentStyle) => (
    <span style={parentStyle}>
        <i
            aria-hidden="true"
            className={`fa fa-2x ${batIconInfo.iconClass}`}
            style={{ transform: 'rotate(180deg)', color: `${batIconInfo.color}` }}
        />
    </span>);

const BAT_ICON_CHARGING = { iconClass: 'fa-plug', color: 'green' };
const BAT_ICON_STYLE = { };
const SMALL_BAT_ICON_STYLE = { fontSize: '8px', marginRight: '5px' };

const batteryIcon = (battery, parentStyle, intl) => {
    if (!isBatteryAvailable(battery) || !isValidBatteryLevel(battery.pr)) {
        return LABELS.UNAVAILABLE.LABEL(intl);
    }

    const batteryLevel = battery.pr;
    let batIconInfo = { iconClass: '', color: '' };

    if (batteryLevel >= 81) {
        batIconInfo = {
            iconClass: 'fa-battery-full',
            color: 'green',
        };
    } else if (batteryLevel >= 10) {
        batIconInfo = {
            iconClass: 'fa-battery-half',
            color: 'orange',
        };
    } else if (batteryLevel >= 0) {
        batIconInfo = {
            iconClass: 'fa-battery-empty',
            color: 'red',
        };
    }

    return batteryIconHtmlTemplate(batIconInfo, parentStyle);
};

export const batteryStatus = (battery, intl) => batteryIcon(battery, BAT_ICON_STYLE, intl);
const batteryStatusSmall = (battery, intl) => (<span>
    {battery.pr} % {batteryIcon(battery, SMALL_BAT_ICON_STYLE, intl)}
</span>);

const batteryChargingStatusSmall = () => (<span>
    {batteryIconHtmlTemplate(BAT_ICON_CHARGING, SMALL_BAT_ICON_STYLE)} charging
</span>);

export const formatTextForUnresponsiveTrackable = (text, isTrackableUnresponsive) => (isTrackableUnresponsive
    ? (
        <span style={{ color: 'red' }}>
            <span style={{ marginRight: '10px' }}>{text}</span>
            <i className="fa fa-exclamation-triangle" />
        </span>
    )
    : text);

export const getKeyValueHtml = (key, value, i) => (
    <div key={i} className="RPP-PopUpRow row">
        <div className="col-md-5 popup-field-label">
            {key}&nbsp;
        </div>
        <div className="col-md-7 popup-field-value multiple-lines-ellipsis">
            <span>{value}</span>
        </div>
    </div>
);

export const getKeyValuesHtml = (key, values, i) => (
    <div key={i} className="RPP-PopUpRow alarms-row row">
        <div className="col-md-12 popup-field-label">
            {key}:&nbsp;
        </div>
        <div className="col-md-12 popup-alarms-field-value multiple-lines-ellipsis">
            {values.map((currentValue, index) => (
                <div key={index} className="popup-alarm-text">
                    {currentValue}
                </div>
            ))}
        </div>
    </div>
);

export const getNodeBatteryLevel = (node, isCategoryAsset = false) => {
    if (node && !_isNil(node.battery)) {
        const { battery } = node;
        if (node.charging) {
            if (isCategoryAsset) {
                if (battery && battery.pr >= 0 && battery.pr <= 100) {
                    return batteryStatusSmall(battery);
                }
                return UNAVAILABLE;
            }
            return batteryChargingStatusSmall();
        } else if (battery && battery.pr >= 0 && battery.pr <= 100) {
            return batteryStatusSmall(battery);
        }
    }
    return UNAVAILABLE;
};

export const calculateTrackableClassName = (trackable, isStationaryTag) => {
    const trackableClassArr = ['trackable-object'];
    if (isStationaryTag(trackable)) {
        trackableClassArr.push('stationary-tag');
    } else {
        trackableClassArr.push('tag');
    }
    if (trackable.isAlarmed) {
        trackableClassArr.push('alarmed');
    }
    if (trackable.isSelected) {
        trackableClassArr.push('selected');
    }
    return trackableClassArr.join(' ');
};

export const calculateLeafletElementClassName = (playbackSpeed, basicClass = 'leaflet-element') => {
    const leafletElementClassArr = [basicClass];
    if (playbackSpeed) {
        leafletElementClassArr.push(`Leaflet-Transition-Speed${playbackSpeed}`);
    }
    return leafletElementClassArr.join(' ');
};

export const clickOnLink = (href) => {
    const link = document.createElement('a');
    link.target = '_blank';
    link.href = href;
    document.body.appendChild(link);
    link.click();
    document.body.removeChild(link);
};

export const getLocalizedRole = (role, intl) => {
    if (role === T2T_TYPE.MONITOR) {
        return TRACKABLE_MODAL.EDIT.FORM.PWS_PARAMS.MONITOR(intl);
    } else if (role === T2T_TYPE.ANNOUNCER) {
        return TRACKABLE_MODAL.EDIT.FORM.PWS_PARAMS.ANNOUNCER(intl);
    } else if (role === T2T_TYPE.DISABLED || role === 'None') {
        return DB_NAMES.NONE(intl);
    }
    return role;
};

export function clickOnLeafletRemoveLayersButton() {
    const removeLayersControl = document.getElementsByClassName('leaflet-draw-edit-remove')[1];
    if (removeLayersControl) {
        removeLayersControl.click();
    }
}

export function clickOnLeafletClearLayerDrawButton() {
    const clearLayersDraw = document.querySelector('[title="Clear all layers"]');
    if (clearLayersDraw) {
        clearLayersDraw.click();
    }
}

export function clickOnLeafletCancelDrawButton() {
    const cancelDraw = document.querySelector('[title="Cancel drawing"]');
    if (cancelDraw) {
        cancelDraw.click();
    }
}

export function clickLeafletDrawRectangleButton() {
    const rectangleDrawControl = document.getElementsByClassName('leaflet-draw-draw-rectangle')[0];
    if (rectangleDrawControl) {
        rectangleDrawControl.click();
    }
}

export function clickLeafletDrawPolygonButton() {
    const polygonDrawControl = document.getElementsByClassName('leaflet-draw-draw-polygon')[0];
    if (polygonDrawControl) {
        polygonDrawControl.click();
    }
}

export function clickLeafletDrawPolylineButton() {
    const polylineDrawControl = document.getElementsByClassName('leaflet-draw-draw-polyline')[0];
    if (polylineDrawControl) {
        polylineDrawControl.click();
    }
}

function getSortDropDownElement() {
    return (
        <Fragment key="sort-drop-down">
            <i className="fa fa-caret-up" key="fa-caret-up" aria-hidden="true" style={{ color: 'rgb(204, 204, 204)', margin: '0 0 0 5px' }} />
            <i className="fa fa-caret-down" key="fa-caret-down" aria-hidden="true" style={{ color: 'rgb(204, 204, 204)', margin: '0 0 0 5px' }} />
        </Fragment>);
}

function getSortDirectionElement(sortDirection) {
    switch (sortDirection) {
        case SortDirection.ASC: {
            return (<i key="asc" className="fa fa-caret-up" aria-hidden="true" style={{ color: 'black', margin: '0 0 0 5px' }} />);
        }
        case SortDirection.DESC: {
            return (<i key="desc" className="fa fa-caret-down" aria-hidden="true" style={{ color: 'black', margin: '0 0 0 5px' }} />);
        }
        default: {
            return (<span />);
        }
    }
}

export function virtualizedHeaderRenderer(props) {
    const { dataKey, label, sortBy, sortDirection, disableSort } = props;
    const showSortIndicator = sortBy === dataKey;
    const children = [
        <span
            className="ReactVirtualized__Table__headerTruncatedText"
            key="label"
            title={typeof label === 'string' ? label : null}
        >
            {label}
        </span>,
    ];
    
    if (showSortIndicator) {
        children.push(getSortDirectionElement(sortDirection));
    } else if (!disableSort) {
        const sortDropDownElement = getSortDropDownElement();
        children.push(sortDropDownElement);
    }
    
    return children;
}

export function getSiteViewClassName(isOpenSideBar, mode) {
    return `${isOpenSideBar ? 'Short-Width-Site-View' : 'col-md-12 Long-Width-Site-View'} ${((mode && mode.name) || '').toLowerCase()}`;
}

export function download(filename, text) {
    const element = document.createElement('a');
    element.setAttribute('href', `data:text/csv;charset=utf-8,${encodeURIComponent(text)}`);
    element.setAttribute('download', filename);
    element.style.display = 'none';
    document.body.appendChild(element);
    element.click();
    document.body.removeChild(element);
}

export function getCategoryText(category, intl) {
    switch (category) {
        case (TRACKABLE_CATEGORY.VEHICLE): {
            return DB_NAMES.VEHICLE(intl);
        }
        case (TRACKABLE_CATEGORY.PERSON): {
            return DB_NAMES.PERSON(intl);
        }
        case (TRACKABLE_CATEGORY.ASSET): {
            return DB_NAMES.ASSET(intl);
        }
        case (TRACKABLE_CATEGORY.UNSPECIFIED): {
            return DB_NAMES.UNSPECIFIED(intl);
        }
        default: {
            return category;
        }
    }
}

/**
 * Development feature used with key SHOW_EXTENDS, show PWS config on trackables and groups table
 * @param rowData
 * @returns {null|*}
 */
export function getRoleIcon(rowData) {
    const LLL = _Get(rowData, 'low_latency_link_config', '');
    const category = _Get(rowData, 'category', '');
    const title = `Category: ${category}, Role: ${LLL.role}, Alarm back: ${LLL.alarm_back}\n`
        + ` Angle: ${!isNaN(LLL.angle) ? LLL.angle : '-'},`
        + ` Delay: ${!isNaN(LLL.delay) ? LLL.delay : '-'},`
        + ` Pin: ${LLL.pin || '-'} \n`
        + ` Safe_distance: ${!isNaN(LLL.safe_distance) ? LLL.safe_distance : '-'},`
        + ` Distance: ${!isNaN(LLL.distance) ? LLL.distance : '-'},`
        + ` Safe_distance: ${!isNaN(LLL.alert_distance) ? LLL.alert_distance : '-'}`;

    if (category === TRACKABLE_CATEGORY.VEHICLE) {
        return (<i key="trackables-getActionsIcons3" title={title} className="fa fa-car inner-table-action" aria-hidden="true" />);
    }
    if (category === TRACKABLE_CATEGORY.PERSON) {
        return (<i key="trackables-getActionsIcons1" title={title} className="fa fa-male inner-table-action" aria-hidden="true" />);
    }
    if (LLL.role === T2T_TYPE.MONITOR) {
        return (<i key="trackables-getActionsIcons2" title={title} className="fa fa-genderless inner-table-action" aria-hidden="true" />);
    }
    if (LLL.role === T2T_TYPE.ANNOUNCER) {
        return (<i key="trackables-getActionsIcons2" title={title} className="fa fa-user inner-table-action" aria-hidden="true" />);
    }
    return null;
}

export function getPaired(rowData) {
    const paired = _Get(rowData, 'paired', '');
    if (paired) {
        return (
            <i
                key="trackables-getActionsIcons2"
                title={`Paired with uid ${paired}`}
                className="fa fa-lock inner-table-action"
                aria-hidden="true"
            />
        );
    }
    return null;
}

function getCornerRibbonHtml(text, align, ribbonClass) {
    return (
        <div className={`corner-ribbon ${align} ${ribbonClass}`}>
            {text}
        </div>
    );
}

export function getLeftCornerRibbonHtml(text, ribbonClass) {
    return getCornerRibbonHtml(text, 'left', ribbonClass);
}

export function getRightCornerRibbonHtml(text, ribbonClass) {
    return getCornerRibbonHtml(text, 'right', ribbonClass);
}

export function onMouseDownScrollPrevent(event) {
    const element = event.target;
    const rect = element.getBoundingClientRect();
    const clientX = event.clientX;
    const isHorizontalScrollbar = clientX >= (rect.left + rect.width - 20);

    if (isHorizontalScrollbar) {
        event.preventDefault();
    }
}
