import deepcopy from 'deepcopy';
import { EMAIL_OF_COMPANY_ADMIN_USER_BY_DEFAULT, SERVER_CONFIG, ZONE_SEVERITY, ZONE_SEVERITY_COLOR, ZONE_TYPE, SITE_MODE } from 'common/constants';
import { getUID } from 'worker';


export function zoneSeverityColor(zone_alarm_level_id) {
    switch (parseInt(zone_alarm_level_id)) {
        case ZONE_SEVERITY.INFORMATION:
            return ZONE_SEVERITY_COLOR.INFORMATION;
        case ZONE_SEVERITY.LOW:
            return ZONE_SEVERITY_COLOR.LOW;
        case ZONE_SEVERITY.GUARDED:
            return ZONE_SEVERITY_COLOR.GUARDED;
        case ZONE_SEVERITY.ELEVATED:
            return ZONE_SEVERITY_COLOR.ELEVATED;
        case ZONE_SEVERITY.HIGH:
            return ZONE_SEVERITY_COLOR.HIGH;
        case ZONE_SEVERITY.SEVERE:
            return ZONE_SEVERITY_COLOR.SEVERE;
        default:
            return '#808080';
    }
}

export function getZoneColor(zone) {
    if (zone.zone_type_name.toLowerCase() === ZONE_TYPE.INFO.name) {
        return ZONE_SEVERITY_COLOR.INFORMATION;
    }
    return zoneSeverityColor(zone.zone_alarm_level);
}

export const getZones = (_zones, _playback, sublocation, isPlayback = false) => {
    if (!sublocation) return [];
    let zones = [];

    if (!isPlayback) {
        zones = _zones;
    } else {
        const playback = _playback;
        const { currentTs } = playback;

        zones = playback.zones.filter((zone) => {
            const createdAt = (new Date(zone.created_at)).getTime();
            const removedAt = (new Date(zone.removed_at)).getTime();
            return (currentTs > createdAt && currentTs < removedAt);
        });
    }
    return zones;
};

const phoneUtil = require('node-phone');

export const separatePhoneNumber = (phoneNumber) => {
    const np = phoneUtil.normalizePhone(phoneNumber);

    if (phoneNumber.replace('+', '').startsWith(np.countryCode.substr(1))) {
        return {
            phoneCode: phoneNumber.startsWith('+') ? np.countryCode : np.countryCode.substr(1),
            phoneNumber: np.phone,
        };
    }
    
    return {
        phoneCode: '',
        phoneNumber,
    };
};


export const mergeTrackablesAndPositions = (newState, positions, Trackable) => {
    if (newState.allUids.length === 0 || !Array.isArray(positions)) {
        return newState;
    }
    positions.forEach((position) => {
        const trackable = newState.byUid[position.dev_id];
        if (trackable && (trackable.tag_config && trackable.tag_config.stationary === false)) {
            const trackableProperties = trackable.getProperties();
            newState.byUid[trackable.uid] = new Trackable({
                ...trackableProperties,
                position,
            });
        }
    });
    return newState;
};

export const makeCopy = item => (deepcopy(item));

export const isMacValid = mac_address => /([A-Fa-f0-9][A-Fa-f0-9]-){5}([A-Fa-f0-9][A-Fa-f0-9])/.test(mac_address);

export const getUUID = () => (getUID());

export const tryParse = (value) => {
    try {
        return JSON.parse(value);
    } catch (e) {
        console.error('Fail to parse json', value);
        return {};
    }
};

export const getHostName = () => `${window.location.protocol}//${window.location.hostname + SERVER_CONFIG[SERVER_CONFIG.CURRENT].PORT}`;

/**
 * Check  if user impersonate user(company user in project scope or lab-user in company/project scope)
 * @param user
 * @returns {boolean}
 */
export const isUserImpersonate = user => (`${(user.email.split('@')[0])}@` === EMAIL_OF_COMPANY_ADMIN_USER_BY_DEFAULT);

/**
 * Check if user script created user lab+0000;
 * @param user
 * @returns {boolean}
 */
export const isUserSuperUser = user => !user.email.includes('@');

export function lengthToDegree(l) {
    return (360 * l) / 40075161.2;
}

export function isScriptCreatedSuperUser(user) {
    if (user.email.startsWith('lab+')) {
        const num = user.email.split('+')[1];
        const parsed = parseInt(num);
        return !isNaN(parsed);
    }
    return false;
}

/**
 * @param data
 * @returns {*}
 */
export function getValue(data) {
    return data && data.value ? data.value : data;
}

/**
 * Check divisible for 0.1
 * @param u
 * @param d
 * @returns {boolean}
 */
export function isDivisible(u, d) {
    const numD = Math.max(u.toString().replace(/^\d+\./, '').length, d.toString()
        .replace(/^\d+\./, '').length);
    u = Math.round(u * Math.pow(10, numD));
    d = Math.round(d * Math.pow(10, numD));
    return (u % d) === 0;
}

/**
 * @function
 * @param {Number} zoom - leaflet map zoom
 * @return {Number} - grid step in meters
 * */
export function getGridStepByZoom(zoom) {
    switch (zoom) {
        case 12:
            return 300;
        case 13:
            return 200;
        case 14:
            return 100;
        case 15:
            return 50;
        case 16:
            return 20;
        case 17:
            return 10;
        case 18:
        case 19:
            return 5;
        default:
            return zoom > 19 ? 1 : 500;
    }
}


export const isZoneEditing = mode => mode === SITE_MODE.EDIT_ZONE;
export const isSpaghettiMode = mode => mode === SITE_MODE.SPAGHETTI;
export const isNearMissMode = mode => mode === SITE_MODE.NEAR_MISS;
export const isPlaybackMode = mode => mode === SITE_MODE.PLAYBACK;
export const isHeatmapMode = mode => mode === SITE_MODE.HEATMAP;
