import moment from 'moment';
import momentTz from 'moment-timezone';
import cronParser from 'cron-parser';
import CronConverter from 'cron-converter';
import _Get from 'lodash/get';
import { durationFormatting, makeCronTimeString } from './formatters';
import { formatDbNameToHumanReadable } from './htmlUtils';


export const minToMs = minutes => (minutes * 60 * 1000);

/**
 *
 * @param time {string|Date}
 * @returns {string} `MM/DD/YYYY HH:mm:ss`
 */
export const timeFormat = (time) => {
    if (!time) {
        return '';
    } else if (typeof time === 'string') {
        time = new Date(time);
    }
    
    let seconds = time.getSeconds();
    let minutes = time.getMinutes();
    let hour = time.getHours();
    const year = time.getFullYear();
    const month = time.getMonth() + 1;
    let day = time.getDate();
    
    if (hour < 10) {
        hour = `0${hour}`;
    }
    if (minutes < 10) {
        minutes = `0${minutes}`;
    }
    if (seconds < 10) {
        seconds = `0${seconds}`;
    }
    if (day < 10) {
        day = `0${day}`;
    }
    if (isNaN(month)) {
        return '';
    }
    return `${month}/${day}/${year} ${hour}:${minutes}:${seconds}`;
};

/**
 *
 * @param arg {string|Date}
 * @returns {string} `HH:mm PM` | `HH:mm AM`
 */
export const timeFormatter = (arg) => {
    let time = arg;
    if (!arg) {
        return '';
    } else if (typeof arg === 'string') {
        time = new Date(arg);
    }
    
    let minutes = time.getMinutes();
    let hour = time.getHours();
    
    const PmAm = hour >= 12 ? 'PM' : 'AM';
    hour = hour > 12 ? hour - 12 : hour;
    if (hour < 10) {
        hour = `0${hour}`;
    }
    if (minutes < 10) {
        minutes = `0${minutes}`;
    }
    
    return `${hour}:${minutes} ${PmAm}`;
};

/**
 *
 * @param arg {string|Date}
 * @returns {string} `MM/DD/YYYY`
 */
export const dateFormatter = (arg) => {
    let time = arg;
    if (!arg) {
        return '';
    } else if (typeof arg === 'string') {
        time = new Date(arg);
    }
    
    const year = time.getFullYear();
    const month = time.getMonth() + 1;
    const day = time.getDate();
    
    return `${month}/${day}/${year}`;
};

/**
 *
 * @param createdAt
 * @returns {string} `HH:mm`
 */
export const getTime = (createdAt) => {
    const date = new Date(createdAt);
    const hours = date.getHours() > 9 ? date.getHours() : `0${date.getHours()}`;
    const minutes = date.getMinutes() > 9 ? date.getMinutes() : `0${date.getMinutes()}`;
    return `${hours}:${minutes}`;
};

export const getTimeZones = () => {
    const now = new Date();
    return momentTz.tz.names().sort((a, b) => {
        const zoneA = moment.tz.zone(a).parse(now);
        const zoneB = moment.tz.zone(b).parse(now);
        return zoneB - zoneA;
    });
};

export const getTimeZoneOfProject = (project) => {
    if (project && project.timezone) {
        return project.timezone;
    } else if (project && project.user_params && project.user_params.timezone) {
        return project.user_params.timezone;
    }
    return 'Etc/GMT';
};

export const getTimeZoneOffset = (project, timezone_name) => {
    if (!timezone_name) {
        const timezone = getTimeZoneOfProject(project);
        if (!timezone) {
            const now = new Date();
            return now.getTimezoneOffset() * -1;
        }
        return -1 * moment.tz.zone(timezone).parse(new Date()); // in minutes
    }
    return -1 * moment.tz.zone(timezone_name).parse(new Date()); // in minutes
};

export const getTimeZoneOffsetByProject = (project) => {
    if (project && project.timezone) {
        return getTimeZoneOffset(project, project.timezone);
    } else if (project && project.user_params && project.user_params.timezone) {
        return getTimeZoneOffset(project, project.user_params.timezone);
    }
    return 0;
};

export const getTimeByFormatAndTimezone = (time, format = 'MM/DD hh:mm:ss a', timeZone = 'Etc/GMT') => momentTz(new Date(time)).tz(timeZone).format(format);

export const getFullDateByTz = (time, timeZone) => getTimeByFormatAndTimezone(time, 'MM/DD/YYYY HH:mm:ss', timeZone);
export const getSyncDateByTz = (time, timeZone) => getTimeByFormatAndTimezone(time, 'MM/DD/YYYY [at] HH:mm', timeZone);
export const getDateByTz = (time, timeZone) => getTimeByFormatAndTimezone(time, 'MM/DD/YYYY', timeZone);
export const getTimeByTz = (time, timeZone) => getTimeByFormatAndTimezone(time, 'hh:mm A', timeZone);

export const getLastReported = (last_heard, intl) => {
    const duration = (last_heard) ? (new Date() - new Date(last_heard)) : NaN;
    return durationFormatting(duration, intl).text;
};

export const millisecondsToHumanReadable = (duration) => {
    const seconds = moment.duration(duration).seconds();
    const minutes = moment.duration(duration).minutes();
    const hours = Math.trunc(moment.duration(duration).asHours());

    const leadingZerosFormat = number => number.toString().padStart(2, '0');

    return `${leadingZerosFormat(hours)}:${leadingZerosFormat(minutes)}:${leadingZerosFormat(seconds)}`;
};

export const durationHumanize = (duration) => {
    if (!duration) return '-';
    const moment_duration = moment.duration(duration);
    const seconds = moment_duration.seconds();
    const minutes = moment_duration.minutes();
    const hours = Math.trunc(moment_duration.asHours());

    let result = '';

    if (hours) {
        result += `${hours}h`;
    }
    if (minutes) {
        result += ` ${minutes}m`;
    }
    if (seconds) {
        result += ` ${seconds}s`;
    }

    return result;
};

function cronToDate(cronString, timezone = 'Etc/GMT') {
    const interval = cronParser.parseExpression(cronString, { tz: timezone });
    const date = interval.next().toString();
    
    return new Date(date);
}

export function getActiveShiftTimeFrame(shift, timezone) {
    if (!shift) return {};
    const start = cronParser.parseExpression(shift.start, { tz: timezone }).prev().toString();
    const end = cronParser.parseExpression(shift.end, { tz: timezone }).next().toString();
    const timeDiff = moment().diff(moment(new Date(start))); // now - start
    const timeElapsed = moment.utc(timeDiff).format('HH:mm');

    return {
        startUtc: momentTz(new Date(start)).tz(timezone).valueOf(),
        endUtc: momentTz(new Date(end)).tz(timezone).valueOf(),
        start: getTimeByFormatAndTimezone(start, 'HH:mm', timezone),
        end: getTimeByFormatAndTimezone(end, 'HH:mm', timezone),
        timeElapsed,
    };
}

export const isCurrentShift = (shift, timezone) => {
    const nextStart = moment(cronToDate(shift.start, timezone));
    const nextEnd = moment(cronToDate(shift.end, timezone));

    return nextStart.isAfter(nextEnd) && shift.active;
}

export const formatTimeToDisplay = (time, format = 'HH:mm') => (moment.isMoment(time) ? time.format(format) : moment(time, format).format(format));

export const getDataForScaling = (start, end, timelineWidth) => {
    const timelineDuration = end - start;
    const startMinutes = new Date(start).getMinutes();
    const endMinutes = new Date(end).getMinutes();

    const minutesToHour = startMinutes ? 60 - startMinutes : 0;

    const fullHours = Math.ceil(timelineDuration / 3600000) - Math.ceil((minutesToHour + endMinutes) / 60);
    const intervalWidth = (timelineWidth * 3600000) / timelineDuration;
    const firstIntervalWidth = startMinutes && ((60 - startMinutes) * 60 * 1000 * timelineWidth) / timelineDuration;
    const lastIntervalWidth = endMinutes && ((endMinutes * 60 * 1000 * timelineWidth) / timelineDuration);
    const firstHour = Math.ceil((start / 3600000)) * 3600000;
    const lastHour = end - (endMinutes * 60 * 1000);

    if (timelineDuration < 3600000 && endMinutes > startMinutes) {
        return { firstIntervalWidth: timelineWidth, intervalWidth: 0, lastIntervalWidth: 0, fullHours: 0, firstHour: start };
    }

    if (timelineDuration < 3600000 && endMinutes > startMinutes) {
        return { firstIntervalWidth, intervalWidth: 0, lastIntervalWidth: 0, fullHours: 0, firstHour: start };
    }

    return { firstIntervalWidth, intervalWidth, lastIntervalWidth, fullHours: fullHours < 0 ? 0 : fullHours, firstHour, lastHour };
};

function getCheckInTime(start, association) {
    if (!association) return null;
    const associationCreate = moment(association.created_at);
    const maximum = moment.max([start, associationCreate]);

    return maximum.valueOf();
}

function formatTimeFrame({ from, to, type }, timezone) {
    return {
        duration: to - from,
        from: getTimeByFormatAndTimezone(from, 'MM/DD/YYYY HH:mm:ss', timezone),
        to: getTimeByFormatAndTimezone(to, 'MM/DD/YYYY HH:mm:ss', timezone),
        from_iso: new Date(from).toISOString(),
        to_iso: new Date(to).toISOString(),
        type,
    };
}

function fillTimelineWithUncheckedPeriods(actions, from, to, timezone) {
    const timeline = [];
    for (let i = 0; i < actions.length; i += 1) {
        const action = actions[i];
        if (action.from > from && i === 0) {
            timeline.push(formatTimeFrame({ from, to: action.from, type: '' }, timezone));
        }
        
        if (i > 0 && actions[i].from > actions[i - 1].to) {
            timeline.push(formatTimeFrame({ from: actions[i - 1].to, to: actions[i].from, type: '' }, timezone));
        }
        
        timeline.push(formatTimeFrame(actions[i], timezone));
    }
    
    return timeline;
}

export function handleTrackableTimeline(uid, trackable, from, to, timezone) {
    const idles = trackable.idle_periods.map(idle => ({ ...idle, type: 'idle' }));
    const activities = trackable.active_periods.map(idle => ({ ...idle, type: 'active' }));
    const unresponsive = trackable.unresponsive_periods.map(idle => ({ ...idle, type: 'unresponsive' }));
    
    const actions = [].concat(idles, activities, unresponsive)
        .map((action => ({ from: moment(action.start).valueOf(), to: moment(action.end).valueOf(), type: action.type })))
        .sort((a, b) => a.from - b.from);
    const timeline = fillTimelineWithUncheckedPeriods(actions, from, to, timezone);
    const totalOnSite = trackable.totalIdle + trackable.totalActive;
    const totalUnresponsive = (to - from) - totalOnSite;
    
    return {
        totalActive: trackable.totalActive,
        totalIdle: trackable.totalIdle,
        name: trackable.name,
        totalUnresponsive,
        totalOnSite,
        timeline,
        from,
        uid,
        to,
    };
}

function checkStatusFromTimeline(timeline) {
    let status = '';
    const timelineIndex = timeline.length - 1;
    if (timeline[timelineIndex].type !== '') {
        status = timeline[timelineIndex].type;
    } else {
        status = 'unassigned';
    }
    return status;
}

export function getAttendanceDashboardTimeline(activities, trackables, associations, from, to, timezone, intl) {
    function handleDriverData(drivers, nonDrivers, state, paired) {
        if (paired && state === 'active') drivers.active += 1;
        if (paired && state === 'idle') drivers.idle += 1;
        if (!paired && state === 'active') nonDrivers.active += 1;
        if (!paired && state === 'idle') nonDrivers.idle += 1;
    }

    if (!activities || !activities.trackableObjects) return [];
    const drivers = { active: 0, idle: 0 };
    const nonDrivers = { active: 0, idle: 0 };
    const timelines = Object.keys(activities.trackableObjects).reduce((memo, uid) => {
        const trackable = trackables[uid];
        const association = associations[uid];
        const trackable_activities = activities.trackableObjects[uid];
        const paired = _Get(trackable, 'paired');
        
        if (trackable_activities.category === 'PERSON' || (trackable && trackable.category === 'PERSON')) {
            const { timeline } = handleTrackableTimeline(uid, trackable_activities, from, to, timezone);
            const status = checkStatusFromTimeline(timeline);
            const checkin_time = getCheckInTime(from, association);
            handleDriverData(drivers, nonDrivers, status, paired);

            const line = {
                uid,
                person_name: trackable_activities.name,
                checkin_time: checkin_time ? getTimeByFormatAndTimezone(checkin_time, 'HH:mm', timezone) : '-',
                status: formatDbNameToHumanReadable(intl, status.toUpperCase()),
                active_time: durationHumanize(trackable_activities.totalActive),
                vehicle: (paired && trackables[paired]) ? trackables[paired].name : '-',
                timeline,
                ...trackable_activities,
            };
            
            memo.push(line);
        }
        return memo;
    }, []);

    return {
        timelines,
        drivers,
        nonDrivers,
    };
}

export function cronFromStartEndTime({ start_time, end_time, weekdays }) {
    if (!weekdays || !weekdays.length || !start_time || !end_time) { return null; }
    const startMoment = moment(start_time);
    const endMoment = moment(end_time);
    let endWeekdays;
    if (endMoment.isBefore(startMoment)) {
        endWeekdays = weekdays.map((day) => {
            if (day === 6) {
                return 0;
            }
            return day + 1;
        });
    } else {
        endWeekdays = weekdays;
    }
    return {
        start: makeCronTimeString(moment(start_time).format('HH:mm'), weekdays),
        end: makeCronTimeString(moment(end_time).format('HH:mm'), endWeekdays),
    };
}

function getDatesFromShift(shift) {
    const start = ((new CronConverter()).fromString(shift.start)).toArray();
    const end = ((new CronConverter()).fromString(shift.end)).toArray();
    const startMinute = start[0][0];
    const startHour = start[1][0];
    const startWeekdays = [...start[4]];
    const endMinute = end[0][0];
    const endHour = end[1][0];
    const endWeekdays = [...end[4]];
    const result = [];
    const isOverNight = ((startHour * 60) + startMinute) > ((endHour * 60) + endMinute);
    
    for (let i = 0; i < startWeekdays.length; i += 1) {
        if (isOverNight && startWeekdays[i] === 6) {
            result.push({
                start: moment().day(startWeekdays[i]).set({ hour: startHour, minute: startMinute, second: 0, millisecond: 0 }).valueOf(),
                end: moment().day(0).add(1, 'week').set({ hour: endHour, minute: endMinute, second: 0, millisecond: 0 })
                    .valueOf(),
            });
        } else if (isOverNight && startWeekdays[i] === 0) {
            result.push({
                start: moment().day(startWeekdays[i]).add(1, 'week').set({ hour: startHour, minute: startMinute, second: 0, millisecond: 0 })
                    .valueOf(),
                end: moment().day(startWeekdays[i] + 1).add(1, 'week').set({ hour: endHour, minute: endMinute, second: 0, millisecond: 0 })
                    .valueOf(),
            });
            result.push({
                start: moment().day(startWeekdays[i]).set({ hour: startHour, minute: startMinute, second: 0, millisecond: 0 }).valueOf(),
                end: moment().day(startWeekdays[i] + 1).set({ hour: endHour, minute: endMinute, second: 0, millisecond: 0 }).valueOf(),
            });
        } else if (startWeekdays[i] === 0) {
            result.push({
                start: moment().day(startWeekdays[i]).add(1, 'week').set({ hour: startHour, minute: startMinute, second: 0, millisecond: 0 })
                    .valueOf(),
                end: moment().day(endWeekdays[i]).add(1, 'week').set({ hour: endHour, minute: endMinute, second: 0, millisecond: 0 })
                    .valueOf(),
            });
            result.push({
                start: moment().day(startWeekdays[i]).set({ hour: startHour, minute: startMinute, second: 0, millisecond: 0 }).valueOf(),
                end: moment().day(endWeekdays[i]).set({ hour: endHour, minute: endMinute, second: 0, millisecond: 0 }).valueOf(),
            });
        } else {
            result.push({
                start: moment().day(startWeekdays[i]).set({ hour: startHour, minute: startMinute, second: 0, millisecond: 0 }).valueOf(),
                end: moment().day(endWeekdays[i]).set({ hour: endHour, minute: endMinute, second: 0, millisecond: 0 }).valueOf(),
            });
        }
    }
    return result;
}

function rangesOverlapped(rangesOne, rangesTwo) {
    if (!rangesOne.length || !rangesTwo.length) return false;
    for (let i = 0; i < rangesOne.length; i += 1) {
        for (let j = 0; j < rangesTwo.length; j += 1) {
            if (rangesTwo[j].start < rangesOne[i].end && rangesTwo[j].end > rangesOne[i].start) {
                return true;
            }
        }
    }
    return false;
}

export function shiftOverlapped(_shift, shifts) {
    const shift = cronFromStartEndTime(_shift);
    if (!shifts || shifts.length === 0 || !shift || !shift.end || !shift.start || !_shift.active) { return false; }
    const shiftRanges = getDatesFromShift(shift);
    for (let i = 0; i < shifts.length; i += 1) {
        if (shifts[i].active && shifts[i].uid !== _shift.uid) {
            const shiftsRanges = getDatesFromShift(shifts[i]);
            if (rangesOverlapped(shiftRanges, shiftsRanges)) { return true; }
        }
    }
    return false;
}

/**
 * @function
 * @param {String} timestamp - ISO 8601 time
 * @param {String | Number} duration - time duration in minutes
 * @param {String} unit - time unit
 * @returns {Boolean}
* */
export function isTsBeforeDuration(timestamp, duration= 0, unit = 'minutes') {
    if (!timestamp) return false;
    const now = moment();
    return moment(timestamp).add(duration, unit).isBefore(now);
}

/**
 * @function
 * @param {String} userRole - user role
 * @param {Array<Object>} usersAccessControl - the list of user Access Control options
 * @param {Number} lastActivityTs - last user activity timestamp in milliseconds
 * @param {Number} now - compared timestamp in milliseconds
 * @returns {Boolean}
* */
export function isSessionExpired(userRole, usersAccessControl, lastActivityTs, now) {
    const currentUserAccessControl = usersAccessControl.find(uac => uac.role === userRole);
    const sessionTimeout = currentUserAccessControl?.sessionTimeout; // in minutes

    if (!sessionTimeout) return false;

    const diff = (now - lastActivityTs) / (1000 * 60);

    return diff > sessionTimeout;
}

/**
 * @param {Number} duration
 * @param {String} unit - minutes e.g. minutes|seconds|hours|days..
 * @returns {String}
* */
export function durationToDaysHumanize(duration, unit) {
    const days = moment.duration(duration, unit).asDays();
    if (days === 1) return `${days} day`;

    return `${days} days`;
}

/**
 * @param {Number} duration
 * @param {String} unit - minutes e.g. minutes|seconds|hours|days..
 * @returns {String}
 * */
export function durationToHoursHumanize(duration, unit) {
    const hours = moment.duration(duration, unit).asHours();
    if (Number.isInteger(hours) && hours === 1) return `${hours} hour`;
    if (Number.isInteger(hours) && hours > 1) return `${hours} hours`;
    const minutes = moment.duration(duration, unit).asMinutes();

    return `${minutes} min`;
}
