import _isEmpty from 'lodash/isEmpty';
import _Get from 'lodash/get';

import Trackable from 'stores/classes/Trackable';
import TrackableFilterHandle from 'stores/classes/TrackableFilterHandle';
import { T2T_TYPE, FILTER } from 'common/constants';

function getPlaybackTrackables(state, sublocation) {
    const { currentTs } = state.playback;
    return state.playback.trackables.filter((trackable) => {
        // Trackable could have many assoc during playback period
        const association = state.playback.associations
            .find((assoc) => {
                const assocCreated = (new Date(assoc.created_at)).getTime();
                const assocRemoved = (new Date(assoc.removed_at)).getTime();
                const toCreated = (new Date(trackable.created_at)).getTime();
                const toRemoved = (new Date(trackable.removed_at)).getTime();

                const isTrackableAssociation = assoc.trob_uid === trackable.uid;
                const associationOpened = assocCreated < currentTs && assocRemoved > currentTs;
                const trackableExists = toCreated < currentTs && toRemoved > currentTs;
                const trackableInSublocation = trackable.position && trackable.position.sl_id === sublocation.uid;
                return (
                    trackableExists && isTrackableAssociation && associationOpened && trackableInSublocation
                    && assoc.isAssociationMacAddressUidEqualTo(trackable.position.mac || trackable.position.mac_address));
            });
        return !!association;
    });
}

function formatVisibleTrackable(_trackable, alarms, groups, selectedItems, selectedAlarms) {
    const trackable = new Trackable(_trackable.getProperties());
    trackable.isSelected = selectedItems ? !!selectedItems.find((item) => item.uid === trackable.uid) : false;
    trackable.groupList = (trackable.groups && !_isEmpty(trackable.groups))
        ? trackable.groups.reduce((memoGroups, groupUid) => {
            const group = groups[groupUid];
            if (group) {
                memoGroups.push(group.name);
            }
            return memoGroups;
        }, []) : [];
    
    trackable.alarms = alarms ? alarms.filter((alarm) => trackable.uid === alarm.obj_uid) : [];
    trackable.isAlarmed = !!trackable.alarms.length;
    if (!trackable.isSelected) {
        trackable.isSelected = selectedAlarms ? !!selectedAlarms.find((alarm) => alarm.obj_uid === trackable.uid) : false;
    }
    
    return trackable;
}

export function getTrackablesBySublocation(state) {
    const sublocation = state.siteView.selectedSublocation;
    const trackables = Object.values(state.entities.trackables.byUid);
    
    const { isOpenPlayBackControl } = state.playback;
    if (!sublocation) {
        return [];
    }
    
    if (!isOpenPlayBackControl) {
        return trackables.filter((trackable) => trackable.position && trackable.position.sl_id === sublocation.uid
            && !!state.entities.associations.byObjUid[trackable.uid]);
    }
    
    return getPlaybackTrackables(state, sublocation);
}

export function getVisibleTrackables(state) {
    const { playback, siteView: { selectedItems }, siteViewFilter: { objectFilter } } = state;
    
    const isTrackablesVisible = !!objectFilter.selectedGroups.length;
    const isPlaybackOpen = playback && playback.isOpenPlayBackControl;
    
    const nodes = state.entities.nodes.byMac;
    const groups = state.entities.groups.byUid;
    const alarms = (isPlaybackOpen ? state.playback.alarms : state.entities.alarms).filter((alarm) => !alarm.acknowledged);
    const selectedAlarms = selectedItems
        ? selectedItems.filter((item) => alarms.some((alarm) => item.uid === alarm.uid))
        : [];
    let hiddenTkoUnresponsiveCount = 0;
    const visibleTrackables = [];
    
    if (isTrackablesVisible) {
        getTrackablesBySublocation(state)
            .forEach((_trackable) => {
                if (_trackable.paired) {
                    const pairedTkoGroups = (state.entities.trackables.byUid[_trackable.paired] || {}).groups || [FILTER.NA];
                    _trackable.pairedGroups = pairedTkoGroups.concat(_trackable.groups || [FILTER.NA]);
                }
                const trackableFilter = new TrackableFilterHandle(_trackable, nodes, objectFilter, isPlaybackOpen);
                if (trackableFilter.isVisible) {
                    const trackable = formatVisibleTrackable(_trackable, alarms, groups, selectedItems, selectedAlarms);
                    if (trackable.paired && _Get(trackable, 'tag_config.t2t_cfg.role') === T2T_TYPE.MONITOR) {
                        const tko = state.entities.trackables.byUid[trackable.paired];
                        trackable.position = tko && tko.position;
                    }
                    visibleTrackables.push(trackable);
                } else if (!trackableFilter._visibleAsUnresponsive) {
                    hiddenTkoUnresponsiveCount += 1;
                }
            });
    }
    
    return { visibleTrackables, hiddenTkoUnresponsiveCount };
}

export const getNodeInfoByMac = (mac_address, nodes) => {
    if (mac_address) {
        const node = nodes[mac_address.toUpperCase()];

        if (node) {
            return {
                battery: node.battery,
                last_heard: node.last_heard,
                charging: node.charging,
            };
        }
    }

    return null;
};
