import axios from 'axios';
import { currentVersion } from 'common/version';
import {
    getLocalStorageValue,
    getSessionStorageValue,
    setSessionStorageKeyValue,
    getQueryParam,
} from 'utils/browserStorage';
import { STORAGE_KEY } from 'common/constants';

export * from './requests';

// todo: duplicate
export const getCurrentFormattedVersion = () => {
    const { versionMajor, versionMiddle, versionMinor } = currentVersion();
    return [versionMajor, versionMiddle, versionMinor].join('.');
};

export const setParamsToHeader = ({
    hostName, token, email, project, company,
}, companyUid) => {
    if (process.env.NODE_ENV === 'production') {
        axios.defaults.headers.common['X-User-Agent'] = `SiteWise v${getCurrentFormattedVersion()}`;
    }

    if (hostName) {
        axios.defaults.baseURL = hostName;
    }

    if (token) {
        axios.defaults.headers.common['X-User-Token'] = token;
    }
    if (email) {
        axios.defaults.headers.common['X-User-Email'] = email;
    }

    if (project) {
        axios.defaults.headers.common['X-User-Project'] = project.uid;
    } else if (axios.defaults.headers.common['X-User-Project']) {
        delete axios.defaults.headers.common['X-User-Project'];
    }

    if (company) {
        axios.defaults.headers.common['X-User-Subcontractor'] = company.uid || null;
    } else {
        axios.defaults.headers.common['X-User-Subcontractor'] = companyUid;
    }
};

export const setTokenToHeader = (token) => {
    axios.defaults.headers.common['X-User-Token'] = token;
};

export const setProjectUidToHeader = (projectUid) => {
    axios.defaults.headers.common['X-User-Project'] = projectUid;
};

export const removeProjectFromHeader = () => {
    delete axios.defaults.headers.common['X-User-Project'];
};

let tokenRefreshing = false;
let failedRequestsQueue = [];

const processQueue = (error, token = null) => {
    failedRequestsQueue.forEach((prom) => {
        if (error) {
            prom.reject(error);
        } else {
            prom.resolve(token);
        }
    });
    
    failedRequestsQueue = [];
    tokenRefreshing = false;
};

const target_server = getQueryParam('target_server');
axios.interceptors.request.use((config) => {
    const commonQueryParams = { target_server };

    // Convert the query params into a string and append to the existing URL
    const url = new URL(config.url, config.baseURL);
    Object.entries(commonQueryParams).forEach(([key, value]) => {
        if (value) {
            url.searchParams.set(key, value); // Ensures no duplicates
        }
    });

    config.url = url.toString().replace(config.baseURL, ''); // Maintain relative URL

    return config;
}, (error) => Promise.reject(error));

axios.interceptors.response.use((response) => response, (error) => {
    const originalRequest = error.config;
    const isRefreshTokenRequest = originalRequest.url.includes('/refresh_token');
    if (error.response && error.response.status === 401 && !isRefreshTokenRequest && !originalRequest._retry) {
        if (tokenRefreshing) {
            return new Promise((resolve, reject) => {
                failedRequestsQueue.push({ resolve, reject });
            }).then((token) => {
                originalRequest.headers['X-User-Token'] = token;
                return axios(originalRequest);
            }).catch((err) => Promise.reject(err));
        }
    
        originalRequest._retry = true;
        tokenRefreshing = true;
        const email = getSessionStorageValue(STORAGE_KEY.USER_EMAIL);
        const token = getSessionStorageValue(STORAGE_KEY.USER_TOKEN);
        
        return new Promise((resolve, reject) => {
            axios.post('/api/v2/auth/refresh_token/', { email, token })
                .then(({ data }) => {
                    axios.defaults.headers.common['X-User-Token'] = data.data.token;
                    originalRequest.headers['X-User-Token'] = data.data.token;
                    setSessionStorageKeyValue(STORAGE_KEY.USER_TOKEN, data.data.token);
                    processQueue(null, data.data.token);
                    if (originalRequest.url.includes('/auth/validate_token')) {
                        originalRequest.data = JSON.stringify({ ...JSON.parse(originalRequest.data), token: data.data.token });
                    }
                    resolve(axios(originalRequest));
                })
                .catch((err) => {
                    processQueue(err, null);
                    reject(err);
                });
        });
    }
    return Promise.reject(error);
});
