import { PLAYBACK } from 'worker/common/constants';
import PlaybackDataMethods from './PlaybackDataMethods';
import PlaybackServerApi from './PlaybackServerApi';

export default class PlaybackHandlers extends PlaybackDataMethods {
    constructor(params) {
        super();
        this.userEmail = params.email;
        this.userToken = params.token;
        this.projectUid = params.project;
        this.hostname = params.hostname;
        this.serverApi = new PlaybackServerApi(params);
        this.historyPos = new Map();
        this.initialPos = new Map();
        this.trackables = [];
        this.finishTs = null;
        this.currentTs = null;
        this.maxTankTs = 0;
        this.maxPosTs = 0;
        this.maxAlarmTs = 0;

        this.firstDownloadHistoryPos = false;
        this.hasPendingRequest = true;
        this.receivedInitialPositions = false;
        this.receivedAllPositions = false;
        this.receivedAllAlarms = false;
    }
    
    _getPlaybackDeltaTime = () => {
        const tankSizeLessThenCritical = (this.maxTankTs - this.currentTs) < PLAYBACK.CRITICAL_LEVEL;
        
        if (this.hasPendingRequest && tankSizeLessThenCritical) {
            this._notifyClientAboutPending();
            return 0;
        }
        return PLAYBACK.DELTA_TIME;
    };
    
    _getHistoryMapKeyTime = (time) => {
        const currentTime = this.currentTs;
        let nextTs = time < (currentTime + PLAYBACK.DELTA_TIME) ? currentTime : null;
    
        // Need to make map custom step size what is defined by PLAYBACK.DELTA_TIME
        const deltaSteps = Math.floor((time - currentTime) / PLAYBACK.DELTA_TIME);
        if (deltaSteps > 0) {
            nextTs = currentTime + (PLAYBACK.DELTA_TIME * deltaSteps);
        }
        return nextTs;
    };
    
    _checkPlaybackState = () => {
        if (this.receivedAllPositions && this.receivedAllAlarms && this.receivedInitialPositions) {
            // Tank size should be determined only when all entities were received
            this.maxTankTs = this.maxPosTs > this.maxAlarmTs ? this.maxPosTs : this.maxAlarmTs;
            this.hasPendingRequest = false;
            this._notifyClientAboutReadyState();
            
            if (this.firstDownloadHistoryPos) {
                // let playing after done getting positions
                this._changeClientPlaybackParams({ isOpenPlayBackControl: true, isReadyToPlay: true });
            } else {
                this._changeClientPlaybackParams({ isReadyToPlay: true });
            }
            // back to initial state
            this.receivedAllPositions = false;
            this.receivedAllAlarms = false;
        }
    };
    
    handlePlaybackMessage = (message) => {
        const { data, subscription } = message;
        
        switch (subscription) {
            case 'pos':
            case '#pos':
                this._handleReceivedPosition(data);
                break;
            case 'alarm':
            case '#alarm':
                this._handleReceivedAlarm(data);
                break;
            case 'initial_pos':
            case '#initial_pos':
                this._handleReceivedInitialPosition(data);
                break;
            default:
                console.warn('[PlaybackHandlers] Playback message from ws did not handled ', message);
        }
    };
    
    generatePlayback = (data) => {
        const playback = { ...data };

        this.clearHistory();
        this.serverApi.subscribe(this.handlePlaybackMessage);
        this.hasPendingRequest = true;
        
        if (playback.currentTs) {
            this.currentTs = playback.currentTs;
            playback.playFrom = playback.currentTs;
            delete playback.currentTs;
        } else {
            this.currentTs = playback.playFrom;
            this.firstDownloadHistoryPos = true;
        }

        this.finishTs = playback.playTo;
        this.sublocation_uid = playback.sublocation_uid;
        this.trackables = playback.trackables;
        
        const playbackSize = playback.playTo - playback.playFrom;
        const diff = (playbackSize < PLAYBACK.MAX_SIZE_TIME) ? playbackSize : PLAYBACK.MAX_SIZE_TIME;
        
        playback.playTo = playback.playFrom + diff;
        playback.finishTs = this.finishTs;
        
        this._changeClientPlaybackParams({ currentTs: this.currentTs });
        this.serverApi.requestInitData(playback);
    };
    
    requestDataFromPlaybackStore = (time) => {
        this.currentTs = time;
        const keyValueHistoryMap = this.historyPos.get(time);
        if (keyValueHistoryMap) {
            const positions = keyValueHistoryMap.pos ? keyValueHistoryMap.pos.values() : [];
            const alarms = keyValueHistoryMap.alarm ? keyValueHistoryMap.alarm.values() : [];
            this._sendDataToClient(positions, alarms);
            this.historyPos.delete(time);
        }
    
        const increasedCurrentTs = time + this._getPlaybackDeltaTime();
        this._changeClientPlaybackParams({ currentTs: increasedCurrentTs });
        
        const tankCurrentSize = this.maxTankTs - this.currentTs;
        const leftOverSize = this.finishTs - this.currentTs;
        const isMaxTsLessFinishTs = this.maxTankTs < this.finishTs;
        const tankLevelLessMinimum = tankCurrentSize < PLAYBACK.MIN_SIZE_TIME;
    
        if (isMaxTsLessFinishTs && tankLevelLessMinimum && !this.hasPendingRequest) {
            const dif = leftOverSize < (PLAYBACK.MAX_SIZE_TIME - tankCurrentSize) ? leftOverSize : PLAYBACK.MAX_SIZE_TIME - tankCurrentSize;
    
            const playTo = this.currentTs + dif;
            const params = {
                playFrom: this.currentTs,
                playTo: playTo > this.finishTs ? this.finishTs : playTo,
                sublocation_uid: this.sublocation_uid,
                trackables: this.trackables || [],
            };
            this.serverApi.pumpData(params);
            this.hasPendingRequest = true;
        }
    };
    
    clearHistory = () => {
        this.serverApi.unsubscribe(this.handlePlaybackMessage);
        this.historyPos.clear();
        
        this.receivedInitialPositions = false;
        this.maxAlarmTs = 0;
        this.maxPosTs = 0;
        this.maxTankTs = 0;
    }
}
