import { HTTP_STATUS } from 'common/constants';
import * as api from './api';
import { formatPos, formatAlarm } from './utils/formatters';

function handleTooManyRequests(err, method) {
    const status = err.response && err.response.status;
    if (status === HTTP_STATUS.TOO_MANY_REQUESTS) {
        setTimeout(method, 1500);
        return true;
    }
    return false;
}
export default class PlaybackServerApi {
    constructor(props) {
        this.userEmail = props.email;
        this.userToken = props.token;
        this.projectUid = props.project;
        this.hostname = props.hostname;
        this.subscribers = [];
        this.queryParams = props.queryParams;

        api.setParamsToHeader({ hostName: props.hostname, token: props.token, email: props.email, project: props.project });
    }
    
    subscribe(fn) {
        this.subscribers.push(fn);
    }
    
    unsubscribe(fn) {
        this.subscribers = this.subscribers.filter((subscriber) => subscriber !== fn);
    }
    
    fire(data) {
        this.subscribers.forEach((subscriber) => {
            subscriber(data);
        });
    }
    
    handleReceivedData(dataSet, subscription, params, formatter = (item) => item) {
        if (dataSet.length === 0) {
            this.fire({ data: { message: 'done', playTo: new Date(params.to).getTime() }, subscription });
        }
        dataSet.forEach((item, index) => {
            this.fire({ data: formatter(item), subscription });
            if (index >= dataSet.length - 1) {
                this.fire({ data: { message: 'done', playTo: new Date(params.to).getTime() }, subscription });
            }
        });
    }

    requestInitialPositions(params) {
        api.getInitialPos(params).then((positions) => {
            this.handleReceivedData(positions, 'initial_pos', params, formatPos);
        }).catch((err) => {
            console.error('[PlaybackServerApi] getInitialPos: ', err);
            if (!handleTooManyRequests(err, () => {
                this.requestInitialPositions(params);
            })) {
                this.handleReceivedData([], 'initial_pos', params);
            }
        });
    }
    
    requestPositions(params) {
        api.getPos(params).then((positions) => {
            this.handleReceivedData(positions, 'pos', params, formatPos);
        }).catch((err) => {
            console.error('[PlaybackServerApi] getPos: ', err);
            if (!handleTooManyRequests(err, () => {
                this.requestPositions(params);
            })) {
                this.handleReceivedData([], 'pos', params);
            }
        });
    }
    
    requestAlarms(params) {
        api.getPlaybackAlarms(params).then((alarms) => {
            this.handleReceivedData(alarms, 'alarm', params, formatAlarm);
        }).catch((err) => {
            console.error('[PlaybackServerApi] getPlaybackAlarms: ', err);
            if (!handleTooManyRequests(err, () => {
                this.requestAlarms(params);
            })) {
                this.handleReceivedData([], 'alarm', params);
            }
        });
    }
    
    requestInitData(params) {
        const from = new Date(params.playFrom).toISOString();
        const to = new Date(params.playTo).toISOString();
        const area_uid = params.sublocation_uid;
        const trackables = params.trackables;
        
        this.requestInitialPositions({ from, to, area_uid, trackables, target_server: this.queryParams.target_server });
        this.requestPositions({ from, to, area_uid, trackables, target_server: this.queryParams.target_server });
        this.requestAlarms({ from, to, area_uid, trackables, sublocation_uid: area_uid, target_server: this.queryParams.target_server });
    }
    
    pumpData(params) {
        const from = new Date(params.playFrom).toISOString();
        const to = new Date(params.playTo).toISOString();
        const area_uid = params.sublocation_uid;
        const trackables = params.trackables;
    
        this.requestPositions({ from, to, area_uid, trackables, target_server: this.queryParams.target_server });
        this.requestAlarms({ from, to, area_uid, trackables, sublocation_uid: area_uid, target_server: this.queryParams.target_server });
    }
}
