const webpack = require('webpack');
const path = require('path');
const HtmlWebpackPlugin = require('html-webpack-plugin');
const CopyPlugin = require('copy-webpack-plugin');
const { CleanWebpackPlugin } = require('clean-webpack-plugin');

const PUBLIC_PATH = (process.env.PUBLIC_PATH || '/').endsWith('/')
    ? process.env.PUBLIC_PATH || '/'
    : (process.env.PUBLIC_PATH || '/') + '/';


const config = {
    entry: {
        app: './src/js/client.js'
    },
    output: {
        path: path.resolve(__dirname, 'build'),
        publicPath: PUBLIC_PATH,
        filename: 'client.min.js',
        chunkFilename: '[chunkhash].chunk.js',
    },
    optimization: {
        splitChunks: {
            cacheGroups: {
                vendor: {
                    chunks: 'initial',
                    name: 'vendor',
                    test: 'vendor',
                    enforce: true
                },
            }
        },
    },
    devServer: {
        open: true,
        host: 'localhost',
    },
    plugins: [
        new HtmlWebpackPlugin({
            inject: false,
            filename: 'index.html',
            template: 'public/index.html'
        }),
        new webpack.DefinePlugin({
            'process.env.NODE_ENV': JSON.stringify(process.env.NODE_ENV),
            'process.env.PUBLIC_PATH': JSON.stringify(process.env.PUBLIC_PATH),
        }),
        new webpack.ProvidePlugin({
            process: 'process/browser'
        }),
        new CleanWebpackPlugin({
            cleanOnceBeforeBuildPatterns: [
                '**/*',
                '!client_config.js',
            ]
        }),
        new CopyPlugin({
            patterns: [
                { from: "public/_client_config.js", to: "_client_config.js" }
            ]
        }),
        new CopyPlugin({
            patterns: [
                { from: "src/images/Warning.jpg", to: "images/Warning.jpg" }
            ]
        }),
        new webpack.ContextReplacementPlugin(/moment[/\\]locale$/, /en-gb|ja|ru|zh-cn/),
    ],
    resolve: {
        extensions: ['.json', '.js', '.scss'],
        alias: {
            joi: 'joi-browser',
            'api': path.resolve(__dirname, 'src/modules/api/'),
            'worker': path.resolve(__dirname, 'src/modules/worker/'),
            'spaghetti': path.resolve(__dirname, 'src/modules/spaghetti/'),
            'playback': path.resolve(__dirname, 'src/modules/playback/'),
            'near-miss-event': path.resolve(__dirname, 'src/modules/near-miss-event/'),
            'common': path.resolve(__dirname, 'src/js/common/'),
            'components': path.resolve(__dirname, 'src/js/components/'),
            'modals': path.resolve(__dirname, 'src/js/modals/'),
            'pages': path.resolve(__dirname, 'src/js/pages/'),
            'stores': path.resolve(__dirname, 'src/js/stores/'),
            'utils': path.resolve(__dirname, 'src/js/utils/'),
            'fonts': path.resolve(__dirname, 'src/fonts/'),
        },
        modules: ['node_modules'],
        fallback: {
            "crypto": false,
            "stream": require.resolve("stream-browserify"),
            "util": require.resolve("util/")
        }
    },
    module: {
        rules: [
            {
                test: /\.(js|jsx)$/i,
                loader: 'babel-loader',
                options: {
                    presets: ['@babel/preset-react']
                }
            },
            {
                test: /\.worker\.(c|m)?js$/i,
                loader: "worker-loader",
                options: {
                    filename: "[name].[contenthash].worker.js",
                },
            },
            {
                test: /\.css$/i,
                use: ['style-loader','css-loader'],
            },
            {
                test: /\.s[ac]ss$/i,
                use: ['style-loader', 'css-loader', 'sass-loader'],
            },
            {
                test: /\.(eot|svg|ttf|woff|woff2|png|jpg|gif)$/i,
                type: 'asset',
            },
        ],
    }
};

module.exports = () => {
    return config;
};
